// Adaba.java
// $Id: Adaba.java 1965 2008-06-09 13:47:22Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
// 
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

// last modified: 
// 01/10/2002:: no closing of frames

import at.iem.adaba.*;
import at.iem.adaba.gui.*;
import at.iem.adaba.transcriber.*;

import at.iem.tools.gui.CustomFileFilter;
import javax.swing.*;
import java.io.File;
import java.awt.event.WindowListener;
import java.awt.event.WindowEvent;

public class Adaba extends JPanel implements WindowListener {
    JFrame frame;
    at.iem.adaba.database.TranscriptionDataBase dataBase;
    at.iem.tools.Quittable guiquit;

    private byte wantTranscriber;

    File file;

    public Adaba(String args[]) throws Exception {
        AdabaShared.createStartupProgressBar();
	parseArgs(args);
        heapTest();

        AdabaShared.createAdabaFont("fonts/Gentium.ttf"); /* just in case, we try to fall back to an embedded font */

	AdabaShared.wantTranscriber=(wantTranscriber>0);
	AdabaShared.read();
	if(file==null)file=AdabaShared.getDBfile();
	AdabaShared.wantTranscriber=(wantTranscriber>0); /* ignore whatever was in the properties */

	AdabaConstants.setLook(0);
	if (file==null){
            AdabaShared.closeSplash();
	    JFileChooser fc = new JFileChooser(".");
	    fc.addChoosableFileFilter(new CustomFileFilter(AdabaShared.getLabel("FileATD"), ".atd"));
	    if (fc.showDialog(this, "adaba")==JFileChooser.APPROVE_OPTION){
		file=fc.getSelectedFile();
		if (!file.exists()){
		    if (!file.getName().toLowerCase().endsWith(".atd"))
			file = new File(file.getParent(), file.getName()+".atd");
		}
	    } else System.exit(0);
	}
	at.iem.tools.Logger.println("opening database");
	dataBase = at.iem.adaba.database.Factory.open(file.getAbsolutePath(), "", "", !AdabaShared.wantTranscriber);
        AdabaShared.setDB(dataBase);
	at.iem.tools.Logger.println("opened database");
	if (dataBase==null){
            AdabaShared.closeSplash();
	    JOptionPane.showMessageDialog(this,AdabaShared.getLabel("ErrorLoadFatal"));
	    System.exit(0);
	}

        checkDataPath();
	at.iem.tools.Logger.println("checked data path");

	if (AdabaShared.wantTranscriber){
	    frame = new JFrame(AdabaShared.getLabel("TranscriberTitle")+
			       " "+AdabaVersion.getTranscriberVersion()+
			       "   -   "+file.toString());
	    TransGUI gui = new TransGUI(frame, dataBase);
	    frame.getContentPane().add("Center", gui);
	    frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
            guiquit=null;
	} else {
	    frame = new JFrame(AdabaShared.getLabel("AdabaTitle")+
			       " "+AdabaVersion.getAdabaVersion()+
			       "   -   "+file.toString());
	    at.iem.adaba.GUI gui = new at.iem.adaba.GUI(dataBase);
            at.iem.adaba.gui.Menu menu = new at.iem.adaba.gui.Menu(gui);
            gui.setHelpSelection(menu);
            menu.setEntrySelector(gui.getResultSelector());

            frame.setJMenuBar(menu);
	    frame.getContentPane().add("Center", gui);
	    frame.setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
            frame.addWindowListener(this);
            guiquit=gui;
	}
	frame.pack();
	frame.setVisible(true);
	frame.toFront();
        {
          java.net.URL imageURL = ClassLoader.getSystemResource("resources/icons/adaba.gif");
          if (imageURL != null) {
                java.awt.Image image = java.awt.Toolkit.getDefaultToolkit().createImage(imageURL);
                if (image != null) {
                  frame.setIconImage(image);
                }
          }
        }
    }

    boolean checkDataPath() {
        File pfad=AdabaShared.getPWD();
        if(pfad!=null) {
            File idfile=new File(pfad, "adaba.id");
            at.iem.tools.Logger.println("searching for "+idfile);

            if(idfile.exists())
                return true;
        }

        AdabaShared.closeSplash();

        /* display a message box to ask to insert the DVD! */
        JFrame frame = null;
        String title=AdabaShared.getLabel("TitleMediumMissing");
        String msg=AdabaShared.getLabel("MsgMediumMissing");
        at.iem.tools.gui.CustomFileFilter cff = new at.iem.tools.gui.CustomFileFilter("adaba-ID file", "adaba", ".id");

        while(true) {
            int result=JOptionPane.showConfirmDialog(frame, msg, title, JOptionPane.YES_NO_CANCEL_OPTION);

            switch(result) {
            case 0:
                {
                    
                    pfad=AdabaShared.getPWD();

                    JFileChooser fc = new JFileChooser(".");
                    fc.setFileFilter(cff);
                    fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
                    if (fc.showOpenDialog(this)==JFileChooser.APPROVE_OPTION){
                        pfad=fc.getSelectedFile().getParentFile();
                    } else {
                        pfad=null;
                    }

                    if(pfad!=null) {
                        File idfile=new File(pfad, "adaba.id");
                        if(idfile.exists()) {
                            AdabaShared.setPWD(pfad);
                            return true;
                        }
                    }
                }
                break;
            case 1:
                return false;
            default:
                System.exit(0);
            }
            
            System.out.println("result: "+result);
        }
    }


    void parseArg(String arg){
        if(arg.startsWith("--logfile=")) {
            String s_logfile=new String(arg.substring(10));
            at.iem.tools.Logger.setLogfile(new File(s_logfile));
        }
	else if(arg.equalsIgnoreCase("--transcriber"))wantTranscriber=1;
	else if(arg.equalsIgnoreCase("--adaba"))wantTranscriber=-1;
	else if(arg.length()>0)file=new File(arg);
    }
    void parseArgs(String args[]){
	file=null;
	if (args.length==0)return;
	
	int i=-1;
	while(++i<args.length)parseArg(args[i]);
    }

    static void heapTest() {
        // Get current size of heap in bytes
        long heapSize = Runtime.getRuntime().totalMemory();
    
        // Get maximum size of heap in bytes. The heap cannot grow beyond this size.
        // Any attempt will result in an OutOfMemoryException.
        long heapMaxSize = Runtime.getRuntime().maxMemory();
        
        // Get amount of free memory within the heap in bytes. This size will increase
        // after garbage collection and decrease as new objects are created.
        long heapFreeSize = Runtime.getRuntime().freeMemory();

        at.iem.tools.Logger.println("current heap size: "+heapSize);
        at.iem.tools.Logger.println("maximum heap size: "+heapMaxSize);
        at.iem.tools.Logger.println("to-free heap size: "+heapFreeSize);
    }

    

    public static void main(String s[]) throws Exception {
	new Adaba(s);
    }

    static final long serialVersionUID = 1L;


    /* WindowListener interface implementation */
    public void 	windowClosing(WindowEvent e){
        heapTest();
        if(null!=guiquit) 
            guiquit.quit();
    }
    public void 	windowActivated(WindowEvent e){ }
    public void 	windowDeactivated(WindowEvent e){ }
    public void 	windowDeiconified(WindowEvent e){ }
    public void 	windowIconified(WindowEvent e){ }
    public void 	windowOpened(WindowEvent e){ }
    public void 	windowClosed(WindowEvent e){ }    
}
