// AdabaProperties.java
// $Id: AdabaProperties.java 1827 2007-06-28 13:28:20Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
// 
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

package at.iem.adaba;

import java.util.Stack;
import java.util.ArrayList;

import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.*;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;

/**
 * <AdabaProperties>
 * liest und schreibt ein Konfigurations-File 
 * file-location: working-directory oder home (?)
 * <home>/.adaba/config.xml unter unix
 * <home>/adaba/config.xml unter windos
 *
 * <home> = System.getProperty("user.home");
 */

import java.io.*;
import at.iem.adaba.AdabaShared;

public class AdabaProperties  extends DefaultHandler {

    private ArrayList<String> sound_editNames   = new ArrayList<String>();
    private ArrayList<String> sound_editScripts = new ArrayList<String>();
    private ArrayList<String> regions  = new ArrayList<String>();
    private ArrayList<String> regionID = new ArrayList<String>();
    private ArrayList<String> entryflags = new ArrayList<String>();

    private static String dummyString;

    private java.util.prefs.Preferences prefs = null;

    public AdabaProperties(File file) {
        property_file=file;
        readPreferences();
        readExternalFile();
    }

    public AdabaProperties() {
        this(null);
    }

    public boolean readPreferences() {
        boolean result1=false, result2=false;
        try {
            at.iem.tools.Logger.println("reading system prefs");
            java.util.prefs.Preferences sprefs=java.util.prefs.Preferences.systemRoot();
            result1=readPreferences(sprefs);
        }  catch (Exception e){
            at.iem.tools.Logger.println("error reading system preferences: " + e);
        }
        
        try {
            at.iem.tools.Logger.println("reading user prefs");
            prefs=java.util.prefs.Preferences.userRoot();
            result2=readPreferences(prefs);
        }  catch (Exception e){
            at.iem.tools.Logger.println("error reading user preferences: " + e);
        }

        at.iem.tools.Logger.println("preferences: " + prefs);


        return (result1||result2);
    }

    private static String readPrefNode(java.util.prefs.Preferences p, String key, String def) {
        try {
            return p.get(key, def);
        } catch (Exception e) {
            at.iem.tools.Logger.println("error querying preference key '"+key+"': " + e);
        }
        return def;
    }
    private static String readPrefNode(java.util.prefs.Preferences p, String key) {
        return readPrefNode(p, key, null);
    }
    private static int readPrefNodeInt(java.util.prefs.Preferences p, String key, int def) {
        try {
            return p.getInt(key, def);
        } catch (Exception e) {
            at.iem.tools.Logger.println("error querying preference key '"+key+"': " + e);
        }
        return def;
    }
    private static boolean readPrefNodeBoolean(java.util.prefs.Preferences p, String key, boolean def) {
        try {
            return p.getBoolean(key, def);
        } catch (Exception e) {
            at.iem.tools.Logger.println("error querying preference key '"+key+"': " + e);
        }
        return def;
    }
    private static void readPrefNodeIPAgroup(java.util.prefs.Preferences p, String path) {
        try {

            if(p.nodeExists(path)) {
                java.util.prefs.Preferences ipaprefs=p.node(path);
                String[]keys=ipaprefs.keys();
                int count=keys.length;
                String[]values=new String[keys.length];
                for(int i=0; i<keys.length; i++) {
                    String value=readPrefNode(ipaprefs, keys[i]);
                     at.iem.tools.Logger.println("IPAprefnode: '"+keys[i]+"': "+value);
                    if(null!=value)
                        AdabaShared.setIPAgroupReplacement(keys[i], value);
                }
            }
        } catch (Exception e) {
            at.iem.tools.Logger.println("error querying preference key (IPAgroup): " + e);
        }
    }

    private boolean readPreferences(java.util.prefs.Preferences p) {
        try {
            p=p.node(AdabaConstants.getPreferenceRoot());
        } catch (Exception e){
            at.iem.tools.Logger.println("error setting preferences-node: " + e);
            return false;
        }

        String fontFamily = readPrefNode(p, "font_family",AdabaShared.AdabaFONT.getFamily());
        int fontStyle = readPrefNodeInt(p, "font_style", AdabaShared.AdabaFONT.getStyle());
        int fontSize = readPrefNodeInt(p, "font_size", AdabaShared.AdabaFONT.getSize());
        AdabaShared.fixMSFonts = readPrefNodeBoolean(p, "font_msfix", AdabaShared.fixMSFonts);

        if(!(null==fontFamily || -1==fontStyle || -1==fontSize)) {
            AdabaShared.AdabaFONT=new java.awt.Font(fontFamily, fontStyle, fontSize);
        }

        readPrefNodeIPAgroup(p, "ipagroup");

        String datapath = readPrefNode(p, "datapath");

        if(null!=datapath) {
            File f=new File(datapath);
            if(f.exists()) {
                AdabaShared.setPWD(f);
            }
        }

        AdabaShared.selectedGuiTab = readPrefNodeInt(p, "open_tab", AdabaShared.selectedGuiTab);

        return true;
    }

    File property_file = null;
    public File getPropertyFile() {
        if(!AdabaShared.wantTranscriber) {
            /* never read the old adaba.xml files if we are not running the transcriber! */
            property_file=null;
            return null;
        }
        if(null!=property_file && property_file.exists())return property_file;
	try {
	    /* 1. try to read config.xml from the current directory */
	    property_file = new File("adaba.xml");
	    if(property_file.exists())return property_file;
	    /* on fail, get it from the users <home>/.adaba/config.xml */
	    property_file = new File(System.getProperty("user.home"), ".adaba.xml");
	    if(property_file.exists())return property_file;
	    /* on fail, try to create it in the users <home> */
	    if(property_file.createNewFile())return property_file;
	} catch (Exception e) {
	    at.iem.tools.Logger.println("error opening ConfigFile: "+e);
	}
	property_file=null;
	return property_file;
    }
    
    public boolean readExternalFile() {
        File f=getPropertyFile();
        if(null==f)
            return false;

        if(!f.isFile() || f.length()<=0) {
            at.iem.tools.Logger.println("bad external property file: '"+f);
            return false;
        }
        try {
	    SAXParserFactory factory = SAXParserFactory.newInstance();
	    factory.setValidating(true);	
	    SAXParser saxParser = factory.newSAXParser();
	    saxParser.parse(f, this);
	} catch (Exception ex) {
	    at.iem.tools.Logger.println("error reading properties: '"+f+"'" + ex);
            return false;
	}
        return true;
    }

    static Stack<String> element_stack;
    static String eName=null; // current element name

    // these are the inherited functions

    public void startDocument() throws SAXException {
	element_stack = new Stack<String>();
    }
    public void endDocument() throws SAXException
    {
	if(sound_editNames.size()>0 && sound_editNames.size()==sound_editScripts.size()){
	    AdabaShared.SoundEditors=new String[sound_editNames.size()][2];
	    for(int i=0; i<sound_editNames.size(); i++){
		AdabaShared.SoundEditors[i][0]=sound_editNames.get(i).toString();
		AdabaShared.SoundEditors[i][1]=sound_editScripts.get(i).toString();
	    }
	}
	if(regions.size()>0 && regions.size()==regionID.size()){
	    AdabaShared.region=new String[regionID.size()][2];
	    for(int i=0; i<regions.size(); i++){
		AdabaShared.region[i][0]=regionID.get(i).toString();
		AdabaShared.region[i][1]=regions.get(i).toString();
	    }
	}
        /**
         * JMZ: disabled EntryFlag storage as this is now in the database itself
         * LATER think about merging!!!
         */
        /*
	if(entryflags.size()>0){
	    AdabaShared.EntryFlag=new String[entryflags.size()];
	    for(int i=0; i<entryflags.size(); i++){
		AdabaShared.EntryFlag[i]=entryflags.get(i).toString();
	    }
	}
        */
    }


    public void startElement(String namespaceURI,
                             String lName, // local name
                             String qName, // qualified name
                             Attributes attrs)
	throws SAXException
    {
	element_stack.push(eName); /* save the current element to the stack */
	/* a startTag for an element */
        eName = lName;
        if ("".equals(eName)) eName = qName; // namespaceAware = false
	if (eName.equals("Font")){
	    if (attrs != null) {
		String name="";
		int style=0;
		int size=12;

                boolean fixMS=AdabaShared.fixMSFonts;

		for (int i = 0; i < attrs.getLength(); i++) {
		    String aName = attrs.getLocalName(i); // Attr name 
		    if ("".equals(aName)) aName = attrs.getQName(i);

		    if("name".equals(aName))name=attrs.getValue(i);
		    else if ("style".equals(aName))style=Integer.parseInt(attrs.getValue(i));
		    else if ("size".equals(aName))size=Integer.parseInt(attrs.getValue(i));
		    else if ("fixMicrosoft".equals(aName))fixMS=Boolean.parseBoolean(attrs.getValue(i));
		    /* 
		       aName: holds the attribute-name
		       (attrs.getValue(i)): holds the attribute-value 
		    */
		}
		AdabaShared.AdabaFONT=new java.awt.Font(name, style, size);
                AdabaShared.fixMSFonts=fixMS;
	    }
	}
	if (eName.equals("Region")){
	    if (attrs != null) {
		String name="";
		String id="";
		
		for (int i = 0; i < attrs.getLength(); i++) {
		    String aName = attrs.getLocalName(i); // Attr name 
		    if ("".equals(aName)) aName = attrs.getQName(i);

		    if("name".equals(aName))name=attrs.getValue(i);
		    else if ("id".equals(aName))id=attrs.getValue(i);
		}
		if(!("".equals(id) || "".equals(name))){
		    regionID.add(id);
		    regions.add(name);
		}
	    }
	}
	if (eName.equals("exec")){
	    if (attrs != null) {
		dummyString="";
		for (int i = 0; i < attrs.getLength(); i++) {
		    String aName = attrs.getLocalName(i); // Attr name 
		    if("".equals(aName))   aName=attrs.getQName(i);
		    if("name".equals(aName))dummyString=attrs.getValue(i);
		}
	    }
	}
	if(eName.equals("Search"))AdabaShared.TranscriberSearch=true;

	if(eName.equals("IPAgroup")){
	    if (attrs != null) {
		String nameString="";
		String abbrString="";
		for (int i = 0; i < attrs.getLength(); i++) {
		    String aName = attrs.getLocalName(i); // Attr name 
		    if("".equals(aName))   aName=attrs.getQName(i);
		    if("name".equals(aName))nameString=attrs.getValue(i);
		    else if("abbr".equals(aName))abbrString=attrs.getValue(i);
		}
		AdabaShared.setIPAgroupReplacement(nameString, abbrString);
	    }
	}
    }

    public void endElement(String namespaceURI,
                           String sName, // simple name
                           String qName  // qualified name
                          )  throws SAXException
    {
	Object eObj=element_stack.pop();
	if(eObj==null)eName=null;
	else eName=eObj.toString(); /* the encapsulating element */
    }

    public void characters(char buf[], int offset, int len)
	throws SAXException    {
        String s = new String(buf, offset, len).trim();
	/* 
	   s: holds the Data within the element
	*/
	if ("".equals(s))return;
	if(eName.equals("adabaProperties")){
	} else if (eName.equals("BaseDir")){
	    at.iem.tools.Logger.println("pwd: "+s);
	    AdabaShared.setPWD(new File(s));
	} else if (eName.equals("Application")){
	    AdabaShared.wantTranscriber="transcriber".equals(s.toLowerCase());
	} else if (eName.equals("SoundFormat")){

	} else if (eName.equals("Font")){
	       at.iem.tools.Logger.println("Font: specify characteristics as arguments instead of text");
	} else if (eName.equals("exec")){
	    if (!"".equals(dummyString)){
		sound_editNames.  add(dummyString);
		sound_editScripts.add(s);
	    }
	} else if (eName.equals("EntryFlag")){
	    if (!"".equals(dummyString)){
		entryflags.add(s);
	    }
	} else if (eName.equals("DataBase")){
	    if (!"".equals(dummyString)){
                at.iem.tools.Logger.println("ignoring database: '"+s+"'");
	    }
	} else {
	    at.iem.tools.Logger.println("hmmm............................"+s);
	}
    }

    public boolean write(){
        boolean result=writePreferences();
        System.out.println("writing prefs: "+result);

        /* we only write to an external XML file when not in deployment mode */
        if(AdabaShared.wantTranscriber)
            return writeExternalFile(!result);

        return result;
    }

    private final boolean usePreferenceSystemRoot = false;

    public boolean writePreferences() {
        boolean result=false;
        if(usePreferenceSystemRoot) {
            try {
                at.iem.tools.Logger.println("writing system prefs");

                java.util.prefs.Preferences sprefs=java.util.prefs.Preferences.systemRoot();
                result=writePreferences(sprefs);
                try {
                    if(null!=prefs)sprefs.flush();
                } catch (java.util.prefs.BackingStoreException e) {
                    at.iem.tools.Logger.println("error saving preferences: " + e);
                    result = false;
                } 
                if(result)return true;
            }  catch (Exception e){
                at.iem.tools.Logger.println("error writing preferences: " + e);
                result = false;
            }
        }
        try {
            at.iem.tools.Logger.println("writing user prefs");

            prefs=java.util.prefs.Preferences.userRoot();
            result=writePreferences(prefs);
            try {
                if(null!=prefs)prefs.flush();
            } catch (java.util.prefs.BackingStoreException e) {
                at.iem.tools.Logger.println("error saving preferences: " + e);
                result=false;
            }
        }  catch (Exception e){
            at.iem.tools.Logger.println("error writing preferences: " + e);
        }
        
        return result;
    }

    private static boolean writePrefNode(java.util.prefs.Preferences p, String key, String val) {        try {
            p.put(key, val);
        } catch (Exception e) {
            at.iem.tools.Logger.println("error writing preference key '"+key+"': " + e);
            return false;
        }
        return true;
    }
    private static boolean writePrefNode(java.util.prefs.Preferences p, String key, int val) {
        try {
            p.putInt(key, val);
        } catch (Exception e) {
            at.iem.tools.Logger.println("error writing preference key '"+key+"': " + e);
            return false;
        }
        return true;
    }

    private static boolean writePrefNode(java.util.prefs.Preferences p, String key, boolean val) {
        try {
            p.putBoolean(key, val);
        } catch (Exception e) {
            at.iem.tools.Logger.println("error writing preference key '"+key+"': " + e);
            return false;
        }
        return true;
    }

    public boolean writePreferences(java.util.prefs.Preferences p) {
        if(p==null)return false;
        try {
            p=p.node(AdabaConstants.getPreferenceRoot());
        } catch (Exception e){
            at.iem.tools.Logger.println("error setting preferences-node: " + e);
            return false;
        }

        boolean result=true;
        if (!writePrefNode(p, "font_family",AdabaShared.AdabaFONT.getFamily()))
            result=false;
        if (!writePrefNode(p, "font_style", AdabaShared.AdabaFONT.getStyle()))
            result=false;
        if(!writePrefNode(p, "font_size",  AdabaShared.AdabaFONT.getSize()))
            result=false;
        if(!writePrefNode(p, "font_msfix",  AdabaShared.fixMSFonts))
            result=false;

        if(null!=AdabaShared.getPWD()) {
            if(!writePrefNode(p, "datapath",   AdabaShared.getPWD().getAbsolutePath()))
                result=false;
        }

        String[][]ipaReplace=AdabaShared.getIPAgroupReplacement();
        if(ipaReplace!=null && ipaReplace.length>0){
            java.util.prefs.Preferences ipaprefs=p.node("ipagroup");
            try {
                ipaprefs.clear();
            } catch (java.util.prefs.BackingStoreException e) {
                at.iem.tools.Logger.println("error clearing IPAgroup preference-node: " + e);
            }
            for(int i=0; i<ipaReplace.length; i++){
                if(!writePrefNode(ipaprefs, ipaReplace[i][0], ipaReplace[i][1]))
                    result=false;
            }
        }

        try {
            p.flush();
        } catch (java.util.prefs.BackingStoreException e) {
            at.iem.tools.Logger.println("error saving preferences: " + e);
            return false;
        }

        return result;
    }

    public boolean writeExternalFile(boolean writeDummy){
	/** write the properties to the file
	 */
	String encoding="UTF-8";
	encoding="ISO-8859-1";
	if(property_file==null)return false;
	try {
	    FileOutputStream ostr = new FileOutputStream(property_file); 
	    OutputStreamWriter out = new OutputStreamWriter(ostr, encoding);
	    String lineEnd =  System.getProperty("line.separator");
	    out.write("<?xml version='1.0' encoding='"+encoding+"' ?>"+lineEnd);
	    out.write("<adabaProperties version='0.1'>"+lineEnd);
	    out.write("  <Application>"+
		      (AdabaShared.wantTranscriber?"transcriber":"adaba")+
		      "</Application>"+lineEnd);
            if(writeDummy) {
                // now we store this in the preference system!
                if(!("".equals(AdabaShared.AdabaFONT))){
                    out.write("  <Font name='"+AdabaShared.AdabaFONT.getFamily()+
                              "' style='"+AdabaShared.AdabaFONT.getStyle()+
                              "' size='"+AdabaShared.AdabaFONT.getSize()+
                              "' fixMicrosoft='"+AdabaShared.fixMSFonts+
                              "'/>"+lineEnd);
                }
                if(AdabaShared.getPWD()!=null){
                    out.write("  <BaseDir>"+AdabaShared.getPWD()+"</BaseDir>"+lineEnd);
                }
            }
	    if(AdabaShared.TranscriberSearch){
		out.write("  <Search/>"+lineEnd);
	    }
	    if(AdabaShared.SoundEditors!=null && AdabaShared.SoundEditors.length>0){
		out.write(lineEnd);
		for(int i=0;i<AdabaShared.SoundEditors.length; i++){
		    out.write("  <exec name='"+AdabaShared.SoundEditors[i][0]+"'>"+
			      AdabaShared.SoundEditors[i][1]+"</exec>"+lineEnd);
		}
		out.write(lineEnd);
	    }
	    if(AdabaShared.region!=null && AdabaShared.region.length>0){
		out.write(lineEnd);
		for(int i=0;i<AdabaShared.region.length; i++){
		    out.write("  <Region id='"+AdabaShared.region[i][0]+"' name='"+
			      AdabaShared.region[i][1]+"'/>"+lineEnd);
		}
		out.write(lineEnd);
	    }
	    if(AdabaShared.EntryFlag!=null && AdabaShared.EntryFlag.length>0){
		out.write(lineEnd);
		for(int i=0;i<AdabaShared.EntryFlag.length; i++){
		    out.write("  <EntryFlag>"+AdabaShared.EntryFlag[i]+"</EntryFlag>"+lineEnd);
		}
		out.write(lineEnd);
	    }
	    
	    String[][]ipaReplace=AdabaShared.getIPAgroupReplacement();
	    if(ipaReplace!=null && ipaReplace.length>0){
		for(int i=0; i<ipaReplace.length; i++){
		    if(ipaReplace[i][1].length()>0)
			out.write("  <IPAgroup name='"+ipaReplace[i][0]
				  +"' abbr='"+ipaReplace[i][1]
				  +"'/>"+lineEnd);
		}
		out.write(lineEnd);
	    }

	    if(AdabaShared.getDB()!=null)out.write("  <DataBase>"+AdabaShared.getDB()+"</DataBase>\n");
	    out.write("</adabaProperties>"+lineEnd);
	    out.flush();
	} catch (Exception e) {
	    at.iem.tools.Logger.println("error writing ConfigFile: "+e);
	    return false;
	}
	return true;
    }


    private boolean clearExternalFile() {
	if(property_file!=null){
            try {
                boolean result=property_file.delete();
                property_file=null;
                return result;
            } catch (Exception e){
                at.iem.tools.Logger.println("error deleting ConfigFile '"+property_file+"': "+e);
               return false;
            }
        }
        return true;
    }
    private boolean clearPrefNode(java.util.prefs.Preferences p, String key) {
        try {
            p.remove(key);
        } catch (Exception e) {
            at.iem.tools.Logger.println("error clearing preference key '"+key+"': " + e);
            return false;
        }
        return true;
    }
    private boolean clearPrefNode(java.util.prefs.Preferences p) {
        try {
            p.clear();
        } catch (Exception e) {
            at.iem.tools.Logger.println("error clearing preference node '"+p+"': " + e);
            return false;
        }
        return true;
    }
    private boolean clearPreferences(java.util.prefs.Preferences p0) {
        if(p0==null)return false;
        java.util.prefs.Preferences p=null;
        try {
            p=p0.node(AdabaConstants.getPreferenceRoot());
        } catch (Exception e){
            at.iem.tools.Logger.println("error setting preferences-node: " + e);
            return false;
        }
        if(p==null)return false;
        

        boolean result=true;
        if (!clearPrefNode(p, "font_family"))
            result=false;
        if (!clearPrefNode(p, "font_style"))
            result=false;
        if(!clearPrefNode(p, "font_size"))
            result=false;
        if(!clearPrefNode(p, "font_msfix"))
            result=false;
        
        java.util.prefs.Preferences ipaprefs=p.node("ipagroup");
        if(!clearPrefNode(ipaprefs))
            result=false;


        try {
            p.flush();
        } catch (java.util.prefs.BackingStoreException e) {
            at.iem.tools.Logger.println("error saving preferences: " + e);
            return false;
        }

        return result;

    }
    private boolean clearPreferences(){
         boolean result=false;
        if(usePreferenceSystemRoot) {
            try {
                at.iem.tools.Logger.println("clearing system prefs");

                java.util.prefs.Preferences sprefs=java.util.prefs.Preferences.systemRoot();
                result=clearPreferences(sprefs);
                try {
                    if(null!=prefs)sprefs.flush();
                } catch (java.util.prefs.BackingStoreException e) {
                    at.iem.tools.Logger.println("error saving preferences: " + e);
                    result = false;
                } 
                if(result)return true;
            }  catch (Exception e){
                at.iem.tools.Logger.println("error writing preferences: " + e);
                result = false;
            }
        }
        try {
            at.iem.tools.Logger.println("clearing user prefs");

            prefs=java.util.prefs.Preferences.userRoot();
            result=clearPreferences(prefs);
            try {
                if(null!=prefs)prefs.flush();
            } catch (java.util.prefs.BackingStoreException e) {
                at.iem.tools.Logger.println("error saving preferences: " + e);
                result=false;
            }
        }  catch (Exception e){
            at.iem.tools.Logger.println("error writing preferences: " + e);
        }
        
        return result;       
    }
    public void clear() {
        clearPreferences();
        clearExternalFile();
    }


   public boolean writePreferencesTabOpen(int i) {
        boolean result=false;
        try {
            at.iem.tools.Logger.println("writing open tab to user prefs: "+i);

            prefs=java.util.prefs.Preferences.userRoot();
            at.iem.tools.Logger.println("writing to prefs: "+prefs);
            if(prefs==null)return false;

            prefs=prefs.node(AdabaConstants.getPreferenceRoot());
            at.iem.tools.Logger.println("writing to Prefs: "+prefs);
            if(prefs==null)return false;

            try {
                result=writePrefNode(prefs, "open_tab", i);
                prefs.flush();
            } catch (java.util.prefs.BackingStoreException e) {
                at.iem.tools.Logger.println("error saving preferences: " + e);
                result=false;
            }
        }  catch (Exception e){
            at.iem.tools.Logger.println("error writing preferences: " + e);
            result=false;
        }
        
        return result;
    }


}
