// AdabaShared.java
// $Id: AdabaShared.java 1966 2008-06-09 13:50:07Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
// 
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

package at.iem.adaba;

import java.awt.Font;
import java.util.HashMap;

import org.xml.sax.helpers.DefaultHandler;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.SAXParser;

import java.awt.Frame;
import java.awt.Toolkit;
import java.net.URL;
import java.io.*;

/**
 * Variablen, die geteilt werden
 */

public class AdabaShared {
    /**
     * this should never be called...
     */
    private AdabaShared(){ }


    private static at.iem.adaba.database.TranscriptionDataBase tdb=null;
    /**
     * get the current database-file
     * @return the current db-file
     */
    public static at.iem.adaba.database.TranscriptionDataBase getDB(){
        return tdb;
    }
    /**
     * set the current database
     * @param db the current db
     */
    public static void setDB(at.iem.adaba.database.TranscriptionDataBase db){
        tdb=db;
    }

    /**
     * the file that holds the database
     */
    static File db_file = new File("db/db.atd");

    /**
     * get the path to the current database-file
     * @return the path to the current db-file
     */
    public static File getDBpath(){
        if(null==db_file)
            return null;
        //System.out.println("DBpath: "+db_file.getParentFile());
        return db_file.getParentFile();
    }
    /**
     * get the current database-file
     * @return the current db-file
     */
    public static File getDBfile(){
        return db_file;
    }
    /**
     * set the current database-file
     * @param f the current db-file
     */
    public static void setDB(File f){
        if(null!=f && f.isFile())db_file=f;
    }


    /**
     * the working directory: 
     * this is either the root of the soundfile-directory, e.g.: path/to/sounds/(T/TE/TEST__AT_W.wav),
     * or an explicit path, e.g. path/to/sounds/T/TE/(TEST__AT_W.wav), <b>in this order!</b>;
     * applications have to manually check whether the file <em>TEST__AT_W.wav</em> is in
     * <em>pwd</em>/T/TE/</em> or in  <em>pwd/</em> (<b>in this order!</b>)
     * if it is <code>null</code>, the current working directory should be used
     */
    static private File pwd=new File("data");

    /**
     * get the working directory
     * @return the working directory
     */
    public static File getPWD(){
        at.iem.tools.Logger.println("getting PWD '"+pwd+"'");
        return pwd;
    }

    /**
     * set the working directory
     * @param path the working directory
     */
    public static void setPWD(File path){
        at.iem.tools.Logger.println("setting PWD to '"+path+"'");
        if(path==null){
            pwd=null;
            return;
        }
        if (path.isDirectory())pwd=path;
        else pwd=path.getParentFile();
    }

    /**
     * whether to start the transcriber or the Adaba-application
     */
    public static boolean wantTranscriber=false;

    /**
     * 
     */
    public static boolean TranscriberSearch=false;

    /**
     * the font to display text (esp: IPA)
     */
    //public static Font AdabaFONT = new Font("Arial Unicode MS", Font.PLAIN, 20);
    public static Font AdabaFONT = new Font("Gentium", Font.PLAIN, 20);

    /**
     * Microsoft's fonts are buggy, as they have broken double-width diacritics;
     * when <code>fixMSFonts</code> is set to <code>true</code>, then we think that
     * we are using such broken fonts and have to take special care
     */
    public static boolean fixMSFonts=false;

    public static void createAdabaFont(String ressourcefont) {
        java.net.URL url=ClassLoader.getSystemResource("resources/"+ressourcefont);
        if(null==url){
            at.iem.tools.Logger.println("'"+ressourcefont+"' not found in system resources");
        }
        java.io.InputStream istream=null;
        try {
            istream = url.openStream();
            try {
                Font font = Font.createFont(Font.TRUETYPE_FONT, istream).deriveFont(Font.PLAIN, 20.f);
                AdabaFONT=font;
            } catch (java.awt.FontFormatException e) {
                at.iem.tools.Logger.println("failed creating font '"+ressourcefont+"': "+e);
            }
           istream.close();
        } catch (Exception e) {
            at.iem.tools.Logger.println("failed reading font '"+ressourcefont+"': "+e);
        }
    }

    /**
     * this holds the externals editors as {name, command}-pairs::
     * the "name" is the name of the script to be displayed.
     * the "command" is a command to be executed when "name" is called; 
     * each occurence of "%s" in the command is substituted by the filename
     */
    public static String[][] SoundEditors;

    public static String[] G_TYPEN = {
	"unbestimmt", 
	"Substantiv",
	"Verb", 
	"Adjektiv",
	"Prposition", 
	"Adverb", 
	"Konjunktion",
	"Pronomen",
	"Mehrwort",
	"andere"};
    public static String[] E_TYPEN = {
	"unbestimmt",
	"Erbwort", 
	"Lehnwort", 
	"Fremdwort", 
	"Eigenname", 
	"andere"};

    public static String[][] region={
	{"AT", "sterreich"},
	{"DE", "Deutschland"}, 
	{"CH", "Schweiz"},
        {"ATBU", "Burgenland"},
        {"ATKA", "Krnten"},
        {"ATNO", "Niedersterreich"},
        {"ATOO", "Obersterreich"},
        {"ATOS", "Oststerreich"},
        {"ATSA", "Salzburg"},
        {"ATST", "Steiermark"},
        {"ATTI", "Tirol"},
        {"ATVO", "Vorarlberg"},
        {"ATWI", "Wien"},
        {"CHDR", "Schweiz"},
        {"DEND", "Norddeutschland"},
    };

    public static String[] EntryFlag={};

    /**
     * add a number of new EntryFlags to the existing ones, eliminating duplicates
     * @param newFlags a String[] of EntryFlags to be added
     */
    public static void addEntryFlags(String[]newFlags) {
        int i=0;
        for(i=0; i<newFlags.length; i++) {
            /* probably not the most efficient way! */
            getEntryFlagIndex(newFlags[i]);
        }
    }

    /** 
     * lookup the index of an EntryFlag; 
     * if the EntryFlag does not yet exist, add it
     */
    public static short getEntryFlagIndex(String flag) {
        int i=0;
      	for(i=0; i<EntryFlag.length; i++)
            if(flag.equals(EntryFlag[i]))return (short)i;
        // couldn't find it!
        String[]newflags=new String[EntryFlag.length+1];
        for(i=0; i<EntryFlag.length; i++)
            newflags[i]=EntryFlag[i];
        newflags[EntryFlag.length]=flag;
        EntryFlag=newflags;
        return (short)(newflags.length-1);
    }


    public static String[][] textTypes={
	{"BIO", "Biographischer Text"},
	{"PRS", "Phonetisch reichhaltige Stze"}, 
	{"LIT", "Literarischer Text"},
	{"NRT", "Nachrichtentext"},
	{"INT", "Interview"},
    };


    /**
     * eine liste von krzeln fr IPAgruppen (damit suchstrings einfacher werden)
     * z.b. {"IPAvowel", "V"}
     */
    private static String[][] IPAgroupReplacements={
	{"IPAvowel", "V"},
	{"IPAconsonant", "K"},
	{"IPAdiacritic", "D"},
	{"IPAsuprasegmental", "S"},
	{"IPAplosive", "P"},
	{"IPAnasal", "N"},
	{"IPAtrill", "R"},
	{"IPAflap", "T"},
	{"IPAfricative", "F"},
        {"IPAlfricative", "Z"},
	{"IPAapproximant", "J"},
        {"IPAlapproximant", "M"},
        {"IPAlateral", "L"},
	{"IPAclick", "X"},
	{"IPAimplosive", "H"},
	{"IPAbilabial", "1"},
	{"IPAlabiodental", "2"},
	{"IPAdental", "3"},
	{"IPAalveolar", "4"},
	{"IPApostalveolar", "5"},
	{"IPAretroflex", "6"},
	{"IPApalatal", "7"},
	{"IPAvelar", "8"},
	{"IPAuvular", "9"},
	{"IPApharyngal", "Y"},
	{"IPAglottal", "G"},
	{"IPAepiglottal", "O"},
	{"IPAejective", "E"},
	{"IPAtone", "I"},
        {"IPAcombined", "C"},
    };

    public static boolean setIPAgroupReplacement(String key, String value){
	for(int i=0; i<IPAgroupReplacements.length; i++){
	    if (key.equals(IPAgroupReplacements[i][0])){
		IPAgroupReplacements[i][1]=value;
		return true;
	    }
	}
	return false;
    }
    public static boolean setIPAgroupReplacement(int idx, String value){
	if(idx<0||idx>=IPAgroupReplacements.length)return false;
	IPAgroupReplacements[idx][1]=value;
	return false;
    }
    public static String getIPAgroupReplacement(String key){
	for(int i=0; i<IPAgroupReplacements.length; i++){
	    if (key.equals(IPAgroupReplacements[i][0]))
		return IPAgroupReplacements[i][1];
	}

	return null;
    }

    public static String[][] getIPAgroupReplacement(){
	String[][]ret=new String[IPAgroupReplacements.length][2];
	for(int i=0; i<IPAgroupReplacements.length; i++){
	    ret[i][0]=IPAgroupReplacements[i][0];
	    ret[i][1]=IPAgroupReplacements[i][1];
	}

	return ret;
    }

    /**
     * set the working directory
     * @param path the working directory
     */
    public static File getPath(String id) {
        File p=getPWD();
        File f=new File(p, id);
        if(f.exists() && f.isDirectory()) {
            return f;
        }

        return p;
    }

    @Deprecated
    public static File getTextsPath(){
        return getPath("texts");
    }

    @Deprecated
    public static File getSoundsPath(){
        return getPath("sounds");
    }



    /**
     * how many search-panes we want
     */
    public static int numSearchPanes = 5;

    /**
     * the dirty-flag indicates that the database has been modified
     */
    public static boolean dirty = false;


    /**
     * internationalization: reading labels,... from files
     */
    public static java.util.ResourceBundle Labels = java.util.ResourceBundle.getBundle("resources/AdabaLabels");

    public static String getLabel(String key) {
        if(key==null)return null;
        
        String s=null;
        try {
            s=Labels.getString(key);
        } catch (java.util.MissingResourceException e) {
            at.iem.tools.Logger.println("resources do not contain key: "+key);
            s=null;
        }
        if (s==null)
            return key;
        return s;
    }


    /**
     * how often (in sec) a backup-should be made
     * 0 means no backup (?)
     */
    //public static int backupTime = 60; // make a backup every minute
    public static int backupTime = -1; // disable backups


    // =================================================================================
    // handling properties

    /**
     * the properties reader...
     */
    private static AdabaProperties properties = null;
    /**
     * read the shared data from a properties file
     * the file is in XML-format
     */


    /** 
     * read the shared data from persistent storage
     */
    public static void read(){
	properties=new AdabaProperties();        
    }
    /** 
     * write the shared data to a persistent storage
     */
    public static void write(){
	if (properties!=null)properties.write();
    }

    /** 
     * clear the properties storages
     */
    public static void clear(){
	if (properties!=null)properties.clear();
    }



    static Frame splashFrame = null;
    public static void openSplash(String ressource){
        closeSplash();
        URL imageURL = ClassLoader.getSystemResource(ressource);
        if (imageURL != null) {
            splashFrame = at.iem.tools.gui.SplashWindow.splash(
                            Toolkit.getDefaultToolkit().createImage(imageURL)
                                              );
        } else {
            at.iem.tools.Logger.println("Splash image not found");
        }
    }

    public static void closeSplash(){
        try {
            Class.forName("AdabaSplash").getMethod("closeSplash").invoke(null);
        } catch (Throwable e) {}
        if (splashFrame != null) splashFrame.dispose();
        splashFrame=null;

        if(pb!=null) {
            pb.done();
            pb=null;
        }
    }

    private static at.iem.tools.gui.ProgressBar pb=null;
    public static void createStartupProgressBar() {
        pb=new at.iem.tools.gui.ProgressBar(getLabel("AdabaLoadingPleaseWait"), null);
        java.awt.Point p=pb.getLocation();
        pb.setLocation((int)(p.getX()),(int)(p.getY()/2));
    }

    public static void exit() {
        if(properties!=null)
            properties.writePreferencesTabOpen(selectedGuiTab);

        System.exit(0);
    }

    public static int selectedGuiTab;

}

