// GUI.java
// $Id: GUI.java 831 2006-03-16 16:21:53Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
// 
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

package at.iem.adaba;

import javax.swing.*;
import java.awt.*;
import  at.iem.tools.gui.VTextIcon;


/**
 * die META-Klasse fr das GUI
 * stellt mehrere tabs fr die unterschiedlichen abfragen dar (wrterbuchsuche, texte,...)
 */

public class GUI extends JPanel implements at.iem.tools.Quittable,  at.iem.tools.gui.Refreshable, at.iem.tools.Selectable,
                                           javax.swing.event.ChangeListener {
    at.iem.adaba.gui.GUI wdbGUI; /* WortDatenBank */
    at.iem.adaba.elan.GUI tdbGUI; /* TextDatenBank */
    at.iem.adaba.elan.GUI etdbGUI; /* TextDatenBank */
    at.iem.adaba.gui.Text einleitungGUI; /* Einleitungstext */

    //at.iem.adaba.gui.Menu menue;
    JTabbedPane TabPane;

    public GUI(at.iem.adaba.database.TranscriptionDataBase db) {
        long tsStart=System.currentTimeMillis();   
        wdbGUI = new at.iem.adaba.gui.GUI(db);
        long tsWDB=System.currentTimeMillis();
        tdbGUI = new at.iem.adaba.elan.GUI(AdabaShared.getPath("texts"), at.iem.adaba.elan.ElanSelector.TYPE_PERSON_TEXT);
        long tsTDB=System.currentTimeMillis();
        etdbGUI = new at.iem.adaba.elan.GUI(AdabaShared.getPath("etexts"), at.iem.adaba.elan.ElanSelector.TYPE_SIMPLE);
        long tsETDB=System.currentTimeMillis();
        einleitungGUI = new at.iem.adaba.gui.Text("resources/texts/start.htm");
        einleitungGUI.setParent(this);
        long tsEinleitung=System.currentTimeMillis();

        at.iem.tools.Logger.println("creating GUI took: "+(tsWDB-tsStart)+"(db)+"+(tsTDB-tsWDB)+"(texts)+"+(tsETDB-tsTDB)+"(etexts)+"+(tsEinleitung-tsETDB)+"(einleitung) = "+(tsEinleitung-tsStart));

	setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	JPanel MainPane = new JPanel();
	TabPane = new JTabbedPane();

	TabPane.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
	MainPane.setLayout(new BoxLayout(MainPane, BoxLayout.Y_AXIS));

        TabPane.addTab(AdabaShared.getLabel("MainTabEinleitung"), einleitungGUI);
        TabPane.addTab(AdabaShared.getLabel("MainTabWordDatabase"), wdbGUI);
        TabPane.addTab(AdabaShared.getLabel("MainTabTextDatabase"), tdbGUI);
        TabPane.addTab(AdabaShared.getLabel("MainTabExtraTextDatabase"), etdbGUI);

        TabPane.addChangeListener(this);

        int selectedtab=AdabaShared.selectedGuiTab;

        try {
            TabPane.setForegroundAt(0, AdabaConstants.AdabaSelectionForeground);
            TabPane.setSelectedIndex(selectedtab);
        } catch (Exception ex) {
            at.iem.tools.Logger.println("accessing tab index "+selectedtab+" failed: "+ex);
        }

        add(TabPane);
    }

    public void setHelpSelection(at.iem.tools.Selectable s) {
        einleitungGUI.setHelp(s);
    }

    public void refresh() {
        wdbGUI.refresh();
        tdbGUI.refresh();
        etdbGUI.refresh();
    }

    public void quit() {
        AdabaShared.exit();
    }

    public void stateChanged(javax.swing.event.ChangeEvent e)
    {
        JTabbedPane tb=new JTabbedPane();
        if(e.getSource()!=null && e.getSource().getClass()==tb.getClass()) {
            tb=(JTabbedPane)(e.getSource());
            for(int i=0; i<tb.getTabCount(); i++) {
                tb.setForegroundAt(i, null);
            }
            tb.setForegroundAt(tb.getSelectedIndex(), AdabaConstants.AdabaSelectionForeground);
        }

        /* store the currently selected */
        AdabaShared.selectedGuiTab = TabPane.getSelectedIndex() ;

        tdbGUI.stop();
        etdbGUI.stop();
    }
    String[] tabIDs={"start", "dict", "texts", "etexts"};
    public void setSelected(String id) {
        at.iem.tools.Logger.println("setting tab to: "+id);
        if(id==null) return;
        //int tabId=tb.getSelectedIndex();
        for(int i=0; i<tabIDs.length; i++) {
            if(id.equals(tabIDs[i])) {
                TabPane.setSelectedIndex(i);
                return;
            }
        }
        at.iem.tools.Logger.println("no tab id found: "+id);
    }

    public void setSelectedIndex(int id) {
        if(id>=0 && id<TabPane.getTabCount());
        TabPane.setSelectedIndex(id);
    }
    
    public int getSelectedIndex() {
        return TabPane.getSelectedIndex();
    }

    public Object[] getSelection() {
        Object[]result=new Object[1];
        result[0]=tabIDs[TabPane.getSelectedIndex()];
        return result;
    }


    public at.iem.tools.Selectable getResultSelector() {
        return wdbGUI.getResultSelector();
    }

    private static final long serialVersionUID = 1L;
}
