// ATDEintrag.java
// $Id: ATDEintrag.java 1390 2007-02-26 16:25:22Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
// 
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

// how to make this work ???
package at.iem.adaba.database.ATD;
import at.iem.adaba.database.*;

import at.iem.adaba.*;

import at.iem.tools.linguistic.IPA;
import at.iem.tools.*;

import java.io.File;
import java.io.OutputStreamWriter;
import java.io.ByteArrayOutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.lang.Short;

/**
 * die Struktur, die alle Information eines Wrterbucheintrags beinhaltet
 */

public class ATDEintrag extends TranscriptionEintrag {
    public ATDEintrag(String key) {
	super(key);
    }
    public ATDEintrag(TranscriptionEintrag eintrag) {
	super(eintrag);
    }
    public ATDEintrag(String key, TranscriptionEintrag eintrag) {
	super(key, eintrag);
    }
    /**
     * make a TranscriptionEintrag
     * the ATDEintrag already has a name
     * the other fields are provided in binary-format
     * @param eintrag the id of the ATDEintrag
     * @param b the binary data
     */
    public ATDEintrag(String eintrag, byte[] b, TranscriptionDBInfo info){
	super(eintrag);
	version=0;
	timestamp=0;
	int laenge  = b.length;
	int magic= (b[0]<<8)+b[1];
	if (b[1]<0)magic+=256;
	if (b[0]<0)magic+=65536;
	if (laenge==magic){
	    interpreteBytes_0(b);
	} else if (magic==0){// this is real magic...
	    int l_version=b[2];
	    this.version=l_version;
            //System.out.println("version "+l_version);
	    switch (l_version){
	    case 1: interpreteBytes_1(b); break;
	    case 2: interpreteBytes_2(b); break;
	    case 3: interpreteBytes_3(b); break;
	    case 4: interpreteBytes_4(b); break;
	    default:
		at.iem.tools.Logger.println("ATDEintrag: '"+eintrag+"' "+
				   "current bytes version #"+l_version+
				   " is not supported!");
	    }
	} else at.iem.tools.Logger.println("couldn't read byte[]-struct!");
        setKey(eintrag); // wortvariant does not seem to work...
    }

    /**
     * make a TranscriptionEintrag
     * the ATDEintrag already has a name
     * the other fields are provided in binary-format
     * @param eintrag the id of the ATDEintrag
     * @param b the binary data
     */
    public ATDEintrag(String key, byte[] b){
        this(key, b, null);
    }

    /**
     * some meta-information about the database we are taken from
     */
    protected TranscriptionDBInfo dbinfo=null;

    /**
     * the binary format we have been created from
     * defaults to the most actual version
     */
    protected int version=4;
 
    /**
     * get the binary-version of this Entry;
     * Entries are stored in binary format in the database; 
     * when the binary is decoded into an instance of this class, the version is set;
     * note: the Entry is always written to the db in the most recent binary format
     * @Deprecated this is implementation specific
     * @return the binary version of this Entry
     */
    int getBinaryVersion(){
	return version;
    }


    /**
     * get the id of this TranscriptionEintrag
     * @return the key of the TranscriptionEintrag
     */
    /*
    protected String getKey(){
	return key;
    }
    */

    /**
     * decode binary data
     * @param b the binary data (version 0)
     */
    private void interpreteBytes_0(byte[] b){
	//	key     = new String (eintrag);
	char laenge = (char)b.length;
	if (b[0]!=(byte)(laenge>>>8))return;
	if (b[1]!=(byte)(laenge))    return;

	int lautALaenge  = b[2]; if(lautALaenge<0)lautALaenge+=256;
	int lautDLaenge  = b[3]; if(lautDLaenge<0)lautDLaenge+=256;
	int lautCHLaenge = b[4]; if(lautCHLaenge<0)lautCHLaenge+=256;
	variant = (b[5]!=0);
	eTyp = b[6];
	gTyp = b[7];
	if (eTyp<0 || eTyp>=AdabaShared.E_TYPEN.length)eTyp=0;
	if (gTyp<0 || gTyp>=AdabaShared.G_TYPEN.length)gTyp=0;

	edited = (b[8]!=0);

	int kommentarlaenge = b[9]; if(kommentarlaenge<0)kommentarlaenge+=256;

	int fileALaenge = b[10]; if(fileALaenge<0)fileALaenge+=256;
	int fileDLaenge = b[11]; if(fileDLaenge<0)fileDLaenge+=256;
	int fileCHLaenge = b[12]; if(fileCHLaenge<0)fileCHLaenge+=256;

	int LAST=12+1;
	int LASTpos=LAST;

	byte[] laut_A = new byte[lautALaenge];
	java.lang.System.arraycopy(b, LASTpos, laut_A, 0, lautALaenge);
	LASTpos+=lautALaenge;
        String laut = StringX.replace(StringX.replace(
                                                      ((new IPA(laut_A)).toString()), 
                                                      "{","[["),
                                      "}","]]");
        ipa.put("AT", new IPA(laut));
        ipa.put((String)null, new IPA(laut)); // the default transcription

	byte[] laut_D = new byte[lautDLaenge];
	java.lang.System.arraycopy(b, LASTpos, laut_D, 0, lautDLaenge);
	LASTpos+=lautDLaenge;
        laut = StringX.replace(StringX.replace(
                                               ((new IPA(laut_D)).toString()), 
                                               "{","[["),
                               "}","]]");
        ipa.put("DE", new IPA(laut));

	byte[] laut_CH = new byte[lautCHLaenge];
	java.lang.System.arraycopy(b, LASTpos, laut_CH, 0, lautCHLaenge);
	LASTpos+=lautCHLaenge;
        laut = StringX.replace(StringX.replace(
                                               ((new IPA(laut_CH)).toString()), 
                                               "{","[["),
                               "}","]]");
        ipa.put("CH", new IPA(laut));


	LASTpos+=fileALaenge;
	LASTpos+=fileDLaenge;
	LASTpos+=fileCHLaenge;

	Kommentar = new String(b, LASTpos, kommentarlaenge);
	LASTpos+=kommentarlaenge;

        fixMS2Unicode();
    }
    /**
     * decode binary data
     * @param b the binary data (version 1)
     */
    private void interpreteBytes_1(byte[] b){
	try {
	    int index=3;
	    int size=0;
	    byte[] buf=null;
	    String s=null;
	    /* timestamp 00000000000*/
	    size=16;
	    buf = new byte[size];System.arraycopy(b, index, buf, 0, size);index+=size;
	    s=new String(buf).trim();
	    timestamp=java.lang.Long.parseLong(s, 16);

	    /*
	     * fertig editiert und aussprache-variant
	     */
	    edited = (b[index]!=0); index++;
	    variant= (b[index]!=0); index++;

	    /* eType */
	    eTyp = b[index];index++;
	    /* gType */
	    gTyp = b[index];index++;

	    /* spezial-flags */
	    size=b[index]; index++; // number of flags stored
	    buf = new byte[size*2];System.arraycopy(b, index, buf, 0, size*2);index+=(size*2);
	    flags=new ArrayList<Short>(size);
	    for(int i=0; i<size*2; i+=2){
                short f = bytes2short(buf[i], buf[i+1]);
                if(null!=dbinfo){
                    String f_s=dbinfo.getEntryFlag(f);
                    if(null!=f_s&&!"".equals(f_s))
                        f=AdabaShared.getEntryFlagIndex(f_s);
                }
		flags.add(new Short(f));
	    }
	    /* comment */
	    size= b[index];index++;
	    buf = new byte[size];System.arraycopy(b, index, buf, 0, size);index+=size;
	    Kommentar=new String(buf, "UTF-8");

	    /*  main-transcription */
	    size= b[index];index++;
	    buf = new byte[size];System.arraycopy(b, index, buf, 0, size);index+=size;
            setIPA((String)null, new String(buf, "UTF-8"));
	    
	    /*  transcription */
	    while(index<b.length){
		size= b[index];index++;
		buf = new byte[size];System.arraycopy(b, index, buf, 0, size);index+=size;
		String region=new String(buf, "UTF-8");
                
		size= b[index];index++;
		buf = new byte[size];System.arraycopy(b, index, buf, 0, size);index+=size;

                ipa.put(region, new IPA(new String(buf, "UTF-8")));
	    }
	    
	} catch (Exception e){
	    at.iem.tools.Logger.println("key='"+key+"':\terror(readbytes1): "+e);
	}
        fixMS2Unicode();
    }

    private static byte[]short2bytes(short s){
	byte[]buf=new byte[2];
	buf[0]=(byte)(s>>>8);
	buf[1]=(byte)(s);
	return buf;
    }
    private static byte[]short2bytes(int i){
	return short2bytes((short)i);
    }
    private static short bytes2short(byte[] b){
	if (b==null || b.length!=2)return 0;
	short s=(short)(b[0]<<8);
	s+=(short)(b[1]);
	return s;
    }
    private static short bytes2short(byte b0, byte b1){
	int s0=b0;
	int s1=b1;
	if(s0<0)s0+=256;
	if(s1<0)s1+=256;
	return (short)((s0<<8)+s1);
    }

    /**
     * decode binary data;
     * changes: pronunciation and comments now have 2 bytes of length-information
     * @param b the binary data (version 2)
     */
    private void interpreteBytes_2(byte[] b){
        //at.iem.tools.Logger.println("interpretebytes2: key='"+key+"'");
	try {
	    int index=3;
	    int size=0;
	    byte[] buf=null;
	    String s=null;
	    /* timestamp 00000000000*/
	    size=16;
	    buf = new byte[size];System.arraycopy(b, index, buf, 0, size);index+=size;
	    s=new String(buf).trim();
	    timestamp=java.lang.Long.parseLong(s, 16);

	    /*
	     * fertig editiert und aussprache-variant
	     */
	    edited = (b[index]!=0); index++;
	    variant= (b[index]!=0); index++;

	    /* eType */
	    eTyp = b[index];index++;
	    /* gType */
	    gTyp = b[index];index++;

	    /* spezial-flags */
	    size=bytes2short((byte)0,b[index]); index++; // number of flags stored
	    buf = new byte[size*2];System.arraycopy(b, index, buf, 0, size*2);index+=(size*2);
	    flags=new ArrayList<Short>(size);

            //at.iem.tools.Logger.println("interpretebytes2");

	    for(int i=0; i<size*2; i+=2){
		flags.add(new Short(bytes2short(buf[i], buf[i+1])));
	    }
	    /* comment */
	    size= bytes2short(b[index], b[index+1]);index+=2;
	    buf = new byte[size];System.arraycopy(b, index, buf, 0, size);index+=size;
	    Kommentar=new String(buf, "UTF-8");

            //at.iem.tools.Logger.println("bytes2: "+key+": starting trans");
	    /*  main-transcription */
	    size= bytes2short(b[index], b[index+1]);index+=2;
	    buf = new byte[size];System.arraycopy(b, index, buf, 0, size);index+=size;
            setIPA((String)null, new String(buf, "UTF-8"));
            //at.iem.tools.Logger.println("bytes2: "+key+": default trans="+new String(buf, "UTF-8"));

	    /*  transcription */
	    while(index<b.length){
                //at.iem.tools.Logger.println("bytes2: "+key+": next trans");

		size= bytes2short((byte)0,b[index]);index++;
		buf = new byte[size];System.arraycopy(b, index, buf, 0, size);index+=size;
		String region=new String(buf, "UTF-8");

		size= bytes2short(b[index], b[index+1]);index+=2;
		buf = new byte[size];System.arraycopy(b, index, buf, 0, size);index+=size;

                //at.iem.tools.Logger.println("bytes2: "+key+": trans["+region+"]="+new String(buf, "UTF-8"));

                ipa.put(region, new IPA(new String(buf, "UTF-8")));
	    }
            //at.iem.tools.Logger.println("bytes2: "+key+": trans done");

	    
	} catch (Exception e){
	    at.iem.tools.Logger.println("key='"+key+"':\terror(readbytes2): "+e);
	}

        fixMS2Unicode();
    }
    /**
     * decode binary data;
     * changes: we also store, whether we (think that we) have soundfiles for this entry; and fixed the ligature...
     * @param b the binary data (version 3)
     */
    private void interpreteBytes_3(byte[] b){
	try {
	    int index=3;
	    int size=0;
	    byte[] buf=null;
	    String s=null;
	    /* timestamp 00000000000*/
	    size=16;
	    buf = new byte[size];System.arraycopy(b, index, buf, 0, size);index+=size;
	    s=new String(buf).trim();
	    timestamp=java.lang.Long.parseLong(s, 16);

	    /*
	     * fertig editiert und aussprache-variant
	     */
	    edited = (b[index]!=0); index++;
	    variant= (b[index]!=0); index++;

	    /* eType */
	    eTyp = b[index];index++;
	    /* gType */
	    gTyp = b[index];index++;

	    /* spezial-flags */
	    size=bytes2short((byte)0,b[index]); index++; // number of flags stored
	    buf = new byte[size*2];System.arraycopy(b, index, buf, 0, size*2);index+=(size*2);
	    flags=new ArrayList<Short>(size);

	    for(int i=0; i<size*2; i+=2){
		flags.add(new Short(bytes2short(buf[i], buf[i+1])));
	    }

            /* we think we have files... */
            have_files=(short)((b[index]!=0)?1:0); index++;

	    /* comment */
	    size= bytes2short(b[index], b[index+1]);index+=2;
	    buf = new byte[size];System.arraycopy(b, index, buf, 0, size);index+=size;
	    Kommentar=new String(buf, "UTF-8");

	    /*  main-transcription */
	    size= bytes2short(b[index], b[index+1]);index+=2;
	    buf = new byte[size];System.arraycopy(b, index, buf, 0, size);index+=size;
            setIPA((String)null, new String(buf, "UTF-8"));
	    
	    /*  transcription */
            //at.iem.tools.Logger.println("key='"+key+"': getting transcriptions from "+index+" to "+b.length);
	    while(index<b.length){
		size= bytes2short(b[index],b[index+1]);index+=2;
                //at.iem.tools.Logger.println("transcription-region of size "+size);

                String region=null;
                if(size>0) {
                    buf = new byte[size];System.arraycopy(b, index, buf, 0, size);index+=size;
                    region=new String(buf, "UTF-8");
                    //at.iem.tools.Logger.println("key='"+key+"':: region='"+region+"'");
                }

		size= bytes2short(b[index], b[index+1]);index+=2;
                //at.iem.tools.Logger.println("transcription of size "+size);
		buf = new byte[size];System.arraycopy(b, index, buf, 0, size);index+=size;

                String ipastring=new String(buf, "UTF-8");
                //at.iem.tools.Logger.println("key='"+key+"':: IPA='"+ipastring+"'");
                ipa.put(region, new IPA(ipastring));
	    }
	    	    
	} catch (Exception e){
	    at.iem.tools.Logger.println("key='"+key+"':\terror(readbytes3): "+e);
	}
    }
    /**
     * decode binary data;
     * changes: we also store whether this entry is a variant (e.g. "weg" and "Weg")
     * @param b the binary data (version 4)
     */
    private void interpreteBytes_4(byte[] b){
	try {
	    int index=3;
	    int size=0;
	    byte[] buf=null;
	    String s=null;
	    /* timestamp 00000000000*/
	    size=16;
	    buf = new byte[size];System.arraycopy(b, index, buf, 0, size);index+=size;
	    s=new String(buf).trim();
	    timestamp=java.lang.Long.parseLong(s, 16);

	    /* the name (might be different from the key) */
	    size= bytes2short(b[index], b[index+1]);index+=2;
	    buf = new byte[size];System.arraycopy(b, index, buf, 0, size);index+=size;
	    setName(new String(buf, "UTF-8"));

            /* wortVariant */
            wortVariant=b[index];index++;
            //at.iem.tools.Logger.println("variant: "+wortVariant);
	    /*
	     * fertig editiert und aussprache-variant
	     */
	    edited = (b[index]!=0); index++;
	    variant= (b[index]!=0); index++;

	    /* eType */
	    eTyp = b[index];index++;
	    /* gType */
	    gTyp = b[index];index++;

	    /* spezial-flags */
	    size=bytes2short((byte)0,b[index]); index++; // number of flags stored
	    buf = new byte[size*2];System.arraycopy(b, index, buf, 0, size*2);index+=(size*2);
	    flags=new ArrayList<Short>(size);

	    for(int i=0; i<size*2; i+=2){
		flags.add(new Short(bytes2short(buf[i], buf[i+1])));
	    }

            /* we think we have files... */
            have_files=(short)((b[index]!=0)?1:0); index++;

	    /* comment */
	    size= bytes2short(b[index], b[index+1]);index+=2;
	    buf = new byte[size];System.arraycopy(b, index, buf, 0, size);index+=size;
	    Kommentar=new String(buf, "UTF-8");

	    /*  main-transcription */
	    size= bytes2short(b[index], b[index+1]);index+=2;
	    buf = new byte[size];System.arraycopy(b, index, buf, 0, size);index+=size;
            setIPA((String)null, new String(buf, "UTF-8"));
	    
	    /*  transcription */
            //at.iem.tools.Logger.println("key='"+key+"': getting transcriptions from "+index+" to "+b.length);
	    while(index<b.length){
		size= bytes2short(b[index],b[index+1]);index+=2;
                //at.iem.tools.Logger.println("transcription-region of size "+size);

                String region=null;
                if(size>0) {
                    buf = new byte[size];System.arraycopy(b, index, buf, 0, size);index+=size;
                    region=new String(buf, "UTF-8");
                    //at.iem.tools.Logger.println("key='"+key+"':: region='"+region+"'");
                }

		size= bytes2short(b[index], b[index+1]);index+=2;
                //at.iem.tools.Logger.println("transcription of size "+size);
		buf = new byte[size];System.arraycopy(b, index, buf, 0, size);index+=size;

                String ipastring=new String(buf, "UTF-8");
                //at.iem.tools.Logger.println("key='"+key+"':: IPA='"+ipastring+"'");
                ipa.put(region, new IPA(ipastring));
	    }
	    	    
	} catch (Exception e){
	    at.iem.tools.Logger.println("key='"+key+"':\terror(readbytes4): "+e);
	}
    }

    private byte[]getIpaBytes(String regionId) {
        IPA i=ipa.get(regionId);
        if(null==i)
            return null;
        
        return i.getBytes("UTF-8");
    }
    private static int outByteWrite(ByteArrayOutputStream out, byte[]b) throws java.io.IOException {
        int len=0;
        if(null==b) {
            len=0;
        } else {
            len=b.length;
        } 

        out.write(short2bytes(len));
        if(len>0)
            out.write(b);

        return len;
    }

    private static int outStringWrite(ByteArrayOutputStream out, String s) throws java.io.IOException {
	return outByteWrite(out, s.getBytes("UTF-8"));
    }

    /**
     * encode this ATDEintrag into binary-data using the format-v3
     * @return an array of bytes containing this ATDEintrag in binary format
     */
    public byte[] getBytes(){ // version 4
	byte [] result = null;
	ByteArrayOutputStream out = new ByteArrayOutputStream(); //Write to an output stream
	try{
	    /* magic number: 00 */
	    out.write((byte)0); out.write((byte)0);
	    /* binary-format version 4 */
	    out.write((byte)4);
	    /* timestamp */
	    String s=java.lang.Long.toHexString(timestamp);
	    String s1=(new String("0000000000000000".getBytes(), 0, 16-s.length())+s).trim();
	    out.write(s1.getBytes());

	    /* the name (might be different from the key) */
	    outStringWrite(out, getName());

            /* wortVariant */
            out.write(wortVariant);

	    /* fertig  editiert und aussprache-variant */
	    out.write((byte)(edited ?1:0));
	    out.write((byte)(variant?1:0));

	    /* eType */
	    out.write(eTyp);
	    /* gType */
	    out.write(gTyp);

	    /* spezial-flags ... */
	    out.write((byte)flags.size());
	    for(int i=0; i<flags.size(); i++){
		out.write(short2bytes(((Short)flags.get(i)).shortValue()));
	    }

            //at.iem.tools.Logger.println("do we have files? "+have_files); 
            long tsStart=System.currentTimeMillis();
            /* we think we have files... */
	    if(true) // set to false to disable the check
		out.write((byte)(hasMedia()?1:0));
	    else
		out.write((byte)0); // set to 0/1/-1 to assume that files are (not) here (:-3s/true/false)
            long tsStop=System.currentTimeMillis();
            //at.iem.tools.Logger.println("detecting files took "+(tsStop-tsStart)+" ms"); 


	    /* comment */
            outStringWrite(out, Kommentar);
	    
            /* main-transcription */
            outByteWrite(out, getIpaBytes((String)null));

	    /* transkriptionen */
            //at.iem.tools.Logger.println("getting IPA-transcriptions for '"+key+"':");
            Iterator keys=ipa.keySet().iterator();
            while(keys.hasNext()) {
                String key=(String)keys.next();
                if(null!=key) { // key==null is handled above!
                    IPA ipa_ = ipa.get(key);
                    if(null!=ipa_) {
                        //at.iem.tools.Logger.println(" - writing IPA for '"+key+"'");

                        outStringWrite(out, key);
                        outByteWrite(out, getIpaBytes(key));
                    }
                }
            }

	    /* writing done: convert to byte[]-buffer */
	    result = out.toByteArray();  

	} catch(Exception e){
	    at.iem.tools.Logger.println("error converting to buffer: "+e);
	    return null;
	}
        //at.iem.tools.Logger.println("ATDEintrag:getBytes("+key+"): "+result.length);
	return result;
    }




}
