// ATDataBase.java
// $Id: ATDataBase.java 1795 2007-06-26 13:54:30Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
// 
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

// 01/10/2002:: "saveas" now changes the current database to the new one
// 02/01/2003:: old backup scheme again: every time the main db is saved, we backup to
//              <atdfile.atd>.bak.# where # is from 0..7
// 21/01/2003:: but now we don't rely on the (cached) jdbm.write()-method any more
//              (it used to fail on crashy Windows(c))
//              We now really copy the files.

package at.iem.adaba.database.ATD;
import at.iem.adaba.database.TranscriptionEintrag;
import at.iem.adaba.database.EintragPattern;
import at.iem.adaba.*;
import at.iem.tools.StringX;

import java.io.File;
import java.io.FileReader;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import org.w3c.tools.dbm.*;

import java.util.Timer;
import java.util.TimerTask;
import java.util.ArrayList;

/**
 * hlt das Wrterbuch 
 * Suchfunktionen 
 */

public class ATDataBase implements at.iem.adaba.database.TranscriptionDataBase, at.iem.adaba.database.Exporter {
    private jdbm db;
    private SearchOrthoDB    odb;
    private SearchPhoneticDB pdb;

    private File file;
    private File odbFile, pdbFile;
    private boolean doBackup;

    int numEntries;

    private Timer timer;

    public ATDataBase(String file, boolean doback)throws Exception
    {
	doBackup=doback;
	if(AdabaShared.backupTime<=0)doBackup=false;
	at.iem.tools.Logger.println("ATdb: start loading");
	if(!load(file)){
            throw (new Exception("ATD: failed to load database"));
        }
	at.iem.tools.Logger.println("ATdb: loaded");
    }
    public ATDataBase(File   file, boolean doback)throws Exception
    {
	doBackup=doback;
	if(AdabaShared.backupTime<=0)doBackup=false;
	if(!load(file)) {
            throw (new Exception("ATD: failed to load database"));
        }
    }
    public ATDataBase(String file)throws Exception
    {
	this(file, true);
    }
    public ATDataBase(File   file)throws Exception
    {
	this(file, true);
    }

    private boolean async = false;
    public void startAsync() {
        async=true;
    }
    public void stopAsync() {
        if(true==async) {
            buildSearchDB(true);
        }
        async=false;
    }


    private void buildSearchDB(boolean doBuild){
        readDBInfo();
	at.iem.tools.Logger.println("buildSearchDB");
	if (doBuild){
            if(!doBackup) {
		at.iem.tools.Logger.println("using prebuilt");
                odb=new SearchOrthoDB(odbFile, db);
                pdb=new SearchPhoneticDB(pdbFile, db);
            } else {
		at.iem.tools.Logger.println("with backups");
                odb=new SearchOrthoDB(db);
                pdb=new SearchPhoneticDB(db);
            }
	}
    }
    private void buildSearchDB(){
	buildSearchDB(true);
    }

    TranscriptionEintrag storeAlternative(byte[] key, 
			  byte[] val, byte[] dupVal,
			  boolean doAlternative){
	String keystring=StringX.fromBytesUTF8(key);
	TranscriptionEintrag altEntry=null;
	try{
	    if(doAlternative){
		/* get current alternative */
		int alternative=-1; /* current alternative */
		int index=keystring.lastIndexOf(' ');
		if(index>0){
		/* might be an alternative */
		    try {
			alternative=java.lang.Integer.parseInt(keystring.substring(index));
		    } catch (Exception ex) {}
		    if (alternative!=-1){
			/* oh, we are already an alternative... */
			keystring=keystring.substring(0, index).trim();
		    }
		/* keystring is now the key without the alternative */
		}
		alternative=1;
		byte[]dummyval=db.lookup(StringX.getBytesUTF8(keystring+' '+alternative));
		while(dummyval!=null){
		    alternative++;
		    dummyval=db.lookup(StringX.getBytesUTF8(keystring+' '+alternative));
		}
		db.store(StringX.getBytesUTF8(keystring+' '+alternative), dupVal, jdbm.STORE_REPLACE);
		altEntry=new ATDEintrag((keystring+' '+alternative), dupVal);
	    }
	    db.store(key, val, jdbm.STORE_REPLACE);
	} catch (Exception e) {
	    at.iem.tools.Logger.println("couldnt store entry (or alternatives) for "+keystring);
	    altEntry=null;
	}
	return altEntry;
    }

    public TranscriptionEintrag replaceKey(TranscriptionEintrag entry, String key){
	String oldkey = entry.getKey();
	TranscriptionEintrag newentry = new TranscriptionEintrag(key, entry);
	insert(newentry);
	try {
	    db.delete(StringX.getBytesUTF8(oldkey));
	} catch (Exception ex) {
	    at.iem.tools.Logger.println("couldn't delete old key\n --> "+ex);
	    return entry;
	}

	save();
	buildSearchDB();
	return newentry;
    }

    public String rename(String oldkey, String newkey) {
        at.iem.tools.Logger.println("renaming '"+oldkey+"' to '"+newkey+"'");
	ATDEintrag e=fetch(oldkey);
        if(null==e) {
		/* oops, the nothing here to rename! */
               at.iem.tools.Logger.println("refusing to rename nonexistant entry: "+oldkey);
               return null;               
           }
	if(null!=fetch(newkey))
	    {
		/* oops, the newkey already exists!!! get rid of it... */
                delete(newkey);
	    }

	try {
	    db.store(StringX.getBytesUTF8(newkey), e.getBytes(), jdbm.STORE_INSERT);
	    save();
	} catch (Exception ex){
	    at.iem.tools.Logger.println("error storing renamed entry...\n--> "+ex);
	    return null;
	}

        /* clean up... */
        delete(oldkey);

	return newkey;
    }

    public TranscriptionEintrag merge(String key, TranscriptionEintrag mergentry) {
	TranscriptionEintrag e=fetch(key);
	if(null==e) {
	    // this entry does not yet exist, create it from scratch!
	    if(insert(key, mergentry)) {
		byte[]bresult=null;
		try {
		    bresult = db.lookup(StringX.getBytesUTF8(key));
		} catch (Exception ex) {
		    at.iem.tools.Logger.println("merging failed for unknown key: '"+key+"'");
		    return null;
		}
		if(bresult!=null&&bresult.length>0) {
		    return new ATDEintrag(key, bresult);
		}
	    }
	    return null;
	}
	/* this entry exists, we have to merge! */
	if(e.merge(mergentry))
	    return e;
	else
	    return null;
    }

    public boolean delete(String s_key){
	byte[]key=StringX.getBytesUTF8(s_key);
	try {
	    db.delete(key);
	}
	catch (Exception ex) {
	    at.iem.tools.Logger.println("error deleting entry...\n-->"+ex);
	    return false;
	}
	return true;
    }

    public void delete(String[] keys){
	if(keys==null || keys.length==0)return;
	for(int i=0; i<keys.length; i++){
	    delete(keys[i]);
	}
    }

    public boolean replace(TranscriptionEintrag entry){
	return replace(entry.getKey(), entry);
    }
    private boolean replace(String key, TranscriptionEintrag entry){
	try {
	    ATDEintrag aentry=new ATDEintrag(entry);
	    db.store(StringX.getBytesUTF8(key), aentry.getBytes(), jdbm.STORE_REPLACE);
	    save();
	} catch (Exception ex){
	    at.iem.tools.Logger.println("error replacing entry...\n-->"+ex);
	    return false;
	}
	return true;
    }
    public boolean insert(TranscriptionEintrag entry){
	return insert(entry.getKey(), entry);
    }

    private boolean insert(String key, TranscriptionEintrag entry) 
    {
	try {
	    ATDEintrag aentry=new ATDEintrag(entry);
	    db.store(StringX.getBytesUTF8(key), aentry.getBytes(), jdbm.STORE_INSERT);
	    save();
	} catch (Exception ex){
	    at.iem.tools.Logger.println("error inserting entry...\n--> "+ex);
	    return false;
	}
	return true;	
    }

    public String add(TranscriptionEintrag entry){
	ATDEintrag aentry=new ATDEintrag(entry);
	String key0=aentry.getName();
	String key=key0;
	byte variant=aentry.getVariant();

	boolean found=true;
	
	/* try to preserver the variant */
	if(variant>0) {
	    key=new String(key0+" ["+variant+"]");
	    found=(null!=fetch(key));

	    if(found)key=key0;
	} 

	while(found) {
	    variant++;
	    key=new String(key0+" ["+variant+"]");
	    found=(null!=fetch(key));
	}
	aentry.setKey(key);
	aentry.setVariant(variant);
	aentry.setName(key0);
	try {
	    db.store(StringX.getBytesUTF8(aentry.getKey()), aentry.getBytes(), jdbm.STORE_INSERT);
	    save();
	} catch (Exception ex){
	    at.iem.tools.Logger.println("error inserting entry...\n--> "+ex);
	    return null;
	}
	return aentry.getKey();
    }

    /**
     * 
     */
    private ATDEintrag fetch(String key) {
	try{
	    byte[] bresult = db.lookup(StringX.getBytesUTF8(key));
	    if(bresult!=null&&bresult.length>0) {
		return new ATDEintrag(key, bresult);
	    }
	} catch (Exception e){
	    at.iem.tools.Logger.println("error searching key: "+key);
	    return null;
	}
	return null;
    }

    /** 
     * stop the search (if running) 
     */
    public void stopSearch() {
        keep_searching=false;
        odb.stopSearch();
        pdb.stopSearch();
    }

    /**
     * a flag for interrupting running searches (from other threads)
     * every function that responds to this flag, OUGHT to set it to <code>true</code> before starting
     */
    private volatile boolean keep_searching=false;


    /**
     * lookup an array of Strings
     * @param key an array of keys to lookup in the database
     * @return an array of TranscriptionEintrag, each entry corresponding to the given entry in the key[]
     */ 
    public TranscriptionEintrag[] lookup(String[] key){
	if (key==null)return null;
        keep_searching=true;
	TranscriptionEintrag[] result = new TranscriptionEintrag[key.length];
	int i = key.length;
	while(i-->0){
	    //at.iem.tools.Logger.println("searching '"+key[i]+"'");
	    result[i]=fetch(key[i]);
            if(!keep_searching)return null;
	}
	return result;
    }
   /**
     * lookup a key
     * @param key a key to lookup in the database
     * @return an array of 1 TranscriptionEintrag  corresponding to the given key, or <code>null</code> if key is not in the db
     */
    public TranscriptionEintrag[] lookup(String key){
	String [] keys=new String[1];
	keys[0]=key;
	return lookup(keys);
    }
   /**
     * lookup all entries that start with a given pattern
     * @param pattern a pattern to search for
     * @return an array keys that start with <code>pattern</code>
     */
    public String[] startsWith(String pattern){
        at.iem.tools.Logger.println("ATD:startsWith("+pattern+")...");
        keep_searching=true;
	String[]results = odb.startsWith(pattern);
        at.iem.tools.Logger.println("ATD:startsWith("+pattern+"): "+results);
	if(results==null||results.length<1)
	    return null;
	java.util.Arrays.sort(results, java.text.Collator.getInstance());
	return results;
    }
   /**
     * lookup all entries that match exactly a given pattern
     * @param pattern a pattern to search for
     * @return an array keys that match <code>pattern</code>
     */
    public String[] isExactly(String pattern){
        at.iem.tools.Logger.println("ATD:isExactly("+pattern+")...");
        keep_searching=true;
	String[]results = odb.findExactly(pattern);
	if(results==null||results.length<1)
	    return null;
	java.util.Arrays.sort(results, java.text.Collator.getInstance());
	return results;
    }
   /**
     * lookup all entries that match a given pattern
     * @param pattern a pattern to search for
     * @param phonetic whether to search in the phonetic-dictionary or in the orthographic one (default)
     * @param gTyp grammatic type
     * @param eTyp etymologic type
     * @param variant if set to <code>true</code> only entries that are labeled "variant" are taken
     * @return an array of keys which match <code>pattern</code>
     * and that refer to TranscriptionEintrag's that match
     * <code>eTyp</code>, <code>eTyp</code> and <code>variant</code>
     * @see #search(String, boolean, int, byte, boolean) search
     */
    public String[] search(String pattern,
			    boolean phonetic,
			    int gTyp, int eTyp,
			    boolean variant) throws Exception {
        return search(pattern, phonetic, gTyp, eTyp, variant, false, null);
    }
   /**
     * lookup all entries that match a given pattern
     * @param pattern a pattern to search for
     * @param phonetic whether to search in the phonetic-dictionary or in the orthographic one (default)
     * @param gTyp grammatic type
     * @param eTyp etymologic type
     * @param variant if set to <code>true</code> only entries that are labeled "variant" are taken
     * @param sounds if set to <code>true</code> only entries with at least one soundfile are taken, <code>false</code> means <em>don't care</em>
     * @return an array of keys which match <code>pattern</code>
     * and that refer to TranscriptionEintrag's that match
     * <code>eTyp</code>, <code>eTyp</code> and <code>variant</code>
     * @see #search(String, boolean, int, byte, boolean) search
     */
    public String[] search(String pattern,
                           boolean phonetic,
                           int gTyp, int eTyp,
                           boolean variant,
                           boolean sounds,
                           ArrayList<Short>flags
                            ) throws Exception {
        keep_searching=true;

	String[] key   = null;
        long tsStart=System.currentTimeMillis();
	if (phonetic) key=pdb.findRegex(pattern);
	else key = odb.findRegex(pattern);
	if (key==null || key.length==0)return null;
        long tsStop=System.currentTimeMillis();
        at.iem.tools.Logger.println("raw results: #"+key.length+" took "+(tsStop-tsStart)+" [ms]");

        if(keep_searching==false)return null;

        /*
         * if we don't have any more restrictions, this could be out final result
         */
        if((gTyp==0) &&
           (eTyp==0) &&
           (sounds==false) &&
           (variant==false) &&
           (flags==null)) {
            java.util.Arrays.sort(key, java.text.Collator.getInstance());
            return key;
        }

        at.iem.tools.Logger.println("argh: do some filtering...");

        /* nopw comes the hard task: filter out all the unwanted stuff based on secondary information
         * (e.g. wordtype,...)
         */
	ArrayList<String> newkey = new ArrayList<String>(key.length);
	for(int i=0; i<key.length; i++){
            if(keep_searching==false)return null;
	    ATDEintrag test = null;
	    try{
                byte[]data=db.lookup(StringX.getBytesUTF8(key[i]));
		test = new ATDEintrag(key[i], data);

	    } catch (Exception e){
		at.iem.tools.Logger.println("ATDatabase.search:: error checking entry( "+i+") "+key[i]+": "+e);
		test=null;
	    }
            if(test==null)continue;

	    while(!newkey.contains(key[i])){
                if(keep_searching==false)return null;
		if(!test.isEtyp(eTyp))break;
		if(!test.isGtyp(gTyp))break;
		if(variant && !test.isVariant())break;
                if(sounds && !test.hasMedia())break;
                if(flags!=null) {
                    boolean flagmatch=true;
                    java.util.Iterator iter=flags.iterator();
                    while(iter.hasNext()) {
                        short flag_s=((Short)iter.next()).shortValue();
                        if(!test.getFlag(flag_s)) {
                            flagmatch=false;
                            break;
                        }
                    }
                    if(!flagmatch)break;
                }

		newkey.add(key[i]);
		break;
	    }
	}
	String[] goodkey = new String[newkey.size()];
	for(int i=0; i<goodkey.length; i++){
            String kk=(String)newkey.get(i);
            if(kk==null)
                at.iem.tools.Logger.println("found null-result: "+i);

            goodkey[i]=kk;
        }
        try {
            if(goodkey!=null)
                java.util.Arrays.sort(goodkey, java.text.Collator.getInstance());
        } catch (Exception ex) {
            at.iem.tools.Logger.println("ATDsearch-sort-exception: "+ex);
        }

	return goodkey;
    }

    /**
     * lookup all entries that match a given pattern
     * @param pattern a pattern to search for
     * @return an array of keys which match <code>pattern</code>
     */
    public String[]  search(EintragPattern pattern) throws Exception
    {
        /* ok, for now this is rather a dummy implementation that does not use the strength of EintragPattern at all;
         * but it is quick and should work
         */
        byte e=0, g=0;
        boolean v=false, s=false;
        ArrayList<Short>flags=null;

        if(pattern.getMatchEType()!=EintragPattern.PATTERN_IGNORE)
            e=pattern.getEType();
        if(pattern.getMatchGType()!=EintragPattern.PATTERN_IGNORE)
            g=pattern.getGType();
        if(pattern.getMatchVariant()!=EintragPattern.PATTERN_IGNORE)
            v=pattern.getVariant();
        if(pattern.getMatchFiles()!=EintragPattern.PATTERN_IGNORE)
            s=pattern.getFiles();
        if(pattern.getMatchFlags()==EintragPattern.PATTERN_EQ)
            flags=pattern.getFlags();

        /*
          this is what we call:
          search(pattern, phonetic, gTyp, eTyp, variant, sounds);
        */
        String result[]=null;

        if(pattern.getMatchKey()!=EintragPattern.PATTERN_IGNORE)
            result=search(pattern.getKey(), false, g, e, v, s, flags);
        else
            if(pattern.getMatchIPA()!=EintragPattern.PATTERN_IGNORE)
                result=search(pattern.getIPA().toString(), true, g, e, v, s, flags);


        

        return result;
    }
    



    public int getNumEntries(){
	int count=0;
	try {
	    java.util.Enumeration e = db.keys();
    	    while (e.hasMoreElements()){
		e.nextElement();
		count++;
	    }
	}catch (Exception ex){
	    at.iem.tools.Logger.println("an error occured!\n"+ex);
	    return (-1);
	}
	return count;
    }

    public String[] getAllEntries(){
	String entry[] = null;
	int count = 0;
	java.util.Enumeration e;
	try {
	    e = db.keys();
	    while (e.hasMoreElements()){
		byte key[] = (byte[]) e.nextElement();
                if(key==null||key.length<1)continue;
		count++;
	    }
	} catch (Exception ex){
	    at.iem.tools.Logger.println("an error occured!\n"+ex);
	}
	try {
	    entry = new String[count];
	    int i=0;
	    e = db.keys();
	    while (e.hasMoreElements()){
		byte key[] = (byte[]) e.nextElement();
                if(key==null||key.length<1)continue;
		if (i<count)entry[i]=StringX.fromBytesUTF8(key);
		i++;
	    }
	} catch (Exception ex){
	    at.iem.tools.Logger.println("an error occured!\n"+ex);
	}
	if(entry!=null && entry.length>0)java.util.Arrays.sort(entry, java.text.Collator.getInstance());
	return entry;
    }


    public boolean isValid(){
	return (file!=null && db!=null && odb!=null && pdb != null);
    }



    public String getName(){
	if (file!=null)return file.toString();
	else return "";
    }

    public java.net.URI getURI(){
	if (file!=null) {
	    try {
		return new java.net.URI(file.toString());
	    } catch (Exception e) {
	    }
	}
	return null;
    }




    public boolean load(File fil){
	file = null;
	try {
	    at.iem.tools.Logger.println("find database");
	    db = new jdbm(at.iem.tools.FileX.fallbackToTempSystemResource(fil));
	    at.iem.tools.Logger.println("found database");
            String filename=fil.toString();
            filename=filename.replaceAll("\\.atd$", "");
	    at.iem.tools.Logger.println("filename="+filename);
	    odbFile=new File(filename+"_ortho.jdb");
	    pdbFile=new File(filename+"_phonetic.jdb");
	    buildSearchDB();
	    file = fil;
	} catch (Exception e) {
	    at.iem.tools.Logger.println(e);
	}
	AdabaShared.setDB(file);
	if (file==null)return false;
	if (doBackup){
	    if (timer==null)timer = new Timer();
	    timer.schedule(new TDBbak(this),
			   AdabaShared.backupTime*1000,
			   AdabaShared.backupTime*1000);
	}
	return true;
    }

    public boolean load(String fil){
	file = null;
	try {
	    at.iem.tools.Logger.println("open jdbm");
	    db = new jdbm(fil);
	    at.iem.tools.Logger.println("opened jdbm");
            String filename=fil.toString();
            filename=filename.replaceAll("\\.atd$", "");
	    at.iem.tools.Logger.println("filename="+filename);
	    odbFile=new File(filename+"_ortho.jdb");
	    pdbFile=new File(filename+"_phonetic.jdb");
	    at.iem.tools.Logger.println("now building search-dbs");
	    buildSearchDB();
	    at.iem.tools.Logger.println("search-dbs done");
	    file = new File (fil);
	} catch (Exception e) {
	    at.iem.tools.Logger.println(e);
	}
	AdabaShared.setDB(file);
	if (file==null)return false;
	return true;
    }

    public void save(){
	try {
	    db.forceWriting(true);
            writeDBInfo();
	    db.save();
	} catch (Exception e) {
	    at.iem.tools.Logger.println(e);
	}
	//backup();
    }

    public void saveas(File fil){
	//	file = null;
	try {
	    jdbm clean = new jdbm(fil);
	    copyToJDBM(clean);
	    clean.save();
	    db = clean; // ?
	    file = fil;
	    AdabaShared.setDB(fil);
	} catch (Exception ex) {
	    at.iem.tools.Logger.println("couldn't create new database...\n--> "+ex);
	    AdabaMessages.showMessage(StringX.replace(AdabaShared.getLabel("ErrorSaveAs"),
						      "%s",  fil.getName()));
	}
    }

    private void copyToJDBM(jdbm newdb) {
        if(null==newdb)return;
	newdb.forceWriting(false);
	try {
	    java.util.Enumeration e = db.keys();
	    int i=0;
            writeDBInfo();
	    while (e.hasMoreElements()){
		byte key[] = (byte[]) e.nextElement();
		if (key!=null && key.length>0) {
		    byte val[] = db.lookup(key);
		    if ( val != null )newdb.store(key, val, jdbm.STORE_REPLACE);
		}
	    }
	} catch (Exception ex) {
	    at.iem.tools.Logger.println("couldn't copy old DB to new DB...\n--> "+ex);
	    AdabaMessages.showMessage(StringX.replace(AdabaShared.getLabel("ErrorCopyDB"),
						      "%s", getName()));
	}
	newdb.forceWriting(true);
    }


    private class TDBbak extends TimerTask{
	private ATDataBase TDB;
	public TDBbak(ATDataBase tdb) {
	    TDB=tdb;
	}

	public void run() {
            if(null!=TDB)
                TDB.backup();
	}
    }

    int backupnum = 0;
    public void backup(){
	if (!doBackup){
	    at.iem.tools.Logger.println("skipping backup!");
	    return;
	}
	//	    File fil = new File(file.getAbsolutePath()+((backupnum==0)?".BAK":"~"));
	File fil = new File(file.getAbsolutePath()+".bak."+backupnum);
	at.iem.tools.Logger.println("doing backup: "+fil.getName());
	/*
	try {
	    jdbm newdb = new jdbm(fil);
	    copyToJDBM(newdb);
	    newdb.save();
	} catch (Exception ex) {
	*/
	save();
	if (!at.iem.tools.FileX.copy(file, fil)){
	    at.iem.tools.Logger.println("couldn't backup ...\n--> ");//+ex);
	    doBackup=AdabaMessages.showYesNoMessage(AdabaShared.getLabel("TitleErrorBackup"),
						    StringX.replace(AdabaShared.getLabel("ErrorBackup"),
								    "%d", ""+backupnum));
	    if (!doBackup)AdabaMessages.showMessage(AdabaShared.getLabel("MessageBackupOff"));
	}

	backupnum++; backupnum%=8;
    }



    public String toText(){
	String allentries[]=getAllEntries();
	String langertext = new String("");
	int i =-1;
	while(++i<allentries.length){
	    try {
		ATDEintrag eintrag=new ATDEintrag(allentries[i],
						      db.lookup(StringX.getBytesUTF8(allentries[i])));
		langertext+=eintrag.toString();
		
	    } catch (Exception e){
		at.iem.tools.Logger.println("an error occured when converting to text at position "+i);
		at.iem.tools.Logger.println(e);
		//return null; /* do not return, just skip the entry */
	    }
	}
	return langertext;
    }


    private void readDBInfo() {
       	try {
	    byte[] info = db.lookup(TranscriptionDBInfo.getKey());
	    if (info==null){
                /* hmm, nothing here */
                at.iem.tools.Logger.println("couldn't find DB-Info");
            } else {
                TranscriptionDBInfo dbinfo = new TranscriptionDBInfo(info);         
            }
        } catch (Exception ex) {
	    at.iem.tools.Logger.println("error reading db-info\n --> "+ex);
	}
    }

    private void writeDBInfo() {
       	try {
            TranscriptionDBInfo dbinfo = new TranscriptionDBInfo();
      	    db.store(TranscriptionDBInfo.getKey(), dbinfo.getBytes(), jdbm.STORE_REPLACE);
        } catch (Exception ex) {
	    at.iem.tools.Logger.println("error writing db-info\n --> "+ex);
	}
    }
   
    /** 
     * export the data to a new database;
     * this includes updating all entries to the newest format
     */
    public void exportATD(File fil){
	exportATD(fil, getAllEntries());
    }
    public void exportATD(File fil, String[]exportentries){

        if(exportentries==null||exportentries.length==0)return;
	try {
	    jdbm clean = new jdbm(fil);
	    
	    try {
		for(int i=0; i<exportentries.length; i++){
		    byte key[] = StringX.getBytesUTF8(exportentries[i]);
                    if(null==key || key.length<1)continue;
                    byte val[] = db.lookup(key);
		    if ( val != null ){
                        String skey=StringX.fromBytesUTF8(key);
                        if(!"".equals(skey)){
                            ATDEintrag exportEintrag=new ATDEintrag(skey, val);
                            /* if "val" is binary-0, the key is stored in "locale" instead of "UTF-8" */
                            if(exportEintrag.getBinaryVersion()==0) {
                                skey=new String(key);
                            }
                            ATDEintrag newentry=new ATDEintrag(skey, exportEintrag);

                            newentry.checkSoundFiles();

                            key=StringX.getBytesUTF8(newentry.getKey());
                            val = newentry.getBytes();
			    clean.store(key, val, jdbm.STORE_INSERT);
                        }
		    }
		}
	    } catch (Exception ex){ 
                at.iem.tools.Logger.println("exportATD:  "+ex);
            }

	    clean.save();
	} catch (Exception ex){
            at.iem.tools.Logger.println("exportATD:: "+ex);
        }
    }


    /** 
     * export the data to a SAMPA-file
     */
    public void exportSAMPA(File fil, String regionID){
	exportSAMPA(fil, getAllEntries(), regionID);
    }
    public void exportSAMPA(File fil, String[]exportentries, String regionID){
	try {
	    at.iem.tools.Logger.println("exporting database as SAMPA");
	    if(exportentries==null||exportentries.length==0)return;
	    FileOutputStream ostr = new FileOutputStream(fil); 
	    OutputStreamWriter out = new OutputStreamWriter(ostr);
	    String lineEnd =  System.getProperty("line.separator");
	    String WORDSEPARATOR=";";
	    
	    try {
		for(int i=0; i<exportentries.length; i++){
		    byte key[] = StringX.getBytesUTF8(exportentries[i]);
                    if(null==key || key.length<1)continue;
                    byte val[] = db.lookup(key);
		    if ( val != null ){
                        String skey=StringX.fromBytesUTF8(key);
                        if(!"".equals(skey)){
                            ATDEintrag exportEintrag=
                                new ATDEintrag(skey, val);
                            String sampa=exportEintrag.getSampa(regionID);
                            if(sampa!=null && sampa.length()>0)
                                out.write(exportentries[i]+WORDSEPARATOR+sampa+lineEnd);
                        }
		    }
		}
	    } catch (Exception ex){ 
                at.iem.tools.Logger.println("exportSAMPA:  "+ex);
            }
	    out.flush();
	} catch (Exception ex){
            at.iem.tools.Logger.println("exportSAMPA:: "+ex);
        }
    }

    private String[] line2wort_sampa(String line){
	String wort;
	String sampa=null;
	String pair[] = new String[2];

	int trenn = line.indexOf(';');
	wort = line.substring(0,trenn);
	sampa = line.substring(trenn+1);

	pair[0]=wort.trim();
	pair[1]=sampa.trim();
	return pair;
    }
    private void importSAMPAfile(File fil, String regionID){
	/* this will only work with umlauts if the current locale is set up correctly ! */
	char WORDSEPARATOR=';'; //'\t';
	char TABULATOR='\t';
	char LINESEPARATOR=10;

	FileReader fr;

	at.iem.tools.Logger.println("trying to import SAMPA-file '"+fil.getName()+"'");

	try {
	    fr = new FileReader(fil);
	} catch (Exception e){
	    at.iem.tools.Logger.println("file not found...");
	    return;
	}

	int i;
	int count=0;
	StringBuffer line;
	String details="";
	int j=0;
	try{
	    while ((i=fr.read())>0){
		while(i>=0 && i!=WORDSEPARATOR && i!=TABULATOR && i<32)i=fr.read();
		if (i<0){
		    String msg=AdabaShared.getLabel("MessageImportedSampa");
		    msg=StringX.replace(msg, "%d", ""+count);
                    if(null!=regionID)
			msg=StringX.replace(msg, "%s", regionID);
		    else msg=StringX.replace(msg, "%s", "---");

		    AdabaMessages.showMessageDetails(msg, details);
		    return;
		}
		line = new StringBuffer();
		line.append((char)i);
		while((i=fr.read())==WORDSEPARATOR || i==TABULATOR || i>=32) {
		    line.append((char)i);
		}
		String [] splitline = line2wort_sampa(new String(line));
		if (addSAMPAentry(splitline[0], splitline[1], regionID, false)){
		    count++;
		    details=new String(details+splitline[0]+'\n');
		}
	    }
	    save();
	} catch (Exception ex){
	    at.iem.tools.Logger.println("an error occured while reading...\n--> "+ex);
	}
	String msg=AdabaShared.getLabel("MessageImportedSampa");
	msg=StringX.replace(msg, "%d", ""+count);
        if(null!=regionID) 
            msg=StringX.replace(msg, "%s", regionID);
        else msg=StringX.replace(msg, "%s", "---");

	AdabaMessages.showMessageDetails(msg, details);
	buildSearchDB();
    }
    public void importSAMPA (File fil, String regionID){
	importSAMPAfile(fil, regionID);
    }
    public void importSAMPA(String filname, String regionID){
	importSAMPA(new File(filname), regionID);
    }
    public void importSAMPA (File fil){
	importSAMPA(fil, (String)null);
    }
    public void importSAMPA(String filname){
	importSAMPA(new File(filname), (String)null);
    }
    
    public void importTXT (File fil){
	/* this will only work with umlauts if the current locale is set up correctly ! */
        char WORDSEPARATOR=';';
        char TABULATOR='\t';
	char LINESEPARATOR=10;

	at.iem.tools.Logger.println("trying to import TXT-file '"+fil.getName()+"'");
        /*
	FileReader fr;
	try {
	    fr = new FileReader(fil);
	} catch (Exception e){
	    at.iem.tools.Logger.println("file not found...");
	    return;
	}
        */
        int count=0;
        String details="";

        try {
	    String encoding="UTF-8";

	    FileInputStream istr = new FileInputStream(fil); 
	    InputStreamReader fr = new InputStreamReader(istr, encoding);

            int i;
            StringBuffer line;
            int j=0;

            try{
                
                while ((i=fr.read())>0){
                    while(i>=0 && i!=WORDSEPARATOR && i!=TABULATOR && i<32)i=fr.read();
                    if (i<0){
                        String msg=AdabaShared.getLabel("MessageImportedTXT");
                        msg=StringX.replace(msg, "%d", ""+count);
                        
                        AdabaMessages.showMessageDetails(msg, details);
                        return;
                    }
                    line = new StringBuffer();
                    line.append((char)i);
                    while((i=fr.read())==WORDSEPARATOR || i==TABULATOR || i>=32) {
                        line.append((char)i);
                    }
                    String sline=new String(line);
                    String [] splitline = sline.split("\t");
                    if (addTXTentry(splitline, false)){
                        count++;
                        details=new String(details+splitline[0]+'\n');
                    }
                }
                save();
            } catch (Exception ex){
                at.iem.tools.Logger.println("an error occured while reading...\n--> "+ex);
            }
        } catch (Exception e) {
            at.iem.tools.Logger.println("error reading file...\n--> "+e);

        }
	String msg=AdabaShared.getLabel("MessageImportedSampa");
	msg=StringX.replace(msg, "%d", ""+count);

	AdabaMessages.showMessageDetails(msg, details);
	buildSearchDB();
    }
    public void importTXT(String filname){
	importTXT(new File(filname));
    }


   // the following is for importing SAMPA-files to the dataBase
    public boolean addSAMPAentry(String eintrag, String sampa, String regionID, boolean makeODB){
	boolean ok=true;
	byte[] beintrag=StringX.getBytesUTF8(eintrag);
	ATDEintrag tentry = new ATDEintrag(eintrag);
	sampa=StringX.replace("[["+sampa+"]]", "[[[[", "[[");
	sampa=StringX.replace(sampa, "]]]]", "]]");
	tentry.setSampa(regionID, sampa);
       	try {
	    byte[] oldentry = db.lookup(beintrag);

	    if (oldentry==null){
		db.store(beintrag, tentry.getBytes(), jdbm.STORE_INSERT);
	    } else {
		ATDEintrag newentry = new ATDEintrag(eintrag, oldentry);
		newentry.setSampa(regionID, sampa);
		db.store(beintrag, newentry.getBytes(), jdbm.STORE_REPLACE);
	    }
	    //int flag =(db.lookup(beintrag)==null?jdbm.STORE_INSERT:jdbm.STORE_REPLACE);
	    //db.store(beintrag, tentry.getBytes(), flag);
	} catch (Exception e){
	    at.iem.tools.Logger.println("error adding entry...'"+eintrag+"'");
	    at.iem.tools.Logger.println(e);
	    ok=false;
	}
	buildSearchDB(makeODB);
	//save();
	return ok;
    }
    public void addSAMPAentry(String eintrag, String sampa, boolean makeODB){
	addSAMPAentry(eintrag, sampa, (String)null, true);
    }
    public void addSAMPAentry(String eintrag, String sampa){
	addSAMPAentry(eintrag, sampa, true);
    }


   // the following is for importing TXT-files to the dataBase
   public boolean addTXTentry(String eintrags[], boolean makeODB){
	boolean ok=true;
        if(null==eintrags||eintrags.length<2)return false;

        String eintrag=eintrags[0];
	byte[] beintrag=StringX.getBytesUTF8(eintrag);

        int count=eintrags.length-2;
        if(count>AdabaShared.region.length)
            count=AdabaShared.region.length;
        
       	try {
	    byte[] oldentry = db.lookup(beintrag);

	    if (oldentry==null){
                ATDEintrag tentry = new ATDEintrag(eintrag);
                if(null!=eintrags[1]&&eintrags[1].length()>0)
                    tentry.setIPA((String)null, eintrags[1]);
                for(int i=0; i<count; i++) {
                    if(null!=eintrags[i+2]&&eintrags[i+2].length()>0)
                        tentry.setIPA(AdabaShared.region[i][0], eintrags[i+2]);
                }
		db.store(beintrag, tentry.getBytes(), jdbm.STORE_INSERT);
	    } else {
		ATDEintrag newentry = new ATDEintrag(eintrag, oldentry);
                if(null!=eintrags[1]&&eintrags[1].length()>0)
                    newentry.setIPA((String)null, eintrags[1]);
                for(int i=0; i<count; i++) {
                    if(null!=eintrags[i+2]&&eintrags[i+2].length()>0)
                        newentry.setIPA(AdabaShared.region[i][0], eintrags[i+2]);
                }
		db.store(beintrag, newentry.getBytes(), jdbm.STORE_REPLACE);
	    }
	} catch (Exception e){
	    at.iem.tools.Logger.println("error adding entry...'"+eintrag+"'");
	    at.iem.tools.Logger.println(e);
	    ok=false;
	}
	buildSearchDB(makeODB);
	//save();
	return ok;
    }
    public void addTXTentry(String[] eintrags){
	addTXTentry(eintrags, true);
    }
    /**
     * import a database;
     * <code>mode</code> controls, how to handle entries, that appear in both the new and the old database
     * <table>
     * <tr><th>mode</th><th>action</th></tr>
     * <tr><td>0</td><td>never overwrite existing entries</td></tr>
     * <tr><td>1</td><td>always overwrite existing entries</td></tr>
     * <tr><td>2</td><td>overwrite existing entries only if the timestamp is newer</td></tr>
     * </table>
     * @param dbfile the database-file to import
     * @param editedOnly if set to <code>true</code> only entries with the flag "edited" are imported
     * @param mode how to handle clashing entries
     * @param storeAsAlternate whether entries that are rejected because of clashes are stored as "alternatives"
     * @return a String[] array of unresolved Entries; <code>null</code> if no clashes occured
     */
    public String[] importATD(File dbfile, 
					   boolean editedOnly, 
					   int mode, 
					   boolean storeAsAlternate){
	File[]dbfiles=new File[1];
	dbfiles[0]=dbfile;
	return importATD(dbfiles, editedOnly, mode, storeAsAlternate);
    }
    /**
     * import a database;
     * <code>mode</code> controls, how to handle entries, that appear in both the new and the old database
     * <table>
     * <tr><th>mode</th><th>action</th></tr>
     * <tr><td>0</td><td>never overwrite existing entries</td></tr>
     * <tr><td>1</td><td>always overwrite existing entries</td></tr>
     * <tr><td>2</td><td>overwrite existing entries only if the timestamp is newer</td></tr>
     * </table>
     * @param dbfile the database-files to import
     * @param editedOnly if set to <code>true</code> only entries with the flag "edited" are imported
     * @param mode how to handle clashing entries
     * @param storeAsAlternate whether entries that are rejected because of clashes are stored as "alternatives"
     * @return an array of unresolved Entries; <code>null</code> if no clashes occured
     */
    public String[] importATD(File[]dbfile, 
					   boolean editedOnly, 
					   int mode, 
					   boolean storeAsAlternate){
        TranscriptionDBInfo new_dbinfo = null;
	final boolean importAll = !editedOnly;
	final int cmode = mode; // mode, but constant
	ArrayList<String> clashes = new ArrayList<String>();
	if(dbfile==null || dbfile.length==0)return null;
	for (int file_index=0; file_index<dbfile.length; file_index++)
	  try {
	    jdbm newdb = new jdbm(dbfile[file_index]);

            try {
                byte[] info = db.lookup(TranscriptionDBInfo.getKey());
                if(info!=null) {
                    new_dbinfo = new TranscriptionDBInfo(info);
                    if(new_dbinfo!=null)
                        AdabaShared.addEntryFlags(new_dbinfo.getEntryFlags());
                }
            } catch (Exception e) {
            }

	    java.util.Enumeration e = newdb.keys();
            final String infoKey=new String(TranscriptionDBInfo.getKey(), "UTF-8");
  
	    while (e.hasMoreElements()){
		byte key[] = (byte[]) e.nextElement();
		at.iem.tools.Logger.println("importing: "+new String(key));
		String keystring=StringX.fromBytesUTF8(key);

                //at.iem.tools.Logger.println("key='"+key+"'\tkeystring='"+keystring+"'\tinfoKey='"+infoKey+"'");

                if(infoKey.equals(keystring)) {
                    at.iem.tools.Logger.println("skipping imported db definition");
                    continue;
                }
		
		byte val[] = newdb.lookup(key);
		if ( val != null ){
		    /* have a look inside: java.lang.ArrayIndexOutOfBoundsException */
		    ATDEintrag newentry = new ATDEintrag(keystring, val, new_dbinfo);
		    if (importAll || newentry.isEdited()) {

			/* if "val" is binary-0, the key is stored in "locale" instead of "UTF-8" */
			if(newentry.getBinaryVersion()==0)newentry.setKey(new String(key));
			key=StringX.getBytesUTF8(newentry.getKey());
			val = newentry.getBytes();

			byte oldval[] = db.lookup(key);
			if (oldval == null) {
			    /* ok, this is a new entry */
			    db.store(key, val, jdbm.STORE_INSERT);
			} else if (java.util.Arrays.equals(val, oldval)) {
			    /* don't do anything, as the entries are the same */
			} else {
			    /* here comes the merging thing... */
			    clashes.add(keystring); // add to the clash-table
			    TranscriptionEintrag altentry;
			    switch(cmode){
			    case 1: /* always overwrite */
				altentry=storeAlternative(key, val, oldval, storeAsAlternate);
				break;
			    case 2: /* overwrite only if newer */
				ATDEintrag oldentry =new ATDEintrag(keystring, oldval);
				if (newentry.getTimestamp()>oldentry.getTimestamp()){
				    altentry=storeAlternative(key, val, oldval, storeAsAlternate);
				} else {
				    altentry=storeAlternative(key, oldval, val, storeAsAlternate);
				}
				break;
			    default: /* never overwrite */
				altentry=storeAlternative(key, oldval, val, storeAsAlternate);
			    }
                        }
		    }
		}
	    }
	} catch (Exception ex) {
	    at.iem.tools.Logger.println("couldn't open new database...\n--> "+ ex);
	}
	buildSearchDB();
	if(clashes.size()==0)return null;
	String[]clashed=new String[clashes.size()];
	for(int i=0;i<clashed.length; i++){
	    clashed[i]=(String)clashes.get(i);
	}
	return clashed;
    }





    /** export a database to the implementor's native storage method (like an "import");
     * no assumption is made on the order of the exported database entries;
     * LATER: should the existing database be deleted?
     * @param source the database to be exported
     * @return <code>true</code> on success, <code>false</code> otherwise
     * @throws ????
     */
    public boolean exportFrom(at.iem.adaba.database.TranscriptionDataBase source) throws Exception
    {
        return exportFrom(source, source.getAllEntries());
    }

    /** export a selection of entries in the database to the implementor's native storage method (like an "import");
     * the order of the exported entries is the same as the order of the exportKeys
     * LATER: should the existing database be deleted?
     * @param source the database to be exported
     * @param exportKeys the keys to be exported
     * @return <code>true</code> on success, <code>false</code> otherwise
     */
    public boolean exportFrom(at.iem.adaba.database.TranscriptionDataBase source, java.lang.String[] exportKeys) throws Exception
    {
        boolean result=true;
        if(null==source)
            return false;
        if(null==exportKeys||exportKeys.length<1)
           return false;
        at.iem.tools.Logger.println("ATD::exportFrom #entries="+exportKeys.length);
        try {
            for(int i=0; i<exportKeys.length; i++) {
                at.iem.tools.Logger.println("ATD::exportFrom key="+exportKeys[i]);
                TranscriptionEintrag[]exportEintrags=source.lookup(exportKeys[i]);
                if(exportEintrags==null) {
                    continue;
                }
                for(int j=0; j<exportEintrags.length; j++) {
                    if(!replace(exportEintrags[j])) {
                        result=false;
                    }
                }
            }
	} catch (Exception ex) {
	    at.iem.tools.Logger.println("ATD::exportFrom failed...\n--> "+ ex);
            result=false;
	}
	buildSearchDB();

        return result;
    }
}
