// SearchDataBase.java
// $Id: SearchDataBase.java 1865 2007-07-11 12:07:47Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
// 
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

package at.iem.adaba.database.ATD;
import at.iem.adaba.*;

import at.iem.tools.StringX;
import org.w3c.tools.dbm.*;
import java.util.HashMap;
import java.util.HashSet;
import java.util.ArrayList;
import java.util.regex.*;

/**
 * hlt eine leichte Struktur in der die tatschliche
 * WortSuche vor sich geht
 * liefert die keys fr das eigentliche Wrterbuch
 *
 * String[] searchwords
 * Hashtable: searchwords->keys
 *
 * ein searchword ist ein "normalisierter" (kleingeschrieben, keine " " oder "-")
 * Sucheintrag
 */

public abstract class SearchDataBase {

    protected String[] searchwords;
    protected HashMap<String,String[]> keyhash;      // searchwords --> Eintrag(Key)

    public SearchDataBase(){}

    /**
     * create a SearchDataBase from the given <code>db</code>
     * @param db the database from which we generate the searchdatabase from
     */
    public SearchDataBase(jdbm db){
	at.iem.tools.Logger.println("trying to build searchtree");
	long tsStart=System.currentTimeMillis();
	buildDB(db);
	long tsEnd=System.currentTimeMillis();
	at.iem.tools.Logger.println("SearchDB: "+(tsEnd-tsStart)+" ms needed to build searchtree");
    }

    /**
     * read the SearchDataBase from the given <code>file</code>;
     * if the file does not exist, build the SearchDataBase from <code>db</code> 
     * and save it in <code>file</code>
     * @param file the file to read the searchdatabase from (and write it too)
     * @param db the database from which we generate the searchdatabase from
     */
    public SearchDataBase(java.io.File file, jdbm db){
	this(file, db, false);
    }
    /**
     * read the SearchDataBase from the given <code>file</code>;
     * if the file does not exist, build the SearchDataBase from <code>db</code> 
     * and save it in <code>file</code>
     * @param file the file to read the searchdatabase from (and write it too)
     * @param db the database from which we generate the searchdatabase from
     * @param rebuild force rebuild of the searchdatabase and store it in <code>file</code>
     */
    public SearchDataBase(java.io.File file, jdbm db, boolean rebuild){
	at.iem.tools.Logger.println("SearchDataBase: rebuild="+rebuild);
        if(!rebuild) {
            /* we don't want to write to search-databases that are stored in the .jar file! */
            file=at.iem.tools.FileX.fallbackToTempSystemResource(file);
        }
	if (file.exists()&&file.canRead()){
	    try {
		at.iem.tools.Logger.println("SearchDataBase: rebuild="+rebuild);
		jdbm mydb= new jdbm(file);
		keyhash=new HashMap<String,String[]>();
		ArrayList<String> keylist = new ArrayList<String>();
		
		java.util.Enumeration keys = mydb.keys();
		while(keys.hasMoreElements()){
		    byte[] bKey = (byte[])keys.nextElement();
		    if(bKey==null)continue;
		    byte[] bVal = mydb.lookup(bKey);

		    String Key = StringX.fromBytesUTF8(bKey);
		    String Val= StringX.fromBytesUTF8(bVal);
		    		    
		    keylist.add(Key);
		    keyhash.put(Key, Val.split("\n"));
		}
		at.iem.tools.Logger.println("SearchDataBase: reading done, now sorting keys");
		searchwords = new String[keylist.size()];
		for (int i=0; i<searchwords.length; i++)searchwords[i]=(String)keylist.get(i);
		//java.util.Arrays.sort(searchwords, java.text.Collator.getInstance());
		at.iem.tools.Logger.println("SearchDataBase: sorting done");

	    } catch (Exception e){
                at.iem.tools.Logger.println("searchdb-exceoption: "+e);

		rebuild=true;
	    }
	} else rebuild=true;

	if(rebuild){
	    buildDB(db);
	    try {
		jdbm mydb = new jdbm(file);

		for(int i=0; i<searchwords.length; i++){
		    String sKey=searchwords[i];
		    if(keyhash.containsKey(sKey)){
			String[]result=(String[])keyhash.get(sKey);

			byte[] bKey=StringX.getBytesUTF8(sKey);
			byte[] bValue=StringX.getBytesUTF8(StringX.merge(result, "\n"));
		
			mydb.store(bKey, bValue, jdbm.STORE_REPLACE);
		    }
		}
		mydb.forceWriting(true);
		mydb.save();
	    } catch (Exception e){
		at.iem.tools.Logger.println(e);		
	    }
	}
    }

    protected void buildDB(jdbm db){
	at.iem.tools.Logger.println("someone called abstract function SearchDataBase::buildDB!");
    }

    public String[] findRegex(String Pattern) throws Exception {
        keep_searching=true;
	Pattern=unifySearchPattern(Pattern);
	at.iem.tools.Logger.println("finding regex: "+Pattern);
	Pattern=at.iem.tools.linguistic.IPA.regex(Pattern);
	java.util.regex.Pattern pattern;
	try {
	    pattern = java.util.regex.Pattern.compile(Pattern);
	} catch(PatternSyntaxException ex){
	    throw new Exception("invalid search pattern", new Throwable(Pattern));
	}
	Matcher matcher=pattern.matcher("");

	int len = searchwords.length;
	String[] ergebnis = new String[len];

	int count = 0;
	for(int i=0; i<searchwords.length; i++){
	    //at.iem.tools.Logger.println("search["+i+"]: "+searchwords[i]);
            if(keep_searching==false)
                return null;
	    if (matcher.reset(searchwords[i]).matches()){
		ergebnis[count]=searchwords[i];
		count++;
	    }
	}
        at.iem.tools.Logger.println("searchdb found: "+count+"; now getting keys...");
        long tsStart=System.currentTimeMillis();
	String[] strippedresult=key2entry(count, ergebnis);
        long tsStop=System.currentTimeMillis();
        at.iem.tools.Logger.println("key2entry took "+(tsStop-tsStart)+" [ms]");
        return strippedresult;
    }

    public String[] findExactly(String Pattern){
	String pattern=unifyString(Pattern);
	if (keyhash.containsKey(pattern)){
	    String [] result = (String[])keyhash.get(pattern);
	    return result;
	}
	return null;
    }
    public String[] startsWith(String Pattern){
	if (searchwords==null || searchwords.length==0)return null;
	String pattern=unifyString(Pattern);

	int len = searchwords.length;
	String[] ergebnis = new String[len];

	int count = 0;
	for(int i=0; i<searchwords.length; i++){
	    if (searchwords[i].startsWith(pattern)){
		ergebnis[count]=searchwords[i];
		count++;
	    }
	}
	return key2entry(count, ergebnis);
    }
    public String[] endsWith(String Pattern){
	if (searchwords==null || searchwords.length==0)return null;
	String pattern=unifyString(Pattern);
	int len = searchwords.length;
	String[] ergebnis = new String[len];

	int count = 0;
	int i = 0;
	while(i<len){
	    if (searchwords[i].endsWith(pattern)){
		ergebnis[count]=searchwords[i];
		count++;
	    }
	    i++;
	}
	return key2entry(count, ergebnis);
    }
    public String[] contains(String Pattern){
	if (searchwords==null || searchwords.length==0)return null;

	String pattern=unifyString(Pattern);
	int len = searchwords.length;
	String[] ergebnis = new String[len];

	int count = 0;
	int i = 0;
	while(i<len){
	    int idx = searchwords[i].indexOf(pattern, 1);
	    if (idx>0 && idx<searchwords[i].length()-pattern.length()){
		ergebnis[count]=searchwords[i];
		count++;
	    }
	    i++;
	}
	return key2entry(count, ergebnis);
    }
    public String[] search(String Pattern, 
			   boolean startswith, 
			   boolean contains, 
			   boolean endswith){

	if (searchwords==null || searchwords.length==0)return null;
	if (Pattern==null || Pattern.length()==0)return null;

	int mask = (startswith?1:0)+(contains?2:0)+(endswith?4:0);

	switch(mask){
	default: break;
	case 1: Pattern=Pattern+".*";
	    break;
	case 2: Pattern=".+"+Pattern+".+";
	    break;
	case 3: Pattern=".*"+Pattern+".+";
	    break;
	case 4: Pattern=".*"+Pattern;
	    break;
	case 5: Pattern="(.*"+Pattern+")|("+Pattern+".*)";
	    break;
	case 6: Pattern=".+"+Pattern+".*";
	    break;
	case 7: Pattern=".*"+Pattern+".*";
	    break;
	}

	try {
	    return findRegex(Pattern);
	} catch(Exception ex){
	    return null;
	}
    }

    private String[] key2entry(int len, String[]key) {
        return key2entry_fast(len, key);
    }

    private String[] key2entry_slow(int len, String[] key) {
        keep_searching=true;
	long tsStart=System.currentTimeMillis();
	ArrayList<String> eintrag = new ArrayList<String>(len);
	if (len<1)return null;
        if(keep_searching==false)return null;
        keep_searching=true;

	for(int i=0; i<len; i++){
            if(keep_searching==false)return null;
	    String [] addkeys = (String[])keyhash.get(key[i]);
	    int L=addkeys.length;
	    for(int j=0; j<L; j++){
                String addkey=addkeys[j];
                if(!eintrag.contains(addkey)) {
                    eintrag.add(addkey);
                }
            }
	}
	len=eintrag.size();
	String[] result = new String[len];
	while(len-->0){
            if(keep_searching==false)return null;
            result[len]=(String)eintrag.get(len);
        }
	return result;
    }

    private String[] key2entry_fast(int len, String[] key) {
        keep_searching=true;
        HashSet hs = new HashSet(len);
        System.out.println("key2entry: "+len+" <= "+key.length);
        System.out.println("keepsearching: "+keep_searching);

        for(int i=0; i<len; i++) {
            if(keep_searching==false)return null;
            at.iem.tools.Logger.println("adding key: "+key[i]);
	    String [] addkeys = (String[])keyhash.get(key[i]);
	    for(int j=0; j<addkeys.length; j++){
                hs.add(addkeys[j]);
            }
        }

        String[]result=new String[hs.size()];

        int count=0;
        java.util.Iterator it=hs.iterator();
        while(it.hasNext()) {
            result[count]=(String)it.next();
            count++;
            if(count>=result.length)
                break;
        }
            
        return result;
    }



    protected String unifyString(String org){
        at.iem.tools.Logger.println("SearchDB: unifyString");
	return org;
    }
    protected String unifySearchPattern(String org){
        at.iem.tools.Logger.println("SearchDB: unifySearchPattern: ...");
	return org;
    }

    /**
     * stop a running search (e.g. from another thread...)
     */
    public void stopSearch() {
        keep_searching=false;
    }

    /**
     * every funtion that listens to the keep_searching flag, OUGHT to set it to <code>true</code> at the beginning
     * so that we cleanup any leftovers
     */
    private volatile boolean keep_searching;

}
