// SearchOrthoDB.java
// $Id: SearchOrthoDB.java 1831 2007-06-28 14:49:48Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
// 
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

package at.iem.adaba.database.ATD;
import at.iem.adaba.database.TranscriptionEintrag;
import at.iem.adaba.*;

import at.iem.tools.StringX;
import org.w3c.tools.dbm.*;
import java.util.HashMap;
import java.util.ArrayList;

/**
 * hlt eine leichte Struktur in der die tatschliche
 * WortSuche vor sich geht
 * liefert die keys fr das eigentliche Wrterbuch
 *
 * String[] searchwords
 * Hashtable: searchwords->keys
 *
 * ein searchword ist ein "normalisierter" (kleingeschrieben, keine " " oder "-")
 * Sucheintrag
 */

public class SearchOrthoDB extends SearchDataBase{

    public SearchOrthoDB(jdbm db){
	super(db);
	at.iem.tools.Logger.println("SearchOrthoDB");
    }
    public SearchOrthoDB(java.io.File file, jdbm db){
	super(file, db);
	at.iem.tools.Logger.println("SearchOrthoDB_file: "+ file);
    }
    protected void buildDB(jdbm db){
        AdabaShared.openSplash("resources/splashOrthographic.gif");
	at.iem.tools.Logger.println("SearchOrthoDB:buildDB");
	keyhash = new HashMap<String,String[]>();// used to initialize to 2...
	ArrayList<String> keylist = new ArrayList<String>();// used to initialize to 2...
	
	int count = 0;
	java.util.Enumeration keys = db.keys();
        long tsStart=System.currentTimeMillis();
	try{
            final String infoKey=new String(TranscriptionDBInfo.getKey(), "UTF-8");
	    int i=0;
	    while(keys.hasMoreElements()){
		String Key = StringX.fromBytesUTF8((byte[])keys.nextElement());
                if(Key.equals(infoKey)){
                    continue;
                }		i++;
		//at.iem.tools.Logger.println("ORTHO found key: "+Key);

		String[] key = key2strings(Key);
		if (key==null)continue;
	    
		int index = key.length;
		while(index-->0){
		    String nkey = new String(key[index]);
		    //at.iem.tools.Logger.println("checking "+nkey);

		    String [] newvals;
		    //at.iem.tools.Logger.println("existing="+keylist.contains(nkey));
		    if (!keylist.contains(nkey))keylist.add(nkey);
		    if (keyhash.containsKey(nkey)){
			String[] oldvals = (String[])keyhash.get(nkey);
			keyhash.remove(nkey);
			newvals = new String[oldvals.length+1];
			if (nkey==Key.toLowerCase()){
			    newvals[0]=Key;
			    System.arraycopy(oldvals, 0, newvals, 1, oldvals.length);
			} else {
			    System.arraycopy(oldvals, 0, newvals, 0, oldvals.length);
			    newvals[oldvals.length]=Key;
			}
		    } else {
			newvals = new String[1];
			newvals[0] = Key;
		    }
		    keyhash.put(nkey, newvals);
		}
	    }
	    at.iem.tools.Logger.println(i+" entries parsed");
	} catch (Exception ex) {
	    at.iem.tools.Logger.println(ex);
	}
        long tsStop=System.currentTimeMillis();
	at.iem.tools.Logger.println("searchDB-infrastructure built in "+(tsStop-tsStart)+" [ms]");

	searchwords = new String[keylist.size()];
	for (int i=0; i<searchwords.length; i++)searchwords[i]=(String)keylist.get(i);
	//java.util.Arrays.sort(searchwords, java.text.Collator.getInstance());
    }

    public String[] key2strings(String key){
	// macht aus einem key-string mehrere sucheintrge (keys), die im org. string
	// durch leerzeichen/bindestriche getrennt sind
	// gibt's da nichts schnelleres/eleganteres ??
	if (key==null)return null;
	key=new String(key.trim().toLowerCase());
	int length = key.length();
	if (length==0)return null;

	String[]prewords = key.split(" -*");

	int count=1; // the complete key

	// check for words with umlauts...
	if(!key.equals(unifyString(key)))count++;
	for(int i=0; i<prewords.length; i++)
	    if(!prewords[i].equals(unifyString(prewords[i])))count++;


	String[]words=new String[count+prewords.length];

	count=0;
	words[count++]=key;
	if(!key.equals(unifyString(key)))words[count++]=unifyString(key);

	for(int i=0; i<prewords.length; i++){
	    words[count++]=prewords[i];
	    if(!prewords[i].equals(unifyString(prewords[i])))words[count++]=unifyString(prewords[i]);
	}

	return words;
    }

    protected String unifyString(String org){
	String nju = new String(org).trim();
	String test="";
	int j=0;
	while(j<nju.length()){
	    if (Character.isLetterOrDigit(nju.charAt(j)))test+=nju.charAt(j);
	    if(nju.charAt(j)=='.')test+='.';
	    j++;
	}
	test=test.trim().toLowerCase();
	
	test=StringX.replace(test, "", "e");
	test=StringX.replace(test, "", "ae");
	test=StringX.replace(test, "", "ae");
	test=StringX.replace(test, "\u0153", "oe");//~
	test=StringX.replace(test, "", "oe");
	test=StringX.replace(test, "", "ue");
	//test=StringX.replace(test, "", "sz");

	test=StringX.replace(test, "...", "\u2026");
	test=StringX.replace(test, "..", "\u2026");
	test=StringX.replace(test, ".", "");
	test=StringX.replace(test, "\u2026", "...");

	return test;
    }

    private String[] unifyKey(String[] okey){
	ArrayList<String> key = new ArrayList<String>(okey.length);
	for(int i=0; i<okey.length; i++){
	    String test=unifyString(okey[i]);
	    boolean existing = (test==null || 
				test.length()==0);  
	    if (test!=null && test.length()!=0 && !key.contains(test))key.add(test);
	}
	String[] nkey=new String[key.size()];
	for(int i=0; i<nkey.length; i++){
	    nkey[i]=(String)key.get(i);
	}
	return nkey;
    }
    protected String unifySearchPattern(String org){
	org=org.trim();
	
	// a naive escaping of regexp-specials
	org=StringX.replace(org, "\\p{IPA", "\1");
	org=StringX.replace(org, "\\P{IPA", "\2");
	
	// better would be, that anything withing "\p{..}" or "\P{..}" would not be lowercased!
	org=org.toLowerCase();
	
	org=StringX.replace(org, "", "e");
	org=StringX.replace(org, "", "ae");
	org=StringX.replace(org, "", "ae");
	org=StringX.replace(org, "", "oe");
	org=StringX.replace(org, "", "ue");
	//org=StringX.replace(org, "", "sz");

	org=StringX.replace(org, "\1", "\\p{IPA");
	org=StringX.replace(org, "\2", "\\P{IPA");

	return org;
    }
}
