// SearchPhoneticDB.java
// $Id: SearchPhoneticDB.java 1831 2007-06-28 14:49:48Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
// 
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

package at.iem.adaba.database.ATD;
import at.iem.adaba.database.TranscriptionEintrag;
import at.iem.adaba.*;

import at.iem.tools.StringX;
import org.w3c.tools.dbm.*;
import java.util.HashMap;
import java.util.ArrayList;

/**
 * hlt eine leichte Struktur in der die tatschliche
 * WortSuche vor sich geht
 * liefert die keys fr das eigentliche Wrterbuch
 *
 * String[] searchwords
 * Hashtable: searchwords->keys
 *
 * ein searchword ist ein "normalisierter" (kleingeschrieben, keine " " oder "-")
 * Sucheintrag
 */

public class SearchPhoneticDB extends SearchDataBase{

    public SearchPhoneticDB(jdbm db){
	super(db);
	at.iem.tools.Logger.println("SearchPhoneticDB");
    }
    public SearchPhoneticDB(java.io.File file, jdbm db){
	super(file, db);
	at.iem.tools.Logger.println("SearchPhoneticDB_file: "+file);
    }
    protected void buildDB(jdbm db){
        AdabaShared.openSplash("resources/splashPhonetic.gif");
	at.iem.tools.Logger.println("SearchPhoneticDB:buildDB");
	
	keyhash = new HashMap<String,String[]>();// used to initialize to 2...
	ArrayList<String> keylist = new ArrayList<String>();// used to initialize to 2...
	
	int count = 0;
	java.util.Enumeration keys = db.keys();
	try{
	    int i=0;
            final String infoKey=new String(TranscriptionDBInfo.getKey(), "UTF-8");
	    while(keys.hasMoreElements()){
		String keystring=StringX.fromBytesUTF8((byte[])keys.nextElement());
                if(keystring.equals(infoKey)){
                    continue;
                }
		byte val[] = db.lookup(StringX.getBytesUTF8(keystring));
		i++;
		if(val!=null){
		    TranscriptionEintrag entry =new ATDEintrag(keystring, val);
                    //at.iem.tools.Logger.println("adding #"+count+": "+keystring); count++;
		    /* go through each and every IPA-entry in the "entry" and store it in the hashtable */
                    at.iem.tools.linguistic.IPA[] ipas = entry.getIPAs();
                    if(ipas!=null) {
                        for(int j=0; j<ipas.length; j++){
                            if(null==ipas[j])continue;
                            String eipa=new String(ipas[j].toString());
                            eipa=StringX.replace(eipa, "...", "\u2026");
                            eipa=StringX.replace(eipa, "..", "\u2026");
                            at.iem.tools.linguistic.IPA ipa=new at.iem.tools.linguistic.IPA(eipa);
                            
                            String[] key=at.iem.tools.SetOperator.union(ipa.getFormulae("[[", "]]"), 
                                                                    ipa.getSubTranscriptions("[[", "]]"));
                            if (key==null)continue;
                            int index = key.length;
                            while(index-->0){
                               String nkey = new String(key[index]);
                                nkey=StringX.replace(nkey, "\u2026", "...");
                                String [] newvals = null;
                                //at.iem.tools.Logger.println(eipa+": "+nkey);
                                if (!keylist.contains(nkey))keylist.add(nkey);
                                if (keyhash.containsKey(nkey)){
                                    String[] oldvals = (String[])keyhash.get(nkey);
                                    /* only add keystring if it is not already in they oldvals ! */
                                    if(!StringX.contains(oldvals, keystring)){
                                        keyhash.remove(nkey);
                                        newvals = new String[oldvals.length+1];
                                        System.arraycopy(oldvals, 0, newvals, 0, oldvals.length);
                                        newvals[oldvals.length]=keystring;
                                    }
                                } else {
                                    newvals = new String[1];
                                    newvals[0] = keystring;
                                }
                                if(newvals!=null)keyhash.put(nkey, newvals);
                            }
                        }
                    }
                }
            }
	    at.iem.tools.Logger.println(i+" entries parsed");
	} catch (Exception ex) {
	    at.iem.tools.Logger.println(ex);
	}
	at.iem.tools.Logger.println("phoneticDB-infrastructure built with "+keylist.size()+" subentries");

	searchwords = new String[keylist.size()];
	for (int i=0; i<searchwords.length; i++)searchwords[i]=(String)keylist.get(i);
	//java.util.Arrays.sort(searchwords, java.text.Collator.getInstance());
    }

    protected String unifyString(String org){
	String test = new String(org);
	/*
	test=StringX.replace(test, "...", "\u2026");
	test=StringX.replace(test, "..", "\u2026");
	test=StringX.replace(test, ".", "");
	test=StringX.replace(test, "\u2026", "...");
	*/
	return test.toUpperCase();
    }

}
