// MasterInfo.java
// $Id: TranscriptionEintrag.java 921 2006-05-16 20:07:50Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
// 
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

// how to make this work ???
package at.iem.adaba.database.ATD;
import at.iem.adaba.*;

import at.iem.tools.linguistic.IPA;
import at.iem.tools.*;

import java.io.File;
import java.io.OutputStreamWriter;
import java.io.ByteArrayOutputStream;
import java.util.ArrayList;
import java.lang.Short;

/**
 * die Struktur, die Meta-Information eines Wrterbuches beinhaltet
 * zb. EntryFlag, lastModified,...
 */

public class TranscriptionDBInfo {
    /**
     * time of last modification
     */
    private long timestamp;

    /**
     * the binary format we have been created from
     * defaults to the most actual version
     */
    private int version=2;

    /**
     * flags available for tagging entries
     */
    public String[] EntryFlag={};

    /**
     * make an empty TranscriptionDBInfo
     * the fields are taken from AdabaShared
     */
    public TranscriptionDBInfo(){
        EntryFlag=AdabaShared.EntryFlag;
    }
    /**
     * make a TranscriptionDBInfo
     * the fields are provided in binary-format
     * @param b the binary data
     */
    public TranscriptionDBInfo(byte[] b){
	//this(eintrag);
	this.version=0;
	timestamp=0;
	int lnge  = b.length;
	int magic= (b[0]<<8)+b[1];
	if (b[1]<0)magic+=256;
	if (b[0]<0)magic+=65536;
	if (lnge==magic){
            at.iem.tools.Logger.println("unknown bytes version");
	} else if (magic==0){// this is real magic...
	    int l_version=b[2];
	    this.version=l_version;
	    switch (l_version){
	    case 2: interpreteBytes_2(b); break;
	    default:
		at.iem.tools.Logger.println("TranscriptionDBInfo: "+
				   "current bytes version #"+l_version+
				   " is not supported!");
	    }
	} else {
            at.iem.tools.Logger.println("couldn't read byte[]-struct!");
            return;
        }

        /* add the current entry flag to the global ones! */
        AdabaShared.EntryFlag=EntryFlag;
    }

    private byte[]short2bytes(short s){
	byte[]buf=new byte[2];
	buf[0]=(byte)(s>>>8);
	buf[1]=(byte)(s);
	return buf;
    }
    private byte[]short2bytes(int i){
	return short2bytes((short)i);
    }
    private short bytes2short(byte[] b){
	if (b==null || b.length!=2)return 0;
	short s=(short)(b[0]<<8);
	s+=(short)(b[1]);
	return s;
    }
    private short bytes2short(byte b0, byte b1){
	int s0=b0;
	int s1=b1;
	if(s0<0)s0+=256;
	if(s1<0)s1+=256;
	return (short)((s0<<8)+s1);
    }

    /**
     * decode binary data;
     * @param b the binary data (version 2)
     */
    private void interpreteBytes_2(byte[] b){
	try {
	    int index=3;
	    int size=0;
	    byte[] buf=null;
	    String s=null;
	    /* timestamp 00000000000*/
	    size=16;
	    buf = new byte[size];System.arraycopy(b, index, buf, 0, size);index+=size;
	    s=new String(buf).trim();
	    timestamp=java.lang.Long.parseLong(s, 16);

	    /* spezial-flags */
	    size=bytes2short(b[index],b[index+1]); index+=2; // number of flags stored
            at.iem.tools.Logger.println("read flags: "+EntryFlag.length);
            if(size>0){
                EntryFlag=new String[size];
            }
            for(int i=0; i<size; i++){
                int flagsize=bytes2short(b[index],b[index+1]); index+=2; // length of flag
                buf = new byte[flagsize];
                System.arraycopy(b, index, buf, 0, flagsize); index+=flagsize;
                EntryFlag[i]=new String(buf, "UTF-8");
                //at.iem.tools.Logger.println("got flag #"+i+": "+new String(buf, "UTF-8"));
            }

	} catch (Exception e){
	    at.iem.tools.Logger.println("error: "+e);
	}
    }

    /**
     * set the timestamp
     * @param ts the timestamp to set to
     */
    public void touch(long ts){
	timestamp=ts;
    }
    /**
     * set the timestamp to now
     */
    public void touch(){
	timestamp=System.currentTimeMillis();
    }

    /**
     * get the timestamp
     * @return the current timestamp
     */
    public long getTimestamp(){
	return timestamp;
    }

    /**
     * get the binary-version of this Entry;
     * Entries are stored in binary format in the database; 
     * when the binary is decoded into an instance of this class, the version is set;
     * note: the Entry is always written to the db in the most recent binary format
     * @return the binary version of this Entry
     */
    public int getBinaryVersion(){
	return version;
    }

    /**
     * encode this TranscriptionDBInfo into binary-data<br>
     * you can specify the binary-format version for backwards-compatibility
     * @param version the version of the binary-format
     * @return an array of bytes containing this TranscriptionDBInfo in binary format
     */
    public byte[] getBytes(int version) {
	switch (version){
        case 2:
	default:
            return getBytes_2();
	}
    }
    /**
     * encode this TranscriptionDBInfo into binary-data using the current format
     * @return an array of bytes containing this TranscriptionDBInfo in binary format
     */
    public byte[] getBytes() { 
	/* get the current version */
	return getBytes(-1);
    }

    /**
     * encode this TranscriptionDBInfo into binary-data using the format-v2
     * @return an array of bytes containing this TranscriptionDBInfo in binary format
     */
    private  byte[] getBytes_2(){ // version 2
	byte [] result = null;		
	ByteArrayOutputStream out = new ByteArrayOutputStream(); //Write to an output stream
	try{
	    /* magic number: 00 */
	    out.write((byte)0); out.write((byte)0);
	    /* binary-format version 2 */
	    out.write((byte)2);

	    /* timestamp */
	    String s=java.lang.Long.toHexString(timestamp);
	    String s1=(new String("0000000000000000".getBytes(), 0, 16-s.length())+s).trim();
	    out.write(s1.getBytes());

	    /* spezial-flags ... */
            //at.iem.tools.Logger.println("write flags: "+EntryFlag.length);
	    out.write(short2bytes(EntryFlag.length));
	    for(int i=0; i<EntryFlag.length; i++){
                byte[] flag=StringX.getBytesUTF8(EntryFlag[i]);
                out.write(short2bytes(flag.length));
 		out.write(flag);
	    }

	    /* writing done: convert to byte[]-buffer */
	    result = out.toByteArray();  

	} catch(Exception e){
	    at.iem.tools.Logger.println("error converting to buffer: "+e);
	    return null;
	}
	return result;
    }

    /** 
     * print some interesting things about this TranscriptionDBInfo
     */
    public void print(){
	at.iem.tools.Logger.println("oops: nothing here yet");
    }

    /**
     * a magic key to store this field in the jdbm-database 
     */
    public static byte[] getKey() {
        return StringX.getBytesUTF8("");
    }


    /**
     * get the EntryFlag of a specified ID
     * @param id the index of this entry-flag as stored in the entries
     * @return the EntryFlag or <code>null</code> if the id is invalid
     */
    public String getEntryFlag(int id) {
        if(null==EntryFlag)
            return null;
        if(id<0||id>=EntryFlag.length)
            return null;
        return EntryFlag[id];
    }

    /**
     * get all EntryFlag's of this database
     * @return the EntryFlags or <code>null</code> if there are none
     */
    public String[]getEntryFlags() {
        return EntryFlag;
    }

     /**
     * add a number of new EntryFlags to the existing ones, eliminating duplicates
     * @param newFlags a String[] of EntryFlags to be added
     */
    public void addEntryFlags(String[]newFlags) {
        int i=0;
        for(i=0; i<newFlags.length; i++) {
            /* probably not the most efficient way! */
            getEntryFlagIndex(newFlags[i]);
        }
    }

    /** 
     * lookup the index of an EntryFlag; 
     * if the EntryFlag does not yet exist, add it
     */
    public short getEntryFlagIndex(String flag) {
        int i=0;
      	for(i=0; i<EntryFlag.length; i++)
            if(flag.equals(EntryFlag[i]))return (short)i;
        // couldn't find it!
        String[]newflags=new String[EntryFlag.length+1];
        for(i=0; i<EntryFlag.length; i++)
            newflags[i]=EntryFlag[i];
        newflags[EntryFlag.length]=flag;
        EntryFlag=newflags;
        return (short)(newflags.length-1);
    } 

  
}
