// TranscriptionEintrag.java
// $Id: TranscriptionEintrag.java 1305 2007-01-25 12:21:44Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
//
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

// how to make this work ???
package at.iem.adaba.database;
import at.iem.adaba.*;

import at.iem.tools.linguistic.IPA;
import at.iem.tools.*;

import java.io.File;
import java.io.OutputStreamWriter;
import java.io.ByteArrayOutputStream;
import java.util.ArrayList;
import java.lang.Short;
import java.util.HashMap;
import java.util.Iterator;


/**
 * die Struktur, die alle Information eines Wrterbucheintrags beinhaltet
 */

public class EintragPattern {
    public static final byte PATTERN_IGNORE  = 0;
    public static final byte PATTERN_EQ   = 1;
    public static final byte PATTERN_NEQ  = 2;
    public static final byte PATTERN_GT   = 3;
    public static final byte PATTERN_LT   = 4;


    /**
     * match the key of an entry
     */
    private String  Key=null;
    private byte matchKey=PATTERN_IGNORE;


    /**
     * get the key of an entry to be matched
     * @return the key-pattern
     */
    public String getKey() { return Key; }
    /**
     * set the key of an entry to be matched
     */
    public void   setKey(String value)
    {
        Key=value;
        if(Key!=null && matchKey==PATTERN_IGNORE) matchKey=PATTERN_EQ;
    }
    public byte   getMatchKey() { return matchKey; }
    public void   setMatchKey(byte matchtype) { matchKey=matchtype; }

    /**
     * match the etymological word type
     */
    private byte    eType=0;
    private byte matchEType=PATTERN_IGNORE;

    public byte getEType() { return eType; }
    public void setEType(byte value)
    {
        eType=value;
        if(eType>0&&matchEType==PATTERN_IGNORE) matchEType=PATTERN_EQ;
        if(eType<=0)matchEType=PATTERN_IGNORE;
    }
    public byte getMatchEType() { return matchEType; }
    public void setMatchEType(byte matchtype) { matchEType = matchtype; }

    /**
     * match the etymological word type
     */
    private byte    gType=0;
    private byte matchGType=PATTERN_IGNORE;

    public byte getGType() { return gType; }
    public void setGType(byte value)
    {
        gType=value;
        if(gType>0&&matchGType==PATTERN_IGNORE) matchGType=PATTERN_EQ;
        if(gType<=0)matchGType=PATTERN_IGNORE;
    }
    public byte getMatchGType() { return matchGType; }
    public void setMatchGType(byte matchtype) { matchGType = matchtype; }

    /**
     * an array of various "special"-flags... (holds the INDEX of each set flag)
     */
    private ArrayList<Short> flags=null;
    private byte matchFlags=PATTERN_IGNORE;

    public ArrayList<Short> getFlags()
    {
        return flags;
    }
    public void setFlags(short value, boolean enable) 
    { 
        ;/* LATER */ 
        if(flags==null)
            flags=new ArrayList<Short>(1);

       if(enable){
	    // add flag
	    if(!flags.contains(value))flags.add(value);
	} else {
	    // remove this flag
	    int idx=-1;
	    while((idx=flags.indexOf(value))>-1)flags.remove(idx);
	}

       if(flags.size()>0)
           matchFlags=PATTERN_EQ;

    }
    public void clearFlags()
    {
        ;/* LATER */
        flags.clear();
        matchFlags=PATTERN_IGNORE;
    }
    public byte getMatchFlags() { return matchFlags; }
    public void setMatchFlags(byte matchtype) { matchFlags = matchtype; }

    /**
     * match any transcription
     */
    private IPA      ipa=null;
    private byte matchIPA=PATTERN_IGNORE;

    public IPA  getIPA() { return ipa; }
    public void setIPA(IPA value)
    {
        ipa=value;
        if(ipa!=null && matchIPA==PATTERN_IGNORE) matchIPA=PATTERN_EQ;
    }
    public byte getMatchIPA() { return matchIPA; }
    public void setMatchIPA(byte matchtype) { matchIPA = matchtype; }

    /**
     * match a specific transcription
     */
    protected HashMap<String, IPA> IPAs;
    protected byte matchIPAs=PATTERN_IGNORE;

    public IPA  getIPAs(String id) { return IPAs.get(id); }
    public void setIPAs(String id, IPA i)
    {
        IPAs.put(id, i);
        if(i!=null && matchIPAs==PATTERN_IGNORE) matchIPAs=PATTERN_EQ;
    }
    public void clearIPAs()
    {
        Iterator keys=IPAs.keySet().iterator();
        while(keys.hasNext())
            IPAs.put((String)keys.next(), null);
        matchIPAs=PATTERN_IGNORE;
    }
    public byte getMatchIPAs() { return matchIPAs; }
    public void setMatchIPAs(byte matchtype) { matchIPAs = matchtype; }

    /**
     * match whether the pronunciations are differing a lot
     */
    private boolean variant=false;
    private byte matchVariant=PATTERN_IGNORE;

    public boolean getVariant() { return variant; }
    public void    setVariant(boolean value)
    {
        variant=value;
        if(matchVariant==PATTERN_IGNORE) matchVariant=PATTERN_EQ;
    }
    public byte    getMatchVariant() { return matchVariant; }
    public void    setMatchVariant(byte matchtype) { matchVariant = matchtype; }

    /**
     * match the editing flag
     */
    private boolean edited;
    private byte matchEdited=PATTERN_IGNORE;

    public boolean getEdited() { return edited; }
    public void    setEdited(boolean value)
    {
        edited=value;
        if(matchEdited==PATTERN_IGNORE) matchEdited=PATTERN_EQ;
    }
    public byte    getMatchEdited() { return matchEdited; }
    public void    setMatchEdited(byte matchtype) { matchEdited = matchtype; }

    /**
     * time of last modification
     */
    private long timeStamp;
    private byte matchTimeStamp=PATTERN_IGNORE;

    public long getTimeStamp() { return timeStamp; }
    public void setTimeStamp(long value)
    {
        timeStamp=value;
        if(timeStamp>0&&matchTimeStamp==PATTERN_IGNORE) matchTimeStamp=PATTERN_EQ;
    }
    public byte getMatchTimeStamp() { return matchTimeStamp; }
    public void setMatchTimeStamp(byte matchtype) { matchTimeStamp = matchtype; }

    /**
     * has at least one sound files
     */
    private boolean have_files=false;
    private byte matchFiles=PATTERN_IGNORE;

    public boolean getFiles() { return have_files; }
    public void    setFiles(boolean value)
    {
        have_files=value;
        if(matchFiles==PATTERN_IGNORE) matchFiles=PATTERN_EQ;
    }
    public byte    getMatchFiles() { return matchFiles; }
    public void    setMatchFiles(byte matchtype) { matchFiles = matchtype; }



    /**
     * make an empty EintragPattern (matches everything)
     */
    public EintragPattern(){
    }
    /**
     * make an EintragPattern that matches a given key (name); 
     * useful for orthographic search
     */
    public EintragPattern(String key){
        this();
        setKey(key);
    }

    /**
     * make an EintragPattern that matches a given transcription; 
     * useful for phonetic search
     */
    public EintragPattern(IPA transcription){
        this();
        setIPA(transcription);
    }

    public boolean matches(TranscriptionEintrag entry)
    {
        at.iem.tools.Logger.println("matching....");
        if((matchKey  !=PATTERN_IGNORE)&&(match(matchKey,   Key,   entry.getName())))return false;
        if((matchEType!=PATTERN_IGNORE)&&(match(matchEType, eType, entry.getEtyp())))return false;
        if((matchGType!=PATTERN_IGNORE)&&(match(matchGType, gType, entry.getGtyp())))return false;
        // flags...
        if((matchFlags!=PATTERN_IGNORE)&&(match(matchFlags, flags, entry.getFlag())))return false;
        if(matchIPA!=PATTERN_IGNORE) {
            at.iem.tools.Logger.println("EintragPattern should match with IPA: not yet implemented!");
            //if(match(matchIPA, ipa, entry.getIPA()))return false;
        }
        if(matchIPAs!=PATTERN_IGNORE){
            Iterator keys=IPAs.keySet().iterator();
            while(keys.hasNext()) {
                String key=(String)keys.next();
                if(!match(matchIPAs, IPAs.get(key), entry.getIPA(key)))return false;
            }
        }
        if((matchVariant!=  PATTERN_IGNORE)&&(match(matchVariant,   variant,    entry.isVariant   ())))return false;
        if((matchEdited!=   PATTERN_IGNORE)&&(match(matchEdited,    edited,     entry.isEdited    ())))return false;
        if((matchTimeStamp!=PATTERN_IGNORE)&&(match(matchTimeStamp, timeStamp,  entry.getTimestamp())))return false;
        if((matchFiles!=    PATTERN_IGNORE)&&(match(matchFiles,     have_files, entry.hasMedia    ())))return false;

	return true;
    }

    private static boolean match(byte matchType, String pattern, String key)
    {
        if(pattern==null)return true;
        if(key==null)return true;

        /* LATER use regex */

	switch(matchType) {
	case PATTERN_NEQ:	    return (pattern.equals(key));
	case PATTERN_EQ:	    return (!pattern.equals(key));
	default:	            return true;
	}
    }
    private static boolean match(byte matchType, IPA pattern, IPA key)
    {
        if(pattern==null)return true;
        if(key==null)return true;

        /* LATER use regex */

	switch(matchType) {
	case PATTERN_NEQ:	    return (pattern.equals(key));
	case PATTERN_EQ:	    return (!pattern.equals(key));
	default:	            return true;
	}
    }
    private static boolean match(byte matchType, boolean pattern, boolean key)
    {
	switch(matchType) {
	case PATTERN_EQ:            return (pattern==key);
	case PATTERN_NEQ:	    return (pattern!=key);
	default:	            return true;
	}
    }
    private static boolean match(byte matchType, int pattern, int key)
    {
	switch(matchType) {
	case PATTERN_EQ:	    return (pattern==key);
	case PATTERN_NEQ:	    return (pattern!=key);
	case PATTERN_GT:	    return (pattern>key);
	case PATTERN_LT:	    return (pattern<key);
        default:                    return true;
	}
    }

    private static boolean match(byte matchType, long pattern, long key)
    {
	switch(matchType) {
	case PATTERN_EQ:	    return (pattern==key);
	case PATTERN_NEQ:	    return (pattern!=key);
	case PATTERN_GT:	    return (pattern>key);
	case PATTERN_LT:	    return (pattern<key);
        default:                    return true;
	}
    }


    //match(matchFlags, flags, entry.getFlag())))
    private static boolean match(byte matchType, ArrayList<Short>pattern, short[] key) {
        at.iem.tools.Logger.println("amtchflags..."+key);
        switch(matchType) {
        case PATTERN_EQ:
            for(int i=0; i<key.length; i++) {
                if(!pattern.contains(key[i]))return false;
            }
            return true;
        case PATTERN_NEQ:
            for(int i=0; i<key.length; i++) {
                if(pattern.contains(key[i]))return false;
            }
            return true;
        default:
        }

        return true;
    }

}
