// TXTfile.java
// $Id: TXTfile.java 1161 2007-01-02 15:50:12Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
//
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

package at.iem.adaba.database.TXT;

import at.iem.adaba.AdabaShared;
import at.iem.adaba.database.*;
import at.iem.tools.linguistic.IPA;
import java.io.OutputStreamWriter;


public class TXTfile implements Exporter {

    private java.io.File file=null;

    public TXTfile(java.io.File f) 
    {
	file=f;
    }

    
    /** export a database to a TXT-file;
     * no assumption is made on the order of the exported database entries
     * @param source the database to be exported
     * @return <code>true</code> on success, <code>false</code> otherwise
     */
    public boolean exportFrom(TranscriptionDataBase source) throws Exception 
    {
	return exportFrom(source, source.getAllEntries());
    }


    /** export a selection of entries to a TXT-file;
     * the order of the exported entries is the same as the order of the exportKeys
     * @param source the database to be exported
     * @param exportKeys the keys to be exported
     * @return <code>true</code> on success, <code>false</code> otherwise
     */
    public boolean exportFrom(TranscriptionDataBase source, java.lang.String[] exportKeys) throws Exception
    {
	at.iem.tools.Logger.println("exporting database to TXT");
        if(exportKeys==null||exportKeys.length==0)return false;
	try {
	    String encoding="UTF-8";
            char WORDSEPARATOR='\t';


	    java.io.FileOutputStream ostr = new java.io.FileOutputStream(file); 
	    OutputStreamWriter out = new OutputStreamWriter(ostr, encoding);
	    String lineEnd =  System.getProperty("line.separator");
	    
	    try {
                for(int i=0; i<exportKeys.length; i++){
                    TranscriptionEintrag[]exportEintrags=source.lookup(exportKeys[i]);
                    for(int j=0; j<exportEintrags.length; j++) {
                        exportEintrag(out, exportEintrags[j], lineEnd, WORDSEPARATOR);
                    }
                    out.write(lineEnd);
                }
	    } catch (Exception ex){ 
                at.iem.tools.Logger.println("exportTXT:  "+ex);
            }
	    out.flush();
	} catch (Exception ex){
            at.iem.tools.Logger.println("exportTXT:: "+ex);
        }
        return true;
    }

    private void exportEintragIPA(OutputStreamWriter out, IPA transcription, char wordSeparator) throws java.io.IOException {
        out.write(wordSeparator);
        if(transcription==null)
            return;

        String transe=transcription.toString();
        if(transe==null)
            return;

        transe=transe.replace(wordSeparator, ' ').trim();
        if(!"".equals(transe)){
            out.write(transe);
        }
    }

    /**
     * write the data of this TranscriptionEintrag to an OutputStream in TXT-format
     * @param out the OutputStreamWriter to write to
     * @return <code>true</code> on success; <code>false</code> otherwise
     */
    public boolean exportEintrag(OutputStreamWriter out, TranscriptionEintrag eintrag, String lineEnd, char wordSeparator){
        if(null==out||null==eintrag) return false;

	try{
            at.iem.tools.Logger.println("file: "+eintrag.getPaths()+"/"+eintrag.getFilename()+"\t"+eintrag.getFullName());
            out.write(eintrag.getName());
            String ipa0=eintrag.getPhonetic();
            /*
            if(null==ipa0)
                exportEintragIPA(out, null, wordSeparator);
            else
                exportEintragIPA(out, new IPA(eintrag.getPhonetic()), wordSeparator);
            */
            exportEintragIPA(out, eintrag.getIPA(null), wordSeparator);

	    if(AdabaShared.region != null){
		for(int i=0; i<AdabaShared.region.length; i++){
                    exportEintragIPA(out, eintrag.getIPA(AdabaShared.region[i][0]), wordSeparator);
                }
	    }
	    out.flush();
	}catch (java.io.IOException e){
            at.iem.tools.Logger.println("exportTxt: "+e);
	    return false;
	}
	return true;
    }
}
