package at.iem.adaba.database;

import at.iem.adaba.AdabaResultInterface;
import java.util.ArrayList;


public class ThreadedSearch extends Thread implements Thread.UncaughtExceptionHandler {
    TranscriptionDataBase db;
    public ThreadedSearch(TranscriptionDataBase db) {
        this.db=db;
        done=false;
        setUncaughtExceptionHandler(this);
    }
    
    EintragPattern pattern;
    boolean done;
    String[]result;
    AdabaResultInterface display;
    boolean awt_lock;

    public void search(EintragPattern pattern) {
        this.pattern=pattern;
        this.display=display;
        awt_lock=true;
        at.iem.tools.Logger.println("ThreadedSearch for '"+pattern+"' started");
        start();
    }

    public void run() {
        result=null;
        done=false;
        wantResult=true;
        try {
            result=db.search(pattern);
        } catch (Exception ex) {
            result=null;
            at.iem.tools.Logger.println("ThreadedSearch::running into exception: "+ex);
            showError(ex);
            
        }
        if(result!=null) {
            at.iem.tools.Logger.println("ThreadedSearch found "+result.length+" results: "+result);
        }

        /* only show the result, if this is till wanted (e.g. nobody "stopped" the search) */

        if(wantResult)
            showResults(result);

        done=true;
    }

    volatile boolean wantResult;

    /** this tries to undo a started search; 
     * the implementation might stop the current search-thread (but need not do so)
     * if it doesn't, it must NOT notify the registered AdabaResultInterface's once there is a result
     */
    public void cancel() {
        if(db!=null)
            db.stopSearch();
        wantResult=false;
    }

    private class AWTSearchResult extends Thread {
        String[]result=null;
        AdabaResultInterface display=null;
        AWTSearchResult(AdabaResultInterface display, String[]result) {
            this.display=display;
            this.result =result;
        }

        public void run() {
            if(display!=null) {
                display.setResult(result);
            }
        }
    }


    private class AWTShowError extends Thread {
        Throwable ex=null;
        AdabaResultInterface display=null;
        AWTShowError(AdabaResultInterface display, Throwable ex) {
            this.display=display;
            this.ex =ex;
        }
        public void run() {
            if(display!=null) {
                display.searchErrorCallback(ex);
            }
        }
    }


    private ArrayList<AdabaResultInterface> m_listeners=null;
    private ArrayList<Boolean> m_awtlock=null;
    /**
     * Add listener to be notified.
     * @param ari the AdabaResultInterface to be notified        System.out.println("alive: "+isAlive());
     * @param awtlock set to <code>true</code> if the AdabaResultInterface should be notified in synch with the AWT-Event dispatching thread
     */
    public void addResultListener(AdabaResultInterface ari, boolean awtlock)
    {
        if(m_listeners==null) {
            m_listeners=new  ArrayList<AdabaResultInterface>();
            m_awtlock  =new  ArrayList<Boolean>();
        }
        m_listeners.add(ari);
        m_awtlock.add(awtlock);
    }
    protected void showResults(String[]results) {
        if(m_listeners==null)return;
        for(int i=0; i<m_listeners.size(); i++){
            AdabaResultInterface ari = (AdabaResultInterface) m_listeners.get(i);
            boolean awtlock=(boolean)m_awtlock.get(i);
	    if(null!=ari) {
		if(awtlock) {
		    AWTSearchResult proxy = new AWTSearchResult(ari, result);
		    try {
			javax.swing.SwingUtilities.invokeAndWait(proxy);
		    } catch (Exception ex) {
			System.out.println("ThreadedSearch::showResults: "+ex);
			ari.setResult(result);
		    }
		} else {
		    ari.setResult(results);
		}
	    }
	}
    }

    protected void showError(Throwable ex) {
        if(m_listeners==null)return;
        for(int i=0; i<m_listeners.size(); i++){
            AdabaResultInterface ari = (AdabaResultInterface) m_listeners.get(i);
            boolean awtlock=(boolean)m_awtlock.get(i);
	    if(null!=ari) {
		if(awtlock) {
		    AWTShowError proxy = new AWTShowError(ari, ex);
		    try {
			javax.swing.SwingUtilities.invokeAndWait(proxy);
		    } catch (Exception e) {
			System.out.println("ThreadedSearch::showResults: "+e);
			ari.searchErrorCallback(ex);
		    }
		} else {
		    ari.searchErrorCallback(ex);
		}
	    }
	}
    }
    


    /**
     * implements Thread.UncaughtExceptionHandler
     */
    public void uncaughtException(Thread t, Throwable e) {
        System.out.println("ThreadedSearch: caught exception in thread "+t+": "+e.getMessage());
        e.printStackTrace();
        System.out.println("ThreadedSearch: the cause is: "+e.getCause());

        cancel();
        showError(e);
    }




}
