// TranscriptionEintrag.java
// $Id: TranscriptionEintrag.java 1967 2008-06-09 13:59:28Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
//
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

// how to make this work ???
package at.iem.adaba.database;
import at.iem.adaba.*;

import at.iem.tools.linguistic.IPA;
import at.iem.tools.*;

import java.io.File;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.lang.Short;

/**
 * die Struktur, die alle Information eines Wrterbucheintrags beinhaltet
 */

public class TranscriptionEintrag {

    /**
     * make an empty TranscriptionEintrag
     */
    public TranscriptionEintrag(){
        name ="";
        key  ="";
        Kommentar="";
        ipa=new HashMap<String, IPA>(1+AdabaShared.region.length);
        ipa.put((String)null, (IPA)null); /* default transcription */

        int i=0;
        if(AdabaShared.region!=null)i=AdabaShared.region.length;
        while(i-->0){
            ipa.put(AdabaShared.region[i][0], null);
        }
        flags = new ArrayList<Short>();
    }

    /**
     * make a basic TranscriptionEintrag
     * the TranscriptionEintrag already has a name but not yet a transcription
     * @param eintrag the id of the TranscriptionEintrag
     */
    public TranscriptionEintrag(String  t_key) {
        this();
        setKey(t_key);
        timestamp=0;
    }

    /**
     * make a TranscriptionEintrag
     * the TranscriptionEintrag already has a name
     * the other fields are copied from a reference objects
     * @param eintrag the id of the TranscriptionEintrag
     * @param oldeintrag the reference TranscriptionEintrag
     */
    public TranscriptionEintrag(TranscriptionEintrag oldeintrag){
        this(oldeintrag.key, oldeintrag);
    }

    /**
     * make a TranscriptionEintrag
     * the TranscriptionEintrag already has a name
     * the other fields are copied from a reference objects
     * @param eintrag the id of the TranscriptionEintrag
     * @param oldeintrag the reference TranscriptionEintrag
     */
    public TranscriptionEintrag(String key, TranscriptionEintrag oldeintrag){
        this(key);
        Kommentar = oldeintrag.Kommentar;
        variant   = oldeintrag.variant;
        eTyp      = oldeintrag.eTyp;
        gTyp      = oldeintrag.gTyp;
        edited    = oldeintrag.edited;

        ipa=new HashMap<String, IPA>(oldeintrag.ipa);

        wortVariant = oldeintrag.wortVariant;
        name = new String(oldeintrag.name);

        if(wortVariant>0)
            setKey(name+" ["+wortVariant+"]");

        Kommentar = new String(oldeintrag.Kommentar);

        flags.addAll(oldeintrag.flags);
     
        // shouldn't we recalculate this? NO
        timestamp = oldeintrag.timestamp;

        // shouldn't we update this?
        have_files = oldeintrag.have_files;
    }

    /**
     * make a TranscriptionEintrag
     * the TranscriptionEintrag already has a name and a basic transcription (in SAMPA)
     * @param eintrag the id of the TranscriptionEintrag
     * @param sampa the SAMPA-encoded basic transcription
     */
    public TranscriptionEintrag(String  eintrag, String  sampa) {
        this(eintrag);
        IPA i=new IPA();
        i.setSAMPA(sampa);
        ipa.put((String)null, i); /* default transcription */
    }


    /**
     *
     */
    public boolean merge(TranscriptionEintrag mergentry) 
    {
        if(null==mergentry)
            return false;

        Kommentar=defaultMerge(Kommentar, mergentry.Kommentar);
        variant  =defaultMerge(variant, mergentry.variant, false);
        eTyp     =defaultMerge(eTyp, mergentry.eTyp, 0);
        gTyp     =defaultMerge(gTyp, mergentry.gTyp, 0);
        edited   =defaultMerge(edited, mergentry.edited, false);
        wortVariant =defaultMerge(wortVariant, mergentry.wortVariant, -1);
        name = defaultMerge(name, mergentry.name);
        have_files = defaultMerge(have_files, mergentry.have_files, -1);

        { /* IPA */
            Iterator keys=ipa.keySet().iterator();
            while(keys.hasNext()) {
                String s_key=(String)keys.next();
                setIPA(s_key, defaultMerge(ipa.get(s_key), mergentry.ipa.get(s_key)));
            }
        }

        { /* flags */
            for(int i=0; i<mergentry.flags.size(); i++){
                setFlag(mergentry.flags.get(i), true);
            }
        }

        touch();

        return true;
    }

    /**
     * der datenbank-key fr diesen eintrag (db-eindeutig!)
     */
    protected String  key;

    /**
     * Eintrag ohne Entitten-Variante
     */
    protected String name;

    /**
     * Entitten-Variante (z.B: Unterscheidung "sure" und "Sure")
     */
    protected byte wortVariant=-1;

    /**
     * allgemeiner Kommentar
     */
    protected String  Kommentar="";

    /**
     * gibt an, ob die regionalen Aussprachen signifikant unterschiedlich sind
     */
    protected boolean variant=false;

    /**
     * Wort-Typus (etymologisch, grammatisch)
     */
    protected byte    eTyp, gTyp;

    /**
     * Transkriptionen (mapping langID->transcription; (langID==null) is for "default" transription
     */
    protected HashMap<String, IPA> ipa;

    /**
     * editing done flag
     */
    protected boolean edited=false;

    /**
     * an array of various "special"-flags... (holds the INDEX of each set flag)
     */
    protected ArrayList<Short> flags;

    /**
     * time of last modification
     */
    protected long timestamp=0;

    /**
     * has at least one sound files
     * "-1" denotes, that we don't know...
     */
    protected short have_files=-1;

    /**
     * set the timestamp
     * @param ts the timestamp to set to
     */
    public void touch(long ts){
        timestamp=ts;
    }

    /**
     * set the timestamp to now
     */
    public void touch(){
        timestamp=System.currentTimeMillis();
    }

    /**
     * get the timestamp
     * @return the current timestamp
     */
    public long getTimestamp(){
        return timestamp;
    }

    /**
     * set the name of this TranscriptionEintrag to a new value
     * @param newname the new name
     */
    public void setName(String newname){
        name=newname;
    }
    /**
     * get the id of this TranscriptionEintrag; no wort-variant information is present!
     * @return the key of the TranscriptionEintrag
     */
    public String getName(){
        return name;
    }

    /**
     * get the fullname of this TranscriptionEintrag: "<name> [variant]", e.g. "Weg [1]"
     * @return the key of the TranscriptionEintrag
     */
    public String getFullName(){
        if(wortVariant>0)
            return new String(name+" ["+wortVariant+"]");
        else
            return name;
    }

    /**
     * @return the variants-index
     */
    public byte getVariant(){
        return wortVariant;
    }

    /**
     * set the variants-index (-1 is unknown)
     * @param variant
     */
    public void setVariant(byte variant){
        wortVariant=variant;
    }

    public void setVariantUnknown(){
        setVariant((byte)-1);
    }


    /**
     * get the transcription for a language region.
     * if there is no transcription for the specified langID, the standard-transcription is returned.
     * @param langID the region-ID of the requested transcription
     * @deprecated always return IPA instead of String
     * @return the transcription in unicode
     */
    @Deprecated
    protected String getIpa(String langID){
        boolean fixMS=AdabaShared.fixMSFonts;
        //at.iem.tools.Logger.println("fixing MS: "+fixMS);
        IPA i=getIPA(langID);
        if(null==i)
            return null;

        return i.toStringMS(fixMS);
    }

    /**
     * get the transcription for a language region.
     * if there is no transcription for the specified langID, the standard-transcription is returned.
     * @param langID the region-ID of the requested transcription
     * @return the transcription in unicode
     */
    public IPA getIPA(String langID){
        if(!ipa.containsKey(langID)) langID=null; // default transcription

        return ipa.get(langID);
    }

    /**
     * get all transcriptions of this Eintrag
     * @return an array of transcriptions
     */
    public IPA[] getIPAs() {
        IPA[] iparray = new IPA[ipa.size()];

        int i=0;
        Iterator keys=ipa.keySet().iterator();
        while(keys.hasNext()) {
            String s_key=(String)keys.next();
            iparray[i]=ipa.get(s_key);
            i++;
        }
        return iparray;
    }

    /**
     * get all region-ID's for which a transcription is defined
     * @return an array of region-IDs
     */
    public String[] getRegions() {
        ArrayList<String> regarray = new ArrayList<String>();
        Iterator keys=ipa.keySet().iterator();
        while(keys.hasNext()) {
            String s_key=(String)keys.next();
            if((s_key!=null)&&(!"".equals(s_key))) {
                regarray.add(s_key);
            }
        }

        if(regarray==null || regarray.size()<1)
            return null;

        String[]result=new String[regarray.size()];
        for(int i=0; i<result.length; i++) {
            result[i]=((String)regarray.get(i));
        }

        if(result!=null && result.length>0)
            java.util.Arrays.sort(result);
        return result;
    }

    /**
     * get the primary phonetic information of a transcription for a language-region.
     * if there is no transcription for the specified langID, the standard-transcription is used.
     * the "primary phonetic" is the first substring of the transcription enclosed within "[[" and "]]"
     * @param langID the region-ID of the requested transcription
     * @return the primary phonetic in unicode
     */
    public String getPhonetic(String langID){
        String phon = getIpa(langID);
        if(phon==null)return null;
        int start=phon.indexOf("[[");
        if(start<0)start=0;
        else start+=2; // skip the 2 characters "[["
        int stop =phon.indexOf("]]", start);
        if(stop<1)stop=phon.length();
        return phon.substring(start, stop);
    }

    /**
     * get the primary phonetic information
     * if the main-transcription contains no information, the other transcriptions are searched
     * @return the primary phonetic in unicode
     * @see #getPhonetic(String)
     */
    public String getPhonetic(){
        int len=0;
        String s_key=null;
        String phon=getPhonetic(s_key);
        if (phon!=null && phon.length()!=0)return phon;

        for(int i=0; i<AdabaShared.region.length; i++) {
            phon=getPhonetic(AdabaShared.region[i][0]);
            if (phon!=null && phon.length()!=0)return phon;
        }

        Iterator keys=ipa.keySet().iterator();
        while(keys.hasNext()) {
            s_key=(String)keys.next();
            if(null!=s_key) { // s_key==null is handled above!
                phon=getPhonetic(s_key);
                if (phon!=null && phon.length()!=0)return phon;
            }
        }
        return null;
    }

    /**
     * get the transcription (in SAMPA) for a specified language-region
     * @param langID the language-region ID;
     * if the ID does not point to a valid transcription, the default transcription is used
     * @return the transcription in SAMPA
     */
    public String getSampa(String langID){
        if(!ipa.containsKey(langID)) langID=null;
        IPA i=ipa.get(langID);
        if(null==i)return null;
        return i.getSAMPA();
    }

    /**
     * get the transcription (in SAMPA) for a specified language-region
     * only parts of the transcription are decoded to SAMPA
     * @param langID the language-region ID;
     * if the ID does not point to a valid transcription, the default transcription is used
     * @param start the starting-delimiter for decoding IPA to SAMPA
     * @param stop the ending-delimiter for decoding IPA to SAMPA
     * @return the transcription in SAMPA (parts)
     */
    public String getSampa(String langID, String start, String stop){
        if(!ipa.containsKey(langID)) langID=null;
        IPA i=ipa.get(langID);
        if(null==i)return null;
        return i.getSAMPA(start, stop);
    }

    /**
     * get, whether editing of this TranscriptionEintrag is considered finished
     * @return <code>true</code> if TranscriptionEintrag is considered finished; else <code>false</code>
     */
    public boolean isEdited(){
        return edited;
    }

    /**
     * get, whether pronunciation-variants are significantly different
     * @return <code>true</code> if variants are different; else <code>false</code>
     */
    public boolean isVariant(){
        return variant;
    }

    /**
     * get, whether one of the special flags is set;
     * the flag is specified by its (short) index
     * @param index the index of the special flag
     * @return <code>true</code> if the specified flag is set, <code>false</code> otherwise
     */
    public boolean getFlag(int index){
        if(flags==null)return false;
        return flags.contains(new Short((short)index));
    }

    /**
     * get all set flags
     * @return an array of the indices of all flags that are set
     */
    public short[] getFlag(){
        short[]buffer=new short[0];
        if(flags!=null){
            buffer = new short[flags.size()];
            for(int i=0; i<buffer.length; i++)buffer[i]=((Short)flags.get(i)).shortValue();
        }
        return buffer;
    }

    /**
     * set a specified flag to a certain state;
     * the flag is specified by its (integer) index
     * @param index the index of the special flag
     * @param state the new boolean value of the flag
     */
    public void setFlag(int index, boolean state){
        if (flags==null)flags=new ArrayList<Short>();
        Short i = new Short((short)index);
        if(state){
            // add flag
            if(!flags.contains(i))flags.add(i);
        } else {
            // remove this flag
            int idx=-1;
            while((idx=flags.indexOf(i))>-1)flags.remove(idx);
        }
    }

    /**
     * get the etymologic type of the TranscriptionEintrag (e.g: Erbwort)
     * @return ID of the etymologic type
     */
    public int getEtyp(){
        return eTyp;
    }

    /**
     * test whether this Eintrag is of the given eType;
     * if the given eType is 0 ("unknown") this returns true too
     * @param e the eType to test against
     * @return <code>true</code> if the given eType matches ours
     */
    public boolean isEtyp(int e){
        return(e==0 || e==eTyp);
    }

    /**
     * test whether this Eintrag is of one of the given eTypes
     * @param e an array of eTypes to be tested against
     * @return <code>true</code> if one of the given eTypes matches ours
     */
    public boolean isEtyp(int[]e){
        for(int i=0; i<e.length; i++)
            if(e[i]==eTyp)return true;
        return false;
    }

    /**
     * get the grammatic type of the TranscriptionEintrag (e.g: Verb)
     * @return ID of the grammatic type
     */
    public int getGtyp(){
        return gTyp;
    }

    /**
     * test whether this Eintrag is of the given gType;
     * if the given gType is 0 ("unknown") this returns true too
     * @param g the gType to test against
     * @return <code>true</code> if the given gType matches ours
     */
    public boolean isGtyp(int g){
        return(g==0 || g==gTyp);
    }

    /**
     * test whether this Eintrag is of one of the given gTypes
     * @param g an array of gTypes to be tested against
     * @return <code>true</code> if one of the given gTypes matches ours
     */
    public boolean isGtyp(int[]g){
        for(int i=0; i<g.length; i++)
            if(g[i]==gTyp)return true;
        return false;
    }

    /**
     * get the comment on this TranscriptionEintrag
     * @return the comment as a String
     */
    public String getKommentar(){
        return Kommentar;
    }

    /**
     * get the estimated soundfile-base for this TranscriptionEintrag
     * <b>without</b> speaker/region-specific parts and extensions<br>
     * (e.g: "test"->"(...)/T/TE/(...)")
     * @return the soundfilebase for this TranscriptionEintrag
     */
    public String[] getPaths(){
        return getPaths(getFilename());
        /*
        String Wort = StringX.replace(key,".", "").trim();
        if (Wort.length()==1)Wort=new String(Wort+' ');
        return at.iem.tools.FileX.makePosixName(Wort.charAt(0)+
                                                File.separator+
                                                Wort.charAt(0)+
                                                Wort.charAt(1));
        */
    }

    protected static String[] getPaths(String key) {
        String Wort = StringX.replace(key,".", "").trim();
        if (Wort.length()==1)Wort=new String(Wort+' ');
        String part1=new String(""+Wort.charAt(0));
        String part2=new String(""+Wort.charAt(0)+Wort.charAt(1));
        String part3=Wort;

        String result[] = new String[8];
        result[0]=at.iem.tools.FileX.makePosixName(part1+File.separator+part2+File.separator+part3); /* T/TE/TEST */
        result[1]=at.iem.tools.FileX.makePosixName(part1+File.separator+part2);  /* T/TE */

        result[2]=at.iem.tools.FileX.makePosixName(part2);  /* TE */
        result[3]=at.iem.tools.FileX.makePosixName(part1);  /* T */
        result[4]=at.iem.tools.FileX.makePosixName(part1+File.separator+part3); /* T/TEST */
        result[5]=at.iem.tools.FileX.makePosixName(part2+File.separator+part3); /* TE/TEST */
        result[6]=at.iem.tools.FileX.makePosixName(part3); /* TEST */
        result[7]="";

        return result;
    }

    /**
     * strip unwanted (e.g: formatting) characters from a String
     * @param wort the String to be stripped
     * @return the stripped String
     */
    protected static String strip_wort(String wort){
        if(wort==null)return null;
        String Wort;
        final int len=wort.length();
        if (len > 2 && wort.charAt(len-2)==' '){
            //char[] CharArray = wort.toCharArray();
            //new String(CharArray, 0, len-2);
            Wort = new String(wort.substring(0, len-1));
        } else Wort = new String(wort);
        Wort=at.iem.tools.FileX.makePosixName(Wort);
        return StringX.replace(Wort, ".", "");
    }

    /**
     * sort an array of filenames.
     * if the file <code>preference</code> exists, it is listed as first item
     * @param orgfiles an array of files to be sorted
     * @param preference the wannabee-number-one
     * @return the sorted array of files
     */
    protected static File[] sortFiles(File[] orgfiles, File preference){
        if (preference==null || !preference.exists())return orgfiles;
        if (orgfiles==null || orgfiles.length==0){
            File[] newfiles = new File[1];
            newfiles[0]=preference;
            return newfiles;
        }
        boolean have = false;
        int i = 0;
        while (i++<orgfiles.length && !have)
            if (preference.compareTo(orgfiles[i])==0)have=true;
        File[] newfiles = null;
        if (!have)newfiles=new File[orgfiles.length+1];
        else newfiles=new File[orgfiles.length];
        newfiles[0]=preference;
        i=0;
        int j=1;
        while (i++<orgfiles.length)
            if(preference.compareTo(orgfiles[i])!=0)newfiles[j++]=orgfiles[i];
        return newfiles;
    }

    /**
     * constructs the "Wort-Entitt" part of the filename;
     * {filename} = {{wortentitt}[_##]}__{regionID}_{{{sex}#}_{x}}.{extension}<br>
     * this function will return {{wortentitt}[_##]}
     * this is everything before the "__" and consists of the (posix-conformant) Eintrag
     * and an optional suffix to separate variants (e.g: "Sure" and "sure")
     * @return first path of the soundfilename for this entry
     */
    public String getFilename(){
        String l_filename = strip_wort(name);
        //at.iem.tools.Logger.println("getting filename for "+name+"..."+wortVariant);
        /*
          if(wortVariant>100)l_filename=l_filename+"_"+wortVariant;
          else if(wortVariant>10)l_filename=l_filename+"_0"+wortVariant;
          else if(wortVariant>0)l_filename=l_filename+"_00"+wortVariant;
        */
        return l_filename;
    }

    /**
     * constructs the constant part of the filename;
     * {filename} = {{wortentitt}[_##]}__{regionID}_{{{sex}#}_{x}}.{extension}<br>
     * this function will return {{wortentitt}[_##]}__{regionID}_{sex}<br>
     * "wortentitt" is the Eintrag of this Object made posix-filename-conformant;
     * "regionID" should be 2 (for nations) or 4 (for regions within nations) characters
     * "sex" is either "M" (male) or "W" (female)
     * @param langID the region ID to construct the filename for
     * @param female <code>true</code> if the speaker is female, <code>false</code> if he is male
     * @return first path of the soundfilename for this entry
     */
    public String getFilename(int langID, boolean female){
        String varstring="";
        if(wortVariant>100)
            varstring="_"+wortVariant;
        else if(wortVariant>10)
            varstring="_0"+wortVariant;
        else if(wortVariant>0)
            varstring="_00"+wortVariant;
        return getFilename()+varstring+"__"+AdabaShared.region[langID][0]+"_"+(female?"W":"M");
    }

    protected File[]listFiles(File path, String filename, String extension){
        at.iem.tools.Logger.println("listFiles: "+path+" ... "+filename+" ... "+extension);
 
        if(path==null){
            return null;
        }
        File[]result=path.listFiles(new at.iem.tools.gui.CustomFileFilter("", filename, extension));
        if(result!=null && result.length>0) {
            java.util.Arrays.sort(result);
            at.iem.tools.Logger.println("listFiles found "+result.length+" files");
        }
        return result;
    }

    private File[] getFile(File path0, String[]paths, String filename, String extension)
    {
        File[]files=null;
        if(null==paths){
            paths=new String[1];
            paths[0]="";
        }
        for(int i=0; i<paths.length; i++) {
            String filenamepath=new String(paths[i]+File.separator+getFilename());
            files=listFiles(new File(path0, filenamepath), filename, extension);
            if(files!=null && files.length>0)return files;
        }
        return null;
    }

    /**
     * constructs a filename for a given language/region and sex for this TranscriptionEintrag.<br>
     * the filename-extension must be specified.
     * @param langID  the region-language id for the constructed filename
     * @param female  <code>true</code> if the file is for the female speaker, <code>false</code> for males
     * @param extension filename-extension
     * @return an array of existing soundfiles conforming to the construcion
     */
    public File[] getFile(int langID, boolean female, String extension){
        // test whether we have stored something already !!!
        String filename=getFilename(langID, female);
        String[]paths=getPaths(getFilename());

        if(extension!=null)extension="."+extension;

        File[]files=null;

        files=getFile(AdabaShared.getSoundsPath(), paths, filename, extension);   if(files!=null && files.length>0)return files;
        files=getFile(AdabaShared.getDBpath(),     paths, filename, extension);   if(files!=null && files.length>0)return files;
        files=getFile(new File("."),               paths, filename, extension);   if(files!=null && files.length>0)return files;

        return null;
    }

    /**
     * check whether we have soundfiles.
     * this is usually stored in the database; if not, we look at the filesystem...
     * @return whether we have files or not
     */
    public boolean hasMedia() {
        at.iem.tools.Logger.println("checking hasMedia()");
        if(have_files<0) {
            have_files=0;
            for(int i=0; i<AdabaShared.region.length; i++) {
                for(int j=0; j<2; j++) {
                    if(null!=getFile(i, (j!=0), null)) {
                        have_files=1;
                        return true;
                    }
                }
            }
        }
        return (have_files>0);
    }

    /**
     * check whether we have soundfiles.
     * this is usually stored in the database; if not, we look at the filesystem...
     * @return whether we have files or not
     */
    public void haveMedia(boolean state) {
        have_files=(byte)((state)?1:0);
    }

    /**
     * check whether we have soundfiles on the filesystem
     * @return whether we have files or not
     */
    public boolean checkSoundFiles() {
        have_files=-1;
        return hasMedia();
    }


    /**
     * get the id of this TranscriptionEintrag
     * @return the key of the TranscriptionEintrag
     * @deprecated keep track of the key yourself!!!
     */
    @Deprecated
    public String getKey(){
        return key;
    }

    /**
     * set the id of this TranscriptionEintrag to a new value (e.g: rename);
     * the new name is automatically parsed to detect variants
     * @param newentry the new id
     */
    public void setKey(String newkey){
        //at.iem.tools.Logger.println("setKey: "+newkey);
        key=newkey.trim();

        int index=key.lastIndexOf(" [");
        int stop =key.indexOf(']', index);

        setVariantUnknown();
        setName(key);
        if(index>0 && stop>index){
            /* might be an alternative */
            try {
                setVariant(java.lang.Byte.parseByte(key.substring(index+2, stop).trim()));
                setName(key.substring(0, index).trim());
                //at.iem.tools.Logger.println("setkey: "+name+" .. "+wortVariant);
            } catch (Exception ex) { }
        }
	
        timestamp=System.currentTimeMillis();
    }

    /**
     * set the transcription (in SAMPA) for a specified language/region.
     * if the region is "-1" the default-transcription is set.
     * @param langID the language-ID for a specific region (or "-1" for standard)
     * @param sampa the SAMPA-encoded transcription
     */
    public void setSampa(String langID, String sampa){
        IPA i=new IPA();
        i.setSAMPA(sampa);
        ipa.put(langID, i);

        timestamp=System.currentTimeMillis();
    }

    /**
     * set the transcription (in SAMPA) for a specified language/region.
     * if the region is "-1" the default-transcription is set.
     * it is assumed, that only parts (within the delimiters)
     * of the given transcription have phonetic information
     * @param langID the language-ID for a specific region (or "-1" for standard)
     * @param sampa the SAMPA-encoded transcription
     * @param start the starting delimiter
     * @param stop the ending delimiter
     */
    public void setSampa(String langID, String sampa, String start, String stop){
        IPA i=new IPA();
        i.setSAMPA(sampa, start, stop);
        ipa.put(langID, i);

        timestamp=System.currentTimeMillis();
    }

    /**
     * set the transcription (in SAMPA) for the main transcription
     * @param sampa the SAMPA-encoded transcription
     */
    public void setSampa(String sampa){
        setSampa((String)null, sampa);
    }

    /**
     * set the transcription (in SAMPA) for the main transcription.
     * it is assumed, that only parts (within the delimiters)
     * of the given transcription have phonetic information
     * @param sampa the SAMPA-encoded transcription
     * @param start the starting delimiter
     * @param stop the ending delimiter
     */
    public void setSampa(String sampa, String start, String stop){
        setSampa((String)null, sampa, start, stop);
    }

    /**
     * set the transcription (in IPA) for a specified language/region.
     * if the region is "null" the default-transcription is set.
     * @param langID the language-ID for a specific region (or "-1" for standard)
     * @param ipastring the IPA-encoded transcription
     * @deprecated use setIPA(String, IPA) instead
     */
    @Deprecated
    public void setIPA(String langID, String ipastring){
        ipa.put(langID, new IPA(ipastring));

        timestamp=System.currentTimeMillis();

    }

    /**
     * set the transcription (in IPA) for the main transcription
     * @param ipastring the IPA-encoded transcription
     * @deprecated use setIPA(IPA) instead
     */
    @Deprecated
    public void setIPA(String ipastring){
        setIPA((String)null, ipastring);
    }

    /**
     * set the transcription (in IPA) for a specified language/region.
     * if the region is "null" the default-transcription is set.
     * @param langID the language-ID for a specific region (or "-1" for standard)
     * @param ipastring the IPA-encoded transcription
     */
    public void setIPA(String langID, IPA ipastring){
        ipa.put(langID, ipastring);

        timestamp=System.currentTimeMillis();
    }

    /**
     * set the transcription (in IPA) for the main transcription
     * @param ipastring the IPA-encoded transcription
     */
    public void setIPA(IPA ipastring){
        setIPA((String)null, ipastring);
    }

    /**
     * copy a specified transcription to all other region-transcriptions
     * if the specified region is invalid, the default transcription is duplicated
     * @param langID the id of the region to copy from
     */
    public void setIPAfrom(String langID){
        IPA i;
        if(ipa.containsKey(langID))
            i=ipa.get(langID);
        else
            i=ipa.get((String)null);


        Iterator keys=ipa.keySet().iterator();
        while(keys.hasNext()) {
            ipa.put((String)keys.next(), i);
        }

        timestamp=System.currentTimeMillis();
    }

    /**
     * set the verbose comment
     * @param comment some exmplanation on this TranscriptionEintrag
     */
    public void setKommentar(String comment){
        Kommentar=comment;
        timestamp=System.currentTimeMillis();
    }

    /**
     * set the "pronuncations are significantly different"-flag
     * @param v the "pronuncations are significantly different"-flag
     */
    public void setVarianz(boolean v){
        variant = v;
        timestamp=System.currentTimeMillis();
    }

    /**
     * set the etymology-type<br>
     * invalid types are set to "0", which means "unkown"
     * @param E the etymology index
     */
    public void setEtyp(int E){
        byte e=(byte)E;
        if (e<0 || e>=AdabaShared.E_TYPEN.length)e=0;
        eTyp=e;
        timestamp=System.currentTimeMillis();
    }

    /**
     * set the grammatic type<br>
     * invalid types are set to "0", which means "unkown"
     * @param G the grammatic index
     */
    public void setGtyp(int G){
        byte g=(byte)G;
        if (g<0 || g>=AdabaShared.G_TYPEN.length)g=0;
        gTyp=g;
        timestamp=System.currentTimeMillis();
    }

    /**
     * set the "editing done"-flag
     * @param ed the "editing done"-flag
     */
    public void setFertig(boolean ed){
        edited=ed;
        timestamp=System.currentTimeMillis();
    }

    /**
     * assume that all IPA-transcriptions are broken, because we used Microsoft fonts for transcription;
     * this function will "fix" these wrong unicode strings.
     * you should execute this function ONCE AND ONLY ONCE!
     */
    protected void fixMS2Unicode() {
        Iterator keys=ipa.keySet().iterator();
        while(keys.hasNext()) {
            String s_key=(String)keys.next();
            IPA i=ipa.get(s_key);
            if(i!=null) {
                i.fixMS2Unicode();
                ipa.put(s_key, i);
            }
        }
    }

    /**
     * split a transcription into several Strings containing <em>only</em> the phonetic information.
     * it is assumed the the phonetic information is enclosed withinin [[ and ]]
     * @param ipa the transcription to split
     * @return an array of pronunciations
     */
    protected static String[] getIPAvariants(IPA ipa){
        if (ipa==null)return null;
        String s_ipa = ipa.toString();
        if (s_ipa==null || s_ipa.length()==0)return null;

        int i=0;
        int dummy=0;
        while((dummy=s_ipa.indexOf("[[", dummy))>=0){
            i++;
            dummy=s_ipa.indexOf("]]", dummy);
            if (dummy==-1)dummy=s_ipa.length();
        }
        dummy=i;
        String iparray[] = new String[i];
        i=-1;
        int start=0, stop=0;
        while(++i<iparray.length){
            start = s_ipa.indexOf("[[", stop);
            stop  = s_ipa.indexOf("]]", start+2);
            if (start<0)break;
            if (stop <0)stop=s_ipa.length();
            iparray[i]=s_ipa.substring(start+2, stop);
        }
	
        return iparray;
    }

    /**
     * convert the data of this TranscriptionEintrag into a String
     * there is no guarantee, that all data is covered;
     * so don't use it for exporting
     * @return a String containing the data
     */
    public String toString(){
        String str = new String();
        str+=key+"\t";

        Iterator keys=ipa.keySet().iterator();
        while(keys.hasNext()) {
            String s_key=(String)keys.next();
            if(null!=s_key) {
                String[] iparray=getIPAvariants(ipa.get(s_key));
                if(iparray!=null){
                    for(int j=0; j<iparray.length; j++){
                        str+=iparray[j];
                        if(j+1!=iparray.length) {
                            str+="/";
                        }
                    }
                    str+="\t";
                }
            }
        }
        str+="\n";
        return str;
    }

    /**
     * print some interesting things about this TranscriptionEintrag
     */
    public void print(){
        String s_key=null;
        at.iem.tools.Logger.println("key: "+key);
        at.iem.tools.Logger.println("laut: "+ipa.get(s_key)+" -- "+ipa.get(s_key).getSAMPA());


        Iterator keys=ipa.keySet().iterator();
        while(keys.hasNext()) {
            s_key=(String)keys.next();
            if(null!=s_key) {
                at.iem.tools.Logger.println("laut("+s_key+"): "+ipa.get(s_key)+" -- "+ipa.get(s_key).getSAMPA());
            }
        }
        at.iem.tools.Logger.println("variant: "+variant);
        at.iem.tools.Logger.println("Kommentar: "+Kommentar);
        at.iem.tools.Logger.println("edited: "+edited);
    }


    /* ******************* importing/exporting ************************ */
    /* LATER move this into subclasses */

    /**
     * write the data of this TranscriptionEintrag to an OutputStream in TXT-format
     * @param out the OutputStreamWriter to write to
     * @return <code>true</code> on success; <code>false</code> otherwise
     */
    public boolean exportTXT(OutputStreamWriter out){
        try{
            char wordSeparator =  '\t';
            at.iem.tools.Logger.println("file: "+getPaths()+"/"+getFilename());
            out.write(key+wordSeparator+getIpa((String)null).replace(wordSeparator, ' ').trim());
            if(ipa!=null && AdabaShared.region != null){
                for(int i=0; i<AdabaShared.region.length; i++){
                    String ipas=getIpa(AdabaShared.region[i][0]);
                    if(null!=ipas) {
                        String transe=ipas.replace(wordSeparator, ' ').trim();
                        if(!"".equals(transe)){
                            out.write(wordSeparator+transe);
                        }
                    }
                }
            }
            out.flush();
        }catch (java.io.IOException e){
            at.iem.tools.Logger.println("exportTxt: "+e);
            return false;
        }
        return true;
    }


    private static String defaultMerge(String older, String newer) 
    {
        if(null==newer)
            return older;
        if(null==older)
            return newer;

        if(!"".equals(newer) && "".equals(older))
            return newer;

        return older;
    }
    private static IPA defaultMerge(IPA older, IPA newer) 
    {
        if(null==newer)
            return older;
        if(null==older)
            return newer;

        if(!newer.equals("") && older.equals(""))
            return newer;

        return older;
    }
    private static boolean defaultMerge(boolean older, boolean newer, boolean defaulter) 
    {
        if(older==defaulter)
            return newer;
        return older;
    }  
    private static byte defaultMerge(byte older, byte newer, int defaulti) 
    {
        byte defaulter=(byte)defaulti;
        if(older==defaulter)
            return newer;
        return older;
    }  
    private static short defaultMerge(short older, short newer, int defaulti) 
    {
        short defaulter=(short)defaulti;
        if(older==defaulter)
            return newer;
        return older;
    }
}
