// XMLfile.java
// $Id: XMLfile.java 1161 2007-01-02 15:50:12Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
//
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

package at.iem.adaba.database.XML;

import at.iem.adaba.AdabaShared;
import at.iem.adaba.database.*;
import at.iem.tools.StringX;
import at.iem.tools.linguistic.IPA;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;

import java.util.ArrayList;

/* i wonder whether it is a good idea to do it in here: */
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.*;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;



public class XMLfile  extends DefaultHandler implements Exporter, Importer {

    private File file=null;

    public XMLfile(File f) 
    {
	file=f;
    }

    
    /** export a database to a XML-file;
     * no assumption is made on the order of the exported database entries
     * @param source the database to be exported
     * @return <code>true</code> on success, <code>false</code> otherwise
     */
    public boolean exportFrom(TranscriptionDataBase source) throws Exception 
    {
	return exportFrom(source, source.getAllEntries());
    }


    /** export a selection of entries to a XML-file;
     * the order of the exported entries is the same as the order of the exportKeys
     * @param source the database to be exported
     * @param exportKeys the keys to be exported
     * @return <code>true</code> on success, <code>false</code> otherwise
     */
    public boolean exportFrom(TranscriptionDataBase source, java.lang.String[] exportKeys) throws Exception
    {
        if(exportKeys==null||source==null)
            return false;

	at.iem.tools.Logger.println("exporting database to XML: "+exportKeys.length);
	try {
	    String encoding="UTF-8";
	    FileOutputStream ostr = new FileOutputStream(file); 
	    OutputStreamWriter out = new OutputStreamWriter(ostr, encoding);
	    String lineEnd =  System.getProperty("line.separator");

	    exportHeader(out, encoding, lineEnd);

	    out.write("<adaba>"+lineEnd+lineEnd);

	    try {
		for(int i=0; i<exportKeys.length; i++){
		    TranscriptionEintrag[]exportEintrags=source.lookup(exportKeys[i]);
		    if(null!=exportEintrags) {
			for(int j=0; j<exportEintrags.length; j++) {
			    exportEintrag(out, exportEintrags[j], lineEnd);
			    
			}
			out.write(lineEnd+lineEnd);
		    } else {
			at.iem.tools.Logger.println("skipping empty entry for '"+exportKeys[i]+"'");
		    }
		}
	    } catch (Exception ex){
		at.iem.tools.Logger.println("xml-exportexception: '"+ex);
	    }
	    out.write("</adaba>"+lineEnd); 
	    out.flush();
	} catch  (Exception ex) {
	    at.iem.tools.Logger.println("couldn't save database as XML...\n--> "+ex);
	    //AdabaMessages.showMessage(StringX.replace(AdabaShared.getLabel("ErrorSaveAs"), "%s", file.getName()));
            return false;
	}
	return true;
    }


    private void exportHeader(OutputStreamWriter out, String encoding, String lineEnd) throws java.io.IOException {
	out.write("<?xml version='1.0' encoding='"+encoding+"' ?>"+lineEnd+
		  "<!DOCTYPE adaba ["+lineEnd+
		  "<!ELEMENT adaba (SoundfileRoot?, entry+)>"+lineEnd+
		  "<!ATTLIST adaba"+lineEnd+
		  "            xml:lang   CDATA    'de'"+lineEnd+
		  ">"+lineEnd+
		  "<!ELEMENT SoundfileRoot (#PCDATA)>"+lineEnd+
		  "<!ELEMENT entry (id, variant?, grammarType, etymologyType, Flag*, pronunciation, phoneticVariant?, done?, media?, timestamp?, comment?, transcription*) >"+lineEnd+
		  "<!ATTLIST entry"+lineEnd+
		  "            type      CDATA    'word'"+lineEnd+
		  "            variant   CDATA    #IMPLIED"+lineEnd+
		  "            xml:lang  CDATA    'de'"+lineEnd+
		  "            version   CDATA    #REQUIRED"+lineEnd+
		  ">"+lineEnd+
		  "<!ELEMENT id (#PCDATA)>"+lineEnd+
		  "<!ELEMENT variant (#PCDATA)>"+lineEnd+
		  "<!ELEMENT grammarType (#PCDATA)>"+lineEnd+
		  "<!ELEMENT etymologyType (#PCDATA)>"+lineEnd+
		  "<!ELEMENT Flag (#PCDATA)>"+lineEnd+
		  "<!ELEMENT pronunciation (#PCDATA)>"+lineEnd+
		  "<!ELEMENT comment (#PCDATA)>"+lineEnd+
		  "<!ELEMENT timestamp (#PCDATA)>"+lineEnd+
		  "<!ELEMENT phoneticVariant EMPTY>"+lineEnd+
		  "<!ELEMENT done EMPTY>"+lineEnd+
		  "<!ELEMENT media (#PCDATA)>"+lineEnd+
		  "<!ELEMENT transcription (pronunciation+, comment?)>"+lineEnd+
		  "<!ATTLIST transcription"+lineEnd+
		  "            region        CDATA    #REQUIRED"+lineEnd+
		  ">"+lineEnd+
		  "]>"+lineEnd+lineEnd);
    }

    /**
     * write the a transcription to an XML-OutputStreamWriter
     * @param out the OutputStreamWriter to write to
     * @param ipa the transcription
     * @region the region-code of the transcription
     * @return <code>true</code> on success; <code>false</code> otherwise
     */
    protected void export_transXML(OutputStreamWriter out, IPA ipa, String region, String lineEnd)
        throws java.io.IOException
    {
	if(region==null) return;
        if(null==ipa) return;
        String ipas=ipa.toString();
        if(null==ipas || "".equals(ipas)) return;
	
	out.write("  <transcription region='"+StringX.toXML(region)+"'>"+lineEnd);
	out.write("    <pronunciation>"+StringX.toXML(ipas)+"</pronunciation>"+lineEnd);
	out.write("  </transcription>"+lineEnd);
    }
    /*
    private void exportEintrag(OutputStreamWriter out, TranscriptionEintrag eintrag) {
	eintrag.exportXML(out);
    }
    */

    /**
     * write the data of this TranscriptionEintrag to an OutputStream in XML-format
     * @param out the OutputStreamWriter to write to
     * @return <code>true</code> on success; <code>false</code> otherwise
     */
    public boolean exportEintrag(OutputStreamWriter out, TranscriptionEintrag eintrag, String lineEnd){
	try{
	    //at.iem.tools.Logger.println("XMLexporting '"+Eintrag+"'");
	    out.write(" <entry version='0.1'>"+lineEnd);
	    out.write("  <id>"+StringX.toXML(eintrag.getName())+"</id>"+lineEnd);

            int wortvariant=eintrag.getVariant();
	    if(wortvariant>0)out.write("  <variant>"+wortvariant+"</variant>"+lineEnd);

	    int gTyp=eintrag.getGtyp();
	    String gtype=new String(""+gTyp);
	    if(gTyp>=0 && gTyp<AdabaShared.G_TYPEN.length)gtype=AdabaShared.G_TYPEN[gTyp];
	    out.write("  <grammarType>"+StringX.toXML(gtype)+"</grammarType>"+lineEnd);

	    int eTyp=eintrag.getEtyp();
	    String etype=new String(""+eTyp);
	    if(eTyp>=0 && eTyp<AdabaShared.E_TYPEN.length)etype=AdabaShared.E_TYPEN[eTyp];
	    out.write("  <etymologyType>"+StringX.toXML(etype)+"</etymologyType>"+lineEnd);


	    short[]flags=eintrag.getFlag();
	    if(flags!=null)for(int i=0; i<flags.length; i++){
		short j = flags[i];
		if(j<AdabaShared.EntryFlag.length&&j>=0)
		    out.write("  <Flag>"+StringX.toXML(AdabaShared.EntryFlag[j])+"</Flag>"+lineEnd);
	    }

            String ipas=eintrag.getIPA((String)null).toString();
            if(null==ipas)ipas=new String("");
	    out.write("  <pronunciation>"+StringX.toXML(ipas)+"</pronunciation>"+lineEnd);

	    if(eintrag.isVariant())out.write("  <phoneticVariant/>"+lineEnd);

	    if(eintrag.isEdited ())out.write("  <done/>"+lineEnd);

	    if(eintrag.hasMedia ())
                out.write("  <media>true</media>"+lineEnd); // we do have media
            else
                out.write("  <media/>"+lineEnd); // we don't have media

	    if(true)   out.write("  <timestamp>"+eintrag.getTimestamp()+"</timestamp>"+lineEnd);

	    String Kommentar=eintrag.getKommentar();
	    if(null!=Kommentar&&!("".equals(Kommentar)))
		out.write("  <comment>"+StringX.toXML(Kommentar)+"</comment>"+lineEnd);

	    String[]regions=eintrag.getRegions();

	    if(regions!=null)for(int i=0; i<regions.length; i++) {
		export_transXML(out, eintrag.getIPA(regions[i]), regions[i], lineEnd);
	    }
            /*
            if(eintrag.checkSoundFiles()) {
                out.write("  <files>true</files>"+lineEnd);
            } else {
                out.write("  <files/>"+lineEnd);
            }
            */
	    out.write(" </entry>"+lineEnd+lineEnd);
	
	    out.flush();
	}catch (Exception e){
            String key=eintrag.getFullName();
	    if(null==key)key=new String("<null>");
            at.iem.tools.Logger.println("error while xml-exporting '"+key+"': '"+e);
	    return false;
	}
	return true;
    }

    /**
     * import the data from an XML file
     * TODO: merging with the current database...
     * @param destination the destination database where we write into
     * @param overwrite overwrite existing entries?
     * @return an array of clashed keys
     */
    public String[]  importTo(TranscriptionDataBase destination, boolean overwrite){
        xml_destination=destination;
        xml_destination.startAsync();
	try {
	    SAXParserFactory factory = SAXParserFactory.newInstance();
	    factory.setValidating(true);	
	    SAXParser saxParser = factory.newSAXParser();
	    saxParser.parse(file, this);
	} catch (Exception e) {
	    at.iem.tools.Logger.println("couldn't read XML-file '"+file+"': "+e);
            destination.stopAsync();

	    return null;
	}

	if(xml_clashes==null || xml_clashes.size()==0)return null;
	String[]clashed=new String[xml_clashes.size()];
	for(int i=0;i<clashed.length; i++){
	    clashed[i]=(String)xml_clashes.get(i);
	}

        xml_destination.stopAsync();

	return clashed;
    }

    TranscriptionDataBase xml_destination;
    /**
     * the element stack
     */
    private ArrayList<String> xml_elements;
    /**
     * the element datastack
     */
    private ArrayList<String> xml_data;

    /**
     * a buffer-entry for the data to be read
     */
    private TranscriptionEintrag xml_eintrag;
    /**
     * a buffer timestamp     */
    private long xml_timestamp;

    /**
     * buffer for the region-code
     */
    String xml_regionID=null;

    /**
     * count the number of imported entries
     */
    private int xml_importcounter;

    /**
     * entries that had problems
     */
    private ArrayList<String> xml_clashes;
    private boolean xml_clash;

    public void startDocument() throws SAXException {
	xml_elements=new ArrayList<String>();
	xml_data=new ArrayList<String>();
	xml_clashes=new ArrayList<String>();
	xml_clash=false;
        xml_importcounter=0;
    }
    public void endDocument() throws SAXException  {
    }
    public void startElement(String namespaceURI,
                             String lName, // local name
                             String qName, // qualified name
                             Attributes attrs
			     ) throws SAXException {
	/* a startTag for an element */
        String eName = ("".equals(lName))?qName:lName;
	if("".equals(eName))throw new SAXException("empty element name!");
	if(xml_elements.size()==0 && !eName.equals("adaba"))throw new SAXException("this is no adaba XML! (element='"+eName+"')");
	xml_data.add("");
	xml_elements.add(eName);
	if ("adaba".equals(eName)){

	} else if ("entry".equals(eName)) {
	    xml_eintrag = new TranscriptionEintrag();
	}else if("done".equals(eName)){
	    xml_eintrag.setFertig(true);
	} else if ("transcription".equals(eName)) {
	    /* get the region-code and evaluate the corresponding ID */
	    String region=null;
	    for (int i = 0; i < attrs.getLength(); i++) {
		String aName = attrs.getLocalName(i); // Attr name 
		if("".equals(aName)) aName = attrs.getQName(i);
		if("region".equals(aName))region=attrs.getValue(i);
	    }
            xml_regionID=region;
	}
    }
    public void endElement  (String namespaceURI,
			     String sName, // simple name
			     String qName  // qualified name
			     ) throws SAXException  {
	String eName=(String)xml_elements.get(xml_elements.size()-1);
	String data=(String)xml_data.get(xml_data.size()-1);
	data=data.trim();

	if("entry".equals(eName)){
	    /* handle the TranscriptionEintrag */
	    xml_eintrag.touch(xml_timestamp);
	    //xml_eintrag.
            xml_importcounter++;
            long tsStart=System.currentTimeMillis();
	    xml_destination.add(xml_eintrag);
            long tsStop=System.currentTimeMillis();
            //at.iem.tools.Logger.println("added entry #"+xml_importcounter+" in "+(tsStop-tsStart)+"[ms]: "+xml_eintrag.getFullName());
	    xml_timestamp=0;
	    xml_regionID=null;
	    //if(xml_clash)xml_clashes.add(xml_eintrag.getKey());
	    xml_clash=false;
	}else if("id".equals(eName)){
	    xml_eintrag.setKey(data);
	}else if("variant".equals(eName)){
            byte wortvariant=-1;
            try {
                wortvariant=java.lang.Byte.parseByte(data);
            } catch (Exception e){
                at.iem.tools.Logger.println("invalid variant: "+data);
            }
	    xml_eintrag.setVariant(wortvariant);
	}else if("grammarType".equals(eName)){
	    int gtype=0;
	    try {
		/* look, whether te value is an integer... */
		gtype=java.lang.Integer.parseInt(data);
	    } catch (Exception e){
		/* no it's a string value... */
		int i=0;
		for(i=0; i<AdabaShared.G_TYPEN.length; i++)
		    if(data.equals(AdabaShared.G_TYPEN[i]))break;
		if(i>=AdabaShared.G_TYPEN.length) {
		    at.iem.tools.Logger.println("adding new g-type: "+data+" for "+xml_eintrag.getFullName());
		    // oops, this gType is not yet in our G_TYPEN[]
		    i=AdabaShared.G_TYPEN.length;
		    String[]sarray=new String[i+1];
		    java.lang.System.arraycopy(AdabaShared.G_TYPEN,0,
					       sarray, 0, 
					       i);
		    AdabaShared.G_TYPEN=sarray;
		    AdabaShared.G_TYPEN[i]=data;
		}
		gtype=i;	
	    }
	    xml_eintrag.setGtyp(gtype);
	}else if("etymologyType".equals(eName)){
	    int etype=0;
	    try {
		/* look, whether te value is an integer... */
		etype=java.lang.Integer.parseInt(data);
	    } catch (Exception e){
		/* no it's a string value... */
		int i=0;
		for(i=0; i<AdabaShared.E_TYPEN.length; i++)
		    if(data.equals(AdabaShared.E_TYPEN[i]))break;
		if(i>=AdabaShared.E_TYPEN.length) {
		    at.iem.tools.Logger.println("adding new e-type: "+data+" for "+xml_eintrag.getFullName());
		    // oops, this etype is not yet in our E_TYPEN[]
		    i=AdabaShared.E_TYPEN.length;
		    String[]sarray=new String[i+1];
		    java.lang.System.arraycopy(AdabaShared.E_TYPEN,0,
					       sarray, 0, 
					       i);
		    AdabaShared.E_TYPEN=sarray;
		    AdabaShared.E_TYPEN[i]=data;
		    /*
		    at.iem.tools.Logger.println("could not determine e-type for "+
				       xml_eintrag.getFullName()+": "+data);
		    i=0;
		    xml_clash=true;
		    */
		}
		etype=i;
	    }
	    xml_eintrag.setEtyp(etype);
	    //xml_eintrag.setEtyp(java.lang.Integer.parseInt(data));
	}else if("Flag".equals(eName)){
	    if(AdabaShared.EntryFlag==null){
		AdabaShared.EntryFlag=new String[1];
		AdabaShared.EntryFlag[0]=data;
		xml_eintrag.setFlag(0, true);
	    } else {
		int i=0;
		for(i=0; i<AdabaShared.EntryFlag.length; i++)
		    if(data.equals(AdabaShared.EntryFlag[i]))break;

		xml_eintrag.setFlag(i, true);

		if(i>=AdabaShared.EntryFlag.length) {
		    /* oops, this flag is not yet in our EntryFlag[] */
		    i=AdabaShared.EntryFlag.length;
		    String[]sarray=new String[i+1];
		    java.lang.System.arraycopy(AdabaShared.EntryFlag,0,
					       sarray, 0, 
					       i);
		    AdabaShared.EntryFlag=sarray;
		    AdabaShared.EntryFlag[i]=data;
		}
	    }
	}else if("comment".equals(eName)){
	    xml_eintrag.setKommentar(data);
	}else if("timestamp".equals(eName)){
	    xml_timestamp=java.lang.Long.parseLong(data);
	}else if("phoneticVariant".equals(eName)){
	    xml_eintrag.setVarianz(true);
	}else if("media".equals(eName)){
            /* if there is NO "media"-element, we assume "don't know"
             * if there IS a "media"-element, we DO know whether we have media or not;
             * if the data of this element is <null> (empty), we have NO media, else we do
             * LATER we might put some information in here, like the root path to the media, or...
             */
            if("1".equals(data) || "true".equalsIgnoreCase(data) || "yes".equalsIgnoreCase(data))
                xml_eintrag.haveMedia(true);
            else
                xml_eintrag.haveMedia(false);

	}else if("pronunciation".equals(eName)){
	    String parent=(String)xml_elements.get(xml_elements.size()-2);
	    if("transcription".equals(parent))xml_eintrag.setIPA(xml_regionID, data);
	    else xml_eintrag.setIPA(data);
	}

	xml_elements.remove(xml_elements.size()-1);
	xml_data.remove(xml_data.size()-1);
    }
    public void characters(char buf[], int offset, int len)
	throws SAXException {
	String data= new String(buf, offset, len).trim();
	if ("".equals(data))return; /* do we really need this ? */

	xml_data.set(xml_data.size()-1, (String)xml_data.get(xml_data.size()-1)+data);
    }
}
