// ElanDatabase.java
// $Id: ElanAnnotationChunk.java 1 2006-09-11 13:22:49Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
//
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

package at.iem.adaba.elan;

import java.util.ArrayList;
import java.util.HashMap;


/* XML import stuff: i wonder whether it is a good idea to do it in here: */
import org.xml.sax.*;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;


public class ElanDatabase extends org.xml.sax.helpers.DefaultHandler {
    ElanTranscription[] transcription;
    java.io.File mediaFile;
    String mediaFileName;

    java.io.File eafFile;
    String title;

    public ElanDatabase(java.io.File eaffile){
        importXML(eaffile);
    }
    public ElanDatabase(String eaffile){
        importXML(new java.io.File(eaffile));
    }

    private boolean importXML(java.io.File file) {
        eafFile=null;
        title=null;
        mediaFile=null;
        mediaFileName=null;
        at.iem.tools.Logger.println("elan importing: "+file);

	try {
	    SAXParserFactory factory = SAXParserFactory.newInstance();
	    factory.setValidating(true);	
	    SAXParser saxParser = factory.newSAXParser();
	    saxParser.parse(file, this);
	} catch (Exception e) {
	    at.iem.tools.Logger.println("couldn't read XML-file '"+file+"': "+e);
	    return false;
	}
        at.iem.tools.Logger.println("elan imported: "+file);

        eafFile=file;
        if(null==title) {
            title=eafFile.getName();
            int index=title.lastIndexOf('.');
            if(index>0)
                title=new String(title.substring(0, index));
        }
        transcription = new ElanTranscription[xml_annotations.size()];

        for(int i=0; i<xml_annotations.size(); i++){
            if(transcription[i]==null){
                transcription[i]=new ElanTranscription();
            }
            ArrayList<AnnoClass> annot = xml_annotations.get(i);
            for(int len=0; len<annot.size(); len++) {
                AnnoClass ac=annot.get(len);
                String data=ac.getData();
                //at.iem.tools.Logger.println("parsing "+len+": "+data);
                String stopstring=(String)map_times.get(ac.stoptime);
                try {
                    int stoptime=java.lang.Integer.parseInt(stopstring);
                    if (!(transcription[i].append(data, stoptime))) {
                        at.iem.tools.Logger.println("error adding entry "+len+": "+data);
                        ac.print();
                    }
                } catch (Exception ex) {
                    at.iem.tools.Logger.println("exception raised when parsing '"+data+"': "+stopstring);
                }
            }
        }

        if(checkMediaFile(file, mediaFileName)) {
            //at.iem.tools.Logger.println("found media-file: "+ mediaFile);
        } else {
            at.iem.tools.Logger.println("could not find media-file: "+mediaFileName+" for edb "+file);
        }


        return true;
    }

    private boolean checkMediaFile(java.io.File xmlfile, String mfileName) {
        if(null==mfileName || ("".equals(mfileName)))
            return false;

        mediaFile = new java.io.File (mfileName);
        if(mediaFile.exists()) {
            return true;
        }


        mediaFile = new java.io.File (xmlfile.getParent(), mediaFile.getName());
        if(mediaFile.exists()) {
            return true;
        }

        mediaFile = new java.io.File (xmlfile.getParent(), mediaFile.getName());
        if(mediaFile.exists()) { return true; }

        mediaFile = new java.io.File (at.iem.tools.StringX.replace(xmlfile.getAbsolutePath(), ".eaf", ".wav"));
        if(mediaFile.exists()) { return true; }

        mediaFile = new java.io.File (at.iem.tools.StringX.replace(xmlfile.getAbsolutePath(), ".eaf", ".mp3"));
        if(mediaFile.exists()) { return true; }

        mediaFile = null;
        return false;
    }

    public java.io.File getEAFile() {
        return eafFile;
    }

    public String getTitle() {
        if(title!=null) {
            return title;
        }

        if(eafFile!=null) {
            return eafFile.getName();
        }

        return null;
    }


    public void print(int id) {
        if(transcription!=null&&id>=0&&id<transcription.length){
            int startid=0;
            int stopid=0;
            startid=transcription[id].positionToId(10);
            stopid =transcription[id].positionToId(170);
            at.iem.tools.Logger.println("printing "+startid+".."+stopid);
            at.iem.tools.Logger.println(transcription[id].getText(startid,stopid));
        }
    }

    public ElanTranscription getTranscription(int id) {
        if(transcription!=null&&id>=0&&id<transcription.length)
            return transcription[id];

        return null;
    }

    /**
     * the element stack
     */
    private ArrayList<String> xml_elements;
    /**
     * the element datastack
     */
    private ArrayList<String> xml_data;

    private HashMap<String,String> map_times;

    private class AnnoClass {
        public String id, starttime, stoptime;
        public String data;

        public AnnoClass(String ID, String start, String stop) {
            id=ID;
            starttime=start;
            stoptime=stop;
            data=null;
        }
        public void setData(String Data){
            data=new String(Data);
        }
        public String getData(){
            // trim all leading and trailing whitespaces, EXCEPT 1 trailing carriage-return (if present)
            return at.iem.tools.StringX.trimBlanks(data);
        }
        public void print() {
            at.iem.tools.Logger.println(data+": "+starttime+".."+stoptime+" ("+id+")");
        }
    }
    private ArrayList<ArrayList<AnnoClass>> xml_annotations;
    private ArrayList<AnnoClass> current_annotation;

    private int xml_tier;

    public void startDocument() throws SAXException {
	xml_elements=new ArrayList<String>();
	xml_data=new ArrayList<String>();

        map_times=new HashMap<String,String> ();
        xml_annotations=new ArrayList<ArrayList<AnnoClass>> ();
        current_annotation=null;

        xml_tier=-1;
    }
    public void endDocument() throws SAXException  {
    }
    public void startElement(String namespaceURI,
                             String lName, // local name
                             String qName, // qualified name
                             Attributes attrs
			     ) throws SAXException {
	/* a startTag for an element */
        String eName = ("".equals(lName))?qName:lName;
	if("".equals(eName))throw new SAXException("empty element name!");
	//if(xml_elements.size()==0 && !eName.equals("adaba"))throw new SAXException("this is no adaba XML! (element='"+eName+"')");
       	if(xml_elements.size()==0 && !eName.equals("ANNOTATION_DOCUMENT"))throw new SAXException("this is no ELAN annotation file! (element='"+eName+"')");

        if("TIME_SLOT".equals(eName)){
            String ts_id=null, ts_value=null;
            for (int i = 0; i < attrs.getLength(); i++) {
		String aName = attrs.getLocalName(i); // Attr name 
		if("".equals(aName)) aName = attrs.getQName(i);
                if("TIME_SLOT_ID".equals(aName))
                    ts_id=attrs.getValue(i);
                else if("TIME_VALUE".equals(aName))
                    ts_value=attrs.getValue(i);
	    }
            if((ts_id==null)||(ts_value==null)||("".equals(ts_id))||("".equals(ts_value))){
            } else {
                map_times.put(ts_id, ts_value);
            }
        } else if ("TIER".equals(eName)) {
            xml_tier++;
            current_annotation=new ArrayList<AnnoClass>();
            xml_annotations.add(current_annotation);
        } else if ("ALIGNABLE_ANNOTATION".equals(eName)) {
            String id=null, ref1=null, ref2=null;            
            for (int i = 0; i < attrs.getLength(); i++) {
		String aName = attrs.getLocalName(i); // Attr name 
		if("".equals(aName)) aName = attrs.getQName(i);
                if("ANNOTATION_ID".equals(aName))
                    id=attrs.getValue(i);
                else if("TIME_SLOT_REF1".equals(aName))
                    ref1=attrs.getValue(i);
                else if("TIME_SLOT_REF2".equals(aName))
                    ref2=attrs.getValue(i);
	    }
            if((id==null)||(ref1==null)||(ref2==null)||
               ("".equals(id))||("".equals(ref1))||("".equals(ref2))
               ||current_annotation==null) {
            } else {
                AnnoClass ac = new AnnoClass(id, ref1, ref2);
                current_annotation.add(ac);
            }
        } else if ("ANNOTATION_VALUE".equals(eName)) {
        } else if ("ANNOTATION_DOCUMENT".equals(eName)) {
            for (int i = 0; i < attrs.getLength(); i++) {
		String aName = attrs.getLocalName(i); // Attr name 
		if("".equals(aName)) aName = attrs.getQName(i);
                if("TITLE".equals(aName))
                    title=attrs.getValue(i);
	    }
        } else if ("ANNOTATION".equals(eName)) {
        } else if ("HEADER".equals(eName)) {
            String timebase=null;
            for (int i = 0; i < attrs.getLength(); i++) {
		String aName = attrs.getLocalName(i); // Attr name 
		if("".equals(aName)) aName = attrs.getQName(i);
                if("TIME_UNITS".equals(aName))
                    timebase=attrs.getValue(i);
	    }
            if((timebase==null)||("".equals(timebase))){
            } else {
                if("milliseconds".equals(timebase)){
                    // OK
                } else {
                    at.iem.tools.Logger.println("unknown time-base: "+timebase);
                }
            }
        } else if ("MEDIA_DESCRIPTOR".equals(eName)) {
            String filename=null;
            for (int i = 0; i < attrs.getLength(); i++) {
		String aName = attrs.getLocalName(i); // Attr name 
		if("".equals(aName)) aName = attrs.getQName(i);
                if("MEDIA_URL".equals(aName))
                    filename=attrs.getValue(i);
	    }
            if((filename==null)||("".equals(filename))){
            } else {
                mediaFileName=filename;
            }
        } else if ("TIME_ORDER".equals(eName)) {
        } else if ("LINGUISTIC_TYPE".equals(eName)) {
        } else if ("LOCALE".equals(eName)) {
        } else if ("CONSTRAINT".equals(eName)) {
       } else
            at.iem.tools.Logger.println("startElement: "+eName);

	xml_data.add("");
	xml_elements.add(eName);

    }
    public void endElement  (String namespaceURI,
			     String sName, // simple name
			     String qName  // qualified name
			     ) throws SAXException  {
	String eName=(String)xml_elements.get(xml_elements.size()-1);
	String data=(String)xml_data.get(xml_data.size()-1);
	//data=data.trim();
        //at.iem.tools.Logger.println("EE: "+eName+": "+data+" ("+(xml_data.size()-1)+")");

        if ("ANNOTATION_VALUE".equals(eName)) {
            //at.iem.tools.Logger.println("ANNOVALUE: "+eName+": "+data+" ("+(xml_data.size()-1)+")");
            if(current_annotation!=null&&current_annotation.size()>0){
                AnnoClass ac = current_annotation.get(current_annotation.size()-1);
                ac.setData(data);
                current_annotation.remove(current_annotation.size()-1);
                current_annotation.add(ac);
             }
        }

	xml_elements.remove(xml_elements.size()-1);
	xml_data.remove(xml_data.size()-1);
    }
    public void characters(char buf[], int offset, int len)
	throws SAXException {
	String data= new String(buf, offset, len);
	if ("".equals(data))return; /* do we really need this ? */
        //at.iem.tools.Logger.println("characters: '"+data+"': "+offset+" "+len);
        int pos=xml_data.size()-1;
        String newdata=(String)xml_data.get(pos)+data;
        //at.iem.tools.Logger.println("characters: '"+newdata+"': "+offset+" "+len);
	xml_data.set(pos, newdata);
        //at.iem.tools.Logger.println("data:: "+(String)xml_data.get(pos)+ "("+pos+")");
    }
}
