// ElanSelectorInterface.java
// $Id: GUITextSelector.java 831 2006-03-16 16:21:53Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
// 
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

package at.iem.adaba.elan;

import java.io.File;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.ArrayList;

public abstract class ElanSelector extends javax.swing.JPanel {
    /**
     * types of ElanSelector implementations...
     */
    public final static int TYPE_SIMPLE = 0;
    public final static int TYPE_PERSON_TEXT = 1;


    /**
     * evaluate the (possibly multidimensional) selection into the so selected file
     * @return the index of the selected filename
     */
    public abstract int getSelectedIndex();

    /**
     * clears all items from this instance
     * currently unimplemented !!!!!
     */
    public abstract void removeAllItems();


    /**
     * add a single elan-file
     * @param f the elan-file to be added
     */
    public void addItem(ElanDatabase edb){
        ElanDatabase[]edbs=new ElanDatabase[1];
        edbs[0]=edb;
        addItem(edbs);
    }


   /**
     * add several elan-files
     * @param f the elan-files to be added
     */
    public abstract void addItem(ElanDatabase edb[]);



    private ArrayList<ActionListener> m_listeners=null;
    /**
     * add an ActionListener that will get notified when the selection changes
     * @param l the ActionListener to be added
     */
    public void addActionListener(java.awt.event.ActionListener l){
        if(m_listeners==null)
            m_listeners=new  ArrayList<ActionListener>();
        m_listeners.add(l);
    }
    /**
     * this notifies all registered ActionListeners, whenever the selection changes
     * @param e the ActionEvent to be sent to all registered ActionListeners
     */
    protected void notifyActionListeners(ActionEvent e) {
        if(m_listeners==null)return;
        for(int i=0; i<m_listeners.size(); i++){
            ActionListener l = (ActionListener) m_listeners.get(i);
            if(l!=null)
                l.actionPerformed(e);
            else {
                at.iem.tools.Logger.println("oops: empty action listener!!");
            }
        }
    }
    
    /**
     * this notifies all registered ActionListeners, whenever the selection changes
     * @param id the event-id to be sent to all registered ActionListeners
     * @param s the event-string to be sent to all registered ActionListeners
     */
    protected void notifyActionListeners(int id, String s) {
        try {
            notifyActionListeners(new ActionEvent(this, id, s));
        } catch (Exception e) {
            at.iem.tools.Logger.println("failed to notify action listeners: "+e);
        }
    }
}
