// GUI.java
// $Id: GUI.java 831 2006-03-16 16:21:53Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
// 
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

package at.iem.adaba.elan;
import at.iem.adaba.AdabaShared;
import at.iem.tools.sound.SoundPlayer;

import javax.swing.*;
import javax.swing.text.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;


/**
 * GUI fr die Darstellung von ELAN-basierten Text-Transkriptionen
 */

public class GUI extends JPanel  implements ActionListener, CaretListener, at.iem.tools.sound. AbstractPlayerStateListener,  at.iem.tools.gui.Refreshable{
    // formatting: JTextField, JTextPane (html!)
    private SimpleAttributeSet style_normal, style_bold;

    private ElanSelector textSelector;
    //    private JTextArea[]   text;
    private JTextPane[]   text;
    private JButton playButton, stopButton, pauseButton;
    private SoundPlayer soundplayer;

    private ElanDatabase[] edb;
    private ElanDatabase edb_current;

    private boolean playing;
    private ImageIcon playicon, pauseicon, stopicon;

    /**
     * Creates a new elan::GUI() which gets initialized by the given path
     * @parm path a directory containing elan-files or an elan-file
     */
    public GUI(String path) {
        this(new java.io.File(path));
    }

    /**
     * Creates a new elan::GUI() which gets initialized by the given path
     * @parm path a directory containing elan-files or an elan-file
     */
    public GUI(java.io.File path) {
        this(path, ElanSelector.TYPE_SIMPLE);
    }
    /**
     * Creates a new elan::GUI() which gets initialized by the given path
     * @parm path a directory containing elan-files or an elan-file
     */
    public GUI(java.io.File path, int selectorType) {
        style_normal = new SimpleAttributeSet();
        style_bold = new SimpleAttributeSet();
        StyleConstants.setBold(style_bold, true);

        text=new JTextPane[2];

        for(int i=0; i<text.length; i++) {
            text[i] = new JTextPane();
            text[i].setEditable(false);
            text[i].addCaretListener(this);
            at.iem.tools.gui.AntiAliasing.enable(text[i]);
            text[i].setBackground(at.iem.adaba.AdabaConstants.AdabaGreyBackground);
       }

        JPanel helppane;
	setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	JPanel MainPane = new JPanel();
        JScrollPane ScrollPane; // = new JScrollPane(MainPane);

        switch(selectorType) {
        case ElanSelector.TYPE_PERSON_TEXT:
            textSelector = new GUITextSelector();
            break;
        default:
            textSelector = new OrderedTextSelector();
        }
        textSelector.addActionListener(this);
        
	java.net.URL iconUrl=ClassLoader.getSystemResource("resources/playbuttons/playBIG.gif");
	if (iconUrl!=null)
	    playicon = new ImageIcon(iconUrl, AdabaShared.getLabel("ButtonPlay"));
	else
	    playicon=new ImageIcon("resources/playbuttons/play.gif", AdabaShared.getLabel("ButtonPlay"));

	iconUrl=ClassLoader.getSystemResource("resources/playbuttons/pauseBIG.gif");
	if (iconUrl!=null)
	    pauseicon = new ImageIcon(iconUrl, AdabaShared.getLabel("ButtonPause"));
	else
	    pauseicon=new ImageIcon("resources/playbuttons/pause.gif", AdabaShared.getLabel("ButtonPause"));

	iconUrl=ClassLoader.getSystemResource("resources/playbuttons/stopBIG.gif");
	if (iconUrl!=null)
	    stopicon = new ImageIcon(iconUrl, AdabaShared.getLabel("ButtonStop"));
	else
	    stopicon=new ImageIcon("resources/playbuttons/stop.gif", AdabaShared.getLabel("ButtonStop"));

        playButton=new JButton(playicon);
	playButton.addActionListener(this);
        playButton.setEnabled(false);

        pauseButton=new JButton(pauseicon);
	pauseButton.addActionListener(this);
        pauseButton.setEnabled(false);

        stopButton=new JButton(stopicon);
	stopButton.addActionListener(this);
        stopButton.setEnabled(false);


        helppane = new JPanel();
        helppane.add(textSelector);
        helppane.add(Box.createRigidArea(playButton.getPreferredSize()));
        helppane.add(playButton);
        helppane.add(Box.createRigidArea(new Dimension(0,20)));
        helppane.add(pauseButton);
        helppane.add(Box.createRigidArea(new Dimension(0,20)));
        helppane.add(stopButton);

        MainPane.add(helppane);
       
        addScrollPane(MainPane, text[0],  AdabaShared.getLabel("TitleTextOrthopane"));
        addScrollPane(MainPane, text[1],  AdabaShared.getLabel("TitleTextPhonopane"));

	MainPane.setLayout(new BoxLayout(MainPane, BoxLayout.Y_AXIS));
        add(MainPane);

        soundplayer = new SoundPlayer();
        soundplayer.addAbstractPlayerStateListener(this);

        open(path);	
    }

    /**
     * adds titled vertical scroll-panes to a component
     * @param pane the parent component
     * @param txt the child component which needs scrollbars
     * @param title a label for the scrollpane
     */
    static void addScrollPane(JComponent pane, JComponent txt, String title) {
        JPanel helppane = new JPanel();
        helppane.setLayout(new BorderLayout());
        helppane.setBorder(BorderFactory.createTitledBorder(
                                         BorderFactory.createEmptyBorder(),
                                         title));

        JScrollPane ScrollPane = new JScrollPane(txt);
        ScrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        ScrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);

        helppane.add(ScrollPane);
        pane.add(helppane);
    }

    /**
     * pre-loads an elan-file (or all elan-files in a directory) into memory; 
     * the first file will be displayed immediately
     * @param file either an elan-file or a directory containing elan-files (*.eaf)
     * @return <code>true</code> on sucess, <code>false</code> otherwise
     */
    public boolean open(java.io.File file) {
        if(textSelector!=null)
            textSelector.removeAllItems();

        if(file==null)return false;

        if(file.isFile()) {
            edb=new ElanDatabase[1];
            edb[0]=new ElanDatabase(file);
        } else if (file.isDirectory()) {
            java.io.File[]files=at.iem.tools.FileX.listFilesRecursive(file, new at.iem.tools.FileFilterRegex(new String(".*\\.eaf"), false, false));
            if(files==null)
                return false;

            java.util.Arrays.sort(files);

            /* probably sort here */
            edb=new ElanDatabase[files.length];
            for(int i=0; i<edb.length; i++) {
                edb[i]=new ElanDatabase(files[i]);
            }
        } else {
            // ??? what file is this???
            return false;
        }

        if(edb==null || edb.length<1)
            return false;

        if(textSelector!=null) {
            textSelector.addItem(edb);
            return update(textSelector.getSelectedIndex());
        }

        return true;
    }
    /**
     * pre-loads an elan-file (or all elan-files in a directory) into memory;
     * the first file will be displayed immediately
     * @param file either an elan-file or a directory containing elan-files (*.eaf)
     * @return <code>true</code> on sucess, <code>false</code> otherwise
     */
    public boolean open(String file) {
        return open(new java.io.File(file));
    }

    /**
     * displays one of the pre-loaded texts
     * @param id the index of the text
     * @return <code>true</code> on success, <code>false</code> if something went wrong
     */
    public boolean update(int id){
        //at.iem.tools.Logger.println("update: "+id+", edb="+edb);
        if(id<0 || edb==null || id>=edb.length)
            return false;

        edb_current=edb[id];
        //at.iem.tools.Logger.println("update: current_ebd="+edb_current);
        boolean local_ignorecaret=ignore_caret; ignore_caret=true;
        for(int i=0; i<text.length; i++){
            String the_text=null;
            ElanTranscription et=edb_current.getTranscription(i);
            if(et!=null) {
                the_text=et.getText();
            }

            text[i].setEnabled(false);
            text[i].setBackground(at.iem.adaba.AdabaConstants.AdabaGreyBackground);
            text[i].setText(the_text);
            text[i].setFont(AdabaShared.AdabaFONT.deriveFont(Font.PLAIN, 15.f));
            if(null!=the_text&&the_text.length()>0) {
                text[i].setEnabled(true);
                text[i].setBackground(Color.WHITE);
            }
	}
	clearSelection();

        selectionStart=selectionStop=0;


        ignore_caret=local_ignorecaret;

        boolean loaded=false;
	if(null!=edb_current.mediaFile) {
	    loaded=soundplayer.load(edb_current.mediaFile);

	    try {
		if(!loaded)
		    loaded=soundplayer.load(edb_current.mediaFile.toURL());
		
	    } catch(java.net.MalformedURLException e) {
	    }
	} else 
	    loaded=false;
	playButton.setEnabled(loaded);
    
        return loaded;
    }

    private void clearSelection() {
        markSelection(-1,-1);
        selectionStart=selectionStop=0;
        startTime=stopTime=0;
    }

    private int selectedField=0;
    private int selectionStart, selectionStop;
    private int startTime=0, stopTime=0;

    /**
     * get the transcription block selected by the current caret position 
     * and set the caret selection to match the boundaries of this transcription block
     * @param id the index of the textpane to be considered
     */
    private void fixSelection(int id) {
        ElanTranscription et=edb_current.getTranscription(id);
        if(et==null)
            return;

        boolean local_ignorecaret=ignore_caret; ignore_caret=true;

        int startChar=text[id].getSelectionStart();
        int stopChar =text[id].getSelectionEnd();

        int startChunk=et.positionToId(startChar);
        int stopChunk =et.positionToId(stopChar);

        startTime=et.getStartTime(startChunk);
        stopTime =et.getStopTime (stopChunk);

        startChar=et.getStartPosition(startChunk);
        stopChar =et.getStopPosition (stopChunk);

        markSelection(startTime, stopTime);

        selectionStart=startChar;
        selectionStop=stopChar;


        ignore_caret=local_ignorecaret;
    }

    private void markSelection(int start, int stop) {
        for(int i=0; i<text.length; i++) {
            markSelection(i, start, stop);
        }

    }
    private void markSelection(int id, int start, int stop) {
        if(text==null||id<0||id>=text.length||text[id]==null)
            return;

 
        boolean local_ignorecaret=ignore_caret; ignore_caret=true;

        try {
            text[id].setCaretPosition (0);
            text[id].moveCaretPosition(text[id].getText().length());
            text[id].setCharacterAttributes(style_normal, true);
        } catch (IllegalArgumentException e) {
            at.iem.tools.Logger.println("ElanGUI: markSelection("+id+", "+start+", "+stop+"):: "+e);
        }



        try {
            ElanTranscription et=edb_current.getTranscription(id);

            if(et!=null && start>=0 && stop>=0) {
		int startChunk=et.timeToId(start);
		int stopChunk =et.timeToId(stop);

                int startChar=et.getStartPosition(startChunk);
                int stopChar =et.getStopPosition (stopChunk);

		at.iem.tools.Logger.println("marking selection ("+startChar+", "+stopChar+") == ["+startChunk +", "+stopChunk+"]");
                if(startChar>-1)                
                    text[id].setCaretPosition (startChar);
                if(stopChar>-1)
                    text[id].moveCaretPosition(stopChar);
                text[id].setCharacterAttributes(style_bold, true);
            } else {
		text[id].setCaretPosition (0);
		text[id].moveCaretPosition(0);  
            }
        } catch (Exception e) {
            at.iem.tools.Logger.println("ElanGUI: markSelection2("+id+", "+start+", "+stop+"):: "+e);
        }
        
        ignore_caret=local_ignorecaret;
    }

    /**
     * copies the selection from one text-container to another;
     * this is done by getting the transcription block selected by the current caret selection in the first text-container,
     * followed by setting the caret selection in the other text-container to match the boundaries of this transcription block;
     * the transcription block may contain several transcription chunks
     * @param oldId the index of the source textpane the selection of which is to be copied
     * @param newId the index of the destination textpane
     */
    private void fixSelection(int oldId, int newId) {
        if(selectionStart==selectionStop)return;
        boolean local_ignorecaret=ignore_caret; ignore_caret=true;

        int startChar=selectionStart+1;
        int stopChar =selectionStop;

        ElanTranscription et_old=edb_current.getTranscription(oldId);
        if(et_old!=null) {
            int startChunk=et_old.positionToId(startChar);
            int stopChunk =et_old.positionToId(stopChar);

            startTime=et_old.getStartTime(startChunk);
            stopTime =et_old.getStopTime (stopChunk);

            ElanTranscription et_new=edb_current.getTranscription(newId);
     
            if(et_new!=null) {
                startChunk=et_new.timeToId(startTime);
                stopChunk =et_new.timeToId(stopTime);
                
                startChar=et_new.getStartPosition(startChunk);
                stopChar =et_new.getStopPosition (stopChunk); 
                startTime=et_new.getStartTime(startChunk);
                stopTime =et_new.getStopTime (stopChunk);

                try {
                    text[newId].setCaretPosition (startChar);
                    text[newId].moveCaretPosition(stopChar);
                    text[newId].setCharacterAttributes(style_bold, true);
                } catch (IllegalArgumentException e) {
                    at.iem.tools.Logger.println(e);
                }
                selectionStart=startChar;
                selectionStop=stopChar;
            }
        }
        
        ignore_caret=local_ignorecaret;
    }
    private int fadeInTime = 500;
    private int fadeOutTime= 500;
    /**
     * plays a part of the soundfile according to the current selection;
     * the selected area is expanded by a certain amount in order to guarantee that the full start and stop of the snippet are audible
     */
     private void playSelection() {
        if(playing){
            setPlaying(false);
            // for some reasons it is necessary to call soundplayer.stop() here and below and NOT at the beginning of this function body
            soundplayer.stop();
            return;
        }
        int realStart=startTime-fadeInTime;
        int realStop=stopTime+fadeOutTime;

        if(realStart<0)realStart=0;
        if(realStop<0)realStop=0;

        soundplayer.stop();
        setPlaying(true);
	at.iem.tools.Logger.println("playSelection: "+realStart+" -> "+realStop);
        if(startTime==stopTime)
            soundplayer.play();
        else
            soundplayer.play(realStart, realStop);
    }

    /* 
     * start the playback (which is not yet running)
     */
    public void startPlaying() {
        int realStart=startTime-fadeInTime;
        int realStop=stopTime+fadeOutTime;

        if(realStart<0)realStart=0;
        if(realStop<0)realStop=0;
        
        soundplayer.stop();

        setPlaying(STATE_UNKNOWN);
	at.iem.tools.Logger.println("playSelection: "+realStart+" -> "+realStop);
        if(startTime==stopTime) /* play the entire soundfile */
            soundplayer.play();
        else
            soundplayer.play(realStart, realStop); 
    }

    /* 
     * stop the playback permanently
     */
    public void stopPlaying() {
        setPlaying(STATE_UNKNOWN);
        soundplayer.stop();
    }

    /* 
     * stop the playback (temporary)
     */
    public void pausePlaying() {
        setPlaying(STATE_UNKNOWN);
        soundplayer.pause();
    }
    /*
     * restart the already running (but paused) playback
     */
    public void resumePlaying() {
        setPlaying(STATE_UNKNOWN);
        soundplayer.resume();
    }

    public void stop() {
        soundplayer.stop();
    }


    /**
     * the listener for button-clicks:<ul>
     * <li>clicking on the "playButton" either starts or stops the playback of the currently selected sound;
     * <li>changing the selected text stops the current playback (if any) and changes the displayed text.</ul>
     * @param e the ActionEvent triggered by a button-press
     */
    public void actionPerformed(ActionEvent e) {
	Object source = e.getSource();
        if(source==playButton) {
            switch(playstate) {
            case STATE_STOPPED:
                startPlaying();
                break;
            case STATE_PLAYING:
                pausePlaying();
                break;
            case STATE_PAUSED:
                resumePlaying();
                break;
            }
        } else if(source==stopButton) {
            stopPlaying();
        } else if(source==pauseButton) {
           switch(playstate) {
            case STATE_PLAYING:
                pausePlaying();
                break;
            case STATE_PAUSED:
                resumePlaying();
                break;
            }
        } else if(source==textSelector) {
            if(playing) {
                stopPlaying();
            }
            int i=textSelector.getSelectedIndex();
            update(i);
        } else 
            at.iem.tools.Logger.println("ElanGUI:: unknown event");

    }

    boolean ignore_caret = false;
    /**
     * the listener for selection changes:
     * whenever the selected text changes, this function gets notified and eventually 
     * corrects the selection to match the transcription chunk boundaries
     * @param e the CaretEvent triggered by a selection change
     */
    public void caretUpdate(CaretEvent e) {
        if(ignore_caret)return;
       	Object source = e.getSource();
        for (int i=0; i<text.length; i++){
            if(source==text[i]){
                if(e.getDot()==e.getMark()){
                    if(selectedField!=i){
                    // active field changed!
                        fixSelection(selectedField, i);
                    } else {
                        clearSelection();
                    }
                } else {
                    fixSelection(i);
                }
                selectedField=i;                
            }
        }
    }

    /**
     * updates the internal state, whether we are currently playing back a soundfile and
     * changes the playButton-icon to reflect this state (and the to-be action);
     * during playback of a soundfile, the playButton looks like a "pause" button;
     * when no playback is active, the playButton looks like a "play" button
     * @param state <code>true</code> for "currently playing", <code>false</code> for "currently stopped"
     */
    private void setPlaying(boolean state) {
        setPlaying((state)?STATE_PLAYING:STATE_STOPPED);
    }

    /**
     * updates the internal state, whether we are currently playing back a soundfile and
     * changes the playButton-icon to reflect this state (and the to-be action);
     * during playback of a soundfile, the playButton looks like a "pause" button;
     * when no playback is active, the playButton looks like a "play" button
     * @param state see playstate
     */
    private void setPlaying(int playstate) {
        this.playstate=playstate;
        at.iem.tools.Logger.println("new playstate: "+playstate);

        switch(playstate) {
        case STATE_PLAYING: /* normal playback */
            //playButton.setIcon(pauseicon);

            playButton.setEnabled(false);
            stopButton.setEnabled(true);
            pauseButton.setEnabled(true);

            //playButton.setToolTipText(AdabaShared.getLabel("ToolTipElanPause"));
            pauseButton.setToolTipText(AdabaShared.getLabel("ToolTipElanPause"));
            stopButton.setToolTipText(AdabaShared.getLabel("ToolTipElanStop"));
            break;
        case STATE_STOPPED: /* stopped */
            //playButton.setIcon(playicon);

            playButton.setEnabled(true);
            stopButton.setEnabled(false);
            pauseButton.setEnabled(false);

            playButton.setToolTipText(AdabaShared.getLabel("ToolTipElanPlay"));
            pauseButton.setToolTipText(AdabaShared.getLabel("ToolTipElanPause"));
            stopButton.setToolTipText(AdabaShared.getLabel("ToolTipElanStopped"));
            break;
        case STATE_PAUSED: /* paused */
            //playButton.setIcon(playicon);

            playButton.setEnabled(true);
            stopButton.setEnabled(true);
            pauseButton.setEnabled(false);

            playButton.setToolTipText(AdabaShared.getLabel("ToolTipElanContinue"));
            //pauseButton.setToolTipText(AdabaShared.getLabel("ToolTipElanContinue"));
            stopButton.setToolTipText(AdabaShared.getLabel("ToolTipElanStop"));
            break;
        default: /* what the... */
            playButton.setEnabled(false);
            stopButton.setEnabled(false);
            pauseButton.setEnabled(false);
            break;
        }
    }

    static final int STATE_UNKNOWN = -1;
    static final int STATE_STOPPED = 0;
    static final int STATE_PLAYING = 1;
    static final int STATE_PAUSED = 2;
    int playstate = STATE_STOPPED;

    /**
     * the listener for playback changes:
     * whenever the playback thread signals a new event, this gets updated
     * @param source the player object emitting the event
     * @param event the AbstractPlayerEvent triggered by the <code>source</code>
     */
    public void stateUpdated(Object source, at.iem.tools.sound.AbstractPlayerEvent event) {
        at.iem.tools.Logger.println("stateupdated: "+event.getCode());
        switch(event.getCode()) {
        case at.iem.tools.sound.AbstractPlayerEvent.PAUSED:
            setPlaying(STATE_PAUSED);
            break;
        case at.iem.tools.sound.AbstractPlayerEvent.STOPPED:
        case at.iem.tools.sound.AbstractPlayerEvent.OPENED:
            setPlaying(STATE_STOPPED);
            break;
        case at.iem.tools.sound.AbstractPlayerEvent.RESUMED:
        case at.iem.tools.sound.AbstractPlayerEvent.PLAYING:
        default:
            setPlaying(STATE_PLAYING);
            break;
        }
    }

    /**
     * implements Refreshable
     * (probably needed to get notified of a new default font)
     */
    public void refresh() {
        for(int i=0; i<text.length; i++){
            text[i].setFont(AdabaShared.AdabaFONT.deriveFont(Font.PLAIN, 15.f));
        }
  
    }
    private static final long serialVersionUID = 1L;
}
