// GUITextSelector.java
// $Id: GUITextSelector.java 831 2006-03-16 16:21:53Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
// 
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

package at.iem.adaba.elan;
import at.iem.adaba.AdabaShared;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import java.io.File;
import java.util.ArrayList;

/**
 * ein RadioButton hnlicher( aber mehr-dimensionaler) Selektor fr verschiedene soundfiles die aus einem elan-file extrahiert werden;
 * die Namen werden generisch aus dem elan-filename erzeugt.
 * file-name: <typ>__<region>__<speaker>.eaf
 *       typ: (BIO|LIT|NRT|PRS|INT)
 *     region: (AT|DE|CH|ATKA|ATST|....)
 *    spkeaer: [MW]_[0-9]+
<+
 * filename: (BIO|LIT|NRT|PRS|INT)__([A-Z]{2}([A-Z]{2})?)__[MW]_[0-9].eaf
 */

public class GUITextSelector extends ElanSelector implements ActionListener, 
                                                       ListSelectionListener
{
    private JRadioButton [] speakerSelection;
    private String[] speakerInfo;

    private JRadioButton[] textSelection;
    private String[] textInfo;

    /**
     * creates an empty GUITextSelector
     */
    public GUITextSelector() {
    }

    /**
     * clears all items from this instance
     * currently unimplemented !!!!!
     */
    public void removeAllItems() {
	at.iem.tools.Logger.println("GUITextSelector:removeAllItems() not yet implemented");
    }

    private File[]  fileList;
    private String[]speakerList;
    private String[]typeList;

    private int[][]file2mapping;

    /**
     * create the internal data representation from a list of files
     */
    private void createData(ElanDatabase[]edb)
    {
	ArrayList<String>persons=new ArrayList<String>(edb.length);
	ArrayList<String>types  =new ArrayList<String>(edb.length);

	fileList=new File[edb.length];
	file2mapping=new int[edb.length][2];

	for(int i=0; i<edb.length; i++) {
            File f=edb[i].getEAFile();
	    fileList[i]=f;

	    String[]parts=extractInfo(f.getName());
	    String typ=expandPersonInfo(parts[0]);
	    String per=expandTextInfo(parts[1]);

	    if(!persons.contains(per)) {
		persons.add(per);
	    }
	    file2mapping[i][0]=persons.indexOf(per);

	    if(!types.contains(typ)) {
		types.add(typ);
	    }
	    file2mapping[i][1]=types.indexOf(typ);
	}

	speakerInfo=fromStringArray(persons);
	textInfo   =fromStringArray(types);

	java.util.Arrays.sort(speakerInfo);
	java.util.Arrays.sort(textInfo);


	persons=new ArrayList<String>();
	types  =new ArrayList<String>();

	for(int i=0; i<speakerInfo.length; i++) {
	    persons.add(speakerInfo[i]);
	}
	for(int i=0; i<textInfo.length; i++) {
	    types.add(textInfo[i]);
	}

	for(int i=0; i<edb.length; i++) {
	    String[]parts=extractInfo(edb[i].getEAFile().getName());
	    String typ=expandPersonInfo(parts[0]);
	    String per=expandTextInfo(parts[1]);

	    file2mapping[i][0]=persons.indexOf(per);
	    file2mapping[i][1]=types.indexOf(typ);
	}


	for(int i=0; i<speakerInfo.length; i++) {
	    at.iem.tools.Logger.println("speaker["+i+"]: "+speakerInfo[i]);
	}

	for(int i=0; i<textInfo.length; i++) {
	    at.iem.tools.Logger.println("text["+i+"]: "+textInfo[i]);
	}

	for(int i=0; i<edb.length; i++) {
	    at.iem.tools.Logger.println("file["+i+"]: "+edb[i].getEAFile()+": ("+file2mapping[i][0]+", "+file2mapping[i][1]+")");
	}
    }

    private static String[]extractInfo(String filename) 
    {
	if(filename.endsWith(".eaf")) {
	    filename=filename.substring(0,filename.lastIndexOf(".eaf"));
	}

	String[]s=filename.split("__");
	if(null==s || s.length<1)
	    return null;

	
	String[]parts=new String[2];
	parts[0]=s[0];
	parts[1]=null;
	for(int i=1; i<s.length; i++) {
	    if(null==parts[1])
		parts[1]=s[i];
	    else
		parts[1]=parts[1]+"__"+s[i];
	}

	return parts;
    }


    /**
     * add several elan-files
     * @param s the elan-files to be added
     */
    public void addItem(ElanDatabase o[]) {
	if(null==o || 0==o.length)return;
        setLayout(new GridLayout(0,2));
	createData(o);

        speakerSelection=new JRadioButton[speakerInfo.length];
	ButtonGroup group = new ButtonGroup();
        JPanel buttonpane = new JPanel();
	buttonpane.setLayout(new BoxLayout(buttonpane, ((true)?BoxLayout.Y_AXIS:BoxLayout.X_AXIS)));

        for(int i=0; i<speakerSelection.length; i++) {
            speakerSelection[i]=new JRadioButton(expandPersonInfo(speakerInfo[i]));
            if(i==0)
                speakerSelection[i].setSelected(true);

            speakerSelection[i].addActionListener(this);
            speakerSelection[i].setActionCommand(speakerInfo[i]);
            group.add(speakerSelection[i]);
            buttonpane.add(speakerSelection[i]);
        }


	JPanel helppane = new JPanel();
	helppane.setLayout(new BorderLayout());
	helppane.setBorder(BorderFactory.createTitledBorder(
                                                                BorderFactory.createEtchedBorder(),
                                                                AdabaShared.getLabel("TitleTextButtonPane")));

	JScrollPane ScrollPane = new JScrollPane(buttonpane);
            
	ScrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
	ScrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
            
	helppane.add(ScrollPane);
	helppane.setPreferredSize(new Dimension(200,150));
	this.add(helppane);

        String[]textInfoEx=new String[textInfo.length];
        for(int i=0; i<textInfo.length; i++) {
            textInfoEx[i]=expandTextInfo(textInfo[i]);
        }
	
	textSelection= new JRadioButton[textInfo.length];
	group = new ButtonGroup();
        JPanel typepane = new JPanel();

        for(int i=0; i<textSelection.length; i++) {
            textSelection[i]=new JRadioButton(expandTextInfo(textInfo[i]));
            if(i==0)
                textSelection[i].setSelected(true);

            textSelection[i].addActionListener(this);
            textSelection[i].setActionCommand(speakerInfo[i]);
            group.add(textSelection[i]);
            typepane.add(textSelection[i]);
        }


	typepane.setLayout(new BoxLayout(typepane, ((true)?BoxLayout.Y_AXIS:BoxLayout.X_AXIS)));
        typepane.setBorder(BorderFactory.createTitledBorder(
                                                              BorderFactory.createEtchedBorder(),
                                                              AdabaShared.getLabel("TitleTextTypePane")));
	
        add(typepane);

	fixSelection();
    }
    /**
     * extract and expand the "person" information from the person-part of a filename
     * @param s person-part of the elan-file to be parsed
     * @return the "person" information
     */
    private static String expandPersonInfo(String s) {
        String persType=s;

        String persTypes[]=s.split("_");
        if(null!=persTypes && persTypes.length>1) {
            String land=persTypes[0];
            for(int j=0; j<AdabaShared.region.length; j++) {
                if(persTypes[0].equals(AdabaShared.region[j][0])){
                    land=AdabaShared.region[j][1];
                    break;
                }
            }

            persType=new String(land+" (");
            for(int j=1; j<persTypes.length; j++) {
                persType+=persTypes[j]+" ";
            }
            persType=persType.trim()+")";
        }
        return persType;
    }


   /**
     * extract and expand the "text" information from a single elan-filenames
     * @param s the elan-file to be parsed
     * @return the "text" information for each of the given elan-files
     */
    private static String expandTextInfo(String s) {
        for(int j=0; j<AdabaShared.textTypes.length; j++) {
            if(s.equals(AdabaShared.textTypes[j][0])){
                return AdabaShared.textTypes[j][1];
            }
        }
        return s;
    }

    /**
     * evaluate the multidimensional selection into the so selected file
     * @return the index of the selected filename
     */
    public int getSelectedIndex() {
        at.iem.tools.Logger.println("calculate selected index");
	int speaker=-1;
	for(int i=0; i<speakerSelection.length; i++) {
            if(speakerSelection[i].isSelected()) {
                speaker=i;
                break;
            }
        }
	int text=-1;
	for(int i=0; i<textSelection.length; i++) {
	    if(textSelection[i].isSelected()) {
                text=i;
		break;
            }
	}


	for(int i=0; i<file2mapping.length; i++) {
	    if(speaker==file2mapping[i][0] && text==file2mapping[i][1])
		return i;
	}
        
        return -1;
    }


    private void fixSelection() 
    {
	int speaker=-1;
	for(int i=0; i<speakerSelection.length; i++) {
            if(speakerSelection[i].isSelected()) {
                speaker=i;
                break;
            }
        }
	int text=-1;
	for(int i=0; i<textSelection.length; i++) {
	    if(textSelection[i].isSelected()) {
                text=i;
            }
	    textSelection[i].setEnabled(false);
	}

	for(int i=0; i<file2mapping.length; i++) {
	    if(speaker==file2mapping[i][0])
		textSelection[file2mapping[i][1]].setEnabled(true);
	}

	if(!textSelection[text].isEnabled()) {
	    for(int i=0; i<textSelection.length; i++) {
		if(textSelection[i].isEnabled()) {
		    textSelection[i].setSelected(true);
		    break;
		}
	    }
	}

	at.iem.tools.Logger.println("selection: "+speaker+" "+text);

    }

    private String selectedSpeaker=null;


    private boolean ignoreAction=false;
    /**
     * whenever the user changes the selection by clicking on one of the RadioButtons this triggers all registered ActionListeners;
     * currently disabled !!!!!!!!!!
     * @param e the ActionEvent triggered by changing the radio-button selection
     */
    public void actionPerformed(ActionEvent e) {
	if(ignoreAction)return;
	ignoreAction=true;
	fixSelection();
        //at.iem.tools.Logger.println("action performed by: "+e);
        if(false==false) {
            selectedSpeaker=e.getActionCommand();
            notifyActionListeners(0, "radio");
        }
	ignoreAction=false;
    }
    /**
     * whenever the user changes the selection by clicking on one the "text"-style list this triggers all registered ActionListeners;
     * @param e the ListSelectionEvent triggered by changing the radio-button selection
     */
    public void valueChanged(ListSelectionEvent e) {
	if(ignoreAction)return;
	ignoreAction=true;
        if(e.getValueIsAdjusting()==false) {
            notifyActionListeners(1, "list");
        }
	ignoreAction=false;
    }


    /**
     * convert an ArrayList of Strings to a String[]
     * @param in the ArrayList of Strings
     * @return an Array of Strings
     */
    private String[]fromStringArray(ArrayList<String>in) {
        if(null==in || in.size()<1)
            return null;
        
	String[] result = new String[in.size()];
        
	for(int i=0; i<result.length; i++){
            result[i]=(String)in.get(i);
        }
	return result;
    }

    private static final long serialVersionUID = 1L;
}
