// OrderedTextSelector.java
// $Id: OrderedTextSelector.java 831 2006-03-16 16:21:53Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
// 
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

package at.iem.adaba.elan;
import at.iem.adaba.AdabaShared;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import java.io.File;
import java.util.ArrayList;

/**
 * ein RadioButton hnlicher( aber mehr-dimensionaler) Selektor fr verschiedene soundfiles die aus einem elan-file extrahiert werden;
 * die Namen werden generisch aus dem elan-filename erzeugt.
 * file-name: <typ>__<region>__<speaker>.eaf
 *       typ: (BIO|LIT|NRT|PRS|INT)
 *     region: (AT|DE|CH|ATKA|ATST|....)
 *    spkeaer: [MW]_[0-9]+
<+
 * filename: (BIO|LIT|NRT|PRS|INT)__([A-Z]{2}([A-Z]{2})?)__[MW]_[0-9].eaf
 */

public class OrderedTextSelector extends ElanSelector implements ActionListener, 
                                                       ListSelectionListener
{
    /**
     * creates an empty OrderedTextSelector
     */
    public OrderedTextSelector() {
    }

    /**
     * clears all items from this instance
     * currently unimplemented !!!!!
     */
    public void removeAllItems() {
	at.iem.tools.Logger.println("OrderedTextSelector:removeAllItems() not yet implemented");
    }

    private File[]  fileList;
    private String[]titleList;

    /**
     * create the internal data representation from a list of files
     */
    private void createData(ElanDatabase[]edb)
    {
        fileList=new File[edb.length];
        titleList=new String[edb.length];
        for (int i=0; i<edb.length; i++) {
            fileList[i]=edb[i].getEAFile();
            titleList[i]=edb[i].getTitle();
        }
    }

    private JRadioButton[] textSelection;

    /**
     * add several elan-files
     * @param s the elan-files to be added
     */
    public void addItem(ElanDatabase o[]) {
	if(null==o || 0==o.length)return;
        setLayout(new GridLayout(0,1));
	createData(o);

        textSelection=new JRadioButton[fileList.length];
	ButtonGroup group = new ButtonGroup();
        JPanel buttonpane = new JPanel();
	buttonpane.setLayout(new BoxLayout(buttonpane, ((true)?BoxLayout.Y_AXIS:BoxLayout.X_AXIS)));

        for(int i=0; i<textSelection.length; i++) {
            textSelection[i]=new JRadioButton(titleList[i]);
            if(i==0)
                textSelection[i].setSelected(true);

            textSelection[i].addActionListener(this);
            textSelection[i].setActionCommand(fileList[i].getName());
            group.add(textSelection[i]);
            buttonpane.add(textSelection[i]);
        }

	JPanel helppane = new JPanel();
	helppane.setLayout(new BorderLayout());
	helppane.setBorder(BorderFactory.createTitledBorder(
                                                                BorderFactory.createEtchedBorder(),
                                                                AdabaShared.getLabel("TitleTextButtonPane")));

	JScrollPane ScrollPane = new JScrollPane(buttonpane);
            
	ScrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
	ScrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
            
	helppane.add(ScrollPane);
	helppane.setPreferredSize(new Dimension(500,150));


	add(helppane);
    }

    /**
     * evaluate the multidimensional selection into the so selected file
     * @return the index of the selected filename
     */
    public int getSelectedIndex() {
        at.iem.tools.Logger.println("calculate selected index");
	int speaker=-1;
	for(int i=0; i<textSelection.length; i++) {
            if(textSelection[i].isSelected()) {
                return i;
            }
        }
        
        return -1;
    }



    private boolean ignoreAction=false;
    /**
     * whenever the user changes the selection by clicking on one of the RadioButtons this triggers all registered ActionListeners;
     * currently disabled !!!!!!!!!!
     * @param e the ActionEvent triggered by changing the radio-button selection
     */
    public void actionPerformed(ActionEvent e) {
	if(ignoreAction)return;
        boolean local_ignoreAction=ignoreAction;
	ignoreAction=true;
        //at.iem.tools.Logger.println("action performed by: "+e);
        if(false==false) {
            notifyActionListeners(0, "radio");
        }
	ignoreAction=local_ignoreAction;
    }

    /**
     * whenever the user changes the selection by clicking on one the "text"-style list this triggers all registered ActionListeners;
     * @param e the ListSelectionEvent triggered by changing the radio-button selection
     */
    public void valueChanged(ListSelectionEvent e) {
	if(ignoreAction)return;
	ignoreAction=true;
        if(e.getValueIsAdjusting()==false) {
            notifyActionListeners(1, "list");
        }
	ignoreAction=false;
    }


    private static final long serialVersionUID = 1L;
}
