// DisplayTranscription.java
// $Id: DisplayTranscription.java 1804 2007-06-26 19:58:00Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
// 
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

package at.iem.adaba.gui;
import at.iem.adaba.*;
import at.iem.adaba.database.*;
import at.iem.tools.sound.*;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.File;

class DisplayTranscription extends JPanel implements ActionListener,  at.iem.tools.gui.Refreshable {
    private int langID;
    private TranscriptionEintrag Entry;
    private AbstractPlayer PlayerM, PlayerW;

    private JButton PlayM, PlayW;
    private JTextField IPAfield;
    private JTextArea  commentField; // commentField not yet used...
    private JComboBox c_variantM, c_variantW;
    private File[] fileW, fileM;

    private Color defaultBgColor;

    private static boolean BUTABOVE=false;

    /**
     * create a DisplayTranscription with soundfile-players and play-buttons for 2 speakers (fe/male) and optional variants
     * @param titl the label of this gui-element
     * @param langId the language ID of this player-gui
     * @param num_variant the number of variants to be selectable at most
     * @param showIPA
     * @param mp3 use an MP3Player instead of a WAVPlayer (if we are not using a more general SoundPlayer that can play everything)
     */
    public DisplayTranscription(String titl, int langId, int num_variant, boolean showIPA, boolean mp3){
	langID=langId;
        PlayerM = new SoundPlayer();
        PlayerW = new SoundPlayer();

	if (num_variant<0)num_variant=0;

        c_variantM = new JComboBox();
        c_variantW = new JComboBox();

	createGUI(titl, showIPA);


        enableVariants(true, num_variant);
        enableVariants(false, num_variant);
    }

   /**
     * create a DisplayTranscription with soundfile-players and play-buttons for 2 speakers (fe/male) and optional variants
     * @param langId the language ID of this player-gui
     */
    public DisplayTranscription(int langId){	
	this("", langId, -1); }
    /**
     * create a DisplayTranscription with soundfile-players and play-buttons for 2 speakers (fe/male) and optional variants
     * @param langId the language ID of this player-gui
     * @param num_variant the number of variants to be selectable at most
     */
    public DisplayTranscription(int langId, int num_variant){
	this("", langId, num_variant, false); }
    /**
     * create a DisplayTranscription with soundfile-players and play-buttons for 2 speakers (fe/male) and optional variants
     * @param langId the language ID of this player-gui
     * @param showIPA add a text-field to display the transcription for this langId
     */
    public DisplayTranscription(int langId, boolean showIPA){
	this("", langId, -1, showIPA); }

    /**
     * create a DisplayTranscription with soundfile-players and play-buttons for 2 speakers (fe/male) and optional variants
     * @param langId the language ID of this player-gui
     * @param num_variant the number of variants to be selectable at most
     * @param showIPA add a text-field to display the transcription for this langId
     */
    public DisplayTranscription(int langId, int num_variant, boolean showIPA){
	this("", langId, num_variant, showIPA); }

    /**
     * create a DisplayTranscription with soundfile-players and play-buttons for 2 speakers (fe/male) and optional variants
     * @param titl the label of this gui-element
     * @param langId the language ID of this player-gui
     */
    public DisplayTranscription(String titl, int langId){	
	this(titl, langId, -1); }
    /**
     * create a DisplayTranscription with soundfile-players and play-buttons for 2 speakers (fe/male) and optional variants
     * @param titl the label of this gui-element
     * @param langId the language ID of this player-gui
     * @param num_variant the number of variants to be selectable at most
     */
    public DisplayTranscription(String titl, int langId, int num_variant){	
	this(titl, langId, num_variant, false); }
    /**
     * create a DisplayTranscription with soundfile-players and play-buttons for 2 speakers (fe/male) and optional variants
     * @param titl the label of this gui-element
     * @param langId the language ID of this player-gui
     * @param showIPA add a text-field to display the transcription for this langId
     */
    public DisplayTranscription(String titl, int langId, boolean showIPA){	
	this(titl, langId, -1, showIPA); }
    /**
     * create a DisplayTranscription with soundfile-players and play-buttons for 2 speakers (fe/male) and optional variants
     * @param titl the label of this gui-element
     * @param langId the language ID of this player-gui
     * @param num_variant the number of variants to be selectable at most
     * @param showIPA add a text-field to display the transcription for this langId
     */
    public DisplayTranscription(String titl, int langId, int num_variant, boolean showIPA){
	this(titl, langId, num_variant, showIPA, false); }

    /**
     * create a playbutton-group (for one sex only) with a variant-selector
     * @param PlayButton the actual "play" button
     * @param imgFile the filename of an image to be used as icon for the PlayButton
     * @param al the ActionListener to be notified by variant-selection
     */
    private JPanel createPlayButton(JPanel pan,
                                    JButton PlayButton,
                                    JComboBox cb, 
				    String imgFile){
	ImageIcon img;
	java.net.URL iconUrl=ClassLoader.getSystemResource(imgFile);
	if (iconUrl!=null)
	    img = new ImageIcon(iconUrl, "Play!");
	else
	    img = new ImageIcon(imgFile, "Play!");
	if (PlayButton==null)
	    PlayButton = new JButton(img);
	else PlayButton.setIcon(img);
	PlayButton.setEnabled(false);
	PlayButton.addActionListener(this);
	PlayButton.setAlignmentX(Component.CENTER_ALIGNMENT);
	pan.add(PlayButton);

        pan.add(Box.createRigidArea(new Dimension(10,1)));
        pan.add(cb);
 
        cb.addActionListener(this);

	return pan;
    }
    /**
     * sets up the GUI stuff for this instance
     * @param titl the label for this gui-element
     * @param haveText add a special text-field to display the phonetic transcription of the current Eintrag
     */
    private void createGUI(String titl, boolean haveText){
	setBorder(BorderFactory.createTitledBorder(
						   BorderFactory.createRaisedBevelBorder(),
						   titl,
						   javax.swing.border.TitledBorder.LEFT,
						   javax.swing.border.TitledBorder.TOP));

        setLayout(new BoxLayout(this,BoxLayout.PAGE_AXIS));

        textPane = new JPanel();
	textPane.setLayout(new BoxLayout(textPane, BoxLayout.PAGE_AXIS));
	IPAfield = new JTextField(20);
        /* turn on font anti-aliasing according to http://mindprod.com/jgloss/antialiasing.html */
        at.iem.tools.gui.AntiAliasing.enable(IPAfield);
	IPAfield.setEditable(false);
	IPAfield.setPreferredSize(new Dimension(400, 30));
	IPAfield.setFont(AdabaShared.AdabaFONT);
	commentField = new JTextArea(3, 10);
	commentField.setEditable(false);

	if (haveText){
	    textPane.add(IPAfield);
	    //	textPane.add(commentField);
	}

	buttonPane = new JPanel();
	PlayW=new JButton();
	PlayM=new JButton();

	buttonPane.setLayout(new BoxLayout(buttonPane,
					   (BUTABOVE)?BoxLayout.PAGE_AXIS:BoxLayout.LINE_AXIS));
	createPlayButton(buttonPane, PlayM, c_variantM, "resources/playbuttons/play_m.gif");
        buttonPane.add(Box.createRigidArea(PlayW.getPreferredSize()));
        createPlayButton(buttonPane, PlayW, c_variantW, "resources/playbuttons/play_f.gif");

        mainPane=new JPanel();
        mainPane.add(textPane);
	mainPane.add(buttonPane);

        add(Box.createVerticalGlue());
        add(mainPane);
        add(Box.createVerticalGlue());

        defaultBgColor=IPAfield.getBackground();

        setColor(false);
    }
    JPanel textPane, buttonPane, mainPane;
    /**
     * update all button-states and display the transcription (if requested);
     * this will disable the play-buttons for each sex where no soundfile exists and enable the variant-selectors if there are several soundfiles;
     */
    private void display(){
	long tsStart=System.currentTimeMillis();
	if (Entry!=null){
            fileW=Entry.getFile(langID, true, null);
            fileM=Entry.getFile(langID, false, null);

	    if (fileW!=null && fileW.length==0)fileW=null;
	    if (fileM!=null && fileM.length==0)fileM=null;
	} else {
	    fileW=null;
	    fileM=null;
	}
	PlayW.setEnabled(false);
        PlayW.setToolTipText(AdabaShared.getLabel("ToolTip_PlayDisabled"));
	PlayM.setEnabled(false);
        PlayM.setToolTipText(AdabaShared.getLabel("ToolTip_PlayDisabled"));

        enableVariants(true, 1);
        enableVariants(false, 1);

	IPAfield.setText("");
	commentField.setText("");

	if (Entry==null)return;

        String langid=AdabaShared.region[langID][0];
        at.iem.tools.linguistic.IPA ipa=Entry.getIPA(langid);
	String IPA = new String("");
        if(null!=ipa)
            IPA=at.iem.tools.linguistic.IPA.simplify(ipa);

	if(AdabaShared.fixMSFonts){
	    IPA=at.iem.tools.linguistic.IPA.fixUnicode2MS(IPA);
	}

	IPA=at.iem.tools.StringX.replace(IPA, "[[", "[");
    	IPA=at.iem.tools.StringX.replace(IPA, "]]", "]").trim();
	if (IPA.lastIndexOf("[")>IPA.lastIndexOf("]"))IPA+=" ]";

	IPAfield.setText(IPA);
        if("".equals(IPA))
            setColor(false);
        else
            setColor(true);

	long tsMid=System.currentTimeMillis();
	if (fileW!=null && fileW.length>0){
	    PlayW.setEnabled(true);
            PlayW.setToolTipText(AdabaShared.getLabel("ToolTip_PlayF"));
            enableVariants(true, fileW.length);
	}
	if (fileM!=null && fileM.length>0){
	    PlayM.setEnabled(true);
            PlayM.setToolTipText(AdabaShared.getLabel("ToolTip_PlayM"));
            enableVariants(false, fileM.length);
	}
	long tsStop=System.currentTimeMillis();
	at.iem.tools.Logger.println("display took "+(tsMid-tsStart)+"+"+(tsStop-tsMid)+"="+(tsStop-tsStart)+" ms");
    }
    private void enableVariants(boolean female, int num_variant) {
        if(female) {
            if(null==c_variantW)c_variantW=new JComboBox();
            enableVariants(c_variantW, num_variant);
        } else {
            if(null==c_variantM)c_variantM=new JComboBox();
            enableVariants(c_variantM, num_variant);
        }
    }

    private void enableVariants(JComboBox cb, int num_variant) {
        cb.removeActionListener(this);
        cb.removeAllItems();

        for(int i=0; i<num_variant; i++) {
            String s = new String(""+(char)(i+65));
            cb.addItem(s);
        }
        cb.setEditable(false);
        cb.setEnabled(num_variant>1);

        if(num_variant>1) {
            cb.setToolTipText(AdabaShared.getLabel("ToolTip_PlayAlternative"));
        } else {
            cb.setToolTipText(AdabaShared.getLabel("ToolTip_PlayAlternative0"));
        }

        cb.addActionListener(this);
    }

    /**
     * make the given TranscriptionEintrag the current one and update the GUI accordingly
     */
    public void set(TranscriptionEintrag entry){
	Entry=entry;
	display();
    }

    /**
     * this gets called, whenever the user changes the selected variant; playback of the selected variant will start immediately
     */
    class RadioListener implements ActionListener {
	int i=0;
	boolean isFemale;
	DisplayTranscription DT;
	RadioListener(DisplayTranscription dt, boolean iF){DT=dt; isFemale=iF; i=0;}
	public void actionPerformed(ActionEvent e) {
	    try { i=(Integer.decode(e.getActionCommand())).intValue(); }
	    catch (NumberFormatException n){i=-1; }
	    DT.play(i, isFemale);
	}
	public int getSelected(){ return i;}
    }
    /**
     * playback the selected variant of a given sex
     */
    private void play(int i, boolean isFemale){
	if (i<0)return;
	if (isFemale){
	    if (i>fileW.length-1)i=0;
	    PlayerW.load(fileW[i]);
	    PlayerW.play();

	} else {
	    if (i>fileM.length-1)i=0;
	    PlayerM.load(fileM[i]);
	    PlayerM.play();
	}
    }
    /*
     * this gets called when the "play" button is called and starts playback of the selected soundfile
     */
    public void actionPerformed(ActionEvent e) {
	Object source = e.getSource();
	if (source == PlayM || source==c_variantM){
            play(c_variantM.getSelectedIndex(), false);
	} else if (source == PlayW || source==c_variantW){
            play(c_variantW.getSelectedIndex(), true);
	} else {  at.iem.tools.Logger.println("an unknown ACTION occured!"); }
    }
    /**
     * update all transcriptions with the new font (or other gui-properties)
     */
    public void refresh() {
	IPAfield.setFont(AdabaShared.AdabaFONT);
    }

    public void setColor(boolean activate) {

        Color c=(activate)?AdabaConstants.TranscriptionBackground:null;
        mainPane.setBackground(c);
        buttonPane.setBackground(c);
        IPAfield.setBackground(c);

        if(activate)
            IPAfield.setBackground(Color.WHITE);

    }

    /* for serialization */
    private static final long serialVersionUID = 1L;
}
