// GUI.java
// $Id: GUI.java 1855 2007-07-03 08:41:44Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
// 
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

package at.iem.adaba.gui;
import  at.iem.adaba.database.*;
import  at.iem.adaba.*;

import javax.swing.*;
import java.awt.*;
import javax.swing.border.*;


/**
 * die META-Klasse fr das GUI
 * stellt HauptAbfrage und Suchabfrage bereit
 */
public class GUI extends JPanel implements AdabaSearchInterface,  at.iem.tools.gui.Refreshable , javax.swing.event.ChangeListener {
    HauptAbfrage haupt;
    SuchAbfrage suche[];
    TranscriptionDataBase dataBase;
    JTabbedPane SuchPane;

    /**
     * creates a new GUI, including a HauptAbfrage and several SuchAbfragen
     * @param db the transcription database to be searched
     */
    public GUI(TranscriptionDataBase db) {
	dataBase = db;

	setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	JPanel MainPane = new JPanel();
        JScrollPane ScrollPane = new JScrollPane(MainPane);
	SuchPane = new JTabbedPane();

	SuchPane.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
	MainPane.setLayout(new BoxLayout(MainPane, BoxLayout.Y_AXIS));

	haupt = new HauptAbfrage(dataBase);
	MainPane.add(haupt);

	MainPane.add(Box.createRigidArea(new Dimension(0, 0)));

	suche = new SuchAbfrage[AdabaShared.numSearchPanes];
	for(int i=0; i<suche.length; i++){
	    suche[i] = new SuchAbfrage(dataBase, (AdabaShowResultInterface)haupt, (AdabaSearchInterface)this, i);
	    SuchPane.addTab(AdabaShared.getLabel("SearchTabTitle")+(i+1), suche[i]);
            SuchPane.setToolTipTextAt(i, AdabaShared.getLabel("ToolTipMetaSearchTabs"));
	}
	JPanel SuchPane2 = new JPanel();// AdabaShared.getLabel("SearchTitle"));
        SuchPane2.setLayout(new BorderLayout());

	JPanel SuchPaneTitled = new JPanel();// AdabaShared.getLabel("SearchTitle"));
        SuchPaneTitled.setBorder(BorderFactory.createTitledBorder(
					BorderFactory.createRaisedBevelBorder(),
					AdabaShared.getLabel("SearchTitle"),
					TitledBorder.LEFT, TitledBorder.TOP,
					new Font("", Font.BOLD, 21)));
        SuchPaneTitled.setLayout(new BorderLayout());

        SuchPaneTitled.add(SuchPane);
	SuchPane2.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));

	SuchPane2.add(SuchPaneTitled);

	MainPane.add(SuchPane2);

        SuchPane.setForegroundAt(0, AdabaConstants.AdabaSelectionForeground);
        SuchPane.addChangeListener(this);
	//add(MainPane);
        add(ScrollPane);
    }

    /**
     * queries the (selected) result(s) of one of the sub SuchAbfragen
     * @param id the identifier (index) if the SuchAbfrage to be queried
     * @param all4none if no results are selected in the sub SuchAbfrage, <code>true</code> will return <em>all</em> rseults, wherase <code>false</code> will return none (<em>null</em>)
     * @return an array of search results, or <em>null</em> if an error occured, the set of results is empty or there were no selected results and <em>all4none</em> was <code>false</code>
     */
    public String[]getResultFromSubSearch(int id, boolean all4none){
	if(suche==null)return null;
	if(id<0||id>=suche.length)return null;
	return suche[id].getSelected(all4none);
    }

    /**
     * queries the selected result(s) of one of the sub SuchAbfragen
     * @param id the identifier (index) if the SuchAbfrage to be queried
     * @return an array of search results, or <em>null</em> if an error occured or the set of selected results is empty
     */
    public String[]getResultFromSubSearch(int id){
	return getResultFromSubSearch(id, false);
    }

    /**
     * queries the selected result(s) of the currently active sub SuchAbfrage
     * @return an array of search results, or <em>null</em> if an error occured or the set of selected results is empty
     */
    public String[]getResultFromSearch(){
	return getResultFromSubSearch(SuchPane.getSelectedIndex());
    }


    /**
     * update all texts and other gui-elements according to the (probably changed) properties
     */
    public void refresh() {
        haupt.refresh();
    }


    public void stateChanged(javax.swing.event.ChangeEvent e)
    {
        JTabbedPane tb=new JTabbedPane();
        if(e.getSource().getClass()==tb.getClass()) {
            tb=(JTabbedPane)(e.getSource());
            for(int i=0; i<tb.getTabCount(); i++) {
                tb.setForegroundAt(i, null);
            }
            tb.setForegroundAt(tb.getSelectedIndex(), AdabaConstants.AdabaSelectionForeground);
        }
    }


    /** 
     * this is used from outside to get the "current selection" of this search-interface:
     * just assume that the "haupt" searchpane is the main-one
     */
    public at.iem.tools.Selectable getResultSelector() {
        return haupt;
    }


    /* for serialization */
    private static final long serialVersionUID = 1L;
}
