// HauptAbfrage.java
// $Id: HauptAbfrage.java 1872 2007-07-11 13:22:02Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
// 
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

package at.iem.adaba.gui;
import  at.iem.adaba.database.*;
import  at.iem.adaba.*;
import  at.iem.tools.sound.AbstractPlayer;

import javax.swing.*;
import java.awt.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.awt.event.*;

import java.util.ArrayList;

/**
 * das GUI-Fenster fr die HauptAbfrage
 * hier wird in erster Linie die Information eines Wrterbucheintrags angezeit
 * mit Abspielmglichkeit, und einer einfachen startsWith()-suche
 */

public class HauptAbfrage extends JPanel implements ActionListener, AdabaShowResultInterface,  at.iem.tools.gui.Refreshable, at.iem.tools.Selectable {
    private JComboBox  SuchEintrag;
    private JTextField TextEintrag;

    private JTextField Lautschrift;
    private JLabel     gKategorie, eKategorie, Varianz, flagLabel[];
    private JTextArea  Kommentar;
    private JCheckBox  isVariant;

    private DisplayTranscription[] Transkription;

    private TranscriptionDataBase    dataBase;
    private TranscriptionEintrag[] results;
    private String[] result_keys;

    /** create a new HauptAbfrage for querying a database and displaying the result en detail
     * @param db the TranscriptionDataBase to be queried
     * @param Eintrag a default TranscriptionEintrag to be displayed
     */
    public HauptAbfrage(TranscriptionDataBase db, TranscriptionEintrag Eintrag) {
	JPanel      HauptPane = new JPanel();
	JPanel      PlayPane  = new JPanel();

	dataBase = db;
	setLayout(new BorderLayout());
	
	SuchEintrag = new JComboBox();
	TextEintrag = new JTextField(10);
	Lautschrift = new JTextField(10);
        /* turn on font anti-aliasing according to http://mindprod.com/jgloss/antialiasing.html */
        at.iem.tools.gui.AntiAliasing.enable(Lautschrift);

	gKategorie  = new JLabel("                 ");
	eKategorie  = new JLabel("                 ");
	Varianz     = new JLabel("                 ");

        flagLabel = new JLabel[2];
        flagLabel[0]       = new JLabel("                 ");
        flagLabel[1]       = new JLabel("                 ");
	
	
	Kommentar   = new JTextArea(3, 20);
	isVariant   = new JCheckBox(AdabaShared.getLabel("MainLabelVariant"), false);

	at.iem.tools.Logger.println("WARNING: HauptAbfrage only displaying the 1st 3 regions!!");
	//Transkription = new DisplayTranscription[AdabaShared.region.length];
	Transkription = new DisplayTranscription[3];

	initEntries(HauptPane, PlayPane);
	addItems(HauptPane, PlayPane);
	
	add(HauptPane, BorderLayout.CENTER);

	setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
	simpleSet(Eintrag);

	SuchEintrag.addActionListener(this);
    }

    /** create a new HauptAbfrage for querying a database and displaying the result en detail
     * @param db the TranscriptionDataBase to be queried
     */
    public HauptAbfrage(TranscriptionDataBase db) { /* for debugging only !!! ??? */
	this(db, null);
    }

    private void initEntries(JPanel HauptPane, JPanel PlayPane){
	HauptPane.setLayout(new BoxLayout(HauptPane, BoxLayout.X_AXIS));
	HauptPane.setBorder(BorderFactory.createTitledBorder(
					BorderFactory.createRaisedBevelBorder(),
					AdabaShared.getLabel("MainTitle"),
					TitledBorder.LEFT, TitledBorder.TOP,
					new Font("", Font.BOLD, 21)));
	PlayPane.setLayout(new BoxLayout(PlayPane, BoxLayout.Y_AXIS));
			
	SuchEintrag.setEditable(true);
	SuchEintrag.setPreferredSize(new Dimension(181, 20));

	SuchEintrag.setFont(new Font("", Font.BOLD, 16));
	TextEintrag.setPreferredSize(new Dimension(181, 20));
	TextEintrag.setFont(new Font("", Font.BOLD, 16));
	TextEintrag.setEditable(false);
	TextEintrag.setHorizontalAlignment(JTextField.CENTER);


	Lautschrift.setEditable(false);
	Lautschrift.setHorizontalAlignment(JTextField.CENTER);
	Lautschrift.setFont(AdabaShared.AdabaFONT.deriveFont(Font.PLAIN, 20.f));
	Lautschrift.setText("");

	Kommentar.setEditable(false);
	Kommentar.setLineWrap(true);
	Kommentar.setWrapStyleWord(true);
	Kommentar.setFont(new Font("", Font.BOLD, 16));
	/*
	Kommentar.setText("Das ist ein Kommentar, der genau ber achtzig Zeichen verfgt."+
			  "Nur zum Testen!!");
	*/
	Kommentar.setBorder(BorderFactory.createLoweredBevelBorder());
	isVariant.setEnabled(false);

	int i = Transkription.length;
	while(i-->0)
	    Transkription[i]=new DisplayTranscription(AdabaShared.region[i][1],
						      i,
						      3,
						      true,
						      false);
    }

    /**
     * a helper function for setting up all the GUI elements
     */
    private void addItems(JPanel HauptPane, JPanel PlayPane){
	JPanel textPane     = new JPanel(); // Suchwort + minimalInformation
	JPanel wordPane     = new JPanel();
	JPanel infoPane     = new JPanel();
        JPanel infoTypePane = new JPanel();
	JPanel commentPane  = new JPanel();


	textPane.setLayout(new BoxLayout(textPane, BoxLayout.Y_AXIS));
	wordPane.setLayout(new BoxLayout(wordPane, BoxLayout.Y_AXIS));

	infoPane.setLayout(new BoxLayout(infoPane, BoxLayout.Y_AXIS));

        infoTypePane.setLayout(new GridLayout(2,2));
	commentPane.setLayout(new BorderLayout());

	wordPane.setBorder(BorderFactory.createTitledBorder(
			BorderFactory.createCompoundBorder(
				     BorderFactory.createLoweredBevelBorder(),
				     BorderFactory.createEmptyBorder(0, 0, 10, 0)),
			AdabaShared.getLabel("MainLabelWord"),
			TitledBorder.LEFT, TitledBorder.TOP,
			new Font("", Font.BOLD, 19)));

	infoPane.setBorder(BorderFactory.createCompoundBorder(
				BorderFactory.createTitledBorder(AdabaShared.getLabel("MainLabelInformation")),
				     BorderFactory.createEmptyBorder(10, 10, 10, 10))); // N,W,S,E
	commentPane.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

	wordPane.add(SuchEintrag);
	wordPane.add(Box.createVerticalStrut(15));
	wordPane.add(TextEintrag);

	infoPane.add(Lautschrift);

        if(false) {
	infoPane.add(gKategorie);
	infoPane.add(eKategorie);
	//infoPane.add(Varianz);
        } else {
            infoTypePane.add(gKategorie);
            infoTypePane.add(flagLabel[0]);
            infoTypePane.add(eKategorie);
            infoTypePane.add(flagLabel[1]);

            infoPane.add(infoTypePane);
        }

	textPane.add(wordPane);
	textPane.add(infoPane);

	commentPane.add(Kommentar);

	int i = 0;
	while(i++<Transkription.length)PlayPane.add(Transkription[i-1]);
     

	HauptPane.add(textPane);
	//	HauptPane.add(commentPane);
	HauptPane.add(PlayPane);
    }

    /**
     * display a single TranscriptionEintrag with all details; the pulldown selector of results is not affected
     * @param Eintrag the TranscriptionEintrag to be displayed
     */
    private void simpleSet(TranscriptionEintrag Eintrag){
        for(int i=0; i<flagLabel.length; i++)
            flagLabel[i].setText("");

	if (Eintrag==null){
	    TextEintrag.setText("");
	    Lautschrift.setText("[ ~ ]");
	    gKategorie.setText("");
	    eKategorie.setText("");
	    Kommentar.setText("");
	    isVariant.setSelected(false);
	    Varianz.setText(AdabaShared.getLabel("MainLabelNotVariant"));
	} else {
	    TextEintrag.setText(Eintrag.getName());
	    Lautschrift.setText("[ "+
				at.iem.tools.linguistic.IPA.simplify(Eintrag.getPhonetic())+
				" ]");

            int gtyp=Eintrag.getGtyp();
            if(gtyp>0)
                gKategorie.setText(AdabaShared.G_TYPEN[gtyp]);
            else
                gKategorie.setText("");

            int etyp=Eintrag.getEtyp();
            if(etyp>0)
                eKategorie.setText(AdabaShared.E_TYPEN[etyp]);
            else
                eKategorie.setText("");

	    Kommentar.setText(Eintrag.getKommentar());
	    isVariant.setSelected(Eintrag.isVariant());
	    Varianz.setText(Eintrag.isVariant()?
			    AdabaShared.getLabel("MainLabelVariant"):
			    AdabaShared.getLabel("MainLabelNotVariant"));

            short[]flags=Eintrag.getFlag();
            for(int i=0; i<flags.length && i<flagLabel.length; i++) {
                if(flags[i]<AdabaShared.EntryFlag.length) {
                    flagLabel[i].setText(AdabaShared.EntryFlag[flags[i]]);
               }
            }
            
	}

	for(int i=0; i<Transkription.length; i++) {
	    long tsStart=System.currentTimeMillis();
	    Transkription[i].set(Eintrag);
	    long tsStop=System.currentTimeMillis();
	    at.iem.tools.Logger.println("setting transcription["+i+"] took "+(tsStop-tsStart)+" ms");
	}
    }

    /**
     * display a single TranscriptionEintrag with all details and set the pulldown selector to only contain this item
     * @param Eintrag the key to the TranscriptionEintrag to be displayed
     */
   public void show(String key){
        result_keys=null;
	SuchEintrag.removeAllItems();
	results=dataBase.lookup(key);
	if(results!=null&&results.length>0){
            result_keys=new String[1];
            result_keys[0]=key;
	    SuchEintrag.addItem(key);
	    simpleSet(results[0]);
	} else simpleSet((TranscriptionEintrag)null);
    }
    /**
     * set the pulldown selector to a set of results and display one of them with all details
     * @param Eintrag an array of keys to TranscriptionEintrags to be made selectable via the pulldown selector
     * @param index the index within the <em>Eintrag</em> to be displayed with all details
     */
    public void show(String[] keys, int index){
        at.iem.tools.Logger.println("HauptAbfrage.show("+index+") of "+((keys!=null)?keys.length:0)+" entries");
        result_keys=null;
	if (SuchEintrag.getItemCount()>0)SuchEintrag.removeAllItems();
	if (keys==null || keys.length==0)return;

        results=dataBase.lookup(keys);
        at.iem.tools.Logger.println("HauptAbfrage.show(): looked up...");
        if(results==null || results.length==0) {
            simpleSet((TranscriptionEintrag)null);
            return;
        }
        result_keys=keys;
	int i=results.length;
	while(i-->0){
            if(results[i]!=null) {
                SuchEintrag.insertItemAt(proxyString(results[i].getName()), 0);
            } else {
                SuchEintrag.insertItemAt(proxyString(null), 0);
            }
	}

	if(index<0||index>=results.length)index=0;
	simpleSet(results[index]);
    }

    /**
     * an alternative object type because multiple strings with the same name make problems!
     * @param item the string that the returned proxy-object will hold
     * @return a proxy String
     */
    private Object proxyString(final String item)  {
	return new Object() { public String toString() { return item; } };
    }

    /**
     * display one of the items currently selectable via the pulldown selector by specifying a pattern
     * @param pattern the pattern to be matched against all items in the pulldown menu
     * @return <code>true</code> if the display succeeded, <code>false</code> otherwise (e.g. because the pattern was not found)
     */
    private boolean activate(int index){
	if(results==null)
	    return false;
	if ((index<0)||(index>=results.length))
            return false;
	TranscriptionEintrag Eintrag=results[index];
        at.iem.tools.Logger.println("HauptAbfrage.activate("+index+"): "+Eintrag);
	simpleSet(Eintrag);
	return true;
    }

    /**
     * add all cached results to the pulldown selector and display the first of these
     */
    private void showResults(){
	SuchEintrag.removeAllItems();
	if (results==null || results.length==0)return;
	int i=results.length;

        while(i-->0){
            if (results[i]!=null){
                SuchEintrag.insertItemAt(proxyString(results[i].getName()),0);
            }
        }

	activate(0);
    }

    /**
     * do a simple search and display the results: 
     * if the pattern has at least 3 characters use it as "anlaut";
     * if the pattern has 2 characters look up all words that match these _exactly_ (to not get too many results);
     * if the pattern has less characters do nothing; 
     */
    private void doSearch(String pattern){
	if (pattern==null)return;
	TranscriptionEintrag[] gefunden = null;
	long tsStart=System.currentTimeMillis();
	at.iem.tools.Logger.println("starting search for "+pattern);
        String[]keys=null;
	switch (pattern.length()){
	case 0:
	    break;
	case 1:
	case 2:
	    keys=dataBase.isExactly(pattern);
	    break;
	default:
	    keys=dataBase.startsWith(pattern);
	    break;
	}
	long tsFound=System.currentTimeMillis();
	at.iem.tools.Logger.println("resultkeys after " + (tsFound-tsStart));

        if(keys==null)return;
        gefunden=dataBase.lookup(keys);

	long tsFound2=System.currentTimeMillis();
	at.iem.tools.Logger.println("results after " + (tsFound2-tsFound));

	if (gefunden==null)return;
        result_keys=keys;
	results = gefunden;
	showResults();
        selectPattern(pattern);

	long tsStop=System.currentTimeMillis();
	at.iem.tools.Logger.println("showresults after another " + (tsStop-tsFound));
    }

    /**
     * make the entry in the SuchEintrag-list that is closest to "pattern" the active entry
     */
    private void selectPattern(String pattern){
        if(null==SuchEintrag)return;
        pattern=pattern.toLowerCase();
            
        for(int i=0; i<SuchEintrag.getItemCount(); i++) {
            Object o=SuchEintrag.getItemAt(i);
            if(null==o)continue;
            String s=(String)o.toString();
            if(null==s)continue;
            if(s.toLowerCase().startsWith(pattern)) {
                activate(i);
                return;
            }
        }
    }

    /**
     * display the TranscriptionEintrag that has been selected via the pulldown menu
     * @param e the ActionEvent triggered by the selection change
     */
    public void actionPerformed(ActionEvent e){
	Object source = e.getSource();
	if (source == SuchEintrag){
            int index=SuchEintrag.getSelectedIndex();
            System.out.println("actionPerformed: "+index);
	    if(index>=-1) {
		if (!activate(index))//we failed to activate the pattern in the list (so it might not be there)
		    {
			//String pattern=(SuchEintrag.getSelectedItem()).toString();
			String pattern=((String)SuchEintrag.getSelectedItem());
			doSearch(pattern);
		    }
	    } else {
                System.out.println("pattern: "+((String)SuchEintrag.getSelectedItem()));
            }
	}
    }

    /**
     * update all text- and gui-elements to the refreshed settings (e.g. fonts)
     */
    public void refresh() {
	Lautschrift.setFont(AdabaShared.AdabaFONT.deriveFont(Font.PLAIN, 20.f));
        for (int i=0; i<Transkription.length; i++) {
            Transkription[i].refresh();
        }
    }

    /** 
     * get the index of the selected item
     */
    public int getSelectedIndex() {
        return SuchEintrag.getSelectedIndex();
    }

    /**
     * get the entire selection
     */
    public Object[] getSelection() {
        return result_keys;
    }


    /** 
     * set the index of the selected item
     */
    public void setSelectedIndex(int id) {
        activate(id);
    }

    /** 
     * select an item by name
     */
    public void setSelected(String id) {
        for(int i=0; i<result_keys.length; i++) {
            if(id.equals(result_keys[i])) {
                activate(i);
            }
        }

        /* oh it is not in the list of active keys;
         * should we search for it??
         */
        // show(id);
    }

    /* for serialization */
    private static final long serialVersionUID = 1L;
}
