// SuchAbfrage.java
// $Id: SuchAbfrage.java 1870 2007-07-11 13:13:38Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
// 
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

package at.iem.adaba.gui;
import  at.iem.adaba.database.*;
import  at.iem.adaba.*;
import  at.iem.tools.gui.VTextIcon;

import javax.swing.*;
import javax.swing.text.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import javax.swing.border.*;

/**
 * der GUI-Teil zum gezielten Suchen von Worteintrgen
 * kann einen gefundenen Eintrag an die Hauptabfrage weiterreichen.
 */
public class SuchAbfrage extends JPanel implements ActionListener, ListSelectionListener, javax.swing.event.ChangeListener,
                                                   AdabaResultInterface, AdabaShowResultInterface
{
    private DefaultListModel Ergebnis;
    private JList ErgebnisListe;
    private JButton Details;

    private TranscriptionDataBase dataBase;
    private AdabaShowResultInterface hauptAbfrage;
    private AdabaSearchInterface suchAbfrage;

    private String[] ergebnis;
    private int selectedResult;

    private DisplayTranscription[] Transkription;

    SuchOrthoAbfrage  soa;
    SuchPhonetAbfrage  spa;
    JLabel resultCount;
    /**
     * creates a new SuchAbfrage;
     * @param db the TranscriptionDataBase to be queried in search operations
     * @param HA the AdabaShowResultInterface we want to pass the information on search results to
     * @param SA the entire AdabaSearchInterface  (of which this class is only a part) to query for other results of other SubSearches
     * @param title a label for this GUI element
     */
    public SuchAbfrage(TranscriptionDataBase db,
		       AdabaShowResultInterface HA, 
		       AdabaSearchInterface SA,
		       String title) {

	//setLayout(new BorderLayout());
	setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
	setBorder(BorderFactory.createEmptyBorder(10,10,10,10));

	dataBase     = db;
	hauptAbfrage = HA;
	suchAbfrage  = SA;
        soa = new SuchOrthoAbfrage(this);
	spa = new SuchPhonetAbfrage(this);
	SuchMengeAbfrage  sma = new SuchMengeAbfrage(this);

	JPanel      MainPane = new JPanel();
	JTabbedPane MainTPane = new JTabbedPane(JTabbedPane.LEFT);

	MainPane.setLayout(new BorderLayout());
        

	VTextIcon textIcon = new VTextIcon(MainTPane, 
					   AdabaShared.getLabel("SearchTabOrthoTitle"),
					   VTextIcon.ROTATE_LEFT);

	MainTPane.addTab(null, textIcon, soa);

	textIcon = new VTextIcon(MainTPane, 
					    AdabaShared.getLabel("SearchTabPhoneticTitle"),
					    VTextIcon.ROTATE_LEFT);
	MainTPane.addTab(null, textIcon, spa);

	textIcon = new VTextIcon(MainTPane, 
					    AdabaShared.getLabel("SearchTabSetTitle"),
					    VTextIcon.ROTATE_LEFT);
	MainTPane.addTab(null, textIcon, sma);

        MainTPane.setToolTipTextAt(0, AdabaShared.getLabel("ToolTipSearchTabOrtho"));
        MainTPane.setToolTipTextAt(1, AdabaShared.getLabel("ToolTipSearchTabPhono"));
        MainTPane.setToolTipTextAt(2, AdabaShared.getLabel("ToolTipSearchTabBoole"));

        MainTPane.setForegroundAt(0, AdabaConstants.AdabaSelectionForeground);
        MainTPane.addChangeListener(this);

        /* getBoundsAt() gets the bounds of the small tabulator-tab */
        //System.out.println("bounds: "+MainTPane.getBoundsAt(0));

	MainPane.add(MainTPane);

	add(MainPane, BorderLayout.WEST);

	Ergebnis         = new DefaultListModel();
	ErgebnisListe    = new JList(Ergebnis);
        ErgebnisListe.setToolTipText(AdabaShared.getLabel("ToolTipShowResults"));
	Details          = new JButton(AdabaShared.getLabel("SearchButtonDetail"));
        Details.setToolTipText(AdabaShared.getLabel("ToolTipShowDetails"));

	ErgebnisListe.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);

        MouseListener mouseListener = new MouseAdapter() {
                public void mouseClicked(MouseEvent e) {
                    if (e.getClickCount() == 2) {
                        showDetails();
                    }
                }
            };
        ErgebnisListe.addMouseListener(mouseListener);

	JPanel ResultPane = new JPanel();
	JScrollPane ResultScrollPane = new JScrollPane(ErgebnisListe);
	ResultScrollPane.setPreferredSize(new Dimension(150,5));
	ResultPane.setBorder(BorderFactory.createTitledBorder(AdabaShared.getLabel("SearchLabelResult")));

	ResultPane.setLayout(new BorderLayout());
        resultCount=new JLabel(AdabaShared.getLabel("SearchLabelResultCount")+": 0");
        ResultPane.add(resultCount, BorderLayout.NORTH);
	ResultPane.add(ResultScrollPane, BorderLayout.CENTER);
	ResultPane.add(Details, BorderLayout.SOUTH);

	Details.addActionListener(this);
	ErgebnisListe.addListSelectionListener(this);

	add(ResultPane, BorderLayout.CENTER);

	at.iem.tools.Logger.println("WARNING: SuchAbfrage only displaying the 1st 3 regions!!");
	//Transkription = new DisplayTranscription[AdabaShared.region.length];
	Transkription = new DisplayTranscription[3];
	int i = Transkription.length;
	while(i-->0)
	    Transkription[i]=new DisplayTranscription(AdabaShared.region[i][1],
 						      i,
						      0,
						      false,
						      false);
	JPanel PlayPane = new JPanel();
	PlayPane.setLayout(new BoxLayout(PlayPane, BoxLayout.Y_AXIS));
	i = 0;
	while(i++<Transkription.length)PlayPane.add(Transkription[i-1]);
	add(PlayPane, BorderLayout.EAST);

    }


    /**
     * creates a new SuchAbfrage;
     * @param db the TranscriptionDataBase to be queried in search operations
     * @param HA the AdabaShowResultInterface we want to pass the information on search results to
     * @param SA the entire AdabaSearchInterface  (of which this class is only a part) to query for other results of other SubSearches
     * @param i an index for a generic (numbered) label for this GUI element
     */
    public SuchAbfrage(TranscriptionDataBase db, AdabaShowResultInterface HA, AdabaSearchInterface SA, int i) {
	this(db, HA, SA, AdabaShared.getLabel("SearchTitle")+" "+i);
    }

    /**
     * creates a new SuchAbfrage;
     * @param db the TranscriptionDataBase to be queried in search operations
     * @param HA the AdabaShowResultInterface we want to pass the information on search results to
     * @param SA the entire AdabaSearchInterface  (of which this class is only a part) to query for other results of other SubSearches
     */
    public SuchAbfrage(TranscriptionDataBase db, AdabaShowResultInterface HA, AdabaSearchInterface SA) {
	this(db, HA, SA, AdabaShared.getLabel("SearchTitle"));
    }

    void showDetails() {
        if (ErgebnisListe.isSelectionEmpty())return;
        int[]indices=ErgebnisListe.getSelectedIndices();
        if(indices==null || indices.length==0)return;
        
        String[]my_results=new String[indices.length];
        for(int i=0;i<my_results.length; i++){
            final int idx=indices[i];
            if(idx<ergebnis.length&&idx>-1)
                my_results[i]=ergebnis[idx];
        }
        hauptAbfrage.show(my_results, -1);
        //hauptAbfrage.show(ergebnis[selectedResult]);
    }

    /**
     * passes the (currently selected) search results to this instances AdabaShowResultInterface
     * @param event the ActionEvent triggered by user-interaction
     */
    public void actionPerformed(ActionEvent event) {
	Object source = event.getSource();
	if (source == Details){
            showDetails();
	} else if (source == pb){
            lock(false);            
        }
    }

    /**
     * get all selected entries;
     * @param all4none if <em>all4none</em> is true and nothing is selected, the complete list of entries is returned; otherwise <em>null</em> is returned
     * @return the selected entries
     */
    String[]getSelected(boolean all4none){
	if(ErgebnisListe.isSelectionEmpty()){
	    if(all4none)return ergebnis;
	    return null;
	}
	int[]indices=ErgebnisListe.getSelectedIndices();
	if(indices==null || indices.length==0)return null;
	String[]my_results=new String[indices.length];
	for(int i=0;i<my_results.length; i++){
	    final int idx=indices[i];
	    if(idx<ergebnis.length&&idx>-1)
		my_results[i]=ergebnis[idx];
	}
	return my_results;
    }
    /**
     * get all selected entries;
     * if nothing is selected <em>null</em> is returned
     * @return the selected entries
     */
    String[]getSelected(){
	return getSelected(false);
    }
    /**
     * get (selected) search result from another search carrying a special <em>index</em>
     * @param index the numeric label of the alternative search
     * @return the selected entries of the other search, or <em>null</em> if this is invalid
     */
    String[]getOtherResults(int index){
	if(suchAbfrage==null)return null;
	return suchAbfrage.getResultFromSubSearch(index, true);
    }

 

    /**
     * this callback function gets called by the active child search whenever it has a new set of results;
     * these results are then displayed in this instance's ErgebnisListe;
     * the first result (of several) is also passed on to the AdabaShowResultInterface
     * @param s_key an array of all results of a search
     */
    public void setErgebnis(String[]s_key){
        //at.iem.tools.Logger.println("setErgebnisString: "+s_key);

	int i = Transkription.length;
	while(i-->0)Transkription[i].set(null);	
	if (ErgebnisListe!=null)ErgebnisListe.clearSelection();
        resultCount.setText(AdabaShared.getLabel("SearchLabelResultCount")+": 0");
	if(Ergebnis==null)return;
	Ergebnis.clear();

	if(s_key==null)return;
	this.ergebnis=s_key;

	TranscriptionEintrag[] e= dataBase.lookup(s_key);
	if(e!=null) {
            resultCount.setText(AdabaShared.getLabel("SearchLabelResultCount")+": "+e.length);
	    for(i=0; i<e.length; i++){
		// use e[i].getKey() ti display full keys
		if(e[i]!=null)Ergebnis.addElement(e[i].getName());
	    }
        }
	
	hauptAbfrage.show(s_key, -1);
    }

    /**
     * get the active transcription database
     */
    TranscriptionDataBase getDB() {
	return dataBase;
    }

    /**
     * a helper function to create a regular-expression for several lautumgebungen
     * @param pattern the core regular-expression
     * @param anlaut whether the <em>pattern</em> can be at the beginning of a string
     * @param inlaut whether the <em>pattern</em> can be within a string
     * @param auslaut whether the <em>pattern</em> can be at the end of a string
     * @return a regular expression String including the information about the lautumgebung
     */
    static String lautumgebungPattern(String pattern, 
				      boolean anlaut, boolean inlaut, boolean auslaut){

	int mask = (anlaut?1:0)+(inlaut?2:0)+(auslaut?4:0);

	switch(mask){
	default: break;
	case 1: pattern=pattern+".*"; // anlaut
	    break;
	case 2: pattern=".+"+pattern+".+"; // inlaut
	    break;
	case 3: pattern=".*"+pattern+".+"; // anlaut||inlaut ( == !auslaut)
	    break;
	case 4: pattern=".*"+pattern; // auslaut
	    break;
	case 5: pattern="(.*"+pattern+")|("+pattern+".*)"; // anlaut || auslaut ( == !inlaut)
	    break;
	case 6: pattern=".+"+pattern+".*"; // inlaut || auslat ( == !anlaut)
	    break;
	case 7: pattern=".*"+pattern+".*"; // anlaut || inlaut || auslaut ( == irgendwo)
	    break;
	}
	return pattern;
    }

    public void show(String key) {
        String[]r=new String[1];
        r[0]=key;
        setErgebnis(r);
    }

    public void show(String[]keys, int index) {
        setErgebnis(keys);
    }

    public void setResult(String[]keys) {
        if(keys!=null) {
            at.iem.tools.Logger.println("SuchAbfrage.setResult("+keys.length+"):");
            for(int i=0; i<keys.length; i++) {
                at.iem.tools.Logger.println("\tkey["+i+"]: "+keys[i]);
            }
        } else {
            at.iem.tools.Logger.println("SuchAbfrage.setResult: "+keys);
        }
        setErgebnis(keys);
        lock(false);
    }


   /**
     * if the selection of the search results changes, the first selected and valid item (if any) is made available for playback
     * @param e the ListSelectionEvent triggered by a selection change
     */
    public void valueChanged(ListSelectionEvent e) {
	if(e.getValueIsAdjusting())return;
	if(ErgebnisListe==null || ergebnis==null)return;

	selectedResult=ErgebnisListe.getSelectedIndex();
	if (selectedResult<0 || selectedResult>=ergebnis.length)return;
	int i = Transkription.length;
	while(i-->0){
            TranscriptionEintrag[] res = dataBase.lookup(ergebnis[selectedResult]);
            if(null!=res)
                Transkription[i].set(res[0]);
        }
    }

    /**
     * someone changed the tab, so we have to change the tablabel-color
     * @param e the ChangeEvent triggered by the tabchange
     */
    public void stateChanged(javax.swing.event.ChangeEvent e)
    {
        // this doesn't seem to work with the vertical labels, though!
        JTabbedPane tb=new JTabbedPane();
        if(e.getSource().getClass()==tb.getClass()) {
            tb=(JTabbedPane)(e.getSource());
            for(int i=0; i<tb.getTabCount(); i++) {
                tb.setForegroundAt(i, null);
            }
            tb.setForegroundAt(tb.getSelectedIndex(), AdabaConstants.AdabaSelectionForeground);
        }
    }

    at.iem.tools.gui.ProgressBar pb;
    public void lock(boolean state) {
        soa.lock(state);
        spa.lock(state);
        if (state) {
            pb=new at.iem.tools.gui.ProgressBar(AdabaShared.getLabel("Searching"), AdabaShared.getLabel("StopSearching"));
            pb.addActionListener(this);
            this.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
            resultCount.setText(AdabaShared.getLabel("SearchLabelResultCount")+": ???"); 
        } else {
            this.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
            if(null!=pb) {
                pb.done();
            }
        }
    }


    /**
     * when the search failed for whatever reasons, 
     * this gets the exception
     */
    public void searchErrorCallback(Throwable ex) {
        lock(false);
        at.iem.tools.Logger.println("SuchAbfrage.searchErrorCallback:: "+ex);
        String text=AdabaShared.getLabel("ErrorRegexPattern");
        text=at.iem.tools.StringX.replace(text, "%s", ex.getCause().getMessage());
        AdabaMessages.showMessage(text);

    }

    /* for serialization */
    private static final long serialVersionUID = 1L;
}
