// SuchMengeAbfrage.java
// $Id: SuchMengeAbfrage.java 1767 2007-06-25 08:45:46Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
// 
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

package at.iem.adaba.gui;
import  at.iem.adaba.database.*;
import  at.iem.adaba.*;
import  at.iem.tools.SetOperator;

import javax.swing.*;
import javax.swing.text.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import javax.swing.border.*;

/**
 * der GUI-Teil fr Mengenoprationen auf Suchergebnisse
 * reicht gefundene Eintrge an "SuchAbfrage" zurck, die es dann darstellt
 */


public class SuchMengeAbfrage extends JPanel implements ActionListener {
    private JButton StartSuche;
    private JComboBox MengeA, MengeB, Operator;

    private SuchAbfrage Elter;

    /**
     * creates a new boolean operator on search results
     * @param elter the parent of this objects controls what to do with any results
     * @param num_SuchAbfrage the number of SearchPanes in the <code>parent</code>; this is the number of Sets that can be used in the boolean operations (exclusive the null set)
     */
    public SuchMengeAbfrage(SuchAbfrage elter, String title, int num_SuchAbfrage) {
	Elter = elter;
	String[] Mengen = new String[num_SuchAbfrage+1];
	Mengen[0]=AdabaShared.getLabel("EmptySet");
	for(int i=1; i<=num_SuchAbfrage; i++)
	    Mengen[i]=AdabaShared.getLabel("SearchTabTitle")+i;

	String[] OperatorCharacters = {"\u222a", "\u2229", "\u2216"};

	//new ImageIcon(existingIcon.getImage().getScaledInstance(width, height, Image.SCALE_DEFAULT));

	ImageIcon[] OperatorIcons = new ImageIcon[3];
	java.net.URL iconUrl=ClassLoader.getSystemResource("resources/setops/union.gif");
        if (iconUrl!=null) {
            OperatorIcons[0] = new ImageIcon(iconUrl, OperatorCharacters[0]);
            //OperatorIcons[0].setToolTipText(AdabaShared.getLabel("ToolTipSearchBoolUnion"));
	    
            iconUrl=ClassLoader.getSystemResource("resources/setops/intersect.gif");
            if (iconUrl!=null)
                OperatorIcons[1] = new ImageIcon(iconUrl, OperatorCharacters[1]);
            else
                OperatorIcons[1] = new ImageIcon("resources/setops/intersect.gif", OperatorCharacters[1]);
            //OperatorIcons[2].setToolTipText(AdabaShared.getLabel("ToolTipSearchBoolIntersect"));
            
            iconUrl=ClassLoader.getSystemResource("resources/setops/minus.gif");
            if (iconUrl!=null)
                OperatorIcons[2] = new ImageIcon(iconUrl, OperatorCharacters[2]);
            else
                OperatorIcons[2] = new ImageIcon("resources/setops/minus.gif", OperatorCharacters[2]);
            // OperatorIcons[2].setToolTipText(AdabaShared.getLabel("ToolTipSearchBoolMinus"));
            
            
            boolean rescale = true;
            if(rescale){
                int width =18;
                int height=18;
                for(int i=0; i<OperatorIcons.length; i++)
                    OperatorIcons[i]=new ImageIcon(OperatorIcons[i].getImage().getScaledInstance(width, height, Image.SCALE_SMOOTH));
            }
            
            
            Operator = new JComboBox(OperatorIcons);
	} else {
	    Operator       = new JComboBox(OperatorCharacters);
	}
        Operator.setToolTipText(AdabaShared.getLabel("ToolTipSearchBoolOperator"));


	setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

	MengeA         = new JComboBox(Mengen);
        MengeA.setToolTipText(AdabaShared.getLabel("ToolTipSearchBoolA"));
	MengeB         = new JComboBox(Mengen);
        MengeB.setToolTipText(AdabaShared.getLabel("ToolTipSearchBoolB"));

	MengeA.setSelectedIndex(0);
	MengeB.setSelectedIndex(0);
	Operator.setSelectedIndex(0);
	
	JPanel MainPane = new JPanel();
	MainPane.setLayout(new BoxLayout(MainPane, BoxLayout.Y_AXIS));

        if(title==null)
            title=AdabaShared.getLabel("SearchLabelEintrag");

        MainPane.setBorder(BorderFactory.createTitledBorder(
                                                            BorderFactory.createLoweredBevelBorder(),
                                                            title,
                                                            TitledBorder.LEFT, TitledBorder.TOP,
                                                            new Font("", Font.BOLD, 19)));

	JPanel SearchPane = new JPanel();

	SearchPane.setBackground((AdabaConstants.SearchBackground));

	SearchPane.add(MengeA);
	SearchPane.add(Operator);
	SearchPane.add(MengeB);

	SearchPane.setLayout(new FlowLayout());
        SearchPane.setAlignmentX(0.5f);

	StartSuche = new JButton(AdabaShared.getLabel("SearchButtonMerge"));
        StartSuche.setToolTipText(AdabaShared.getLabel("ToolTipSearchBoolDoit"));
	StartSuche.addActionListener(this);
        StartSuche.setAlignmentX(0.5f);

        MainPane.add(Box.createVerticalGlue());
        MainPane.add(Box.createRigidArea(new Dimension(10,10)));
	MainPane.add(SearchPane);
        MainPane.add(Box.createRigidArea(new Dimension(10,10)));
	MainPane.add(StartSuche);
        MainPane.add(Box.createRigidArea(new Dimension(10,10)));
        MainPane.add(Box.createVerticalGlue());

        add(MainPane);
    }
    /**
     * creates a new boolean operator on search results; for convenience this uses a-priori knowledge about the number of needed search-panes via AdabaShared
     * @param elter the parent of this objects controls what to do with any results
     */
    public SuchMengeAbfrage(SuchAbfrage elter, String title) {
	this(elter, title, AdabaShared.numSearchPanes);
    }
    /**
     * creates a new boolean operator on search results; for convenience this uses a-priori knowledge about the number of needed search-panes via AdabaShared
     * @param elter the parent of this objects controls what to do with any results
     */
    public SuchMengeAbfrage(SuchAbfrage elter) {
	this(elter,  AdabaShared.getLabel("SearchSetTitle"), AdabaShared.numSearchPanes);
    }

    /**
     * when pressing the "evaluate" button, the two selected sets are merged by applying the selected set operator;
     * the result is send to the parent
     * @param event notification, that the "evaluate" button has been pressed
     */
    public void actionPerformed(ActionEvent event) {
	Object source = event.getSource();
	if (source == StartSuche){
	    String[]setA=Elter.getOtherResults(MengeA.getSelectedIndex()-1);
	    String[]setB=Elter.getOtherResults(MengeB.getSelectedIndex()-1);

	    String[]result = new String[0];
	    int op=Operator.getSelectedIndex();
	    if (op==0){ // union
		result=SetOperator.union(setA, setB);
	    } else if (op==1) { // intersect
		result=SetOperator.intersect(setA, setB);
	    } else if (op==2) { // minus
		result=SetOperator.minus(setA, setB);
	    }
    	    Elter.setErgebnis(result);
	}
    }

    /* for serialization */
    private static final long serialVersionUID = 1L;
}
