// SuchOrthoAbfrage.java
// $Id: SuchOrthoAbfrage.java 1792 2007-06-26 13:52:26Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
// 
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

package at.iem.adaba.gui;
import  at.iem.adaba.database.*;
import  at.iem.adaba.*;

import javax.swing.*;
import javax.swing.text.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import javax.swing.border.*;

/**
 * der GUI-Teil zum gezielten Suchen von Worteintrgen nach orthographischen Kriterien
 * reicht gefundene Eintrge an "SuchAbfrage" zurck, die es dann darstellt
 */
public class SuchOrthoAbfrage extends JPanel implements ActionListener {
    private JButton StartSuche;
    private JTextField EingabeFeld;
    private JComboBox GrammTyp, EtymTyp, FlagTyp;
    private JCheckBox AnLautend, InLautend, AusLautend, AudioOnly;
    private JCheckBox fuzzy, nurVariant;

    private JRadioButton masterdict, audiodict;

    protected SuchAbfrage Elter;

    /**
     * creates a new SuchOrthoAbfrage for orthographic searches
     * @param elter the meta SuchAbfrage (which knows nothing about search algorithms) to which we pass our results
     * @param title a label for this GUI element
     */
    public SuchOrthoAbfrage(SuchAbfrage elter, String title) {
	Elter = elter;

	GrammTyp         = new JComboBox(AdabaShared.G_TYPEN);
        GrammTyp.setToolTipText(AdabaShared.getLabel("ToolTipSearchGrammTyp"));
	EtymTyp          = new JComboBox(AdabaShared.E_TYPEN);
        EtymTyp.setToolTipText(AdabaShared.getLabel("ToolTipSearchEtymTyp"));

        String[]flags=new String[AdabaShared.EntryFlag.length+1];
        flags[0]=new String(AdabaShared.getLabel("SearchLabelFlagUnknown"));
        for(int i=0; i<AdabaShared.EntryFlag.length; i++) {
            flags[i+1]=AdabaShared.EntryFlag[i];
        }
	FlagTyp          = new JComboBox(flags);
        FlagTyp.setToolTipText(AdabaShared.getLabel("ToolTipSearchFlagTyp"));

	AnLautend        = new JCheckBox(AdabaShared.getLabel("SearchButtonAnlaut"));
        AnLautend.setToolTipText(AdabaShared.getLabel("ToolTipSearchAnlaut"));
	InLautend        = new JCheckBox(AdabaShared.getLabel("SearchButtonInlaut"));
        InLautend.setToolTipText(AdabaShared.getLabel("ToolTipSearchInlaut"));
	AusLautend       = new JCheckBox(AdabaShared.getLabel("SearchButtonAuslaut"));
        AusLautend.setToolTipText(AdabaShared.getLabel("ToolTipSearchAuslaut"));

	fuzzy            = new JCheckBox(AdabaShared.getLabel("SearchButtonFuzzy"));
        fuzzy.setToolTipText(AdabaShared.getLabel("ToolTipSearchFuzzy"));
	nurVariant       = new JCheckBox(AdabaShared.getLabel("SearchButtonVariant"));
        nurVariant.setToolTipText(AdabaShared.getLabel("ToolTipSearchVariant"));

        ButtonGroup bg = new ButtonGroup();

	masterdict = new JRadioButton(AdabaShared.getLabel("SearchButtonMasterdict"));
        masterdict.setSelected(true);
        masterdict.setToolTipText(AdabaShared.getLabel("ToolTipSearchDBSelectionFull"));
        bg.add(masterdict);

	audiodict = new JRadioButton(AdabaShared.getLabel("SearchButtonAudiodict"));
        audiodict.setSelected(false);
        audiodict.setToolTipText(AdabaShared.getLabel("ToolTipSearchDBSelectionAudio"));
        bg.add(audiodict);

        masterdict.addActionListener(this);
        audiodict.addActionListener(this);

        GrammTyp.setSelectedIndex(0);
        EtymTyp.setSelectedIndex(0);
        FlagTyp.setSelectedIndex(0);
	
        AnLautend.setSelected(true);
        InLautend.setSelected(false);
        AusLautend.setSelected(false);
	    
        fuzzy.setSelected(false);
        nurVariant.setSelected(false);

        enableEtymGram(false);

        JPanel SearchPane = new JPanel();
        JPanel OptionPane = new JPanel();


        SearchPane.setBackground((AdabaConstants.SearchBackground));
        if(title==null)
            title=AdabaShared.getLabel("SearchLabelEintrag");

        SearchPane.setBorder(BorderFactory.createTitledBorder(
                                                              BorderFactory.createLoweredBevelBorder(),
                                                              title,
                                                              TitledBorder.LEFT, TitledBorder.TOP,
                                                              new Font("", Font.BOLD, 19)));
	    
        EingabeFeld      = new JTextField(15);
        setPatternToolTip(AdabaShared.getLabel("ToolTipSearchInput"));
        EingabeFeld.addActionListener(this);
        SearchPane.add(EingabeFeld);

        StartSuche       = new JButton(AdabaShared.getLabel("SearchButtonStart"));
        StartSuche.setToolTipText(AdabaShared.getLabel("ToolTipSearchDoit"));
        StartSuche.addActionListener(this);
        SearchPane.add(StartSuche);
        JPanel KategoriePane = new JPanel();
        JPanel LautUmgebungPane = new JPanel();
        JPanel AudioOnlyPane = new JPanel();
        JPanel CoreOptionPane = new JPanel();

        OptionPane.setBorder(BorderFactory.createTitledBorder(
                                                              AdabaShared.getLabel("SearchLabelOptions"))
                             );
        OptionPane.setBorder(BorderFactory.createTitledBorder(
                                                              BorderFactory.createLineBorder(Color.gray),
                                                              AdabaShared.getLabel("SearchLabelOptions"),
                                                              TitledBorder.LEFT, TitledBorder.TOP,
                                                              new Font("", Font.BOLD, 15))
                             );
        //OptionPane.setLayout(new BorderLayout());
        OptionPane.setLayout(new GridLayout(1,0));
        //OptionPane.setLayout(new BoxLayout(OptionPane, BoxLayout.X_AXIS));
   
        KategoriePane.setBorder(BorderFactory.createTitledBorder(
                                                                 AdabaShared.getLabel("SearchLabelCategory"))
                                );
        KategoriePane.setLayout(new BoxLayout(KategoriePane, BoxLayout.Y_AXIS));

        KategoriePane.add(new JLabel(AdabaShared.getLabel("SearchLabelGrammatic")));
        KategoriePane.add(GrammTyp);
        KategoriePane.add(new JLabel(AdabaShared.getLabel("SearchLabelEtymologic")));
        KategoriePane.add(EtymTyp);
        KategoriePane.add(new JLabel(AdabaShared.getLabel("SearchLabelFlags")));
        KategoriePane.add(FlagTyp);

        LautUmgebungPane.setBorder(BorderFactory.createTitledBorder(
                                                                    AdabaShared.getLabel("SearchLabelEnvironment"))
                                   );
        //LautUmgebungPane.setLayout(new BoxLayout(LautUmgebungPane, BoxLayout.Y_AXIS));
        LautUmgebungPane.setLayout(new GridLayout(0,1));
        LautUmgebungPane.add(AnLautend);
        LautUmgebungPane.add(InLautend);
        LautUmgebungPane.add(AusLautend);

        CoreOptionPane.setBorder(BorderFactory.createTitledBorder(
                                                                  AdabaShared.getLabel("SearchLabelDatabaseSelection"))
                                 );
        CoreOptionPane.setLayout(new GridLayout(0,1));
        CoreOptionPane.add(masterdict);
        CoreOptionPane.add(audiodict);

        //CoreOptionPane.add(nurVariant);
        //CoreOptionPane.add(fuzzy);

        
        JPanel CheckBoxPane = new JPanel();
        CheckBoxPane.setLayout(new BoxLayout(CheckBoxPane, BoxLayout.Y_AXIS));
        CheckBoxPane.add(LautUmgebungPane);
        CheckBoxPane.add(CoreOptionPane);

        OptionPane.add(KategoriePane);
        OptionPane.add(CheckBoxPane);
        //OptionPane.add(LautUmgebungPane, BorderLayout.CENTER);
        //OptionPane.add(CoreOptionPane, BorderLayout.SOUTH);

        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
        
        add(SearchPane);
        add(OptionPane);

    }

    /**
     * creates a new SuchOrthoAbfrage for orthographic searches with a generic label
     * @param elter the meta SuchAbfrage (which knows nothing about search algorithms) to which we pass our results
     */
    public SuchOrthoAbfrage(SuchAbfrage elter) {
	this(elter, AdabaShared.getLabel("SearchOrthoTitle"));
    }

    /**
     * start a search when the "doSearch" button has been pressed
     *  @param event the ActionEvent triggered by pressing the "doSearch" button
     */
    public void actionPerformed(ActionEvent event) {
	Object source = event.getSource();
	if (source == StartSuche||source == EingabeFeld)
            doSearch();
        else if (source == masterdict || source==audiodict)
            enableEtymGram(audiodict.isSelected());
    }

    /**
     * whenever we select the gesamtWoerterbuch, we disable the grammatic and etymologic criteria
     * because (in the current database!) the non-audio entries lack these tags!
     * @param <tt>false</tt> disables the criteria, but saves the current state and restores them the next time they are enabled
     */
    int old_gtype=0, old_etype=0;
    private void enableEtymGram(boolean state) {
        GrammTyp.setEnabled(state);
        EtymTyp.setEnabled(state);
        if(state) {
            GrammTyp.setSelectedIndex(old_gtype);
  	    EtymTyp.setSelectedIndex(old_etype);
        } else {
            old_gtype=GrammTyp.getSelectedIndex();
            old_etype=EtymTyp.getSelectedIndex();
            GrammTyp.setSelectedIndex(0);
  	    EtymTyp.setSelectedIndex(0);

        }
    }

    /**
     * create a search-pattern from the settings in the GUI
     * @return an EintragPattern that holds all relevant information to do a search
     */
    protected EintragPattern makePattern() {
	String pattern = getPattern();

        if("".equals(pattern)) {
            if (JOptionPane.CANCEL_OPTION == JOptionPane.showConfirmDialog(this,
                                                                           AdabaShared.getLabel("WarningSearchAll"),
                                                                           AdabaShared.getLabel("TitleSearchAll"),
                                                                           JOptionPane.CANCEL_OPTION,
                                                                           JOptionPane.WARNING_MESSAGE))
                return null;
            return null;
        }

        EintragPattern epattern=new EintragPattern(Elter.lautumgebungPattern(pattern,
                                                                             getLautumgebungAn(),
                                                                             getLautumgebungIn(),
                                                                             getLautumgebungAus()
                                                                             ));
        epattern.setGType(getGType());
        epattern.setEType(getEType());
        
        short flag=getNeedFlag();
        if(flag>=0) {
            epattern.setFlags(flag, true);
        }

        epattern.setVariant(getNeedVariant());
        epattern.setFiles(getNeedMedia());

        return epattern;        
    }

    /** 
     * search the transcription database according to the constraints given by the user;
     * the result is passed to the parent SuchAbfrage
     */
    void doSearch(){
        EintragPattern epattern=makePattern();

        if(epattern!=null) {
            Elter.lock(true);
            searcher=new ThreadedSearch(Elter.getDB());
            searcher.setPriority(Thread.MAX_PRIORITY);
            searcher.addResultListener(Elter, true);
            searcher.search(epattern);
        }
    }
    ThreadedSearch searcher;

    void lock(boolean dolock) {
        if(searcher!=null) {
            searcher.cancel();
        }
        StartSuche.setEnabled(!dolock);
    }


    protected void setPatternToolTip(String tt) {
        if(EingabeFeld!=null)
            EingabeFeld.setToolTipText(tt);
    }


    /**
     * get the search pattern the user entered
     */
    protected String getPattern() {
        return EingabeFeld.getText();
    }

    protected byte getEType() {
        int etyp_index=EtymTyp.getSelectedIndex();
        
        if(etyp_index>=0 && etyp_index<AdabaShared.E_TYPEN.length) {
            if("Fremdwort".equals(AdabaShared.E_TYPEN[etyp_index])) {
                for(int i=0; i<AdabaShared.E_TYPEN.length; i++) {
                    if("Lehnwort".equals(AdabaShared.E_TYPEN[i])) {
                        etyp_index=i;
                        continue;
                    }
                }
            }
        }
        return (byte)etyp_index;
    }

    protected byte getGType() {
        return (byte)(GrammTyp.getSelectedIndex());
    }

    protected short getNeedFlag() {
        return (short)(FlagTyp.getSelectedIndex()-1);
    }

    protected boolean getNeedVariant() {
        return nurVariant.isSelected();
    }

    protected boolean getNeedMedia() {
        return audiodict.isSelected();
    }

    protected boolean getLautumgebungAn(){
        return AnLautend.isSelected();
    }
    protected boolean getLautumgebungIn(){
        return InLautend.isSelected();
    }
    protected boolean getLautumgebungAus(){
        return AusLautend.isSelected();
    }
    /* for serialization */
    private static final long serialVersionUID = 1L;
}
