// SuchOrthoAbfrage.java
// $Id: SuchPhonetAbfrage.java 1776 2007-06-25 16:03:16Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
// 
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

package at.iem.adaba.gui;
import  at.iem.adaba.database.*;
import  at.iem.adaba.*;

import javax.swing.*;
import javax.swing.text.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import javax.swing.border.*;

/**
 * der GUI-Teil zum gezielten Suchen von Worteintrgen nach orthographischen Kriterien
 * reicht gefundene Eintrge an "SuchAbfrage" zurck, die es dann darstellt
 */


public class SuchPhonetAbfrage extends SuchOrthoAbfrage {

    /**
     * creates a new SuchPhonetAbfrage for phonetic searches
     * @param elter the meta SuchAbfrage (which knows nothing about search algorithms) to which we pass our results
     * @param title a label for this GUI element
     */
    public SuchPhonetAbfrage(SuchAbfrage elter, String title) {
        super(elter, title);
        setPatternToolTip(AdabaShared.getLabel("ToolTipSearchInputIPA"));
    }
    /**
     * creates a new SuchPhonetAbfrage for phonetic searches with a generic label
     * @param elter the meta SuchAbfrage (which knows nothing about search algorithms) to which we pass our results
     */
    public SuchPhonetAbfrage(SuchAbfrage elter) {
	this(elter, AdabaShared.getLabel("SearchPhoneticTitle"));
    }

    /**
     * replace user-defined shortcuts in a regular expression by more meaningful ones (e.g. "V" by "\p{IPAvowel}")
     * @param pattern the original regular expression with shortcuts
     */
    private String replaceIPAgroups(String pattern){
	String[][]rep1=AdabaShared.getIPAgroupReplacement();
	int count=0;
	for(int i=0; i<rep1.length; i++){if(!"".equals(rep1[i][1]))count++;}
	String[][]replac=new String[count][2];
	count=0;
	for(int i=0; i<rep1.length; i++){if(!"".equals(rep1[i][1])){
		replac[count][0]=rep1[i][1];
		replac[count][1]="\\p{"+rep1[i][0]+"}";
		count++;
	    }
	}
	
	return at.iem.tools.StringX.replace(pattern, replac);
    }

    /**
     * create a search-pattern from the settings in the GUI
     * @return an EintragPattern that holds all relevant information to do a search
     */
    protected EintragPattern makePattern() {
	String pattern = replaceIPAgroups(getPattern());
        if("".equals(pattern)) {
            if (JOptionPane.CANCEL_OPTION == JOptionPane.showConfirmDialog(this,
                                                                           AdabaShared.getLabel("WarningSearchAll"),
                                                                           AdabaShared.getLabel("TitleSearchAll"),
                                                                           JOptionPane.CANCEL_OPTION,
                                                                           JOptionPane.WARNING_MESSAGE))
                return null;
            return null;
        }

        EintragPattern epattern=new EintragPattern(new 
                                                   at.iem.tools.linguistic.IPA(Elter.lautumgebungPattern(pattern,
                                                                                                         getLautumgebungAn(),
                                                                                                         getLautumgebungIn(),
                                                                                                         getLautumgebungAus()
                                                                                                         )
                                                                               )
                                                   );
        
        epattern.setGType(getGType());
        epattern.setEType(getEType());
        
        short flag=getNeedFlag();
        if(flag>=0) {
            epattern.setFlags(flag, true);
        }

        epattern.setVariant(getNeedVariant());
        epattern.setFiles(getNeedMedia());
        
        return epattern;
    }

    /* for serialization */
    private static final long serialVersionUID = 1L;
}
