// TransGUI.java
// $Id: TransGUI.java 1804 2007-06-26 19:58:00Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
//
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

package at.iem.adaba.transcriber;
import at.iem.adaba.database.*;
import at.iem.adaba.*;
import at.iem.adaba.gui.SuchAbfrage;
import at.iem.tools.StringX;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;


/**
 * die Klasse fr das TransGUI
 */

public class TransGUI extends JPanel implements ActionListener,
						ListSelectionListener,
						AdabaShowResultInterface{

    private final boolean sync=false;


    TranscriptionDataBase dataBase;
    TransMenu     menue;

    private JFrame parent_frame;
    private JButton doneButton;
    private JButton fertigButton;
    private JButton angleichenButton;
    private JCheckBox varianz;
    private JCheckBox bearbeitet;
    private JComboBox SuchWort;
    private JTextField Wort;
    private TransPanel trans[];
    private JComboBox gTyp, eTyp, FlagMenu;
    private DefaultListModel Flags;
    private JList FlagListe;
    private JButton FlagDelete;
    private String selectedFlag;
    private JTextField SAMPAfield;
    private JTextField IPAfield;

    private JTextField Comment;

    private JLabel dirty;
    private ImageIcon dirtyIcon, cleanIcon;

    private TranscriptionEintrag currentEintrag;
    private String[] currentKeys;
    private int currentIndex;


    private boolean disableActions;

    private SuchAbfrage suche;
    private JFrame search_frame;

    public TransGUI(JFrame frame, TranscriptionDataBase db) throws Exception{
	this(db);
	parent_frame=frame;
    }

    /**
     * create a new TransGUI - the main window for the transcriber
     * @param db the TranscriptionDataBase we want to edit
     */
    public TransGUI(TranscriptionDataBase db) throws Exception{
        if(!sync) {
            at.iem.tools.Logger.println("WARNING: TransGUI: db-sync disabled!!");
        }


	/*   |--------------------------------------------|
	 *   |             MENU                           |
	 *   |============================================|
	 *   | WortPane                                   |
	 *   |--------------------------------------------|
	 *   | TypPane | FlagPane | TransPane | WortOKPane|
	 *   |============================================|
	 *   | TranscriptionPane                         ^|
	 *   | TranscriptionPane                         !|
	 *   |============================================|
	 *   | Comment                                    |
	 *   |--------------------------------------------|
	 */
	dataBase = db;

	JPanel MainPane = new JPanel();
        //JScrollPane ScrollPane = new JScrollPane(MainPane);

	setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	MainPane.setLayout(new BoxLayout(MainPane, BoxLayout.Y_AXIS));

	menue=new TransMenu(this);
        MainPane.add(menue);

	// holds: Suchwort, Wort
	JPanel WortPane         = new JPanel(new GridLayout(1,0));
	WortPane.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));

	JPanel SuchWortPane = new JPanel(new BorderLayout());
	SuchWortPane.setBorder(BorderFactory.createCompoundBorder(
					BorderFactory.createTitledBorder(AdabaShared.getLabel("LabelSearchWordPane")),
					BorderFactory.createEmptyBorder(5,5,5,5)));
	
	SuchWort=new JComboBox();
	SuchWort.setEditable(true);
	SuchWort.setToolTipText(AdabaShared.getLabel("ToolTip_Search"));
	SuchWort.addActionListener(this);
	SuchWortPane.add(SuchWort);
	WortPane.add(SuchWortPane);

	JPanel ThisWortPane = new JPanel(new BorderLayout());
	ThisWortPane.setBorder(BorderFactory.createCompoundBorder(
					BorderFactory.createTitledBorder(AdabaShared.getLabel("LabelCurrentWordPane")),
					BorderFactory.createEmptyBorder(5,5,5,5)));
	Wort = new JTextField(15);
	Wort.setEditable(false);
	Wort.setAlignmentX(Component.CENTER_ALIGNMENT);
	Wort.setToolTipText(AdabaShared.getLabel("ToolTip_EditableWord"));
	ThisWortPane.add(Wort);
	WortPane.add(ThisWortPane);
	MainPane.add(WortPane);

	MainPane.add(Box.createRigidArea(new Dimension(0,10)));

	/* EditPane */

	// holds: TypPane, FlagPane, TransPane, WortOKPane
	JPanel EditPane         = new JPanel();
	EditPane.setLayout(new BoxLayout(EditPane,BoxLayout.X_AXIS));
	EditPane.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));

	JPanel TypFlagPane = new JPanel();
	TypFlagPane.setLayout(new BoxLayout(TypFlagPane,BoxLayout.X_AXIS));
	TypFlagPane.setBorder(BorderFactory.createCompoundBorder(
					BorderFactory.createTitledBorder(AdabaShared.getLabel("LabelTypFlagPane")),
					BorderFactory.createEmptyBorder(5,5,5,5)));

	// holds: eTyp, gTyp
	JPanel TypPane = new JPanel(new GridLayout(0,1));
	eTyp = new JComboBox(AdabaShared.E_TYPEN);
	gTyp = new JComboBox(AdabaShared.G_TYPEN);
	eTyp.setToolTipText(AdabaShared.getLabel("ToolTip_etymologic"));
	gTyp.setToolTipText(AdabaShared.getLabel("ToolTip_grammatic"));
	gTyp.addActionListener(this);
	eTyp.addActionListener(this);

	TypPane.add(Box.createVerticalGlue());
	TypPane.add(eTyp);
	TypPane.add(Box.createVerticalGlue());
	TypPane.add(gTyp);
	TypPane.add(Box.createVerticalGlue());
	TypFlagPane.add(Box.createHorizontalGlue());
	TypFlagPane.add(TypPane);	
	TypFlagPane.add(Box.createRigidArea(new Dimension(5,0)));

	// holds: FlagScrollPane, FlagMenu, FlagDelete
	JPanel FlagPane         = new JPanel(new BorderLayout());

	Flags = new DefaultListModel();
	FlagListe = new JList(Flags);
	FlagListe.addListSelectionListener(this);
	FlagListe.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	JScrollPane FlagScrollPane = new JScrollPane(FlagListe);
	FlagListe.setToolTipText(AdabaShared.getLabel("ToolTip_Flags"));
	FlagMenu = new JComboBox(AdabaShared.EntryFlag);
	FlagMenu.addActionListener(this);
	FlagMenu.setToolTipText(AdabaShared.getLabel("ToolTip_FlagAdd"));
	FlagDelete = new JButton(AdabaShared.getLabel("ButtonDelete"));
	FlagDelete.addActionListener(this);
	FlagDelete.setEnabled(false);
	FlagDelete.setToolTipText(AdabaShared.getLabel("ToolTip_FlagDelete"));
	FlagPane.add(FlagScrollPane, BorderLayout.PAGE_START);

	FlagPane.add(FlagMenu, BorderLayout.LINE_START);
	TypFlagPane.add(Box.createHorizontalGlue(), BorderLayout.CENTER);
	FlagPane.add(FlagDelete, BorderLayout.LINE_END);
	TypFlagPane.add(FlagPane);


	// holds: SAMPA, IPA, varianz, angleichenButton
	JPanel TransPane        = new JPanel();
	TransPane.setLayout(new BoxLayout(TransPane, BoxLayout.Y_AXIS));


	TransPane.setBorder(BorderFactory.createCompoundBorder(
		BorderFactory.createTitledBorder(AdabaShared.getLabel("LabelMainTransPane")),
		BorderFactory.createEmptyBorder(5,5,5,5)));

	SAMPAfield = new JTextField();
	SAMPAfield.setEditable(false);
	SAMPAfield.addActionListener(this);
        at.iem.tools.gui.AntiAliasing.enable(SAMPAfield);

	IPAfield   = new JTextField();
	IPAfield.setEditable(false);
        at.iem.tools.gui.AntiAliasing.enable(IPAfield);

	SAMPAfield.setToolTipText(AdabaShared.getLabel("ToolTip_SAMPA"));
	IPAfield.setToolTipText(AdabaShared.getLabel("ToolTip_IPA"));

	TransPane.add(Box.createVerticalGlue());
	JPanel pane= new JPanel(new BorderLayout());
	//pane.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEmptyBorder(),"SAMPA"));
	pane.add(SAMPAfield);
	TransPane.add(pane);
	TransPane.add(Box.createVerticalGlue());

	pane= new JPanel(new BorderLayout());
	//pane.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEmptyBorder(),"IPA"));
	pane.add(Box.createVerticalGlue());

	pane.add(IPAfield);
	TransPane.add(pane);
	TransPane.add(Box.createVerticalGlue());
	TransPane.add(Box.createRigidArea(new Dimension(0,5)));

	JPanel TransSubPane     = new JPanel(new GridLayout(1,0));
	varianz=new JCheckBox(AdabaShared.getLabel("ButtonIsVariant"));
	varianz.setSelected(false);
	varianz.setEnabled(false);
	varianz.setAlignmentX(Component.CENTER_ALIGNMENT);
	varianz.setToolTipText(AdabaShared.getLabel("ToolTip_isVariant"));
	varianz.addActionListener(this);
	TransSubPane.add(varianz);

	angleichenButton = new JButton(AdabaShared.getLabel("ButtonApplyTranscription"));
	angleichenButton.setToolTipText(AdabaShared.getLabel("ToolTip_applyTranscription"));
	angleichenButton.setAlignmentX(Component.CENTER_ALIGNMENT);
	angleichenButton.addActionListener(this);
	angleichenButton.setEnabled(false);

	TransSubPane.add(angleichenButton);
	TransPane.add(Box.createVerticalGlue());

	TransPane.add(TransSubPane);
	TransPane.add(Box.createVerticalGlue());

	/*  WortOKPane */
	// holds: fertigButton, doneButton, bearbeitet, dirty
	JPanel WortOKPane       = new JPanel(new GridLayout(1,0));
	WortOKPane.setBorder(BorderFactory.createCompoundBorder(
			BorderFactory.createTitledBorder(
                                     AdabaShared.getLabel("LabelWortOKPane")),
			BorderFactory.createEmptyBorder(5,5,5,5)));
	JPanel WortOKButPane = new JPanel();
	WortOKButPane.setLayout(new BoxLayout(WortOKButPane, BoxLayout.Y_AXIS));
    	JPanel WortOKSubPane = new JPanel();
	WortOKSubPane.setLayout(new GridLayout(2, 1));
	WortOKSubPane.setBorder(BorderFactory.createTitledBorder(""));

	fertigButton = new JButton(AdabaShared.getLabel("ButtonFinished"));
	fertigButton.setToolTipText(AdabaShared.getLabel("ToolTip_editDone"));
	fertigButton.setAlignmentX(Component.CENTER_ALIGNMENT);
	fertigButton.addActionListener(this);
	fertigButton.setEnabled(false);
	WortOKButPane.add(Box.createVerticalGlue());
	WortOKButPane.add(fertigButton);
	WortOKButPane.add(Box.createVerticalGlue());

	bearbeitet=new JCheckBox(AdabaShared.getLabel("ButtonEdited"));
	bearbeitet.setSelected(false);
	bearbeitet.setEnabled(false);
	bearbeitet.setToolTipText(AdabaShared.getLabel("ToolTip_edited"));
	WortOKSubPane.add(bearbeitet);
	//	WortOKSubPane.add(Box.createHorizontalGlue());

	doneButton = new JButton(AdabaShared.getLabel("ButtonDone"));
	doneButton.setToolTipText(AdabaShared.getLabel("ToolTip_editOK"));
	doneButton.setAlignmentX(Component.CENTER_ALIGNMENT);
	doneButton.addActionListener(this);
	doneButton.setEnabled(false);
	//WortOKSubPane.add(Box.createHorizontalGlue(),BorderLayout.LINE_END);
	WortOKSubPane.add(doneButton);

	WortOKButPane.add(WortOKSubPane);
	WortOKButPane.add(Box.createVerticalGlue());

	WortOKPane.add(WortOKButPane);

	/* dirty - image */
	java.net.URL iconUrl=ClassLoader.getSystemResource("resources/editstates/clean.gif");
	if (iconUrl!=null)
	     cleanIcon = new ImageIcon(iconUrl);
	else cleanIcon = new ImageIcon("resources/editstates/clean.gif");

	iconUrl=ClassLoader.getSystemResource("resources/editstates/dirty.gif");
	if (iconUrl!=null)
	     dirtyIcon = new ImageIcon(iconUrl);
	else dirtyIcon = new ImageIcon("resources/editstates/dirty.gif");

	dirty = new JLabel(cleanIcon);
	dirty.setToolTipText(AdabaShared.getLabel("ToolTip_dirty"));
	WortOKPane.add(dirty);


	EditPane.add(TransPane);
	EditPane.add(Box.createHorizontalGlue());
	EditPane.add(TypFlagPane);
	EditPane.add(Box.createHorizontalGlue());
	EditPane.add(WortOKPane);

	MainPane.add(EditPane);

	MainPane.add(Box.createRigidArea(new Dimension(0,10)));

	// holds: a number of "TransPanel"s
	JPanel TranscriptionPane = new JPanel(new GridLayout(0,1));
	TranscriptionPane.setSize(0,800);

	at.iem.tools.Logger.println("TransGUI: only displaying 3 transcriptions!!!");
	//trans = new TransPanel[AdabaShared.region.length];
	trans = new TransPanel[3];
	for(int i=0; i<trans.length; i++){
	    trans[i]=new TransPanel(this, i);
	    TranscriptionPane.add(trans[i]);
	}

	JScrollPane TranscriptionScrollPane = new JScrollPane(TranscriptionPane);
	MainPane.add(TranscriptionScrollPane);

	MainPane.add(Box.createRigidArea(new Dimension(0,10)));

	// holds the comment

	Comment = new JTextField();
	Comment.setEditable(false);
	Comment.setAlignmentX(Component.CENTER_ALIGNMENT);
	Comment.setToolTipText(AdabaShared.getLabel("ToolTip_Comment"));
	Comment.addActionListener(this);

	setIPAFont(AdabaShared.AdabaFONT);
	MainPane.add(Comment);

	AdabaMessages.setParent(this);
	AdabaShared.dirty=false;
	disableActions=false;

	search_frame = new JFrame();
	suche = new SuchAbfrage(db, (AdabaShowResultInterface)this, null, "");
	search_frame.getContentPane().add(suche);
	search_frame.pack();
	search_frame.setTitle(AdabaShared.getLabel("TranscriberSearchTitle"));
	search_frame.setVisible(AdabaShared.TranscriberSearch);
        {
	  java.net.URL imageURL = ClassLoader.getSystemResource("resources/icons/search.gif");
          if (imageURL != null) {
            java.awt.Image image = java.awt.Toolkit.getDefaultToolkit().createImage(imageURL);
                if (image != null) {
                  search_frame.setIconImage(image);
                }
          }
        }

        //add(ScrollPane);
        add(MainPane);
    }
    /**
     * save the current entry into the database
     * @param fertig flag as "done"
     * @return the dirty flag
     */
    private boolean saveEntry(boolean fertig){
        at.iem.tools.Logger.println("saving entry: "+fertig);
	String display_name = Wort.getText();
	boolean l_dirty = AdabaShared.dirty;
	if (fertig)currentEintrag.setFertig(true);
	else currentEintrag.setFertig(bearbeitet.isSelected());

	for (int i=0; i<trans.length; i++){
	    currentEintrag.setSampa(AdabaShared.region[i][0], trans[i].get()[0], "[[", "]]");
	}
	currentEintrag.setVarianz(varianz.isSelected());
	currentEintrag.setEtyp(eTyp.getSelectedIndex());
	currentEintrag.setGtyp(gTyp.getSelectedIndex());
	currentEintrag.setKommentar(Comment.getText());
	currentEintrag.setSampa(SAMPAfield.getText());

	int index=SuchWort.getSelectedIndex();



	if (!display_name.equals(currentEintrag.getFullName())){
	    /* user renamed the Eintrag-key */

	    TranscriptionEintrag Ein = currentEintrag;
	    Object[] options = {AdabaShared.getLabel("ButtonReplace"),
				AdabaShared.getLabel("ButtonInsert")};
	    boolean replace=false;
	    String text=AdabaShared.getLabel("MessageEntryReplace");
	    text = StringX.replace(text, "%s0", currentEintrag.getName());
	    text = StringX.replace(text, "%s1", display_name);

	    replace=(JOptionPane.YES_OPTION == JOptionPane.showOptionDialog(
				      this,
				      text,
				      "",
				      JOptionPane.YES_NO_OPTION,
				      JOptionPane.WARNING_MESSAGE,
				      null, options, options[0]));
	
	    if (index==-1)index=0;
	    if (SuchWort.getItemCount()>0)
		SuchWort.removeItemAt(index);
	    SuchWort.insertItemAt(display_name, index);
	
            if(sync)dataBase.startAsync();

	    if (replace) {
		/* change the name of the entry (deleting the old one! */
		/* add the renamed entry to the db */
		/* delete the original entry */
		at.iem.tools.Logger.println("replacing old entry");
		dataBase.delete(currentKeys[index]);
	    } else{ // insert
		/* add a new entry derived from the given one */
		at.iem.tools.Logger.println("inserting new entry");
	    }
	    TranscriptionEintrag entry=new TranscriptionEintrag(display_name, Ein);
	    entry.setName(display_name);
	    entry.setVariant((byte)(-1));
	    currentKeys[index]=dataBase.add(entry);

            if(sync)dataBase.stopAsync();

	} else {
	    /* changing the current entry */
	    /* store the entry into the database (as a new entry) */
	    /* delete the old entry */
	    /* re-key the new entry to the old key */
	    at.iem.tools.Logger.println("storing entry in db");
	    dataBase.delete(currentKeys[index]);
	    String newkey=dataBase.add(currentEintrag);
	    if(newkey==null) {
		at.iem.tools.Logger.println("adding returned null-key! ("+currentKeys[index]+")");
	    } else {
		String dbkey=dataBase.rename(newkey, currentKeys[index]);
		at.iem.tools.Logger.println("renamed '"+currentKeys[index]+"' to '"+dbkey+"' ("+newkey+")");
		currentKeys[index]=dbkey;
	    }
	    //l_dirty=(!(null!=newkey && null==dataBase.rename(newkey, oldkey)));
	}

	l_dirty=(null==currentKeys[index]);
	currentEintrag=dataBase.lookup(currentKeys[index])[0];

	at.iem.tools.Logger.println("dirty="+l_dirty);

	AdabaShared.dirty=l_dirty;
	return l_dirty;
    }
    /**
     * user interaction
     */
    public void actionPerformed(ActionEvent e) {
	if (disableActions)return;
	disableActions=true;
	boolean l_dirty=AdabaShared.dirty;
	Object source = e.getSource();
	if (source==doneButton){
            at.iem.tools.Logger.println("done button!");
	    l_dirty=saveEntry(false);
	    showEntry();
	} else if (source==fertigButton){
            at.iem.tools.Logger.println("fertig button!");
	    l_dirty=saveEntry(true);
	    showEntry();
	} else if (source==SuchWort) {
	    if (AdabaShared.dirty){
		int value = JOptionPane.showConfirmDialog(this,
					      AdabaShared.getLabel("MessageEntryModified"),
					      "",
					      JOptionPane.YES_NO_CANCEL_OPTION,
					      JOptionPane.WARNING_MESSAGE);
		if (value!=JOptionPane.NO_OPTION){
		    if (value==JOptionPane.YES_OPTION)l_dirty=saveEntry(false);
		    else {
			disableActions=false;
			return;
		    }
		} else {
                    l_dirty=false;
                }
	    }
	    AdabaShared.dirty=l_dirty;


	    int idx=SuchWort.getSelectedIndex();
	    if (idx<0) {
		String pattern=((String)SuchWort.getSelectedItem());
		if (pattern==null){
		    disableActions=false;
		    return;
		}
		String[] gefunden=null;
		switch (pattern.length()){
		case 0:
		    break;
		case 1:
		case 2:
		    gefunden = dataBase.isExactly(pattern);
		    break;
		default:
		    gefunden = dataBase.startsWith(pattern);
		    break;
		}
		if (gefunden==null){
		    disableActions=false;
		    return;
		}
		show(gefunden);
	    } else {
		showEntry(idx);
	    }
	} else if (source==angleichenButton){
	    String laut=IPAfield.getText();
	    currentEintrag.setIPA((String)null, "[["+laut+"]]");
	    currentEintrag.setIPAfrom((String)null);
	    currentEintrag.setEtyp(eTyp.getSelectedIndex());
	    currentEintrag.setGtyp(gTyp.getSelectedIndex());
	    currentEintrag.setKommentar(Comment.getText());
	    AdabaShared.dirty=true;
	    showEntry();
	} else if (source==varianz || source==gTyp || source==eTyp || source==Comment) {
	    if (currentEintrag.isVariant()!=varianz.isSelected() ||
		currentEintrag.getEtyp()!=eTyp.getSelectedIndex() ||
		currentEintrag.getGtyp()!=gTyp.getSelectedIndex() ||
		!currentEintrag.getKommentar().trim().equals(Comment.getText().trim()))
		AdabaShared.dirty=true;
	} else if (source==FlagMenu) {
	    if(currentEintrag==null || FlagMenu==null){
		disableActions=false;
		return;
	    }
	    currentEintrag.setFlag(FlagMenu.getSelectedIndex(), true);
	    AdabaShared.dirty=true;
	    showFlags();
	} else if (source==FlagDelete) {
	    if (FlagListe.isSelectionEmpty() || selectedFlag==null|| "".equals(selectedFlag)){
		disableActions=false;
		return;
	    }
	    int index=-1;
	    for(int i=0; i<AdabaShared.EntryFlag.length; i++){
		if(selectedFlag.equals(AdabaShared.EntryFlag[i])){
		    index=i;
		    break;
		}
	    }
	    if(index!=-1){
		currentEintrag.setFlag(index, false);
		AdabaShared.dirty=true;
	    }
	    showFlags();
	} else if (source==SAMPAfield){
	    at.iem.tools.linguistic.IPA ipa = new at.iem.tools.linguistic.IPA();
	    ipa.setSAMPA(SAMPAfield.getText());

	    if(!ipa.equals(currentEintrag.getIPA((String)null))){
		currentEintrag.setIPA(ipa);
                ipa = currentEintrag.getIPA((String)null);
                if(ipa!=null) {

                    String ipastring=ipa.toString();

                    IPAfield.setText(ipastring);
                    SAMPAfield.setText(currentEintrag.getSampa((String)null));
                    AdabaShared.dirty=true;
                } else {
                    at.iem.tools.Logger.println("TransGUI:: ipa became 'null' all of a sudden!");
                }

	    }

	} else
	    at.iem.tools.Logger.println("TransGUI:: unknown event");
	setDirty(AdabaShared.dirty);
	disableActions=false;
    }

    public void valueChanged(ListSelectionEvent e) {
	FlagDelete.setEnabled(false);
	selectedFlag="";
	if (e.getValueIsAdjusting())return;
	selectedFlag=(String)FlagListe.getSelectedValue();
	if(selectedFlag!=null && !"".equals(selectedFlag))FlagDelete.setEnabled(true);
    }
    /**
     * update the display of the current entry
     */
    public void showEntry(){
	showEntry(currentIndex);
    }

    /**
     * update the display of the current entry
     */
    public void showEntry(int index){
	if(index<0 || index>currentKeys.length) {
	    index=-1;
	    currentEintrag=null;
	} else {
	    TranscriptionEintrag[]results=dataBase.lookup(currentKeys[index]);
	    currentEintrag=results[0];
	}
	currentIndex=index;

	if (currentEintrag==null){
	    disableActions=true;
	    bearbeitet.setSelected(false);
	    Wort.setText("");
	    Wort.setEditable(false);
	    angleichenButton.setEnabled(false);
	    varianz.setEnabled(false);
	    doneButton.setEnabled(false);
	    fertigButton.setEnabled(false);
	    bearbeitet.setEnabled(false);
	    SAMPAfield.setText("");
	    SAMPAfield.setEditable(false);
	    IPAfield.setText("");
	    Flags.clear();

	    for(int i=0; i<trans.length; i++)trans[i].set(null);
	    varianz.setSelected(false);
	    eTyp.setSelectedIndex(0);
	    gTyp.setSelectedIndex(0);
	    Comment.setText(null);
	    Comment.setEditable(false);
	    disableActions=false;
	    return;
	}

	disableActions=true;
	String eintrag=currentEintrag.getFullName();
	// doneButton
	bearbeitet.setSelected(currentEintrag.isEdited());
	Wort.setText(eintrag);
	Wort.setEditable(true);
	angleichenButton.setEnabled(true);
	varianz.setEnabled(true);
	doneButton.setEnabled(true);
	fertigButton.setEnabled(true);
	bearbeitet.setEnabled(true);

	SAMPAfield.setEditable(true);
        String phon=currentEintrag.getPhonetic();
        String sampaphon=null;
        if(phon!=null)sampaphon=at.iem.tools.linguistic.IPA.ipa2sampa(phon);
	SAMPAfield.setText(sampaphon);
	IPAfield.setText(phon);

	showFlags();

	for(int i=0; i<trans.length; i++)trans[i].set(currentEintrag);

	varianz.setSelected(currentEintrag.isVariant());

	int typ=currentEintrag.getEtyp();
	if(typ<0)typ=0;	if(typ>=eTyp.getItemCount())typ=0;
	eTyp.setSelectedIndex(typ);

	typ=currentEintrag.getGtyp();
	if(typ<0)typ=0;	if(typ>=gTyp.getItemCount())typ=0;
	gTyp.setSelectedIndex(typ);

	Comment.setText(currentEintrag.getKommentar());
	Comment.setEditable(true);
	disableActions=false;
    }

    /**
     * display via a BIG icon whether the current entry is"dirty"
     * @param dirt the current entry has been touched
     */
    public void setDirty(boolean dirt){
	if (dirt){
	    dirty.setIcon(dirtyIcon);
	} else {
	    dirty.setIcon(cleanIcon);
	}	
    }
    /**
     * display all "flags" of the current entry
     */
    private void showFlags(){
	if(currentEintrag==null)return;
	short[] buffer=currentEintrag.getFlag();
	if (buffer==null)return;
	Flags.clear();
	for(int i=0; i<buffer.length; i++){
	    if(buffer[i]>=0 && buffer[i]<AdabaShared.EntryFlag.length)
		Flags.addElement(AdabaShared.EntryFlag[buffer[i]]);
	}
    }

    /**
     * make the given TranscriptionEintrag the current one and display it
     * @param entry the TranscriptionEintrag to be displayed
     */
    public void show(String key){
	String[]dummyKeys=new String[1];
	dummyKeys[0]=new String(key);
	show(dummyKeys, 0);
    }
    /**
     * make the given TranscriptionEintrags selectable and display the first one
     * @param entry an array of new TranscriptionEintrags
     */
    public void show(String[] entry){
	show(entry, 0);
    }

    /**
     * make the given TranscriptionEintrags selectable and display one of them
     * @param keys an array of new TranscriptionEintrags
     * @param index the index of the to-be-current TranscriptionEintrag within the array
     * (if index is out of range, the 1st TranscriptionEintrags will be displayed)
     */
    public void show(String[] keys, int index){
	at.iem.tools.Logger.println("showing key#"+index+" of " + keys.length);
	if(null==SuchWort)return;
	if (SuchWort.getItemCount()>0)SuchWort.removeAllItems();
	if(keys==null || keys.length<1) {
	    currentKeys=null;
	    showEntry(-1);
	    return;
	}
	currentKeys=new String[keys.length];
	for(int i=0; i<currentKeys.length; i++) {
	    currentKeys[i]=keys[i];
	}

	if (keys.length==0){
	    showEntry(-1);
	    return;
	}

	TranscriptionEintrag[]results=dataBase.lookup(currentKeys);
	if(results==null || results.length<1) {
	    showEntry(-1);
	    return;
	}

	for(int i=0; i<results.length; i++) {
	    if(results[i]!=null) {
		at.iem.tools.Logger.println("showing:  "+results[i].getFullName());
		SuchWort.addItem(proxyString(results[i].getFullName()));
	    } else {
		SuchWort.addItem(proxyString(null));
	    }
	}
	at.iem.tools.Logger.println("current keys done__");
	if(index<0||index>=keys.length)index=0;
	showEntry(index);
    }


    /**
     * an alternative object type because multiple strings with the same name make problems!
     * @param item the string that the returned proxy-object will hold
     * @return a proxy String
     */
    private Object proxyString(final String item)  {
	return new Object() { public String toString() { return item; } };
    }

    /**
     * set the window-title of the transcriber window
     * @param titl the new window title
     */
    void setTitle(String titl){
	if(parent_frame==null)return;
	if (titl!=null&&titl.length()>0)
	    parent_frame.setTitle(AdabaShared.getLabel("TranscriberTitle")+
				  " "+AdabaVersion.getTranscriberVersion()+
				  "\t-\t"+titl);
	else parent_frame.setTitle(AdabaShared.getLabel("TranscriberTitle")+
				   " "+AdabaVersion.getTranscriberVersion());
    }
    /**
     * change the font for the IPA-entries
     */
    void  setIPAFont(Font newfont){
	if (newfont==null)return;
	AdabaShared.AdabaFONT = newfont;

	Wort.setFont(AdabaShared.AdabaFONT);
	Comment.setFont(AdabaShared.AdabaFONT);

	for (int i=0; i<trans.length; i++)trans[i].refreshFont();
	SAMPAfield.setFont(AdabaShared.AdabaFONT);
	IPAfield.setFont(AdabaShared.AdabaFONT);	

	if (parent_frame!=null)parent_frame.pack();

    }
    /**
     * refresh the display
     */
    void refresh(){
	boolean l_dirty=AdabaShared.dirty;
	setIPAFont(AdabaShared.AdabaFONT);
	FlagMenu.removeAllItems();
	for(int i=0; i<AdabaShared.EntryFlag.length; i++)FlagMenu.addItem(AdabaShared.EntryFlag[i]);
	AdabaShared.dirty=l_dirty;
	setDirty(l_dirty);
	search_frame.setVisible(AdabaShared.TranscriberSearch);	
    }
    /**
     * clear the current search results and set it to and display the given Eintrag
     * @param key the key to the new current TranscriptionEintrag
     */
    public void clearSet(String key){
	if(SuchWort.getItemCount()>0)SuchWort.removeAllItems();
	if (key!=null){
	    show(key);
	}
    }
    /**
     * clear the current search results and set it to the given Eintrag; display the given index
     * @param keys the keys to the new TranscriptionEintrags
     * @param index the index within the TranscriptionEintrag[] to be displayed
     */
    public void clearSet(String[] keys, int index){
	if(suche!=null)
            suche.setErgebnis(keys);
	show(keys);
	if(keys==null || index < 0 || index >= keys.length)return;
	show(keys[index]);	
    }
    /**
     * delete the current Eintrag from the TranscriptionDataBase
     */
    public void singleDelete(){
	TranscriptionEintrag eintrag=currentEintrag;
	String seintrag=eintrag.getFullName();

	String message=AdabaShared.getLabel("MessageEntryDelete");
	message=StringX.replace(message, "%s", seintrag);
	if (JOptionPane.YES_OPTION == JOptionPane.showConfirmDialog(
					      this,
					      message,
					      AdabaShared.getLabel("TitleEntryDelete"),
					      JOptionPane.YES_NO_OPTION,
					      JOptionPane.WARNING_MESSAGE)){
	    if(sync)dataBase.startAsync();
	    dataBase.delete(currentKeys[currentIndex]);
	    if(sync)dataBase.stopAsync();

	    SuchWort.removeItemAt(currentIndex);
	}
    }
    /**
     * delete all Eintrags which are currently in the pulldown-selector
     */
    public void selectedDelete(){
	String message=AdabaShared.getLabel("MessageEntrysDelete");
	String seintrag="";
	if(currentKeys==null || currentKeys.length<1)
	    return;
	TranscriptionEintrag[]selection=dataBase.lookup(currentKeys);
	for(int i=0;i<selection.length;i++) {
	    seintrag+=selection[i].getFullName()+"\n";
	}
	message=StringX.replace(message, "%s", seintrag.trim());
	
	if (JOptionPane.YES_OPTION == JOptionPane.showConfirmDialog(
					      this,
					      message,
					      AdabaShared.getLabel("TitleEntryDelete"),
					      JOptionPane.YES_NO_OPTION,
					      JOptionPane.WARNING_MESSAGE)){
            if(sync)dataBase.startAsync();
	    dataBase.delete(getSelectedEntries());
	    if(sync)dataBase.stopAsync();
	    SuchWort.removeAllItems();

	    showEntry(-1);
	}
    }
    /**
     * export all Eintrags which are currently in the pulldown-selector into a new TranscriptionDataBase
     */
    public void selectedExport(){
	JFileChooser fc = new JFileChooser(".");
	fc.setFileFilter(new at.iem.tools.gui.CustomFileFilter(AdabaShared.getLabel("FileATD"),
							       ".atd"));
	if (fc.showSaveDialog(this)==JFileChooser.APPROVE_OPTION){
	    java.io.File fil = fc.getSelectedFile();
	    if (!fil.getName().toLowerCase().endsWith(".atd")){
		String fname=fil.getName()+".atd";
		fil=new java.io.File(fil.getParent(), fname);
	    }
	    String warning = (fil.exists()?
			      StringX.replace(AdabaShared.getLabel("WarningOverwriteDB"),
					      "%s", fil.getName()):
			      StringX.replace(AdabaShared.getLabel("WarningSaveDB"),
					      "%s", fil.getName()));

	    if (JOptionPane.YES_OPTION == JOptionPane.showConfirmDialog(
									this,
									warning,
									AdabaShared.getLabel("TitleSaveAs"),
									JOptionPane.YES_NO_OPTION,
									JOptionPane.WARNING_MESSAGE)){
		dataBase.exportATD(fil, getSelectedEntries());
	    }
	}
    }
    /**
     * export all Eintrags which are currently in the pulldown-selector into a new SAMPA-File
     */
    public void selectedExportSAMPA(){
	JFileChooser fc = new JFileChooser(".");
	fc.setFileFilter(new at.iem.tools.gui.CustomFileFilter(AdabaShared.getLabel("FileSAMPA"),
							       ".txt"));
	if (fc.showSaveDialog(this)==JFileChooser.APPROVE_OPTION){
	    java.io.File fil = fc.getSelectedFile();
	    if (!fil.getName().toLowerCase().endsWith(".txt")){
		String fname=fil.getName()+".txt";
		fil=new java.io.File(fil.getParent(), fname);
	    }
	    String warning = (fil.exists()?
			      StringX.replace(AdabaShared.getLabel("WarningOverwriteFile"),
					      "%s", fil.getName()):
			      StringX.replace(AdabaShared.getLabel("WarningSaveDB"),
					      "%s", fil.getName()));

	    if (JOptionPane.YES_OPTION == JOptionPane.showConfirmDialog(
									this,
									warning,
									AdabaShared.getLabel("TitleSaveAs"),
									JOptionPane.YES_NO_OPTION,
									JOptionPane.WARNING_MESSAGE)){
		Object[] possibilities = new Object[AdabaShared.region.length+1];
		possibilities[0]=AdabaShared.getLabel("LabelStandardTranscription");
		for (int i=1; i<possibilities.length; i++)possibilities[i]=AdabaShared.region[i-1][1];
		String s = (String)JOptionPane.showInputDialog(
					this,
					AdabaShared.getLabel("MessageExportSampaRegion"),
					AdabaShared.getLabel("MessageSampaRegionTitle"),
					JOptionPane.PLAIN_MESSAGE,
					null,
					possibilities,
					AdabaShared.getLabel("LabelStandardTranscription"));
		
		//If a string was returned, say so.
		if(s==null || "".equals(s))s="";
		String langid=null;
		for (int i=0; i<AdabaShared.region.length; i++)
		    if(s.equals(AdabaShared.region[i][1])){
			langid=AdabaShared.region[i][0];
			break;
		    }
		dataBase.exportSAMPA(fil, getSelectedEntries(), langid);
	    }
	}
    }
    /**
     * query the Eintrags currently in the pulldown-selector
     * @return a String[] of the keys to the selected Eintrags
     */
    String[] getSelectedEntries(){
	if(SuchWort==null || currentKeys==null || currentKeys.length<1)return null;
	return currentKeys;
    }

    /* for serialization */
    private static final long serialVersionUID = 1L;
}
