// TransPanel.java
// $Id: TransPanel.java 1967 2008-06-09 13:59:28Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
// 
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

// 02.10.2002: dirty-flag
// 05.01.2003: Generalisierung des LandIDs
//             getFile/Path so halbwegs im TranscriptionEintrag

package at.iem.adaba.transcriber;
import at.iem.adaba.database.*;
import at.iem.adaba.*;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import at.iem.tools.StringX;
import at.iem.tools.FileX;

import at.iem.tools.gui.*;


import java.io.*;
/**
 * die Klasse fr das TransPanel;
 * beinhaltet alle fr eine Region relevanten Daten (transcription, play-section, external-application section,...)
 */
public class TransPanel extends JPanel implements ActionListener{
    JTextField SAMPAfield;
    JTextField IPAfield;
    JButton    FileSelector1, FileSelector2;
    JButton    PlayButton1, PlayButton2;
    File       PlayFile1, PlayFile2;
    //    private String langID1, langID2;
    private int langID;
    private String WortName;
    int    alternateFiles1, alternateFiles2;
    JLabel alt1, alt2;

    JButton EditButton1, EditButton2; /* to start PRAAT,... */
    JComboBox EditCombo1, EditCombo2; /* to start PRAAT,... */

    private TransGUI parent;

    private TranscriptionEintrag Eintrag;

    private at.iem.tools.sound.AbstractPlayer SoundPlayer;

    /**
     * creates a new TransPanel of a certain language and within a TransGUI
     * @param Parent the TransGUI where this instance lives in
     * @param langId the language id of this TransPanel
     */
    public TransPanel(TransGUI Parent,  int langId){
	this(Parent, AdabaShared.region[langId][1], langId);
    }
   
    /**
     * creates a new TransPanel of a certain language and within a TransGUI
     * @param Parent the TransGUI where this instance lives in
     * @param title the title for this gui-element
     * @param langId the language id of this TransPanel
     */
    public TransPanel(TransGUI Parent, String title, int langId)//String langId1, String langId2)
    {
	parent = Parent;
	Eintrag=null;
	SoundPlayer = new at.iem.tools.sound.SoundPlayer();

	JPanel     IPApane, SAMPApane, LautPane, PLAY1pane, PLAY2pane;
	ImageIcon playicon_f, playicon_m;

	java.net.URL iconUrl=ClassLoader.getSystemResource("resources/playbuttons/play_f.gif");
	if (iconUrl!=null)
	    playicon_f = new ImageIcon(iconUrl, AdabaShared.getLabel("ButtonPlay"));
	else
	    playicon_f=new ImageIcon("resources/playbuttons/play_f.gif", AdabaShared.getLabel("ButtonPlay"));

	iconUrl=ClassLoader.getSystemResource("resources/playbuttons/play_m.gif");
	if (iconUrl!=null)
	    playicon_m = new ImageIcon(iconUrl, AdabaShared.getLabel("ButtonPlay"));
	else
	    playicon_m=new ImageIcon("resources/playbuttons/play_m.gif", AdabaShared.getLabel("ButtonPlay"));

	WortName = null;

	SAMPApane = new JPanel();
	SAMPApane.setLayout(new BorderLayout());
	SAMPApane.setBorder(BorderFactory.createTitledBorder(
				     BorderFactory.createEmptyBorder(),
				     AdabaShared.getLabel("TitleSAMPApane")));
	IPApane = new JPanel();
	IPApane.setLayout(new BorderLayout());
	IPApane.setBorder(BorderFactory.createTitledBorder(
				   BorderFactory.createEmptyBorder(),
				   AdabaShared.getLabel("TitleIPApane")));
	PLAY1pane = new JPanel();
	PLAY1pane.setBorder(BorderFactory.createEmptyBorder());
	//PLAY1pane.setLayout(new BoxLayout(PLAY1pane, BoxLayout.X_AXIS));

	PLAY2pane = new JPanel();
	PLAY2pane.setBorder(BorderFactory.createEmptyBorder());
	//PLAY2pane.setLayout(new BoxLayout(PLAY2pane, BoxLayout.X_AXIS));

	JPanel PLAYpane = new JPanel();
	PLAYpane.setBorder(BorderFactory.createEmptyBorder());
	PLAYpane.setLayout(new BoxLayout(PLAYpane, BoxLayout.Y_AXIS));

	SAMPAfield=new JTextField(); //20
	SAMPApane.add(SAMPAfield);
	IPAfield=new JTextField(); //20
	IPAfield.setEditable(false);
        at.iem.tools.gui.AntiAliasing.enable(SAMPAfield);
        at.iem.tools.gui.AntiAliasing.enable(IPAfield);

	refreshFont();

	SAMPAfield.setToolTipText(AdabaShared.getLabel("ToolTip_SAMPAregional"));
	IPAfield.setToolTipText(AdabaShared.getLabel("ToolTip_IPAregional"));

	IPApane.add(IPAfield, BorderLayout.CENTER);
	IPApane.add(Box.createRigidArea(new Dimension(0,40)), BorderLayout.EAST);
	FileSelector1=new JButton(AdabaShared.getLabel("LabelNoFile"));
	FileSelector2=new JButton(AdabaShared.getLabel("LabelNoFile"));
	PlayFile1=null;
	PlayFile2=null;
	alternateFiles1=alternateFiles1=0;

	PlayButton1=new JButton(playicon_m);
	PlayButton2=new JButton(playicon_f);

	alt1 = new JLabel();
	alt2 = new JLabel();

	PLAY1pane.add(PlayButton1);
	PLAY1pane.add(Box.createRigidArea(new Dimension(10,0)));
	PLAY1pane.add(alt1);
	PLAY1pane.add(FileSelector1);

	PLAY2pane.add(PlayButton2);
	PLAY2pane.add(Box.createRigidArea(new Dimension(10,0)));
	PLAY2pane.add(alt2);
	PLAY2pane.add(FileSelector2);

	EditButton1=new JButton();
	EditButton2=new JButton();

        EditCombo1=new JComboBox();
        EditCombo2=new JComboBox();

	EditButton1.setToolTipText(AdabaShared.getLabel("ToolTip_externalScript"));
	EditButton2.setToolTipText(AdabaShared.getLabel("ToolTip_externalScript"));
	EditCombo1.setToolTipText(AdabaShared.getLabel("ToolTip_externalScript"));
	EditCombo2.setToolTipText(AdabaShared.getLabel("ToolTip_externalScript"));

	if (AdabaShared.SoundEditors!=null) {
	    switch(AdabaShared.SoundEditors.length){
	    case 0: break;
	    case 1:
		EditButton1=new JButton(AdabaShared.SoundEditors[0][0]);
		EditButton2=new JButton(AdabaShared.SoundEditors[0][0]);
		PLAY1pane.add(EditButton1);
		PLAY2pane.add(EditButton2);
		break;
	    default:
		for(int i=0; i<AdabaShared.SoundEditors.length; i++){
                    //at.iem.tools.Logger.println("adding: "+AdabaShared.SoundEditors[i][0]);
		    EditCombo1.addItem(AdabaShared.SoundEditors[i][0]);
		    EditCombo2.addItem(AdabaShared.SoundEditors[i][0]);
		}
                {
                    JPanel epane=new JPanel();
                    epane.add(EditCombo1);
                    PLAY1pane.add(epane);
                    epane=new JPanel();
                    epane.add(EditCombo2);
                    PLAY2pane.add(epane);

                }
                break;
	    }
	}

	PlayButton1.setAlignmentX(Component.CENTER_ALIGNMENT);
	PlayButton2.setAlignmentX(Component.CENTER_ALIGNMENT);
	FileSelector1.setAlignmentX(Component.CENTER_ALIGNMENT);
	FileSelector2.setAlignmentX(Component.CENTER_ALIGNMENT);

	EditButton1.setAlignmentX(Component.CENTER_ALIGNMENT);
	EditButton2.setAlignmentX(Component.CENTER_ALIGNMENT);
	EditCombo1.setAlignmentX(Component.CENTER_ALIGNMENT);
	EditCombo2.setAlignmentX(Component.CENTER_ALIGNMENT);


	/*
	langID1 = langId1;
	langID2 = langId2;
	*/
	langID = langId;

	PlayButton1.setToolTipText(AdabaShared.getLabel("ToolTip_PlayM"));
	PlayButton2.setToolTipText(AdabaShared.getLabel("ToolTip_PlayF"));
	alt1.setToolTipText(AdabaShared.getLabel("ToolTip_alternateNumber"));
	alt2.setToolTipText(AdabaShared.getLabel("ToolTip_alternateNumber"));

	FileSelector1.setToolTipText(AdabaShared.getLabel("ToolTip_FileM"));
	FileSelector2.setToolTipText(AdabaShared.getLabel("ToolTip_FileF"));

	setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
	setBorder(BorderFactory.createTitledBorder(title));

	LautPane = new JPanel();
	LautPane.setLayout(new BoxLayout(LautPane, BoxLayout.Y_AXIS));
	LautPane.add(SAMPApane);
	LautPane.add(IPApane);
	add(LautPane);

	PLAYpane.add(PLAY1pane);
	PLAYpane.add(Box.createRigidArea(new Dimension(0,20)));
	PLAYpane.add(PLAY2pane);

	add(Box.createRigidArea(new Dimension(50,0)));
	add(PLAYpane);
	add(Box.createRigidArea(new Dimension(50,0)));

	SAMPAfield.addActionListener(this);
	FileSelector1.addActionListener(this);
	FileSelector2.addActionListener(this);

	PlayButton1.addActionListener(this);
	PlayButton2.addActionListener(this);

	EditButton1.addActionListener(this);
	EditButton2.addActionListener(this);
	EditCombo1.addActionListener(this);
	EditCombo2.addActionListener(this);

	SAMPAfield.setEditable(false);
	PlayButton1.setEnabled(false);
	PlayButton2.setEnabled(false);
	EditButton1.setEnabled(false);
	EditButton2.setEnabled(false);
	EditCombo1.setEnabled(false);
	EditCombo2.setEnabled(false);

	FileSelector1.setEnabled(false);
 	FileSelector2.setEnabled(false);
    }
    /**
     * simplifies IPA strings by replacing some double-characters by 2 separate characters
     * @param ipa the IPA-string to be simplified
     * @return a simplified version of the IPA-string
     */
    private String simpleIpa(at.iem.tools.linguistic.IPA ipa){
	// some Unicode-fonts do not display following characters
	if (ipa==null)return null;
	String ipa2 = new String(ipa.toString());
	ipa2 = StringX.replace(ipa2, "\u02ab", "lz");      // lz
	ipa2 = StringX.replace(ipa2, "\u02aa", "ls");      // ls
	ipa2 = StringX.replace(ipa2, "\u02a9", "f\u014b"); // fN
	return ipa2;
    }
    /**
     * complexifies IPA strings by replacing some 2-character groups by double-characters
     * @param ipa the IPA-string to be complexified
     * @return a complexified version of the IPA-string
     */
    private String complexIpa(String ipa){
	if (ipa==null)return null;
	// some Unicode-fonts do not display following characters
	String ipa2 = new String(ipa);
	ipa2 = StringX.replace(ipa2, "lz", "\u02ab");      // lz
	ipa2 = StringX.replace(ipa2, "ls", "\u02aa");      // ls
	ipa2 = StringX.replace(ipa2, "f\u014b", "\u02a9"); // fN
	return ipa2;
    }
    /**
     * display all the information of a given Eintrag that is relevant for us 
     * and store other information about this Eintrag for later use (e.g. soundfiles,...)
     * @param eintrag the TranscriptionEintrag that we display
     */
    public void set(TranscriptionEintrag eintrag){
	Eintrag = eintrag;
	PlayFile1=PlayFile2=null;
	alternateFiles1=alternateFiles2=0;

	if (eintrag==null){
	    PlayButton1.setEnabled(false);
	    EditButton1.setEnabled(false);
	    EditCombo1.setEnabled(false);
	    FileSelector1.setText(AdabaShared.getLabel("LabelNoFile"));
	    PlayButton2.setEnabled(false);
	    EditButton2.setEnabled(false);
	    EditCombo2.setEnabled(false);
	    FileSelector2.setText(AdabaShared.getLabel("LabelNoFile"));
	    SAMPAfield.setText("");
	    IPAfield.setText("");
	    SAMPAfield.setEditable(false);
	    FileSelector1.setEnabled(false);
	    FileSelector2.setEnabled(false);
	    return;
	}
	
	WortName = eintrag.getFilename();
	File[]fils=Eintrag.getFile(langID, false, null);
	if (fils!=null){
	    alternateFiles1=fils.length;
	    PlayFile1=fils[0];
	}
	if (alternateFiles1>1)
	    alt1.setText(alternateFiles1+":");
	else alt1.setText("  ");

	if (PlayFile1!=null){
	    FileSelector1.setText(PlayFile1.getName());
	    //at.iem.tools.Logger.println("found file: "+PlayFile1.getAbsolutePath());
	    PlayButton1.setEnabled(true);
	    EditButton1.setEnabled(true);
	    EditCombo1.setEnabled(true);
	}
	else {
	    FileSelector1.setText(AdabaShared.getLabel("LabelNoFile"));
	    PlayButton1.setEnabled(false);
	    EditButton1.setEnabled(false);
	    EditCombo1.setEnabled(false);
	}

	// then the female files
	fils=Eintrag.getFile(langID, true, null);
	if (fils!=null){
	    alternateFiles2=fils.length;
	    PlayFile2=fils[0];
	}
	if (alternateFiles2>1)
	    alt2.setText(alternateFiles2+":");
	else alt2.setText("  ");

	if (PlayFile2!=null){
	    FileSelector2.setText(PlayFile2.getName());
	    PlayButton2.setEnabled(true);
	    EditButton2.setEnabled(true);
	    EditCombo2.setEnabled(true);
	}
	else {
	    FileSelector2.setText(AdabaShared.getLabel("LabelNoFile"));
	    PlayButton2.setEnabled(false);
	    EditButton2.setEnabled(false);
	    EditCombo2.setEnabled(false);
	}

        String langid=AdabaShared.region[langID][0];
	SAMPAfield.setText(eintrag.getSampa(langid, "[[", "]]"));
	IPAfield.setText(simpleIpa(eintrag.getIPA(langid)));


	SAMPAfield.setEditable(true);
	FileSelector1.setEnabled(true);
	FileSelector2.setEnabled(true);
    }
    /**
     * get the transcription in sampa and ipa of the current entry
     * @return a String[2] array; the sampa transcription is stored in field#0, and the ipa-transcription is stored in field#1
     */
    public String[] get(){
	String[] ret = new String[2];
	ret[0] = SAMPAfield.getText();
	ret[1] = complexIpa(IPAfield.getText());
	return ret;
    }
    /**
     * user interaction
     */
    public void actionPerformed(ActionEvent e) {
	Object source = e.getSource();
	if (source==SAMPAfield){
	    at.iem.tools.linguistic.IPA ipa = new at.iem.tools.linguistic.IPA();
	    ipa.setSAMPA(SAMPAfield.getText(), "[[", "]]");
	    IPAfield.setText(simpleIpa(ipa));
	    AdabaShared.dirty=true;
	    parent.setDirty(true);
	} else if (source==FileSelector1 || source==FileSelector2){
	    boolean female=(source==FileSelector2);
	    String Wort = new String (WortName);
	    Wort = StringX.replace(Wort, ".", "");
	    Wort = FileX.makePosixName(Wort); // achtung !!

      String paths[]=Eintrag.getPaths();
      String path="";
      if(null!=paths) {
          if(paths.length>1)
              path=paths[1];
          else
              path=paths[0];
      }
	    JFileChooser fc = new JFileChooser(path);
	    fc.addChoosableFileFilter(new CustomFileFilter("WAV-Files", ".WAV"));
	    fc.addChoosableFileFilter(new CustomFileFilter("'"+Wort.toUpperCase()+"*.WAV'-Files",
							   Wort, ".WAV"));
 
	    if (!female){
		if (alternateFiles1>1)
		    fc.addChoosableFileFilter(new CustomFileFilter("Alternativen ("+Wort+")",
								   Eintrag.getFilename(langID, female),
								   ".WAV"));

		if (PlayFile1!=null)fc.setSelectedFile(PlayFile1);
		if(fc.showOpenDialog(FileSelector1)==JFileChooser.APPROVE_OPTION){
		    java.io.File file=fc.getSelectedFile();
		    FileSelector1.setText("file: "+file.getName());
		    PlayFile1 = file;
		    PlayButton1.setEnabled(true);
		    EditButton1.setEnabled(true);
		    EditCombo1.setEnabled(true);
		}
	    } else {
		if (alternateFiles2>1)
		    fc.addChoosableFileFilter(new CustomFileFilter("Alternativen ("+Wort+")",
								   Eintrag.getFilename(langID, female),
								   ".WAV"));
		if (PlayFile2!=null)fc.setSelectedFile(PlayFile2);
		if(fc.showOpenDialog(FileSelector2)==JFileChooser.APPROVE_OPTION){
		    java.io.File file=fc.getSelectedFile();
		    FileSelector2.setText("file: "+file.getName());
		    PlayFile2 = file;
		    PlayButton2.setEnabled(true);
		    EditButton2.setEnabled(true);
		    EditCombo2.setEnabled(true);
		}
	    }
	    AdabaShared.dirty=true;
	    parent.setDirty(true);
	} else if (source==PlayButton1){
	    if (PlayFile1!=null){
		SoundPlayer.load(PlayFile1);
		SoundPlayer.play();
	    }
	} else if (source==PlayButton2){
	    if (PlayFile2!=null){
		SoundPlayer.load(PlayFile2);
		SoundPlayer.play();
	    }
	} else if (source==EditButton1){
	    if (PlayFile1!=null){
		try {
		    Runtime.getRuntime().exec(StringX.replace(AdabaShared.SoundEditors[0][1],
							      "%s", PlayFile1.getAbsolutePath()));
		} catch (Exception ex) {
		    at.iem.tools.Logger.println("an error occured on execution: "+ex);
		}
	    }
	} else if (source==EditButton2){
	    if (PlayFile2!=null){
		try {
		    Runtime.getRuntime().exec(StringX.replace(AdabaShared.SoundEditors[0][1],
							      "%s", PlayFile2.getAbsolutePath()));
		} catch (Exception ex) {
		    at.iem.tools.Logger.println("an error occured on execution: "+ex);
		}
	    }
	} else if (source==EditCombo1 || source==EditCombo2){
	    JComboBox combo=(source==EditCombo1)?EditCombo1:EditCombo2;
	    File playfile=(source==EditCombo1)?PlayFile1:PlayFile2;
	    try {
		String execstring=StringX.replace(AdabaShared.SoundEditors[combo.getSelectedIndex()][1],
						  "%s", playfile.getAbsolutePath());
		Runtime.getRuntime().exec(execstring);
	    } catch (Exception ex) {
		at.iem.tools.Logger.println("an error occured on execution: "+ex);
	    }
	} else at.iem.tools.Logger.println("TransPanel:: unknown event");
    }
    /**
     * refresh the fonts in case the user has selected another one via the properties
     */
    void refreshFont(){
	SAMPAfield.setFont(AdabaShared.AdabaFONT);
	IPAfield.setFont(AdabaShared.AdabaFONT);
    }

    /* for serialization */
    private static final long serialVersionUID = 1L;

}
