// TransGUI.java
// $Id: TransProperties.java 1962 2008-06-03 14:24:00Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
// 
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file


// 01/10/2002:: auto-save when hitting "OK" or "fertig"
// 03/10/2002:: auto-save jetzt in TranscriptionDatabase bei insert/replace
//              dirty-flag

package at.iem.adaba.transcriber;
import at.iem.adaba.*;
import at.iem.tools.StringX;

import javax.swing.*;
import javax.swing.table.*;
import java.awt.*;
import java.awt.event.*;

// testing
import java.io.*;

/**
 * die Klasse fr die Transcription Properties
 */

public class TransProperties extends JDialog  implements ActionListener, ItemListener{
    JTable regions_table, flags_table, scripts_table, characters_table;
    DefaultTableModel regions_model, scripts_model, flags_model, characters_model;

    JCheckBox transcriberButton, searchButton, fixMSfontsButton;
    JRadioButton[] pathButtons;

    String pathstring="---";
    int pathmode=0;

    JComboBox fontName, fontSize, fontStyle;


   class PropTableModel extends DefaultTableModel {
       int[]editable_columns;
       PropTableModel(String[][] data, String[]header, int[]editable_columns) {
           super(data, header);
           this.editable_columns=editable_columns;
       }



        public boolean isCellEditable(int row, int col) {
            if(editable_columns==null||editable_columns.length<1)
                return true;

            for(int i=0; i<editable_columns.length; i++) {
                if (col==editable_columns[i])
                    return true;
            }
            return false;
        }
    }

    /**
     * creates a new TransProperties class for setting properties in the transcriber 
     */
    public TransProperties(){
	setModal(true);
        setTitle(AdabaShared.getLabel("PropertiesTitle"));

	JPanel masterpane = new JPanel(new BorderLayout());
	JPanel main = new JPanel(new GridLayout(0,1));
	JPanel regions = new JPanel(new GridLayout(0,1));
	JPanel flags = new JPanel(new GridLayout(0,1));
	JPanel scripts = new JPanel(new GridLayout(0,1));
	JPanel characters = new  JPanel(new GridLayout(0,1));
	JTabbedPane tabs = new JTabbedPane();

	tabs.addTab(AdabaShared.getLabel("PropertiesMain"), main);
        if(AdabaShared.wantTranscriber) {
            tabs.addTab(AdabaShared.getLabel("PropertiesRegions"), regions);
            tabs.addTab(AdabaShared.getLabel("PropertiesFlags"), flags);
            tabs.addTab(AdabaShared.getLabel("PropertiesScripts"), scripts);
        }
	tabs.addTab(AdabaShared.getLabel("PropertiesCharacters"), characters);

	/* MAIN */
        if(AdabaShared.wantTranscriber) {
            JPanel pathpane = new JPanel(new GridLayout(0,1));
            pathpane.setBorder(BorderFactory.createCompoundBorder(
				BorderFactory.createTitledBorder(AdabaShared.getLabel("PropertiesPathTitle")),
				BorderFactory.createEmptyBorder(5,5,5,5)));
            pathButtons = new JRadioButton[2];
            pathButtons[0]=new JRadioButton(AdabaShared.getLabel("PropertiesPathDefault"));
            pathButtons[1]=new JRadioButton(StringX.replace(AdabaShared.getLabel("PropertiesSoundPath"),
							"%s", "---"));

            int selectedPath=0;
            at.iem.tools.Logger.println("global-dir: "+AdabaShared.getPWD());
            if (AdabaShared.getPWD()!=null){
                selectedPath=1;
                pathButtons[1].setSelected(true);
                pathButtons[1].setText(StringX.replace(AdabaShared.getLabel("PropertiesSoundPath"),
                                                       "%s", AdabaShared.getPWD().getAbsolutePath()));
            }
            pathButtons[selectedPath].setSelected(true);

            
            //Group the radio buttons.
            ButtonGroup pathgroup = new ButtonGroup();
            for(int i=0; i<pathButtons.length; i++){
                pathgroup.add(pathButtons[i]);
                pathpane.add (pathButtons[i]);
                pathButtons[i].setActionCommand("path"+i);
                pathButtons[i].addActionListener(this);
            }
            pathpane.setPreferredSize(new Dimension(0, 400));
            main.add(pathpane);
        }

	JPanel fontPanel = new JPanel();
	fontPanel.setBorder(BorderFactory.createCompoundBorder(
		BorderFactory.createTitledBorder(AdabaShared.getLabel("PropertiesFontTitle")),
		BorderFactory.createEmptyBorder(5,5,5,5)));
	
	String envfonts[] = GraphicsEnvironment.getLocalGraphicsEnvironment().getAvailableFontFamilyNames();
	fontName = new JComboBox(envfonts);
	fontPanel.add(fontName);

	fontSize = new JComboBox( new Object[]{ "10", "12", "14", "16", "18", "20", "22", "24"} );
	fontSize.setMaximumRowCount( 9 );
	fontPanel.add(fontSize);

	fontStyle = new JComboBox( new Object[]{
	    "PLAIN",
	    "BOLD",
	    "ITALIC",
	    "BOLD & ITALIC"} );
	fontStyle.setMaximumRowCount( 9 );
	fontPanel.add(fontStyle);

	fontName.setSelectedItem(AdabaShared.AdabaFONT.getName());
	fontSize.setSelectedItem(AdabaShared.AdabaFONT.getSize()+"");
	fontStyle.setSelectedIndex(AdabaShared.AdabaFONT.getStyle());

	main.add(fontPanel);
        fixMSfontsButton = new JCheckBox(AdabaShared.getLabel("PropertiesFixMSFonts"));
        fixMSfontsButton.setToolTipText(AdabaShared.getLabel("ToolTipsFixMSFonts"));
        fixMSfontsButton.setSelected(AdabaShared.fixMSFonts);
        main.add(fixMSfontsButton);        

        if(AdabaShared.wantTranscriber) {
            JPanel appPane = new JPanel();
            transcriberButton = new JCheckBox(AdabaShared.getLabel("PropertiesWhichApplication"));
            transcriberButton.setSelected(AdabaShared.wantTranscriber);
            appPane.add(transcriberButton);
            searchButton = new JCheckBox(AdabaShared.getLabel("PropertiesWantSearch"));
            searchButton.setSelected(AdabaShared.TranscriberSearch);
            appPane.add(searchButton);
            
            main.add(appPane);
        }


	regions_model = new DefaultTableModel( AdabaShared.region, 
				     new String[]{AdabaShared.getLabel("PropertiesRegionTable0"), 
						  AdabaShared.getLabel("PropertiesRegionTable1")});
	regions_table = new JTable(regions_model);
	init_table(regions, regions_table, regions_model, "region");
	
	String[][]eflags=new String[ AdabaShared.EntryFlag.length][1];
	for(int i=0; i<eflags.length; i++)eflags[i][0]=AdabaShared.EntryFlag[i];
	flags_model = new DefaultTableModel( eflags,new String[]{AdabaShared.getLabel("PropertiesFlags")} );
	flags_table = new JTable(flags_model);
	init_table(flags, flags_table, flags_model, "flag");

	scripts_model = new DefaultTableModel(AdabaShared.SoundEditors, 
					      new String[]{AdabaShared.getLabel("PropertiesScriptsTable0"), 
						  AdabaShared.getLabel("PropertiesScriptsTable1")});
	scripts_table = new JTable(scripts_model);
	init_table(scripts, scripts_table, scripts_model, "script");



	// IPAgroups
	String[][]chars_=AdabaShared.getIPAgroupReplacement();

        String[][]chars =new String[chars_.length][3];
	for(int i=0; i<chars.length; i++){
	    chars[i][0]=AdabaShared.getLabel("PropertiesChar_"+chars_[i][0]);
            chars[i][1]=chars_[i][1];
            chars[i][2]=at.iem.tools.StringX.merge(at.iem.tools.linguistic.IPA.getGroup(chars_[i][0]), " ");
	}
	
	characters_model = new PropTableModel(chars,
                                              new String[]{AdabaShared.getLabel("PropertiesCharTableGroup"), 
                                                           AdabaShared.getLabel("PropertiesCharTableAbbrev"),
                                                           AdabaShared.getLabel("PropertiesCharTableChars"),
                                              },
                                              new int[]{1});
	characters_table = new JTable(characters_model);
        characters_table.setFont( AdabaShared.AdabaFONT.deriveFont( characters_table.getFont().getSize2D()) );
        characters_table.setRowHeight((int)(1.5* characters_table.getRowHeight()));

        //characters_table.setAutoResizeMode (JTable.AUTO_RESIZE_OFF);
        //characters_table.setFont(AdabaShared.AdabaFONT);
        setupTable(characters_table);

	JScrollPane scrollPane = new JScrollPane(characters_table);
	//	table.setPreferredScrollableViewportSize(new Dimension(500, 70));
	characters.add(scrollPane);



	masterpane.add(tabs, BorderLayout.CENTER);
	tabs.setPreferredSize(new Dimension(520, 440));


	JPanel button_pane= new JPanel();
	JButton button; 
  /*
  button=new JButton(AdabaShared.getLabel("ButtonOK"));
	button.setActionCommand("OK");
	button.addActionListener(this);
	button_pane.add(button, BorderLayout.WEST);
  */
	button=new JButton(AdabaShared.getLabel("ButtonSave"));
	button.setActionCommand("save");
	button.addActionListener(this);
	button_pane.add(button, BorderLayout.CENTER);
	button=new JButton(AdabaShared.getLabel("ButtonCancel"));
	button.setActionCommand("cancel");
	button.addActionListener(this);
	button_pane.add(button, BorderLayout.EAST);
        masterpane.add(button_pane, BorderLayout.PAGE_END);

	getContentPane().add(masterpane);
	pack();
	setVisible(true);

    }


    private static void setupTable(JTable table) {
        DefaultTableCellRenderer renderer = new DefaultTableCellRenderer();
        renderer.setHorizontalAlignment(JLabel.CENTER);
        at.iem.tools.gui.AntiAliasing.enable(renderer);
        for(int i=0; i<table.getColumnModel().getColumnCount(); i++) {
            table.getColumnModel().getColumn(i).setCellRenderer(renderer);
        }
        table.setFont(AdabaShared.AdabaFONT);
    }

    private void init_main(){

    }
    /**
     * helper function to set up editable tables for key-value pairs
     * @param pane the JPanel we want to put the table in
     * @param table the table
     * @param model (unused)
     * @param id an id to identify this table when it triggers ActionEvents
     */
    private void init_table(JPanel pane, JTable table, DefaultTableModel model, String id){
	JPanel buttons= new JPanel();

        table.setFont(AdabaShared.AdabaFONT);
	table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

	JScrollPane scrollPane = new JScrollPane(table);
	//	table.setPreferredScrollableViewportSize(new Dimension(500, 70));
	pane.add(scrollPane);
	JButton button=new JButton("up");
	button.setActionCommand(id+"_up");
	button.addActionListener(this);
	buttons.add(button);
	button=new JButton("down");
	button.setActionCommand(id+"_down");
	button.addActionListener(this);
	buttons.add(button);
	button=new JButton("add");
	button.setActionCommand(id+"_add");
	button.addActionListener(this);
	buttons.add(button);
	button=new JButton("remove");
	button.setActionCommand(id+"_del");
	button.addActionListener(this);
	buttons.add(button);

	pane.add(buttons);
    }
    /**
     * user interaction
     * @param e the ActionEvent describing what the user did
     */
    public void actionPerformed(ActionEvent e) {
	String cmd = e.getActionCommand();
	String[] empty_row={"", ""};
	if(cmd.equals("")){

	}else if(cmd.equals("region_up")){
	    int row=regions_table.getSelectedRow();
	    if(row>0){
		regions_model.moveRow(row, row, row-1);
		regions_table.setRowSelectionInterval(row-1, row-1);
	    }
	}else if(cmd.equals("region_down")){
	    int row=regions_table.getSelectedRow();
	    if(row!=-1 && row<regions_model.getRowCount()-1){
		regions_model.moveRow(row, row, row+1);
		regions_table.setRowSelectionInterval(row+1, row+1);
	    }
	}else if(cmd.equals("region_add")){
	    regions_model.addRow(empty_row);
	}else if(cmd.equals("region_del")){
	    int row=regions_table.getSelectedRow();
	    if(row!=-1)regions_model.removeRow(row);

	}else if(cmd.equals("flag_up")){
	    int row=flags_table.getSelectedRow();
	    if(row>0){
		flags_model.moveRow(row, row, row-1);
		flags_table.setRowSelectionInterval(row-1, row-1);
	    }
	}else if(cmd.equals("flag_down")){
	    int row=flags_table.getSelectedRow();
	    if(row!=-1 && row<flags_model.getRowCount()-1){
		flags_model.moveRow(row, row, row+1);
		flags_table.setRowSelectionInterval(row+1, row+1);
	    }
	}else if(cmd.equals("flag_add")){
	    flags_model.addRow(empty_row);
	}else if(cmd.equals("flag_del")){
	    int row=flags_table.getSelectedRow();
	    if(row!=-1)flags_model.removeRow(row);

	}else if(cmd.equals("script_up")){
	    int row=scripts_table.getSelectedRow();
	    if(row>0){
		scripts_model.moveRow(row, row, row-1);
		scripts_table.setRowSelectionInterval(row-1, row-1);
	    }
	}else if(cmd.equals("script_down")){
	    int row=scripts_table.getSelectedRow();
	    if(row!=-1 && row<scripts_model.getRowCount()-1){
		scripts_model.moveRow(row, row, row+1);
		scripts_table.setRowSelectionInterval(row+1, row+1);
	    }
	}else if(cmd.equals("script_add")){
	    scripts_model.addRow(empty_row);
	}else if(cmd.equals("script_del")){
	    int row=scripts_table.getSelectedRow();
	    if(row!=-1)scripts_model.removeRow(row);
	}else if(cmd.equals("OK") || cmd.equals("save")){
	    String fname = (String)fontName.getSelectedItem();
	    int fstyle   = fontStyle.getSelectedIndex();
	    int fsize = (new Integer((String)fontSize.getSelectedItem())).intValue(); 
	    AdabaShared.AdabaFONT = new Font(fname, fstyle, fsize);
	    if ("---".equals(pathstring))pathmode=0;
	    switch (pathmode) {
	    case 1:
		AdabaShared.setPWD(new File(pathstring));
		break;
	    default:
		AdabaShared.setPWD((File)null);
	    }

            AdabaShared.fixMSFonts=fixMSfontsButton.isSelected();

            if(AdabaShared.wantTranscriber) {

                AdabaShared.wantTranscriber =  transcriberButton.isSelected();
                AdabaShared.TranscriberSearch = searchButton.isSelected();

                //this is to prevent the currently edited cell
                if(regions_table!=null && regions_table.getCellEditor()!=null)
                    regions_table.getCellEditor().stopCellEditing();
                AdabaShared.region = new String[regions_table.getRowCount()][2];
                for(int i=0; i<regions_table.getRowCount(); i++){
                    AdabaShared.region[i][0]=(String)regions_table.getValueAt(i, 0);
                    AdabaShared.region[i][1]=(String)regions_table.getValueAt(i, 1);
                }

                //this is to prevent the currently edited cell
                if(flags_table!=null && flags_table.getCellEditor()!=null)
                    flags_table.getCellEditor().stopCellEditing();
                AdabaShared.EntryFlag = new String[flags_table.getRowCount()];
                for(int i=0; i<flags_table.getRowCount(); i++){
                    AdabaShared.EntryFlag[i]=(String)flags_table.getValueAt(i, 0);
                }

                //this is to prevent the currently edited cell
                if(scripts_table!=null && scripts_table.getCellEditor()!=null)
                    scripts_table.getCellEditor().stopCellEditing();
                AdabaShared.SoundEditors = new String[scripts_table.getRowCount()][2];
                for(int i=0; i<scripts_table.getRowCount(); i++){
                    AdabaShared.SoundEditors[i][0]=(String)scripts_table.getValueAt(i, 0);
                    AdabaShared.SoundEditors[i][1]=(String)scripts_table.getValueAt(i, 1);
                    //at.iem.tools.Logger.println("script: '"+AdabaShared.SoundEditors[i][0]+
                    //		   "' --> '"+	AdabaShared.SoundEditors[i][1]+"'");
                }
            }
            
	    //this is to prevent the currently edited cell
	    if(characters_table!=null && characters_table.getCellEditor()!=null)
		characters_table.getCellEditor().stopCellEditing();
	    for(int i=0; i<characters_table.getRowCount(); i++)
		AdabaShared.setIPAgroupReplacement(i ,(String)characters_table.getValueAt(i, 1));

	    if ("save".equals(cmd))AdabaShared.write();
	    dispose();
	}else if(cmd.equals("cancel")){
	    dispose();
	}else if(cmd.equals("path0")){
	    pathmode=0;
	}else if(cmd.equals("path1")){ // sound-root
	    JFileChooser fc = new JFileChooser(".");
	    fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
	    if (fc.showOpenDialog(this)==JFileChooser.APPROVE_OPTION){
		pathstring=fc.getSelectedFile().getAbsolutePath();
		String path=StringX.replace(AdabaShared.getLabel("PropertiesSoundPath"),
					    "%s", pathstring);
		pathmode=1;
		pathButtons[1].setText(path);
	    } else pathButtons[pathmode].setSelected(true);
	} else {

	}
	
    } 

    /** 
     * Listens to the check boxes. 
     * @param e the ItemEvent triggered by changes to the check boxes
     */
    public void itemStateChanged(ItemEvent e) {
        int index = 0;
        char c = '-';
        Object source = e.getItemSelectable();

	if (source == pathButtons[0]){

	} else if (source == pathButtons[1]){

	} else {

	}
    }


    /* for serialization */
    private static final long serialVersionUID = 1L;
}
