// FileX.java
// $Id: FileX.java 1508 2007-04-25 13:20:31Z zmoelnig $
// Institute for Electronic Music and Acoustics, Graz (IEM)
//
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

// eXtended String-operations

package at.iem.tools;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;

public class FileX {
    public static String getExtension(File f) {
	String ext = null;
        String s = f.getName();
        int i = s.lastIndexOf('.');
	
        if (i > 0 &&  i < s.length() - 1) {
            ext = s.substring(i+1);
        }
        return ext;
    }

    /** i really don't know whether this is POSIX (guess it is not).
     * anyhow, (german) umlauts are converted to ascii,
     * spaces are replaced by underscores
     * pipes and quotes are ignored
     * and the whole filename will be returned uppercase
     */
    public static String makePosixName(String s){
	String Wort = s.trim();
	Wort = StringX.replace(Wort, "", "sz");
	Wort = Wort.toUpperCase().trim();
	Wort = Wort.replace('', 'E').replace('', 'E').replace('', 'E');
	Wort = Wort.replace('', 'A').replace('', 'A').replace('', 'A');
	Wort = Wort.replace('', 'O').replace('', 'O').replace('', 'O');
	Wort = StringX.replace(Wort, "", "AE");
	Wort = StringX.replace(Wort, "", "OE");
	Wort = StringX.replace(Wort, "\u0152", "OE"); // ~
	Wort = StringX.replace(Wort, "", "UE");
	Wort = StringX.replace(Wort, "|", "");
	Wort = StringX.replace(Wort, "'", "");

	Wort = Wort.replace(' ', '_');

	return Wort.trim();
    }

    public static boolean copy(File from, File to){
	//if(to.exists())return false; // the user should care for this
	if(!from.exists()||!from.isFile())return false;
	FileInputStream  in  = null;
	FileOutputStream out = null;
	try {
	    in  = new FileInputStream (from);
	    out = new FileOutputStream(to);
	    int b=0;
	    /*
	    while(b!=-1){
		b=in.read();
		out.write(b);
	    }
	    */
	    for(b=in.read(); b!=-1; b=in.read())out.write(b);
	    in.close();
	    out.close();
	} catch (Exception ex){
	    return false;
	}
		
	return true;
    }


    private static void listFilesRecursive_append(java.util.ArrayList<File>result,
                                                  File path, 
                                                  java.io.FilenameFilter filter) {
        File[]files=path.listFiles(filter);
        for(int i=0; i<files.length; i++){
            if(files[i].isDirectory())
                listFilesRecursive_append(result, files[i], filter);
            else
                result.add(files[i]);
        }
    }
    public static File[] listFilesRecursive(File path, java.io.FilenameFilter filter) {
        java.util.ArrayList<File>filelist = new java.util.ArrayList<File>();
        listFilesRecursive_append(filelist, path, filter);
        File[]result=new File[filelist.size()];
        for(int i=0; i<result.length; i++){
            result[i]=(File)filelist.get(i);
        }
        return result;
    }

    /**
     * copies a file from the system-ressources (e.g. within the .jar archive) to a temporary file
     */
    public static File getSystemResourceTempFile(String name) {
        java.net.URL url=ClassLoader.getSystemResource("resources/"+name);
        if(null==url){
            System.err.println("'"+name+"' not found in system resources");
            return null;
        }

        java.io.InputStream istream;
        java.io.FileOutputStream ostream;
        File file;

        try {
            istream = url.openStream();
        } catch (java.io.IOException e) {
            System.err.println("failed opening input stream: "+e);
            return null;
        }
         try {
             String extension=FileX.getExtension(new File(name));
             file = java.io.File.createTempFile("iem", "."+extension);
             file.deleteOnExit();
             System.out.println("created temp file: "+file);
             ostream = new java.io.FileOutputStream (file);
         } catch (java.io.IOException e) {
             System.err.println("failed opening output stream: "+e);
             return null;
         }

        int oneChar, count=0;
        
        try {
            int bytesread=0;
            byte[] b = new byte[1024];
            while ((bytesread=istream.read(b))>0) {
                ostream.write(b, 0, bytesread);
                count+=bytesread;
            }
        } catch (Exception e) {
            System.err.println("failed to copy file at byte "+count);
            System.err.println("error was: "+e);

            return null;
        }
        try {
            istream.close();
            ostream.close();
        } catch (java.io.IOException e) {
            System.err.println("error closing stream: "+e);
            return null;
        }

        return file;
    }

    /**
     * if <code>file</code> exists, return <code>file</code>.<br>
     * if it does not exist, try to find a System Resource of the same name; 
     * if this resource exists, copy it into a temporary file and return it;
     * if it does not exist, the original File is returned
     */
    public static File fallbackToTempSystemResource(File file) {
        if(file==null || file.exists())
            return file;
        System.out.println("really fallBackToSystemResource: "+file);
        File f=getSystemResourceTempFile(file.toString());
        if(f!=null)
            return f;

        return file;

    }

}

