// SetOperator.java
// $Id: SetOperator.java 825 2006-03-16 11:57:33Z zmoelnig $
// Institute for Electronic Music and Acoustics, Graz (IEM)
//
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

// operations on sets

package at.iem.tools;

import java.util.ArrayList;

/**
 * operations on sets: &#x222A, &#x2229,  &#x2216
 */


public class SetOperator {

    /**
     * a union of 2 sets
     * @param A the 1st set to be merged
     * @param B the 2nd set to be merged
     * @return the union of A and B: A &#x222A B
     */ 
    public static Object[] union(Object[] A, Object[] B){
	if(B==null||B.length==0)return A;
	if(A==null||A.length==0)return B;

	ArrayList<Object> AB = new ArrayList<Object>(A.length+B.length);

	for(int i=0; i<A.length; i++)
	    AB.add(A[i]);
	for(int i=0; i<B.length; i++)
	    if(!AB.contains(B[i]))AB.add(B[i]);
	

	return AB.toArray();
    }
   /**
     * @see #union(Object[], Object[]) union
     */ 
    public static String[] union(String[] A, String[] B){
	if(B==null||B.length==0)return A;
	if(A==null||A.length==0)return B;

	ArrayList<String> AB = new ArrayList<String>(A.length+B.length);

	for(int i=0; i<A.length; i++)
	    AB.add(A[i]);
	for(int i=0; i<B.length; i++)
	    if(!AB.contains(B[i]))AB.add(B[i]);

	String[]result=new String[AB.size()];
	for(int i=0; i<result.length; i++)
	    result[i]=(String)AB.get(i);
	
	return result;
    }

    /**
     * an intersection of 2 sets
     * @param A the 1st set to be merged
     * @param B the 2nd set to be merged
     * @return the intersection of A and B: A &#x2229 B
     */ 
    public static Object[] intersect(Object[] A, Object[] B){
	if(B==null||B.length==0)return null;
	if(A==null||A.length==0)return null;


	ArrayList<Object> Al = new ArrayList<Object>(A.length);

	for(int i=0; i<A.length; i++)
	    Al.add(A[i]);


	ArrayList<Object> AB = new ArrayList<Object>((A.length<B.length)?A.length:B.length);

	for(int i=0; i<B.length; i++)
	    if(Al.contains(B[i]))AB.add(B[i]);
	
	return AB.toArray();
    }
   /**
     * @see #intersect(Object[], Object[]) intersect
     */ 
    public static String[] intersect(String[] A, String[] B){
	if(B==null||B.length==0)return null;
	if(A==null||A.length==0)return null;


	ArrayList<String> Al = new ArrayList<String>(A.length);

	for(int i=0; i<A.length; i++)
	    Al.add(A[i]);


	ArrayList<String> AB = new ArrayList<String>((A.length<B.length)?A.length:B.length);

	for(int i=0; i<B.length; i++)
	    if(Al.contains(B[i]))AB.add(B[i]);

	String[]result=new String[AB.size()];
	for(int i=0; i<result.length; i++)
	    result[i]=(String)AB.get(i);
	
	return result;
    }
    /**
     * a subtraction of 2 sets
     * @param A the basic set
     * @param B the set that is to be subtracted from A
     * @return the subtraction of B from A: A &#x2216 B
     */ 
    public static Object[] minus(Object[] A, Object[] B){
	if(B==null||B.length==0)return A;
	if(A==null||A.length==0)return null;
	
	ArrayList<Object> AB = new ArrayList<Object>(A.length);

	for(int i=0; i<A.length; i++)
	    AB.add(A[i]);
	for(int i=0; i<B.length; i++)
	    if(AB.contains(B[i]))AB.remove(AB.indexOf(B[i]));
	
	return AB.toArray();
    }
   /**
     * @see #minus(Object[], Object[]) minus
     */
    public static String[] minus(String[] A, String[] B){
	if(B==null||B.length==0)return A;
	if(A==null||A.length==0)return null;
	
	ArrayList<String> AB = new ArrayList<String>(A.length);

	for(int i=0; i<A.length; i++)
	    AB.add(A[i]);
	for(int i=0; i<B.length; i++)
	    if(AB.contains(B[i]))AB.remove(AB.indexOf(B[i]));
	
	String[]result=new String[AB.size()];
	for(int i=0; i<result.length; i++)
	    result[i]=(String)AB.get(i);
	
	return result;
    }
}




