// CustomFileFilter.java
// $Id: CustomFileFilter.java 783 2005-01-27 12:31:43Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
// 
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

package at.iem.tools.gui;
import java.io.File;
import java.io.FilenameFilter;
import javax.swing.filechooser.FileFilter;

/**
 * a FileFilter.<br>
 * both the start of the filename and the ending (extension,...) can be specified
 */

public class CustomFileFilter extends FileFilter implements FilenameFilter{
    /** 
     * a verbose description for the files to be filtered
     */
    private String description;
    /** 
     * the ending of the filename (could be extension or more=
     */
    private String endung;
    /**
     * the beginning of the filename
     */
    private String startung;
    /**
     * create a filter with for filenames ending with a certain string
     * @param desc a verbose description of the filter
     * @param End the filename ending
     */
    public CustomFileFilter(String desc, String End){
	startung = null;
	endung = End.toLowerCase();;
	description=desc;
    }
    /**
     * create a filter for filenames ending with a certain string
     * @param End the filename ending
     */
    public CustomFileFilter(String End){
	endung = End.toLowerCase();
	startung = null;
	description = null;
    }
    /**
     * create a filter for filenames starting and ending with certain strings
     * @param desc a verbose description of the filter
     * @param Start the filename beginning
     * @param End the filename ending
     */
    public CustomFileFilter( String desc, String Start, String End){
	startung = (Start!=null)?Start.toLowerCase():null;
	endung = (End!=null)?End.toLowerCase():null;
	description=desc;
    }

    public boolean accept(File path, String Name){
	String name = Name.toLowerCase();

	if (startung==null){
	    if (endung==null)return true;
	    else return (name.endsWith(endung));
	} else {
	    if (endung==null)return name.startsWith(startung);
	    else return (name.startsWith(startung)&&name.endsWith(endung));
	}
    }

    public boolean accept(File file){
	if (file.isDirectory())return true;
	return accept(file, file.getName());
    }

     // The description of this filter
    public String getDescription() {
	if (description!=null){
	    return description;
	} else {
	    return "Custom Files ("+((startung==null)?"":startung)+"*"+((endung==null)?"":endung)+")";
	}
    }
    public void setDescription(String desc) {
	description=desc;
    }

}
