// FontChooser.java
// $Id: FontChooser.java 829 2006-03-16 15:24:27Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
// 
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

// choose one of those fonts installed on your system
package at.iem.tools.gui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.font.*;
import java.util.Vector;
import java.beans.*; //Property change stuff

/**
 * like a JFileChooser, just for fonts...
 */

public class FontChooser extends JDialog{
    private Font myfont;
    private JComboBox fonts, sizes, styles;
    private JOptionPane optionPane;

    public FontChooser(JFrame parent){
	super(parent, true);

	JPanel fontPanel = new JPanel();
	//Font newFont = getFont().deriveFont(1);
	GraphicsEnvironment gEnv = GraphicsEnvironment.getLocalGraphicsEnvironment();
	String envfonts[] = gEnv.getAvailableFontFamilyNames();
	java.util.Vector<String> vector = new java.util.Vector<String>();
	for ( int i = 1; i < envfonts.length; i++ )
	    vector.addElement(envfonts[i]);
	fonts = new JComboBox( vector );
	fonts.setMaximumRowCount( 9 );
	//	    fontchoice = envfonts[0];
	fontPanel.add(fonts);

	sizes = new JComboBox( new Object[]{ "10", "12", "14", "16", "18", "20", "22", "24"} );
	sizes.setMaximumRowCount( 9 );
	fontPanel.add(sizes);

	styles = new JComboBox( new Object[]{
	    "PLAIN",
	    "BOLD",
	    "ITALIC",
	    "BOLD & ITALIC"} );
	styles.setMaximumRowCount( 9 );
	fontPanel.add(styles);

	optionPane = new JOptionPane(fontPanel,
                                   JOptionPane.QUESTION_MESSAGE,
                                    JOptionPane.OK_CANCEL_OPTION);
	
	setContentPane(optionPane);
	setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent we) {
		    /*
		     * Instead of directly closing the window,
		     * we're going to change the JOptionPane's
		     * value property.
		     */
                    optionPane.setValue(new Integer(JOptionPane.CLOSED_OPTION));
		}
	    });
	optionPane.addPropertyChangeListener(new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent e) {
		    String prop = e.getPropertyName();
		    
		    if (isVisible() 
			&& (e.getSource() == optionPane)
			&& (prop.equals(JOptionPane.VALUE_PROPERTY) ||
			    prop.equals(JOptionPane.INPUT_VALUE_PROPERTY))) {
			if (optionPane.getValue() == JOptionPane.UNINITIALIZED_VALUE) {
			    //ignore reset
			    return;
			}
			int value = ((Integer)optionPane.getValue()).intValue();
			optionPane.setValue(JOptionPane.UNINITIALIZED_VALUE);
			if (value==0) { // ???
			    String f = (String)fonts.getSelectedItem();
			    int st   = styles.getSelectedIndex();
			    int size = (new Integer((String)sizes.getSelectedItem())).intValue(); 
			    myfont = new Font(f, st, size);
			} else myfont = null;
			
			setVisible(false);
		    }
		}
	    });
    }
    public Font getFont(){
	return myfont;
    }
    public void setDefaultFont(Font def){
	if (def==null)return;
	
	System.out.println("FONT: "+def.getName()+" "+def.getSize()+" "+def.getStyle());

	fonts.setSelectedItem(def.getName());
	if (!((String)fonts.getSelectedItem()).equals(def.getName())){
	    fonts.insertItemAt(def.getName(), 0);
	    fonts.setSelectedIndex(0);
	}
	
	sizes.setSelectedItem(def.getSize()+"");
	if (!((String)sizes.getSelectedItem()).equals(def.getSize()+"")){
	    sizes.insertItemAt(def.getSize()+"", 0); 
	    sizes.setSelectedIndex(0);
	}
	styles.setSelectedIndex(def.getStyle());
    }
    public static Font showFontChooser(JFrame parent, Font defaultfont){
	FontChooser fc = new FontChooser(parent);
	fc.pack();
	fc.setDefaultFont(defaultfont);
	fc.setVisible(true);
	
	return fc.getFont();

    }
    static final long serialVersionUID = 1L;
}
