// TextWindow.java
// $Id: TextWindow.java 829 2006-03-16 15:24:27Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
// 
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

// display a text from a file in a window
// add a "done" button to close the window
package at.iem.tools.gui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;

public class ProgressBar extends JDialog implements Runnable, ActionListener, WindowListener {
    String title;
    JProgressBar pb;
    JButton but;

    boolean running;

    public ProgressBar(){
        this(null);
 
    }
    public ProgressBar(String title){
        this(title, null);
    }

    public ProgressBar(String title, String donestring){
        this.title=title;
        running=true;
        
        JPanel pane=new JPanel();
        pane.setLayout(new BoxLayout(pane, BoxLayout.PAGE_AXIS));

        pane.add(Box.createRigidArea(new Dimension(0,5)));
        pb = new JProgressBar();
        pb.setIndeterminate(true);
        pb.setAlignmentX(Component.CENTER_ALIGNMENT);

        pane.add(pb);

        pane.add(Box.createRigidArea(new Dimension(0,5)));

        if(donestring!=null) {
            but=new JButton(donestring);
            but.addActionListener(this);
            but.setAlignmentX(Component.CENTER_ALIGNMENT);
            pane.add(but);
            pane.add(Box.createRigidArea(new Dimension(0,5)));
            setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        } else {
            setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
        }

	setLabel(title);
	setTitle(title);
	setLocationRelativeTo(null); // position at the middle of screen
        setAlwaysOnTop(true);

        addWindowListener(this);

        getContentPane().add(pane);

        Thread thread = new Thread(this);
        thread.setPriority(thread.MIN_PRIORITY);
        thread.start();

        pack();
        setVisible(true);
    }

    public void run() {
        while(running) {
            try {
                Thread.sleep(100);
            } catch (Exception ex) {

            }
        }
    }

    public void setLabel(String label) {
        pb.setString(label);
    }

    public void done() {
        running=false;
        setVisible(false);
        dispose();
    }

    private ArrayList<ActionListener> m_listeners=null;
    /**
     * Add listener to be notified.
     * @param al
     */
    public void addActionListener(ActionListener al)
    {
        if(m_listeners==null)
            m_listeners=new  ArrayList<ActionListener>();
        m_listeners.add(al);
    }
    protected void notifyActionListeners(int id, String command) {
        ActionEvent e=new ActionEvent(this, id, command);
        if(m_listeners==null)return;
        for(int i=0; i<m_listeners.size(); i++){
            ActionListener al = (ActionListener) m_listeners.get(i);
            al.actionPerformed(e);
        }
    }

    void userStopped() {
        if(running) {
            notifyActionListeners(0, "stop");
            done();
        }
    }

    /**
     * the user pressed the "stop" button
     * @param event the ActionEvent triggered by user-interaction
     */
    public void actionPerformed(ActionEvent event) {
        at.iem.tools.Logger.println("ProgressBar: action");
        userStopped();
    }

    public void 	windowActivated(WindowEvent e){}
    public void 	windowClosed(WindowEvent e){
        at.iem.tools.Logger.println("window closed!");
        userStopped();
    }
    public void 	windowClosing(WindowEvent e){}
    public void 	windowDeactivated(WindowEvent e){}
    public void 	windowDeiconified(WindowEvent e){}
    public void 	windowIconified(WindowEvent e){}
    public void 	windowOpened(WindowEvent e){}


}

