// TextWindow.java
// $Id: TextWindow.java 829 2006-03-16 15:24:27Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
// 
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

// display a text from a file in a window
// add a "done" button to close the window
package at.iem.tools.gui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.font.*;
import java.net.URL;
import javax.swing.event.HyperlinkListener;
import javax.swing.event.HyperlinkEvent;

public class TextWindow  implements HyperlinkListener {
    public TextWindow(URL url, String title, String done, Font font){
        this(url, title, done, font, false);
    }
    private java.net.URL url;
    JDialog dialog;
    JEditorPane editorPane;
    public TextWindow(URL url, String title, String done, Font font, boolean modal){
        JFrame frame=null;
        if(url==null)return;
        this.url=url;

        try {
            editorPane = new JEditorPane(url);
        } catch (java.io.IOException e) {
            at.iem.tools.Logger.println("Attempted to initialize TextWindow with a bad URL: " + url);
            return;
        }

        editorPane.setEditable(false);
        at.iem.tools.gui.AntiAliasing.enable(editorPane);
        editorPane.addHyperlinkListener(this);

        if(null!=font)
            editorPane.setFont(font);

        JScrollPane editorScrollPane = new JScrollPane(editorPane);
        editorScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        editorScrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
        editorScrollPane.setMinimumSize(new java.awt.Dimension(10, 10));
        editorScrollPane.setPreferredSize(new java.awt.Dimension(640, 480));

        dialog = new JDialog(frame, title, modal);
        JButton donebut=new JButton(done);
        
        donebut.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    dialog.setVisible(false);
                    dialog.dispose();
                }
            });
        JPanel closePanel = new JPanel();
        closePanel.setLayout(new BoxLayout(closePanel, BoxLayout.LINE_AXIS));
        closePanel.add(Box.createHorizontalGlue());
        closePanel.add(donebut);
        closePanel.add(Box.createHorizontalGlue());
        closePanel.setBorder(BorderFactory.createEmptyBorder(0,0,5,5));

        JPanel contentPane = new JPanel(new BorderLayout());
        contentPane.add(editorScrollPane, BorderLayout.CENTER);
        contentPane.add(closePanel, BorderLayout.PAGE_END);
        contentPane.setOpaque(true);
        dialog.setContentPane(contentPane);
        dialog.setSize(contentPane.getPreferredSize());

        dialog.setVisible(true);
    }

    private void clicked(java.net.URL url) {
        at.iem.tools.Logger.println("clicked URL: " + url);
        at.iem.tools.Logger.println("this    URL: " + this.url);
        if(null==url||null==this.url)
            return;
        at.iem.tools.Logger.println("same-base: " + (url.sameFile(this.url)) + "\tsame-url: "+(url.equals(this.url)));
        if(url.sameFile(this.url) && !url.equals(this.url)) {
            /* this is a local reference (e.g. "#A") */
            setURL(url);
            return;
        }
    }

    public void hyperlinkUpdate(HyperlinkEvent e) {
        HyperlinkEvent.EventType typ=e.getEventType();
        if(typ==HyperlinkEvent.EventType.ACTIVATED) {
            clicked(e.getURL());
        } else if(typ==HyperlinkEvent.EventType.ENTERED) {
            //clicked(e.getURL());
        } else if (typ==HyperlinkEvent.EventType.EXITED) {

        }
    }

    public void setURL(java.net.URL url) {
        /* this is a local reference (e.g. "#A") */
        at.iem.tools.Logger.println("open URL: " + url);
        try {
            if(editorPane!=null) {
                editorPane.setPage(url);
                dialog.setVisible(true);
            }
            this.url=url;
        } catch (Exception ex) {
            at.iem.tools.Logger.println("URI-exception: "+ex);
        }
    }
}
