// SoundPlayer.java
// $Id: AbstractPlayer.java 1145 2006-12-18 15:21:31Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
// 
// this code is licensed under the Gnu-General Public License
// see the LICENSE.txt file

package at.iem.tools.sound;

import java.io.File;
import java.net.URL;
import java.util.ArrayList;

public abstract class AbstractPlayer {
    protected URL SoundFile;
    /** 
     * if the child-class supports reading from URLs,
     * this shouldn't be overriden.
     * If something additionally has to be done,
     * maybe overwrite do_load()
     */
    public boolean load(String filename){
	if (filename==null)return false;
	try{
	    URL context = new URL("file:" + System.getProperty("user.dir") + "/");
	    SoundFile = new URL(context, filename);
	}catch (Exception ex){
	    System.err.println(ex);
	    return false;
	}
	return do_load();
    }
    public boolean load(File file){
	if (file==null)return false;
	try{
	    SoundFile = new URL("file:"+file.getAbsolutePath());
	}catch (Exception ex){
	    System.err.println(ex);
	    return false;
	}
	return do_load();
    }
    public boolean load(URL url){
	if (url==null)return false;
	SoundFile = url;
	return do_load();
    }
    /**
     * this can should overwritten by the child-class if needed.
     * Perform everything you need for pre-loading a soundfile from the URL
     */
    protected boolean do_load(){
	return true;
    }


    /**
     * play the sound from the URL SoundFile
     */
    abstract public boolean play();

    /**
     * play the sound from the URL SoundFile
     * @param start starttime in mseconds
     * @return "true" on success, "false" if playback failed or seeking unsupported
     */
    public boolean play(long start){
        return false;
    }
    /**
     * play the sound from the URL SoundFile
     * @param start starttime in mseconds
     * @param stop  stoptime in mseconds
     
     * @return "true" on success, "false" if playback failed or seeking unsupported
     */
    public boolean play(long start, long stop){
        return false;
    }
    
    /**
     * stop playing (i dont use "stop", because of multithreaded applets...)
     */
    public boolean pause(){
	return false;
    }

    private ArrayList<AbstractPlayerStateListener> m_listeners=null;
    /**
     * Add listener to be notified.
     * @param bpl
     */
    public void addAbstractPlayerStateListener(AbstractPlayerStateListener bpl)
    {
        if(m_listeners==null)
            m_listeners=new  ArrayList<AbstractPlayerStateListener>();
        m_listeners.add(bpl);
    }
    protected void notifyAbstractPlayerStateListeners(AbstractPlayerEvent e) {
        if(m_listeners==null)return;
        for(int i=0; i<m_listeners.size(); i++){
            AbstractPlayerStateListener bpl = (AbstractPlayerStateListener) m_listeners.get(i);
            bpl.stateUpdated(this, e);
        }
    }
}
