// SoundPlayer.java
// $Id: SoundPlayer.java 1786 2007-06-26 09:27:22Z zmoelnig $
// (c) Copyright IOhannes m zmlnig
// Institute for Electronic Music and Acoustics, Graz (IEM)
// 
// this code is licensed under the GNU General Public License (GnuGPL)
// see the LICENSE.txt file
//
// as a matter of facts, this code has just been ripped off SoundPlayer-example from the SUN-tutorials

package at.iem.tools.sound;

import java.io.File;
import java.net.URL;
import javazoom.jlgui.basicplayer.*;

public class SoundPlayer extends AbstractPlayer implements BasicPlayerListener{
    BasicPlayer player;
    BasicController control;

    float ms2bytes = 0.f;

    long length=-1;

    public SoundPlayer(){
	player = new BasicPlayer();
	control = (BasicController) player;
        player.addBasicPlayerListener(this);
    }
    public boolean load(String filename){
        at.iem.tools.Logger.println("SoundPlayer.loadstring: "+filename);
	if (filename==null)return false;
	try {
	    control.open(new File(filename));
	} catch (BasicPlayerException e){
	    System.err.println(e);
	    return false;
	}
	return true;
    }
    public boolean load(File file){
        at.iem.tools.Logger.println("SoundPlayer.loadfile: "+file);
	try {
	    control.open(file);
	} catch (BasicPlayerException e){
	    System.err.println(e);
	    return false;
	}
	return true;
    }

    public boolean load(URL url){
        at.iem.tools.Logger.println("SoundPlayer.loadurl: "+url);
	try {
	    control.open(url);
	} catch (BasicPlayerException e){
	    System.err.println(e);
	    return false;
	}
	return true;
    }

    public boolean play(){
        length =-1;
	try {
	    control.play();
	} catch (BasicPlayerException e){
	    System.err.println(e);
	    return false;
	}
	return true;
    }

    public boolean play(long start){
        long startbytes=(long)(start*ms2bytes);
        //if(startbytes>sound_bytes)return false;
        length =-1;
	try {
	    if(startbytes>0)
		control.seek(startbytes);
	    control.play();
	} catch (BasicPlayerException e){
	    System.err.println(e);
	    return false;
	}
	return true;
    }

    public boolean play(long start, long stop){
        long startbytes=(long)(start*ms2bytes);
        //if(startbytes>sound_bytes)return false;
        length =stop-start;
        System.out.println("SoundPlayer: playing from "+startbytes+" for "+length+" ms");
	try {
	    long s=0;
	    if(startbytes>0)
		s=control.seek(startbytes);
	    control.play();
	} catch (BasicPlayerException e){
	    System.err.println(e);
	    return false;
	}
	return true;
    }
    public boolean resume(){
	try {
	    control.resume();
	} catch (BasicPlayerException e){
	    System.err.println(e);
	    return false;
	}
	return true;
    }
    public boolean pause(){
	try {
	    control.pause();
	} catch (BasicPlayerException e){
	    System.err.println(e);
	    return false;
	}
	return true;
    }
    public boolean stop(){
        length =-1;
	try {
	    control.stop();
	} catch (BasicPlayerException e){
	    System.err.println(e);
	    return false;
	}
	return true;
    }
    public void setController(BasicController controller){
        //System.out.println("setController");
    }
    public void stateUpdated(BasicPlayerEvent event){
        //System.out.println("SoundPlayer:stateUpdated: "+event);
        AbstractPlayerEvent aevent;
        switch(event.getCode()) {
        case(BasicPlayerEvent.OPENING): aevent=new AbstractPlayerEvent(AbstractPlayerEvent.OPENING); 
            break;
        case(BasicPlayerEvent.OPENED): aevent=new AbstractPlayerEvent(AbstractPlayerEvent.OPENED); 
            break;
        case(BasicPlayerEvent.PLAYING): aevent=new AbstractPlayerEvent(AbstractPlayerEvent.PLAYING); 
            break;
        case(BasicPlayerEvent.STOPPED): aevent=new AbstractPlayerEvent(AbstractPlayerEvent.STOPPED);
            break;
        case(BasicPlayerEvent.PAUSED): aevent=new AbstractPlayerEvent(AbstractPlayerEvent.PAUSED); 
            break;
        case(BasicPlayerEvent.RESUMED): aevent=new AbstractPlayerEvent(AbstractPlayerEvent.RESUMED);
            break;
        case(BasicPlayerEvent.SEEKING): aevent=new AbstractPlayerEvent(AbstractPlayerEvent.SEEKING);
            break;
        case(BasicPlayerEvent.SEEKED): aevent=new AbstractPlayerEvent(AbstractPlayerEvent.SEEKED); 
            try {
                control.play();
            } catch (BasicPlayerException e){
                System.err.println(e);
            } 
            break;
        case(BasicPlayerEvent.EOM): aevent=new AbstractPlayerEvent(AbstractPlayerEvent.EOM); 
            break;
        case(BasicPlayerEvent.PAN): aevent=new AbstractPlayerEvent(AbstractPlayerEvent.PAN); 
            break;
        case(BasicPlayerEvent.GAIN): aevent=new AbstractPlayerEvent(AbstractPlayerEvent.GAIN); 
            break;
        default:
            aevent=new AbstractPlayerEvent(AbstractPlayerEvent.UNKNOWN); break;
        }

        notifyAbstractPlayerStateListeners(aevent);
    }
    public void progress(int bytesread, long microseconds, byte[] pcmdata, java.util.Map properties){
        //System.out.println("progress: "+microseconds+"usec ("+bytesread+"): "+(length*1000));
        //System.out.println("map: "+properties);

        if((length>0)&&(microseconds>=length*1000)){
            System.out.println("end reached: stopped at "+microseconds+"[us] > "+length+"[ms]");
            stop();
        }
    }
    public void opened(Object stream, java.util.Map properties){
	//System.out.println("opened stream: "+properties);

        String typ = (String)properties.get("audio.type");
        //System.out.println("TYPE: "+typ);
	ms2bytes=0;

        try {
            //System.out.println("...");
	    long sound_bytes=(Integer)properties.get(new String("audio.length.bytes"));
            //System.out.println("...+: "+sound_bytes);
	    long sound_length=(Long)properties.get(new String("duration")); /* [us] */
            //System.out.println("...-");
             ms2bytes=(float)sound_bytes*1000/sound_length;
             //System.out.println("OGG="+sound_bytes+": "+sound_length);
        } catch (Exception e) {
            //System.out.println("hmm:"+e);
	}

	try {	    
	    /* total bytes */
	    long sound_bytes=(Integer)properties.get(new String("audio.length.bytes"));
	    /* length in ms */
	    float sound_length=(Integer)(properties.get(new String("audio.length.frames")))/
		(Float)(properties.get(new String("audio.framerate.fps")))*1000;
	    //System.out.println("bytes: "+sound_bytes+" : "+sound_length);
	    
	    ms2bytes=(float)sound_bytes/sound_length;
	} catch (Exception e) {

	}
	//System.out.println("opened done: ms2bytes="+ms2bytes);
    }


}
